!--------------------------------------------------------------------------------------------------!
!   CP2K: A general program to perform molecular dynamics simulations                              !
!   Copyright (C) 2000 - 2016  CP2K developers group                                               !
!--------------------------------------------------------------------------------------------------!


! **************************************************************************************************
!> \brief various utilities that regard array of different kinds:
!>      output, allocation,...
!> \note
!>     ____              _ _     __  __           _ _  __         _____ _     _       _____ _ _      _
!>    |  _ \  ___  _ __ ( ) |_  |  \/  | ___   __| (_)/ _|_   _  |_   _| |__ (_)___  |  ___(_) | ___| |
!>    | | | |/ _ \| '_ \|/| __| | |\/| |/ _ \ / _` | | |_| | | |   | | | '_ \| / __| | |_  | | |/ _ \ |
!>    | |_| | (_) | | | | | |_  | |  | | (_) | (_| | |  _| |_| |   | | | | | | \__ \ |  _| | | |  __/_|
!>    |____/ \___/|_| |_|  \__| |_|  |_|\___/ \__,_|_|_|  \__, |   |_| |_| |_|_|___/ |_|   |_|_|\___(_)
!>                                                        |___/
!>      ____ _                  ___                              _ _       _       _
!>     / ___| | ___  ___  ___  |_ _|_ __ ___  _ __ ___   ___  __| (_) __ _| |_ ___| |_   _
!>    | |   | |/ _ \/ __|/ _ \  | || '_ ` _ \| '_ ` _ \ / _ \/ _` | |/ _` | __/ _ \ | | | |
!>    | |___| | (_) \__ \  __/  | || | | | | | | | | | |  __/ (_| | | (_| | ||  __/ | |_| |
!>     \____|_|\___/|___/\___| |___|_| |_| |_|_| |_| |_|\___|\__,_|_|\__,_|\__\___|_|\__, |
!>                                                                                   |___/
!>     _____ _     _       _____ _ _      _
!>    |_   _| |__ (_)___  |  ___(_) | ___| |
!>      | | | '_ \| / __| | |_  | | |/ _ \ |
!>      | | | | | | \__ \ |  _| | | |  __/_|
!>      |_| |_| |_|_|___/ |_|   |_|_|\___(_)
!>
!>      This is a template
!>
!>      **** DO NOT MODIFY THE .F FILES ****
!>      modify the .template and .instantition instead
!>
!>      maybe it is not a good idea mixing output and memeory utils...
!> \par History
!>      12.2001 first version [fawzi]
!>      3.2002 templatized [fawzi]
!> \author Fawzi Mohamed
! **************************************************************************************************
MODULE cp_array_utils_logical
   USE cp_log_handling,                 ONLY: cp_to_string
   ! dummy line needed for coverage broadcasting
   USE machine,                         ONLY: m_flush
! Basic use statements and preprocessor macros
! should be included in the use statements

  USE base_hooks,                      ONLY: cp__a,&
                                             cp__b,&
                                             cp__w,&
                                             cp__l,&
                                             cp_abort,&
                                             cp_warn,&
                                             timeset,&
                                             timestop


! Dangerous: Full path can be arbitrarily long and might overflow Fortran line.


! The MARK_USED macro can be used to mark an argument/variable as used.
! It is intended to make it possible to switch on -Werror=unused-dummy-argument,
! but deal elegantly with e.g. library wrapper routines that take arguments only used if the library is linked in. 
! This code should be valid for any Fortran variable, is always standard conforming,
! and will be optimized away completely by the compiler

   IMPLICIT NONE
   PRIVATE

   LOGICAL, PRIVATE, PARAMETER :: debug_this_module = .TRUE.
   CHARACTER(len=*), PARAMETER, PRIVATE :: moduleN = 'cp_array_utils_logical'

   ! types
   PUBLIC :: cp_1d_logical_p_type, cp_2d_logical_p_type, cp_3d_logical_p_type

   ! generic interfaces
   PUBLIC :: cp_guarantee_size

   ! the underlying procedures
   PUBLIC :: cp_1d_logical_guarantee_size, &
             cp_1d_logical_write, cp_2d_logical_write, &
             cp_2d_logical_guarantee_size, &
             cp_1d_logical_bsearch

   INTERFACE cp_guarantee_size
      MODULE PROCEDURE cp_1d_logical_guarantee_size, &
         cp_2d_logical_guarantee_size
   END INTERFACE

!***

! **************************************************************************************************
!> \brief represent a pointer to a 1d array
!> \par History
!>      02.2003 created [fawzi]
!> \author fawzi
! **************************************************************************************************
   TYPE cp_1d_logical_p_type
      LOGICAL, DIMENSION(:), POINTER :: array
   END TYPE cp_1d_logical_p_type

! **************************************************************************************************
!> \brief represent a pointer to a 2d array
!> \par History
!>      02.2003 created [fawzi]
!> \author fawzi
! **************************************************************************************************
   TYPE cp_2d_logical_p_type
      LOGICAL, DIMENSION(:, :), POINTER :: array
   END TYPE cp_2d_logical_p_type

! **************************************************************************************************
!> \brief represent a pointer to a 3d array
!> \par History
!>      02.2003 created [fawzi]
!> \author fawzi
! **************************************************************************************************
   TYPE cp_3d_logical_p_type
      LOGICAL, DIMENSION(:, :, :), POINTER :: array
   END TYPE cp_3d_logical_p_type

CONTAINS

! **************************************************************************************************
!> \brief writes an array to the given unit
!> \param array the array to write
!> \param unit_nr the unit to write to (defaults to the standard out)
!> \param el_format the format of a single element
!> \par History
!>      4.2002 created [fawzi]
!> \author Fawzi Mohamed
!> \note
!>      maybe I will move to a comma separated paretized list
! **************************************************************************************************
   SUBROUTINE cp_1d_logical_write(array, unit_nr, el_format)
      LOGICAL, INTENT(in)                                :: array(:)
      INTEGER, INTENT(in)                                :: unit_nr
      CHARACTER(len=*), INTENT(in), OPTIONAL             :: el_format

      CHARACTER(len=*), PARAMETER :: defaultFormat = "(l1)", routineN = 'cp_1d_logical_write', &
         routineP = moduleN//':'//routineN

      INTEGER                                            :: i, iostat

      WRITE (unit=unit_nr, fmt="('( ')", advance="no", iostat=iostat)
      IF(.NOT.(iostat == 0))CALL cp__a("common/cp_array_utils_logical.F",126)
      IF (PRESENT(el_format)) THEN
         DO i = 1, SIZE(array)-1
            WRITE (unit=unit_nr, fmt=el_format, advance="no") array(i)
            IF (MOD(i, 5) .EQ. 0) THEN ! only a few elements per line
               WRITE (unit=unit_nr, fmt="(',')")
            ELSE
               WRITE (unit=unit_nr, fmt="(',')", advance="no")
            ENDIF
         END DO
         IF (SIZE(array) > 0) &
            WRITE (unit=unit_nr, fmt=el_format, advance="no") array(SIZE(array))
      ELSE
         DO i = 1, SIZE(array)-1
            WRITE (unit=unit_nr, fmt=defaultFormat, advance="no") array(i)
            IF (MOD(i, 5) .EQ. 0) THEN ! only a few elements per line
               WRITE (unit=unit_nr, fmt="(',')")
            ELSE
               WRITE (unit=unit_nr, fmt="(',')", advance="no")
            ENDIF
         END DO
         IF (SIZE(array) > 0) &
            WRITE (unit=unit_nr, fmt=defaultFormat, advance="no") array(SIZE(array))
      END IF
      WRITE (unit=unit_nr, fmt="(' )')")
      CALL m_flush(unit_nr)

   END SUBROUTINE cp_1d_logical_write

! **************************************************************************************************
!> \brief writes an array to the given unit
!> \param array the array to write
!> \param unit_nr the unit to write to (defaults to the standard out)
!> \param el_format the format of a single element
!> \par History
!>      4.2002 created [fawzi]
!> \author Fawzi Mohamed
!> \note
!>      maybe I will move to a comma separated parentized list
! **************************************************************************************************
   SUBROUTINE cp_2d_logical_write(array, unit_nr, el_format)
      LOGICAL, INTENT(in)                                :: array(:, :)
      INTEGER, INTENT(in)                                :: unit_nr
      CHARACTER(len=*), INTENT(in), OPTIONAL             :: el_format

      CHARACTER(len=*), PARAMETER :: defaultFormat = "(l1)", routineN = 'cp_2d_logical_write', &
         routineP = moduleN//':'//routineN

      CHARACTER(len=10)                                  :: nRiga
      CHARACTER(len=200)                                 :: fmtstr
      INTEGER                                            :: i, iostat

      nRiga = cp_to_string(SIZE(array, 2))
      DO i = 1, SIZE(array, 1)
         IF (PRESENT(el_format)) THEN
            fmtstr = '(" ",'//nRiga//el_format//')'
            WRITE (unit=unit_nr, fmt=fmtstr, iostat=iostat) array(i, :)
         ELSE
            fmtstr = '(" ",'//nRiga//defaultFormat//')'
            WRITE (unit=unit_nr, fmt=fmtstr, iostat=iostat) array(i, :)
         END IF
         IF(.NOT.(iostat == 0))CALL cp__a("common/cp_array_utils_logical.F",187)
      END DO
      CALL m_flush(unit_nr)
   END SUBROUTINE cp_2d_logical_write

! **************************************************************************************************
!> \brief If the size of the array is changes reallocate it.
!>      Issues a warning when the size changes (but not on allocation
!>      and deallocation).
!>
!>      The data is NOT preserved (if you want to preserve the data see
!>      the realloc in the module memory_utilities)
!> \param array the array to reallocate if necessary
!> \param n the wanted size
!> \par History
!>      12.2001 first version [fawzi]
!>      3.2002 templatized [fawzi]
!> \author Fawzi Mohamed
!> \note
!>      this is a different behaviour than the realloc in the module
!>      memory_utilities. It is quite low level
! **************************************************************************************************
   SUBROUTINE cp_1d_logical_guarantee_size(array, n)
      LOGICAL, POINTER                                   :: array(:)
      INTEGER, INTENT(in)                                :: n

      CHARACTER(len=*), PARAMETER :: routineN = 'cp_1d_logical_guarantee_size', &
         routineP = moduleN//':'//routineN

      IF(.NOT.(n >= 0))CALL cp__a("common/cp_array_utils_logical.F",216)
      IF (ASSOCIATED(array)) THEN
         IF (SIZE(array) /= n) THEN
            CALL cp__w("common/cp_array_utils_logical.F",219,'size has changed')
            DEALLOCATE (array)
         END IF
      END IF
      IF (.NOT. ASSOCIATED(array)) THEN
         ALLOCATE (array(n))
      END IF
   END SUBROUTINE cp_1d_logical_guarantee_size

! **************************************************************************************************
!> \brief If the size of the array is changes reallocate it.
!>      Issues a warning when the size changes (but not on allocation
!>      and deallocation).
!>
!>      The data is NOT preserved (if you want to preserve the data see
!>      the realloc in the module memory_utilities)
!> \param array the array to reallocate if necessary
!> \param n_rows the wanted number of rows
!> \param n_cols the wanted number of cols
!> \par History
!>      5.2001 first version [fawzi]
!> \author Fawzi Mohamed
!> \note
!>      this is a different behaviour than the realloc in the module
!>      memory_utilities. It is quite low level
! **************************************************************************************************
   SUBROUTINE cp_2d_logical_guarantee_size(array, n_rows, n_cols)
      LOGICAL, POINTER                                   :: array(:, :)
      INTEGER, INTENT(in)                                :: n_rows, n_cols

      CHARACTER(len=*), PARAMETER :: routineN = 'cp_2d_logical_guarantee_size', &
         routineP = moduleN//':'//routineN

      IF(.NOT.(n_cols >= 0))CALL cp__a("common/cp_array_utils_logical.F",252)
      IF(.NOT.(n_rows >= 0))CALL cp__a("common/cp_array_utils_logical.F",253)
      IF (ASSOCIATED(array)) THEN
         IF (SIZE(array, 1) /= n_rows .OR. SIZE(array, 2) /= n_cols) THEN
            CALL cp__w("common/cp_array_utils_logical.F",256,'size has changed')
            DEALLOCATE (array)
         END IF
      END IF
      IF (.NOT. ASSOCIATED(array)) THEN
         ALLOCATE (array(n_rows, n_cols))
      END IF
   END SUBROUTINE cp_2d_logical_guarantee_size

! **************************************************************************************************
!> \brief returns the index at which the element el should be inserted in the
!>      array to keep it ordered (array(i)>=el).
!>      If the element is bigger than all the elements in the array returns
!>      the last index+1.
!> \param array the array to search
!> \param el the element to look for
!> \param l_index the lower index for binary search (defaults to 1)
!> \param u_index the upper index for binary search (defaults to size(array))
!> \retval res ...
!> \par History
!>      06.2003 created [fawzi]
!> \author Fawzi Mohamed
!> \note
!>      the array should be ordered in growing order
! **************************************************************************************************
   FUNCTION cp_1d_logical_bsearch(array, el, l_index, u_index) &
      RESULT(res)
      LOGICAL, INTENT(in)                                :: array(:), el
      INTEGER, INTENT(in), OPTIONAL                      :: l_index, u_index
      INTEGER                                            :: res

      CHARACTER(len=*), PARAMETER :: routineN = 'cp_1d_logical_bsearch', &
         routineP = moduleN//':'//routineN

      INTEGER                                            :: aindex, lindex, uindex

      lindex = 1
      uindex = SIZE(array)
      IF (PRESENT(l_index)) lindex = l_index
      IF (PRESENT(u_index)) uindex = u_index
      DO WHILE (lindex <= uindex)
         aindex = (lindex+uindex)/2
         IF (( (.not.array(aindex)).and. el )) THEN
            lindex = aindex+1
         ELSE
            uindex = aindex-1
         END IF
      END DO
      res = lindex
   END FUNCTION cp_1d_logical_bsearch

   ! template def put here so that line numbers in template and derived
   ! files are almost the same (multi-line use change it a bit)
   ! [template(type1,nametype1,USE,defaultFormatType1,lessQ,defines)]
! ARGS:
!  USE = "! dummy line needed for coverage broadcasting"
!  defaultFormatType1 = ""(l1)""
!  defines =
!    "#define CP_L_LESS_Q(el1,el2) ( (.not.el1).and.el2 )
!     "
!  lessQ = "CP_L_LESS_Q"
!  nametype1 = "logical"
!  type1 = "logical"

END MODULE cp_array_utils_logical
!--------------------------------------------------------------------------------------------------!
!   CP2K: A general program to perform molecular dynamics simulations                              !
!   Copyright (C) 2000 - 2016  CP2K developers group                                               !
!--------------------------------------------------------------------------------------------------!

! ideally users test for the optional __GRID_CORE macro definition
! otherwise, we provide some reasonable defaults
! the included fortran files represent the same kernel, with a fortran representation that optimizes best for a given compiler




  SUBROUTINE collocate_core_default(grid,coef_xyz,pol_x,pol_y,pol_z,map,sphere_bounds,lp,cmax,gridbounds)
    USE kinds,                        ONLY: dp
    INTEGER, INTENT(IN)                      :: sphere_bounds(*), lp
    REAL(dp), INTENT(IN) :: coef_xyz(((lp+1)*(lp+2)*(lp+3))/6)
    INTEGER, INTENT(IN)                      :: cmax
    REAL(dp), INTENT(IN)                     :: pol_x(0:lp,-cmax:cmax), &
                                                pol_y(1:2,0:lp,-cmax:0), &
                                                pol_z(1:2,0:lp,-cmax:0)
    INTEGER, INTENT(IN)                      :: map(-cmax:cmax,1:3), &
                                                gridbounds(2,3)
    REAL(dp), INTENT(INOUT) :: grid(gridbounds(1,1):gridbounds(2,1), &
      gridbounds(1,2):gridbounds(2,2), gridbounds(1,3):gridbounds(2,3))

    INTEGER                                  :: i, ig, igmax, igmin, j, j2, &
                                                jg, jg2, jgmin, k, k2, kg, &
                                                kg2, kgmin, lxp, lxy, lxyz, &
                                                lyp, lzp, sci
    REAL(dp)                                 :: coef_x(4,0:lp), &
                                                coef_xy(2,(lp+1)*(lp+2)/2), &
                                                s01, s02, s03, s04

    sci=1

    kgmin=sphere_bounds(sci)
    sci=sci+1
    DO kg=kgmin,0
       kg2=1-kg
       k=map(kg,3)
       k2=map(kg2,3)

       coef_xy=0.0_dp
       lxyz = 0
       DO lzp=0,lp
          lxy=0
          DO lyp=0,lp-lzp
             DO lxp=0,lp-lzp-lyp
                lxyz=lxyz+1 ; lxy=lxy+1
                coef_xy(1,lxy)=coef_xy(1,lxy)+coef_xyz(lxyz)*pol_z(1,lzp,kg)
                coef_xy(2,lxy)=coef_xy(2,lxy)+coef_xyz(lxyz)*pol_z(2,lzp,kg)
             ENDDO
             lxy=lxy+lzp
          ENDDO
       ENDDO

       jgmin=sphere_bounds(sci)
       sci=sci+1
       DO jg=jgmin,0
          jg2=1-jg
          j=map(jg,2)
          j2=map(jg2,2)
          igmin=sphere_bounds(sci)
          sci=sci+1
          igmax=1-igmin

          coef_x=0.0_dp
          lxy=0
          DO lyp=0,lp
          DO lxp=0,lp-lyp
             lxy=lxy+1
             coef_x(1,lxp)=coef_x(1,lxp)+coef_xy(1,lxy)*pol_y(1,lyp,jg)
             coef_x(2,lxp)=coef_x(2,lxp)+coef_xy(2,lxy)*pol_y(1,lyp,jg)
             coef_x(3,lxp)=coef_x(3,lxp)+coef_xy(1,lxy)*pol_y(2,lyp,jg)
             coef_x(4,lxp)=coef_x(4,lxp)+coef_xy(2,lxy)*pol_y(2,lyp,jg)
          ENDDO
          ENDDO

          DO ig=igmin,igmax
             i=map(ig,1)
             s01=0.0_dp
             s02=0.0_dp
             s03=0.0_dp
             s04=0.0_dp
             DO lxp=0,lp
                s01=s01+coef_x(1,lxp)*pol_x(lxp,ig)
                s02=s02+coef_x(2,lxp)*pol_x(lxp,ig)
                s03=s03+coef_x(3,lxp)*pol_x(lxp,ig)
                s04=s04+coef_x(4,lxp)*pol_x(lxp,ig)
             ENDDO
             grid(i,j,k) = grid(i,j,k)     + s01
             grid(i,j2,k) = grid(i,j2,k)   + s03
             grid(i,j,k2) = grid(i,j,k2)   + s02
             grid(i,j2,k2) = grid(i,j2,k2) + s04
          END DO

       END DO
    END DO

  END SUBROUTINE collocate_core_default
  SUBROUTINE collocate_core_0(grid,coef_xyz,pol_x,pol_y,pol_z,map,sphere_bounds,cmax,gridbounds)
    USE kinds,                        ONLY: dp
    INTEGER, INTENT(IN)                      :: sphere_bounds(*), cmax, &
                                                map(-cmax:cmax,1:3), &
                                                gridbounds(2,3)
    REAL(dp), INTENT(INOUT) :: grid(gridbounds(1,1):gridbounds(2,1), &
      gridbounds(1,2):gridbounds(2,2), gridbounds(1,3):gridbounds(2,3))
    INTEGER, PARAMETER                       :: lp = 0
    REAL(dp), INTENT(IN) :: pol_x(0:lp,-cmax:cmax), pol_y(1:2,0:lp,-cmax:0), &
      pol_z(1:2,0:lp,-cmax:0), coef_xyz(((lp+1)*(lp+2)*(lp+3))/6)

    INTEGER                                  :: i, ig, igmax, igmin, j, j2, &
                                                jg, jg2, jgmin, k, k2, kg, &
                                                kg2, kgmin, lxp, lxy, lyp, sci
    REAL(dp)                                 :: coef_x(4,0:lp), &
                                                coef_xy(2,(lp+1)*(lp+2)/2), &
                                                s01, s02, s03, s04

    sci=1

    kgmin=sphere_bounds(sci)
    sci=sci+1
    DO kg=kgmin,0
       kg2=1-kg
       k=map(kg,3)
       k2=map(kg2,3)
       coef_xy=0.0_dp
                coef_xy(:,1)=coef_xy(:,1)+coef_xyz(1)*pol_z(:,0,kg)
       jgmin=sphere_bounds(sci)
       sci=sci+1
       DO jg=jgmin,0
          jg2=1-jg
          j=map(jg,2)
          j2=map(jg2,2)
          igmin=sphere_bounds(sci)
          sci=sci+1
          igmax=1-igmin
          coef_x=0.0_dp
          lxy=0
          DO lyp=0,lp
          DO lxp=0,lp-lyp
             lxy=lxy+1
             coef_x(1,lxp)=coef_x(1,lxp)+coef_xy(1,lxy)*pol_y(1,lyp,jg)
             coef_x(2,lxp)=coef_x(2,lxp)+coef_xy(2,lxy)*pol_y(1,lyp,jg)
             coef_x(3,lxp)=coef_x(3,lxp)+coef_xy(1,lxy)*pol_y(2,lyp,jg)
             coef_x(4,lxp)=coef_x(4,lxp)+coef_xy(2,lxy)*pol_y(2,lyp,jg)
          ENDDO
          ENDDO
          DO ig=igmin,igmax
             i=map(ig,1)
             s01=0.0_dp
             s02=0.0_dp
             s03=0.0_dp
             s04=0.0_dp
             DO lxp=0,lp
                s01=s01+coef_x(1,lxp)*pol_x(lxp,ig)
                s02=s02+coef_x(2,lxp)*pol_x(lxp,ig)
                s03=s03+coef_x(3,lxp)*pol_x(lxp,ig)
                s04=s04+coef_x(4,lxp)*pol_x(lxp,ig)
             ENDDO
             grid(i,j,k) = grid(i,j,k)     + s01
             grid(i,j2,k) = grid(i,j2,k)   + s03
             grid(i,j,k2) = grid(i,j,k2)   + s02
             grid(i,j2,k2) = grid(i,j2,k2) + s04
          END DO
       END DO
    END DO

  END SUBROUTINE collocate_core_0
  SUBROUTINE collocate_core_1(grid,coef_xyz,pol_x,pol_y,pol_z,map,sphere_bounds,cmax,gridbounds)
    USE kinds,                        ONLY: dp
    INTEGER, INTENT(IN)                      :: sphere_bounds(*), cmax, &
                                                map(-cmax:cmax,1:3), &
                                                gridbounds(2,3)
    REAL(dp), INTENT(INOUT) :: grid(gridbounds(1,1):gridbounds(2,1), &
      gridbounds(1,2):gridbounds(2,2), gridbounds(1,3):gridbounds(2,3))
    INTEGER, PARAMETER                       :: lp = 1
    REAL(dp), INTENT(IN) :: pol_x(0:lp,-cmax:cmax), pol_y(1:2,0:lp,-cmax:0), &
      pol_z(1:2,0:lp,-cmax:0), coef_xyz(((lp+1)*(lp+2)*(lp+3))/6)

    INTEGER                                  :: i, ig, igmax, igmin, j, j2, &
                                                jg, jg2, jgmin, k, k2, kg, &
                                                kg2, kgmin, lxp, sci
    REAL(dp)                                 :: coef_x(4,0:lp), &
                                                coef_xy(2,(lp+1)*(lp+2)/2), &
                                                s01, s02, s03, s04

    sci=1

    kgmin=sphere_bounds(sci)
    sci=sci+1
    DO kg=kgmin,0
       kg2=1-kg
       k=map(kg,3)
       k2=map(kg2,3)
       coef_xy=0.0_dp
                coef_xy(:,1)=coef_xy(:,1)+coef_xyz(1)*pol_z(:,0,kg)
                coef_xy(:,2)=coef_xy(:,2)+coef_xyz(2)*pol_z(:,0,kg)
                coef_xy(:,3)=coef_xy(:,3)+coef_xyz(3)*pol_z(:,0,kg)
                coef_xy(:,1)=coef_xy(:,1)+coef_xyz(4)*pol_z(:,1,kg)
       jgmin=sphere_bounds(sci)
       sci=sci+1
       DO jg=jgmin,0
          jg2=1-jg
          j=map(jg,2)
          j2=map(jg2,2)
          igmin=sphere_bounds(sci)
          sci=sci+1
          igmax=1-igmin
          coef_x=0.0_dp
             coef_x(1:2,0)=coef_x(1:2,0)+coef_xy(1:2,1)*pol_y(1,0,jg)
             coef_x(3:4,0)=coef_x(3:4,0)+coef_xy(1:2,1)*pol_y(2,0,jg)
             coef_x(1:2,1)=coef_x(1:2,1)+coef_xy(1:2,2)*pol_y(1,0,jg)
             coef_x(3:4,1)=coef_x(3:4,1)+coef_xy(1:2,2)*pol_y(2,0,jg)
             coef_x(1:2,0)=coef_x(1:2,0)+coef_xy(1:2,3)*pol_y(1,1,jg)
             coef_x(3:4,0)=coef_x(3:4,0)+coef_xy(1:2,3)*pol_y(2,1,jg)
          DO ig=igmin,igmax
             i=map(ig,1)
             s01=0.0_dp
             s02=0.0_dp
             s03=0.0_dp
             s04=0.0_dp
             DO lxp=0,lp
                s01=s01+coef_x(1,lxp)*pol_x(lxp,ig)
                s02=s02+coef_x(2,lxp)*pol_x(lxp,ig)
                s03=s03+coef_x(3,lxp)*pol_x(lxp,ig)
                s04=s04+coef_x(4,lxp)*pol_x(lxp,ig)
             ENDDO
             grid(i,j,k) = grid(i,j,k)     + s01
             grid(i,j2,k) = grid(i,j2,k)   + s03
             grid(i,j,k2) = grid(i,j,k2)   + s02
             grid(i,j2,k2) = grid(i,j2,k2) + s04
          END DO
       END DO
    END DO

  END SUBROUTINE collocate_core_1
  SUBROUTINE collocate_core_2(grid,coef_xyz,pol_x,pol_y,pol_z,map,sphere_bounds,cmax,gridbounds)
    USE kinds,                        ONLY: dp
    INTEGER, INTENT(IN)                      :: sphere_bounds(*), cmax, &
                                                map(-cmax:cmax,1:3), &
                                                gridbounds(2,3)
    REAL(dp), INTENT(INOUT) :: grid(gridbounds(1,1):gridbounds(2,1), &
      gridbounds(1,2):gridbounds(2,2), gridbounds(1,3):gridbounds(2,3))
    INTEGER, PARAMETER                       :: lp = 2
    REAL(dp), INTENT(IN) :: pol_x(0:lp,-cmax:cmax), pol_y(1:2,0:lp,-cmax:0), &
      pol_z(1:2,0:lp,-cmax:0), coef_xyz(((lp+1)*(lp+2)*(lp+3))/6)

    INTEGER                                  :: i, ig, igmax, igmin, j, j2, &
                                                jg, jg2, jgmin, k, k2, kg, &
                                                kg2, kgmin, sci
    REAL(dp)                                 :: coef_x(4,0:lp), &
                                                coef_xy(2,(lp+1)*(lp+2)/2), &
                                                s01, s02, s03, s04

    sci=1

    kgmin=sphere_bounds(sci)
    sci=sci+1
    DO kg=kgmin,0
       kg2=1-kg
       k=map(kg,3)
       k2=map(kg2,3)
       coef_xy=0.0_dp
                coef_xy(1,1)=coef_xy(1,1)+coef_xyz(1)*pol_z(1,0,kg)
                coef_xy(2,1)=coef_xy(2,1)+coef_xyz(1)*pol_z(2,0,kg)
                coef_xy(1,2)=coef_xy(1,2)+coef_xyz(2)*pol_z(1,0,kg)
                coef_xy(2,2)=coef_xy(2,2)+coef_xyz(2)*pol_z(2,0,kg)
                coef_xy(1,3)=coef_xy(1,3)+coef_xyz(3)*pol_z(1,0,kg)
                coef_xy(2,3)=coef_xy(2,3)+coef_xyz(3)*pol_z(2,0,kg)
                coef_xy(1,4)=coef_xy(1,4)+coef_xyz(4)*pol_z(1,0,kg)
                coef_xy(2,4)=coef_xy(2,4)+coef_xyz(4)*pol_z(2,0,kg)
                coef_xy(1,5)=coef_xy(1,5)+coef_xyz(5)*pol_z(1,0,kg)
                coef_xy(2,5)=coef_xy(2,5)+coef_xyz(5)*pol_z(2,0,kg)
                coef_xy(1,6)=coef_xy(1,6)+coef_xyz(6)*pol_z(1,0,kg)
                coef_xy(2,6)=coef_xy(2,6)+coef_xyz(6)*pol_z(2,0,kg)
                coef_xy(1,1)=coef_xy(1,1)+coef_xyz(7)*pol_z(1,1,kg)
                coef_xy(2,1)=coef_xy(2,1)+coef_xyz(7)*pol_z(2,1,kg)
                coef_xy(1,2)=coef_xy(1,2)+coef_xyz(8)*pol_z(1,1,kg)
                coef_xy(2,2)=coef_xy(2,2)+coef_xyz(8)*pol_z(2,1,kg)
                coef_xy(1,4)=coef_xy(1,4)+coef_xyz(9)*pol_z(1,1,kg)
                coef_xy(2,4)=coef_xy(2,4)+coef_xyz(9)*pol_z(2,1,kg)
                coef_xy(1,1)=coef_xy(1,1)+coef_xyz(10)*pol_z(1,2,kg)
                coef_xy(2,1)=coef_xy(2,1)+coef_xyz(10)*pol_z(2,2,kg)
       jgmin=sphere_bounds(sci)
       sci=sci+1
       DO jg=jgmin,0
          jg2=1-jg
          j=map(jg,2)
          j2=map(jg2,2)
          igmin=sphere_bounds(sci)
          sci=sci+1
          igmax=1-igmin
          coef_x=0.0_dp
             coef_x(1:2,0)=coef_x(1:2,0)+coef_xy(1:2,1)*pol_y(1,0,jg)
             coef_x(3:4,0)=coef_x(3:4,0)+coef_xy(1:2,1)*pol_y(2,0,jg)
             coef_x(1:2,1)=coef_x(1:2,1)+coef_xy(1:2,2)*pol_y(1,0,jg)
             coef_x(3:4,1)=coef_x(3:4,1)+coef_xy(1:2,2)*pol_y(2,0,jg)
             coef_x(1:2,2)=coef_x(1:2,2)+coef_xy(1:2,3)*pol_y(1,0,jg)
             coef_x(3:4,2)=coef_x(3:4,2)+coef_xy(1:2,3)*pol_y(2,0,jg)
             coef_x(1:2,0)=coef_x(1:2,0)+coef_xy(1:2,4)*pol_y(1,1,jg)
             coef_x(3:4,0)=coef_x(3:4,0)+coef_xy(1:2,4)*pol_y(2,1,jg)
             coef_x(1:2,1)=coef_x(1:2,1)+coef_xy(1:2,5)*pol_y(1,1,jg)
             coef_x(3:4,1)=coef_x(3:4,1)+coef_xy(1:2,5)*pol_y(2,1,jg)
             coef_x(1:2,0)=coef_x(1:2,0)+coef_xy(1:2,6)*pol_y(1,2,jg)
             coef_x(3:4,0)=coef_x(3:4,0)+coef_xy(1:2,6)*pol_y(2,2,jg)
          DO ig=igmin,igmax
             i=map(ig,1)
             s01=0.0_dp
             s02=0.0_dp
             s03=0.0_dp
             s04=0.0_dp
                s01=s01+coef_x(1,0)*pol_x(0,ig)
                s02=s02+coef_x(2,0)*pol_x(0,ig)
                s03=s03+coef_x(3,0)*pol_x(0,ig)
                s04=s04+coef_x(4,0)*pol_x(0,ig)
                s01=s01+coef_x(1,1)*pol_x(1,ig)
                s02=s02+coef_x(2,1)*pol_x(1,ig)
                s03=s03+coef_x(3,1)*pol_x(1,ig)
                s04=s04+coef_x(4,1)*pol_x(1,ig)
                s01=s01+coef_x(1,2)*pol_x(2,ig)
                s02=s02+coef_x(2,2)*pol_x(2,ig)
                s03=s03+coef_x(3,2)*pol_x(2,ig)
                s04=s04+coef_x(4,2)*pol_x(2,ig)
             grid(i,j,k) = grid(i,j,k)     + s01
             grid(i,j2,k) = grid(i,j2,k)   + s03
             grid(i,j,k2) = grid(i,j,k2)   + s02
             grid(i,j2,k2) = grid(i,j2,k2) + s04
          END DO
       END DO
    END DO

  END SUBROUTINE collocate_core_2
  SUBROUTINE collocate_core_3(grid,coef_xyz,pol_x,pol_y,pol_z,map,sphere_bounds,cmax,gridbounds)
    USE kinds,                        ONLY: dp
    INTEGER, INTENT(IN)                      :: sphere_bounds(*), cmax, &
                                                map(-cmax:cmax,1:3), &
                                                gridbounds(2,3)
    REAL(dp), INTENT(INOUT) :: grid(gridbounds(1,1):gridbounds(2,1), &
      gridbounds(1,2):gridbounds(2,2), gridbounds(1,3):gridbounds(2,3))
    INTEGER, PARAMETER                       :: lp = 3
    REAL(dp), INTENT(IN) :: pol_x(0:lp,-cmax:cmax), pol_y(1:2,0:lp,-cmax:0), &
      pol_z(1:2,0:lp,-cmax:0), coef_xyz(((lp+1)*(lp+2)*(lp+3))/6)

    INTEGER                                  :: i, ig, igmax, igmin, j, j2, &
                                                jg, jg2, jgmin, k, k2, kg, &
                                                kg2, kgmin, lxp, sci
    REAL(dp)                                 :: coef_x(4,0:lp), &
                                                coef_xy(2,(lp+1)*(lp+2)/2), &
                                                s01, s02, s03, s04

    sci=1

    kgmin=sphere_bounds(sci)
    sci=sci+1
    DO kg=kgmin,0
       kg2=1-kg
       k=map(kg,3)
       k2=map(kg2,3)
       coef_xy=0.0_dp
                coef_xy(:,1)=coef_xy(:,1)+coef_xyz(1)*pol_z(:,0,kg)
                coef_xy(:,2)=coef_xy(:,2)+coef_xyz(2)*pol_z(:,0,kg)
                coef_xy(:,3)=coef_xy(:,3)+coef_xyz(3)*pol_z(:,0,kg)
                coef_xy(:,4)=coef_xy(:,4)+coef_xyz(4)*pol_z(:,0,kg)
                coef_xy(:,5)=coef_xy(:,5)+coef_xyz(5)*pol_z(:,0,kg)
                coef_xy(:,6)=coef_xy(:,6)+coef_xyz(6)*pol_z(:,0,kg)
                coef_xy(:,7)=coef_xy(:,7)+coef_xyz(7)*pol_z(:,0,kg)
                coef_xy(:,8)=coef_xy(:,8)+coef_xyz(8)*pol_z(:,0,kg)
                coef_xy(:,9)=coef_xy(:,9)+coef_xyz(9)*pol_z(:,0,kg)
                coef_xy(:,10)=coef_xy(:,10)+coef_xyz(10)*pol_z(:,0,kg)
                coef_xy(:,1)=coef_xy(:,1)+coef_xyz(11)*pol_z(:,1,kg)
                coef_xy(:,2)=coef_xy(:,2)+coef_xyz(12)*pol_z(:,1,kg)
                coef_xy(:,3)=coef_xy(:,3)+coef_xyz(13)*pol_z(:,1,kg)
                coef_xy(:,5)=coef_xy(:,5)+coef_xyz(14)*pol_z(:,1,kg)
                coef_xy(:,6)=coef_xy(:,6)+coef_xyz(15)*pol_z(:,1,kg)
                coef_xy(:,8)=coef_xy(:,8)+coef_xyz(16)*pol_z(:,1,kg)
                coef_xy(:,1)=coef_xy(:,1)+coef_xyz(17)*pol_z(:,2,kg)
                coef_xy(:,2)=coef_xy(:,2)+coef_xyz(18)*pol_z(:,2,kg)
                coef_xy(:,5)=coef_xy(:,5)+coef_xyz(19)*pol_z(:,2,kg)
                coef_xy(:,1)=coef_xy(:,1)+coef_xyz(20)*pol_z(:,3,kg)
       jgmin=sphere_bounds(sci)
       sci=sci+1
       DO jg=jgmin,0
          jg2=1-jg
          j=map(jg,2)
          j2=map(jg2,2)
          igmin=sphere_bounds(sci)
          sci=sci+1
          igmax=1-igmin
          coef_x=0.0_dp
             coef_x(1,0)=coef_x(1,0)+coef_xy(1,1)*pol_y(1,0,jg)
             coef_x(2,0)=coef_x(2,0)+coef_xy(2,1)*pol_y(1,0,jg)
             coef_x(3,0)=coef_x(3,0)+coef_xy(1,1)*pol_y(2,0,jg)
             coef_x(4,0)=coef_x(4,0)+coef_xy(2,1)*pol_y(2,0,jg)
             coef_x(1,1)=coef_x(1,1)+coef_xy(1,2)*pol_y(1,0,jg)
             coef_x(2,1)=coef_x(2,1)+coef_xy(2,2)*pol_y(1,0,jg)
             coef_x(3,1)=coef_x(3,1)+coef_xy(1,2)*pol_y(2,0,jg)
             coef_x(4,1)=coef_x(4,1)+coef_xy(2,2)*pol_y(2,0,jg)
             coef_x(1,2)=coef_x(1,2)+coef_xy(1,3)*pol_y(1,0,jg)
             coef_x(2,2)=coef_x(2,2)+coef_xy(2,3)*pol_y(1,0,jg)
             coef_x(3,2)=coef_x(3,2)+coef_xy(1,3)*pol_y(2,0,jg)
             coef_x(4,2)=coef_x(4,2)+coef_xy(2,3)*pol_y(2,0,jg)
             coef_x(1,3)=coef_x(1,3)+coef_xy(1,4)*pol_y(1,0,jg)
             coef_x(2,3)=coef_x(2,3)+coef_xy(2,4)*pol_y(1,0,jg)
             coef_x(3,3)=coef_x(3,3)+coef_xy(1,4)*pol_y(2,0,jg)
             coef_x(4,3)=coef_x(4,3)+coef_xy(2,4)*pol_y(2,0,jg)
             coef_x(1,0)=coef_x(1,0)+coef_xy(1,5)*pol_y(1,1,jg)
             coef_x(2,0)=coef_x(2,0)+coef_xy(2,5)*pol_y(1,1,jg)
             coef_x(3,0)=coef_x(3,0)+coef_xy(1,5)*pol_y(2,1,jg)
             coef_x(4,0)=coef_x(4,0)+coef_xy(2,5)*pol_y(2,1,jg)
             coef_x(1,1)=coef_x(1,1)+coef_xy(1,6)*pol_y(1,1,jg)
             coef_x(2,1)=coef_x(2,1)+coef_xy(2,6)*pol_y(1,1,jg)
             coef_x(3,1)=coef_x(3,1)+coef_xy(1,6)*pol_y(2,1,jg)
             coef_x(4,1)=coef_x(4,1)+coef_xy(2,6)*pol_y(2,1,jg)
             coef_x(1,2)=coef_x(1,2)+coef_xy(1,7)*pol_y(1,1,jg)
             coef_x(2,2)=coef_x(2,2)+coef_xy(2,7)*pol_y(1,1,jg)
             coef_x(3,2)=coef_x(3,2)+coef_xy(1,7)*pol_y(2,1,jg)
             coef_x(4,2)=coef_x(4,2)+coef_xy(2,7)*pol_y(2,1,jg)
             coef_x(1,0)=coef_x(1,0)+coef_xy(1,8)*pol_y(1,2,jg)
             coef_x(2,0)=coef_x(2,0)+coef_xy(2,8)*pol_y(1,2,jg)
             coef_x(3,0)=coef_x(3,0)+coef_xy(1,8)*pol_y(2,2,jg)
             coef_x(4,0)=coef_x(4,0)+coef_xy(2,8)*pol_y(2,2,jg)
             coef_x(1,1)=coef_x(1,1)+coef_xy(1,9)*pol_y(1,2,jg)
             coef_x(2,1)=coef_x(2,1)+coef_xy(2,9)*pol_y(1,2,jg)
             coef_x(3,1)=coef_x(3,1)+coef_xy(1,9)*pol_y(2,2,jg)
             coef_x(4,1)=coef_x(4,1)+coef_xy(2,9)*pol_y(2,2,jg)
             coef_x(1,0)=coef_x(1,0)+coef_xy(1,10)*pol_y(1,3,jg)
             coef_x(2,0)=coef_x(2,0)+coef_xy(2,10)*pol_y(1,3,jg)
             coef_x(3,0)=coef_x(3,0)+coef_xy(1,10)*pol_y(2,3,jg)
             coef_x(4,0)=coef_x(4,0)+coef_xy(2,10)*pol_y(2,3,jg)
          DO ig=igmin,igmax
             i=map(ig,1)
             s01=0.0_dp
             s02=0.0_dp
             s03=0.0_dp
             s04=0.0_dp
             DO lxp=0,lp
                s01=s01+coef_x(1,lxp)*pol_x(lxp,ig)
                s02=s02+coef_x(2,lxp)*pol_x(lxp,ig)
                s03=s03+coef_x(3,lxp)*pol_x(lxp,ig)
                s04=s04+coef_x(4,lxp)*pol_x(lxp,ig)
             ENDDO
             grid(i,j,k) = grid(i,j,k)     + s01
             grid(i,j2,k) = grid(i,j2,k)   + s03
             grid(i,j,k2) = grid(i,j,k2)   + s02
             grid(i,j2,k2) = grid(i,j2,k2) + s04
          END DO
       END DO
    END DO

  END SUBROUTINE collocate_core_3
  SUBROUTINE collocate_core_4(grid,coef_xyz,pol_x,pol_y,pol_z,map,sphere_bounds,cmax,gridbounds)
    USE kinds,                        ONLY: dp
    INTEGER, INTENT(IN)                      :: sphere_bounds(*), cmax, &
                                                map(-cmax:cmax,1:3), &
                                                gridbounds(2,3)
    REAL(dp), INTENT(INOUT) :: grid(gridbounds(1,1):gridbounds(2,1), &
      gridbounds(1,2):gridbounds(2,2), gridbounds(1,3):gridbounds(2,3))
    INTEGER, PARAMETER                       :: lp = 4
    REAL(dp), INTENT(IN) :: pol_x(0:lp,-cmax:cmax), pol_y(1:2,0:lp,-cmax:0), &
      pol_z(1:2,0:lp,-cmax:0), coef_xyz(((lp+1)*(lp+2)*(lp+3))/6)

    INTEGER                                  :: i, ig, igmax, igmin, j, j2, &
                                                jg, jg2, jgmin, k, k2, kg, &
                                                kg2, kgmin, sci
    REAL(dp)                                 :: coef_x(4,0:lp), &
                                                coef_xy(2,(lp+1)*(lp+2)/2), &
                                                s01, s02, s03, s04

    sci=1

    kgmin=sphere_bounds(sci)
    sci=sci+1
    DO kg=kgmin,0
       kg2=1-kg
       k=map(kg,3)
       k2=map(kg2,3)
       coef_xy=0.0_dp
                coef_xy(1,1)=coef_xy(1,1)+coef_xyz(1)*pol_z(1,0,kg)
                coef_xy(2,1)=coef_xy(2,1)+coef_xyz(1)*pol_z(2,0,kg)
                coef_xy(1,2)=coef_xy(1,2)+coef_xyz(2)*pol_z(1,0,kg)
                coef_xy(2,2)=coef_xy(2,2)+coef_xyz(2)*pol_z(2,0,kg)
                coef_xy(1,3)=coef_xy(1,3)+coef_xyz(3)*pol_z(1,0,kg)
                coef_xy(2,3)=coef_xy(2,3)+coef_xyz(3)*pol_z(2,0,kg)
                coef_xy(1,4)=coef_xy(1,4)+coef_xyz(4)*pol_z(1,0,kg)
                coef_xy(2,4)=coef_xy(2,4)+coef_xyz(4)*pol_z(2,0,kg)
                coef_xy(1,5)=coef_xy(1,5)+coef_xyz(5)*pol_z(1,0,kg)
                coef_xy(2,5)=coef_xy(2,5)+coef_xyz(5)*pol_z(2,0,kg)
                coef_xy(1,6)=coef_xy(1,6)+coef_xyz(6)*pol_z(1,0,kg)
                coef_xy(2,6)=coef_xy(2,6)+coef_xyz(6)*pol_z(2,0,kg)
                coef_xy(1,7)=coef_xy(1,7)+coef_xyz(7)*pol_z(1,0,kg)
                coef_xy(2,7)=coef_xy(2,7)+coef_xyz(7)*pol_z(2,0,kg)
                coef_xy(1,8)=coef_xy(1,8)+coef_xyz(8)*pol_z(1,0,kg)
                coef_xy(2,8)=coef_xy(2,8)+coef_xyz(8)*pol_z(2,0,kg)
                coef_xy(1,9)=coef_xy(1,9)+coef_xyz(9)*pol_z(1,0,kg)
                coef_xy(2,9)=coef_xy(2,9)+coef_xyz(9)*pol_z(2,0,kg)
                coef_xy(1,10)=coef_xy(1,10)+coef_xyz(10)*pol_z(1,0,kg)
                coef_xy(2,10)=coef_xy(2,10)+coef_xyz(10)*pol_z(2,0,kg)
                coef_xy(1,11)=coef_xy(1,11)+coef_xyz(11)*pol_z(1,0,kg)
                coef_xy(2,11)=coef_xy(2,11)+coef_xyz(11)*pol_z(2,0,kg)
                coef_xy(1,12)=coef_xy(1,12)+coef_xyz(12)*pol_z(1,0,kg)
                coef_xy(2,12)=coef_xy(2,12)+coef_xyz(12)*pol_z(2,0,kg)
                coef_xy(1,13)=coef_xy(1,13)+coef_xyz(13)*pol_z(1,0,kg)
                coef_xy(2,13)=coef_xy(2,13)+coef_xyz(13)*pol_z(2,0,kg)
                coef_xy(1,14)=coef_xy(1,14)+coef_xyz(14)*pol_z(1,0,kg)
                coef_xy(2,14)=coef_xy(2,14)+coef_xyz(14)*pol_z(2,0,kg)
                coef_xy(1,15)=coef_xy(1,15)+coef_xyz(15)*pol_z(1,0,kg)
                coef_xy(2,15)=coef_xy(2,15)+coef_xyz(15)*pol_z(2,0,kg)
                coef_xy(1,1)=coef_xy(1,1)+coef_xyz(16)*pol_z(1,1,kg)
                coef_xy(2,1)=coef_xy(2,1)+coef_xyz(16)*pol_z(2,1,kg)
                coef_xy(1,2)=coef_xy(1,2)+coef_xyz(17)*pol_z(1,1,kg)
                coef_xy(2,2)=coef_xy(2,2)+coef_xyz(17)*pol_z(2,1,kg)
                coef_xy(1,3)=coef_xy(1,3)+coef_xyz(18)*pol_z(1,1,kg)
                coef_xy(2,3)=coef_xy(2,3)+coef_xyz(18)*pol_z(2,1,kg)
                coef_xy(1,4)=coef_xy(1,4)+coef_xyz(19)*pol_z(1,1,kg)
                coef_xy(2,4)=coef_xy(2,4)+coef_xyz(19)*pol_z(2,1,kg)
                coef_xy(1,6)=coef_xy(1,6)+coef_xyz(20)*pol_z(1,1,kg)
                coef_xy(2,6)=coef_xy(2,6)+coef_xyz(20)*pol_z(2,1,kg)
                coef_xy(1,7)=coef_xy(1,7)+coef_xyz(21)*pol_z(1,1,kg)
                coef_xy(2,7)=coef_xy(2,7)+coef_xyz(21)*pol_z(2,1,kg)
                coef_xy(1,8)=coef_xy(1,8)+coef_xyz(22)*pol_z(1,1,kg)
                coef_xy(2,8)=coef_xy(2,8)+coef_xyz(22)*pol_z(2,1,kg)
                coef_xy(1,10)=coef_xy(1,10)+coef_xyz(23)*pol_z(1,1,kg)
                coef_xy(2,10)=coef_xy(2,10)+coef_xyz(23)*pol_z(2,1,kg)
                coef_xy(1,11)=coef_xy(1,11)+coef_xyz(24)*pol_z(1,1,kg)
                coef_xy(2,11)=coef_xy(2,11)+coef_xyz(24)*pol_z(2,1,kg)
                coef_xy(1,13)=coef_xy(1,13)+coef_xyz(25)*pol_z(1,1,kg)
                coef_xy(2,13)=coef_xy(2,13)+coef_xyz(25)*pol_z(2,1,kg)
                coef_xy(1,1)=coef_xy(1,1)+coef_xyz(26)*pol_z(1,2,kg)
                coef_xy(2,1)=coef_xy(2,1)+coef_xyz(26)*pol_z(2,2,kg)
                coef_xy(1,2)=coef_xy(1,2)+coef_xyz(27)*pol_z(1,2,kg)
                coef_xy(2,2)=coef_xy(2,2)+coef_xyz(27)*pol_z(2,2,kg)
                coef_xy(1,3)=coef_xy(1,3)+coef_xyz(28)*pol_z(1,2,kg)
                coef_xy(2,3)=coef_xy(2,3)+coef_xyz(28)*pol_z(2,2,kg)
                coef_xy(1,6)=coef_xy(1,6)+coef_xyz(29)*pol_z(1,2,kg)
                coef_xy(2,6)=coef_xy(2,6)+coef_xyz(29)*pol_z(2,2,kg)
                coef_xy(1,7)=coef_xy(1,7)+coef_xyz(30)*pol_z(1,2,kg)
                coef_xy(2,7)=coef_xy(2,7)+coef_xyz(30)*pol_z(2,2,kg)
                coef_xy(1,10)=coef_xy(1,10)+coef_xyz(31)*pol_z(1,2,kg)
                coef_xy(2,10)=coef_xy(2,10)+coef_xyz(31)*pol_z(2,2,kg)
                coef_xy(1,1)=coef_xy(1,1)+coef_xyz(32)*pol_z(1,3,kg)
                coef_xy(2,1)=coef_xy(2,1)+coef_xyz(32)*pol_z(2,3,kg)
                coef_xy(1,2)=coef_xy(1,2)+coef_xyz(33)*pol_z(1,3,kg)
                coef_xy(2,2)=coef_xy(2,2)+coef_xyz(33)*pol_z(2,3,kg)
                coef_xy(1,6)=coef_xy(1,6)+coef_xyz(34)*pol_z(1,3,kg)
                coef_xy(2,6)=coef_xy(2,6)+coef_xyz(34)*pol_z(2,3,kg)
                coef_xy(1,1)=coef_xy(1,1)+coef_xyz(35)*pol_z(1,4,kg)
                coef_xy(2,1)=coef_xy(2,1)+coef_xyz(35)*pol_z(2,4,kg)
       jgmin=sphere_bounds(sci)
       sci=sci+1
       DO jg=jgmin,0
          jg2=1-jg
          j=map(jg,2)
          j2=map(jg2,2)
          igmin=sphere_bounds(sci)
          sci=sci+1
          igmax=1-igmin
          coef_x=0.0_dp
             coef_x(1,0)=coef_x(1,0)+coef_xy(1,1)*pol_y(1,0,jg)
             coef_x(2,0)=coef_x(2,0)+coef_xy(2,1)*pol_y(1,0,jg)
             coef_x(3,0)=coef_x(3,0)+coef_xy(1,1)*pol_y(2,0,jg)
             coef_x(4,0)=coef_x(4,0)+coef_xy(2,1)*pol_y(2,0,jg)
             coef_x(1,1)=coef_x(1,1)+coef_xy(1,2)*pol_y(1,0,jg)
             coef_x(2,1)=coef_x(2,1)+coef_xy(2,2)*pol_y(1,0,jg)
             coef_x(3,1)=coef_x(3,1)+coef_xy(1,2)*pol_y(2,0,jg)
             coef_x(4,1)=coef_x(4,1)+coef_xy(2,2)*pol_y(2,0,jg)
             coef_x(1,2)=coef_x(1,2)+coef_xy(1,3)*pol_y(1,0,jg)
             coef_x(2,2)=coef_x(2,2)+coef_xy(2,3)*pol_y(1,0,jg)
             coef_x(3,2)=coef_x(3,2)+coef_xy(1,3)*pol_y(2,0,jg)
             coef_x(4,2)=coef_x(4,2)+coef_xy(2,3)*pol_y(2,0,jg)
             coef_x(1,3)=coef_x(1,3)+coef_xy(1,4)*pol_y(1,0,jg)
             coef_x(2,3)=coef_x(2,3)+coef_xy(2,4)*pol_y(1,0,jg)
             coef_x(3,3)=coef_x(3,3)+coef_xy(1,4)*pol_y(2,0,jg)
             coef_x(4,3)=coef_x(4,3)+coef_xy(2,4)*pol_y(2,0,jg)
             coef_x(1,4)=coef_x(1,4)+coef_xy(1,5)*pol_y(1,0,jg)
             coef_x(2,4)=coef_x(2,4)+coef_xy(2,5)*pol_y(1,0,jg)
             coef_x(3,4)=coef_x(3,4)+coef_xy(1,5)*pol_y(2,0,jg)
             coef_x(4,4)=coef_x(4,4)+coef_xy(2,5)*pol_y(2,0,jg)
             coef_x(1,0)=coef_x(1,0)+coef_xy(1,6)*pol_y(1,1,jg)
             coef_x(2,0)=coef_x(2,0)+coef_xy(2,6)*pol_y(1,1,jg)
             coef_x(3,0)=coef_x(3,0)+coef_xy(1,6)*pol_y(2,1,jg)
             coef_x(4,0)=coef_x(4,0)+coef_xy(2,6)*pol_y(2,1,jg)
             coef_x(1,1)=coef_x(1,1)+coef_xy(1,7)*pol_y(1,1,jg)
             coef_x(2,1)=coef_x(2,1)+coef_xy(2,7)*pol_y(1,1,jg)
             coef_x(3,1)=coef_x(3,1)+coef_xy(1,7)*pol_y(2,1,jg)
             coef_x(4,1)=coef_x(4,1)+coef_xy(2,7)*pol_y(2,1,jg)
             coef_x(1,2)=coef_x(1,2)+coef_xy(1,8)*pol_y(1,1,jg)
             coef_x(2,2)=coef_x(2,2)+coef_xy(2,8)*pol_y(1,1,jg)
             coef_x(3,2)=coef_x(3,2)+coef_xy(1,8)*pol_y(2,1,jg)
             coef_x(4,2)=coef_x(4,2)+coef_xy(2,8)*pol_y(2,1,jg)
             coef_x(1,3)=coef_x(1,3)+coef_xy(1,9)*pol_y(1,1,jg)
             coef_x(2,3)=coef_x(2,3)+coef_xy(2,9)*pol_y(1,1,jg)
             coef_x(3,3)=coef_x(3,3)+coef_xy(1,9)*pol_y(2,1,jg)
             coef_x(4,3)=coef_x(4,3)+coef_xy(2,9)*pol_y(2,1,jg)
             coef_x(1,0)=coef_x(1,0)+coef_xy(1,10)*pol_y(1,2,jg)
             coef_x(2,0)=coef_x(2,0)+coef_xy(2,10)*pol_y(1,2,jg)
             coef_x(3,0)=coef_x(3,0)+coef_xy(1,10)*pol_y(2,2,jg)
             coef_x(4,0)=coef_x(4,0)+coef_xy(2,10)*pol_y(2,2,jg)
             coef_x(1,1)=coef_x(1,1)+coef_xy(1,11)*pol_y(1,2,jg)
             coef_x(2,1)=coef_x(2,1)+coef_xy(2,11)*pol_y(1,2,jg)
             coef_x(3,1)=coef_x(3,1)+coef_xy(1,11)*pol_y(2,2,jg)
             coef_x(4,1)=coef_x(4,1)+coef_xy(2,11)*pol_y(2,2,jg)
             coef_x(1,2)=coef_x(1,2)+coef_xy(1,12)*pol_y(1,2,jg)
             coef_x(2,2)=coef_x(2,2)+coef_xy(2,12)*pol_y(1,2,jg)
             coef_x(3,2)=coef_x(3,2)+coef_xy(1,12)*pol_y(2,2,jg)
             coef_x(4,2)=coef_x(4,2)+coef_xy(2,12)*pol_y(2,2,jg)
             coef_x(1,0)=coef_x(1,0)+coef_xy(1,13)*pol_y(1,3,jg)
             coef_x(2,0)=coef_x(2,0)+coef_xy(2,13)*pol_y(1,3,jg)
             coef_x(3,0)=coef_x(3,0)+coef_xy(1,13)*pol_y(2,3,jg)
             coef_x(4,0)=coef_x(4,0)+coef_xy(2,13)*pol_y(2,3,jg)
             coef_x(1,1)=coef_x(1,1)+coef_xy(1,14)*pol_y(1,3,jg)
             coef_x(2,1)=coef_x(2,1)+coef_xy(2,14)*pol_y(1,3,jg)
             coef_x(3,1)=coef_x(3,1)+coef_xy(1,14)*pol_y(2,3,jg)
             coef_x(4,1)=coef_x(4,1)+coef_xy(2,14)*pol_y(2,3,jg)
             coef_x(1,0)=coef_x(1,0)+coef_xy(1,15)*pol_y(1,4,jg)
             coef_x(2,0)=coef_x(2,0)+coef_xy(2,15)*pol_y(1,4,jg)
             coef_x(3,0)=coef_x(3,0)+coef_xy(1,15)*pol_y(2,4,jg)
             coef_x(4,0)=coef_x(4,0)+coef_xy(2,15)*pol_y(2,4,jg)
          DO ig=igmin,igmax
             i=map(ig,1)
             s01=0.0_dp
             s02=0.0_dp
             s03=0.0_dp
             s04=0.0_dp
                s01=s01+coef_x(1,0)*pol_x(0,ig)
                s02=s02+coef_x(2,0)*pol_x(0,ig)
                s03=s03+coef_x(3,0)*pol_x(0,ig)
                s04=s04+coef_x(4,0)*pol_x(0,ig)
                s01=s01+coef_x(1,1)*pol_x(1,ig)
                s02=s02+coef_x(2,1)*pol_x(1,ig)
                s03=s03+coef_x(3,1)*pol_x(1,ig)
                s04=s04+coef_x(4,1)*pol_x(1,ig)
                s01=s01+coef_x(1,2)*pol_x(2,ig)
                s02=s02+coef_x(2,2)*pol_x(2,ig)
                s03=s03+coef_x(3,2)*pol_x(2,ig)
                s04=s04+coef_x(4,2)*pol_x(2,ig)
                s01=s01+coef_x(1,3)*pol_x(3,ig)
                s02=s02+coef_x(2,3)*pol_x(3,ig)
                s03=s03+coef_x(3,3)*pol_x(3,ig)
                s04=s04+coef_x(4,3)*pol_x(3,ig)
                s01=s01+coef_x(1,4)*pol_x(4,ig)
                s02=s02+coef_x(2,4)*pol_x(4,ig)
                s03=s03+coef_x(3,4)*pol_x(4,ig)
                s04=s04+coef_x(4,4)*pol_x(4,ig)
             grid(i,j,k) = grid(i,j,k)     + s01
             grid(i,j2,k) = grid(i,j2,k)   + s03
             grid(i,j,k2) = grid(i,j,k2)   + s02
             grid(i,j2,k2) = grid(i,j2,k2) + s04
          END DO
       END DO
    END DO

  END SUBROUTINE collocate_core_4
  SUBROUTINE collocate_core_5(grid,coef_xyz,pol_x,pol_y,pol_z,map,sphere_bounds,cmax,gridbounds)
    USE kinds,                        ONLY: dp
    INTEGER, INTENT(IN)                      :: sphere_bounds(*), cmax, &
                                                map(-cmax:cmax,1:3), &
                                                gridbounds(2,3)
    REAL(dp), INTENT(INOUT) :: grid(gridbounds(1,1):gridbounds(2,1), &
      gridbounds(1,2):gridbounds(2,2), gridbounds(1,3):gridbounds(2,3))
    INTEGER, PARAMETER                       :: lp = 5
    REAL(dp), INTENT(IN) :: pol_x(0:lp,-cmax:cmax), pol_y(1:2,0:lp,-cmax:0), &
      pol_z(1:2,0:lp,-cmax:0), coef_xyz(((lp+1)*(lp+2)*(lp+3))/6)

    INTEGER                                  :: i, ig, igmax, igmin, j, j2, &
                                                jg, jg2, jgmin, k, k2, kg, &
                                                kg2, kgmin, lxp, sci
    REAL(dp)                                 :: coef_x(4,0:lp), &
                                                coef_xy(2,(lp+1)*(lp+2)/2), &
                                                s01, s02, s03, s04

    sci=1

    kgmin=sphere_bounds(sci)
    sci=sci+1
    DO kg=kgmin,0
       kg2=1-kg
       k=map(kg,3)
       k2=map(kg2,3)
       coef_xy=0.0_dp
                coef_xy(1,1)=coef_xy(1,1)+coef_xyz(1)*pol_z(1,0,kg)
                coef_xy(2,1)=coef_xy(2,1)+coef_xyz(1)*pol_z(2,0,kg)
                coef_xy(1,2)=coef_xy(1,2)+coef_xyz(2)*pol_z(1,0,kg)
                coef_xy(2,2)=coef_xy(2,2)+coef_xyz(2)*pol_z(2,0,kg)
                coef_xy(1,3)=coef_xy(1,3)+coef_xyz(3)*pol_z(1,0,kg)
                coef_xy(2,3)=coef_xy(2,3)+coef_xyz(3)*pol_z(2,0,kg)
                coef_xy(1,4)=coef_xy(1,4)+coef_xyz(4)*pol_z(1,0,kg)
                coef_xy(2,4)=coef_xy(2,4)+coef_xyz(4)*pol_z(2,0,kg)
                coef_xy(1,5)=coef_xy(1,5)+coef_xyz(5)*pol_z(1,0,kg)
                coef_xy(2,5)=coef_xy(2,5)+coef_xyz(5)*pol_z(2,0,kg)
                coef_xy(1,6)=coef_xy(1,6)+coef_xyz(6)*pol_z(1,0,kg)
                coef_xy(2,6)=coef_xy(2,6)+coef_xyz(6)*pol_z(2,0,kg)
                coef_xy(1,7)=coef_xy(1,7)+coef_xyz(7)*pol_z(1,0,kg)
                coef_xy(2,7)=coef_xy(2,7)+coef_xyz(7)*pol_z(2,0,kg)
                coef_xy(1,8)=coef_xy(1,8)+coef_xyz(8)*pol_z(1,0,kg)
                coef_xy(2,8)=coef_xy(2,8)+coef_xyz(8)*pol_z(2,0,kg)
                coef_xy(1,9)=coef_xy(1,9)+coef_xyz(9)*pol_z(1,0,kg)
                coef_xy(2,9)=coef_xy(2,9)+coef_xyz(9)*pol_z(2,0,kg)
                coef_xy(1,10)=coef_xy(1,10)+coef_xyz(10)*pol_z(1,0,kg)
                coef_xy(2,10)=coef_xy(2,10)+coef_xyz(10)*pol_z(2,0,kg)
                coef_xy(1,11)=coef_xy(1,11)+coef_xyz(11)*pol_z(1,0,kg)
                coef_xy(2,11)=coef_xy(2,11)+coef_xyz(11)*pol_z(2,0,kg)
                coef_xy(1,12)=coef_xy(1,12)+coef_xyz(12)*pol_z(1,0,kg)
                coef_xy(2,12)=coef_xy(2,12)+coef_xyz(12)*pol_z(2,0,kg)
                coef_xy(1,13)=coef_xy(1,13)+coef_xyz(13)*pol_z(1,0,kg)
                coef_xy(2,13)=coef_xy(2,13)+coef_xyz(13)*pol_z(2,0,kg)
                coef_xy(1,14)=coef_xy(1,14)+coef_xyz(14)*pol_z(1,0,kg)
                coef_xy(2,14)=coef_xy(2,14)+coef_xyz(14)*pol_z(2,0,kg)
                coef_xy(1,15)=coef_xy(1,15)+coef_xyz(15)*pol_z(1,0,kg)
                coef_xy(2,15)=coef_xy(2,15)+coef_xyz(15)*pol_z(2,0,kg)
                coef_xy(1,16)=coef_xy(1,16)+coef_xyz(16)*pol_z(1,0,kg)
                coef_xy(2,16)=coef_xy(2,16)+coef_xyz(16)*pol_z(2,0,kg)
                coef_xy(1,17)=coef_xy(1,17)+coef_xyz(17)*pol_z(1,0,kg)
                coef_xy(2,17)=coef_xy(2,17)+coef_xyz(17)*pol_z(2,0,kg)
                coef_xy(1,18)=coef_xy(1,18)+coef_xyz(18)*pol_z(1,0,kg)
                coef_xy(2,18)=coef_xy(2,18)+coef_xyz(18)*pol_z(2,0,kg)
                coef_xy(1,19)=coef_xy(1,19)+coef_xyz(19)*pol_z(1,0,kg)
                coef_xy(2,19)=coef_xy(2,19)+coef_xyz(19)*pol_z(2,0,kg)
                coef_xy(1,20)=coef_xy(1,20)+coef_xyz(20)*pol_z(1,0,kg)
                coef_xy(2,20)=coef_xy(2,20)+coef_xyz(20)*pol_z(2,0,kg)
                coef_xy(1,21)=coef_xy(1,21)+coef_xyz(21)*pol_z(1,0,kg)
                coef_xy(2,21)=coef_xy(2,21)+coef_xyz(21)*pol_z(2,0,kg)
                coef_xy(1,1)=coef_xy(1,1)+coef_xyz(22)*pol_z(1,1,kg)
                coef_xy(2,1)=coef_xy(2,1)+coef_xyz(22)*pol_z(2,1,kg)
                coef_xy(1,2)=coef_xy(1,2)+coef_xyz(23)*pol_z(1,1,kg)
                coef_xy(2,2)=coef_xy(2,2)+coef_xyz(23)*pol_z(2,1,kg)
                coef_xy(1,3)=coef_xy(1,3)+coef_xyz(24)*pol_z(1,1,kg)
                coef_xy(2,3)=coef_xy(2,3)+coef_xyz(24)*pol_z(2,1,kg)
                coef_xy(1,4)=coef_xy(1,4)+coef_xyz(25)*pol_z(1,1,kg)
                coef_xy(2,4)=coef_xy(2,4)+coef_xyz(25)*pol_z(2,1,kg)
                coef_xy(1,5)=coef_xy(1,5)+coef_xyz(26)*pol_z(1,1,kg)
                coef_xy(2,5)=coef_xy(2,5)+coef_xyz(26)*pol_z(2,1,kg)
                coef_xy(1,7)=coef_xy(1,7)+coef_xyz(27)*pol_z(1,1,kg)
                coef_xy(2,7)=coef_xy(2,7)+coef_xyz(27)*pol_z(2,1,kg)
                coef_xy(1,8)=coef_xy(1,8)+coef_xyz(28)*pol_z(1,1,kg)
                coef_xy(2,8)=coef_xy(2,8)+coef_xyz(28)*pol_z(2,1,kg)
                coef_xy(1,9)=coef_xy(1,9)+coef_xyz(29)*pol_z(1,1,kg)
                coef_xy(2,9)=coef_xy(2,9)+coef_xyz(29)*pol_z(2,1,kg)
                coef_xy(1,10)=coef_xy(1,10)+coef_xyz(30)*pol_z(1,1,kg)
                coef_xy(2,10)=coef_xy(2,10)+coef_xyz(30)*pol_z(2,1,kg)
                coef_xy(1,12)=coef_xy(1,12)+coef_xyz(31)*pol_z(1,1,kg)
                coef_xy(2,12)=coef_xy(2,12)+coef_xyz(31)*pol_z(2,1,kg)
                coef_xy(1,13)=coef_xy(1,13)+coef_xyz(32)*pol_z(1,1,kg)
                coef_xy(2,13)=coef_xy(2,13)+coef_xyz(32)*pol_z(2,1,kg)
                coef_xy(1,14)=coef_xy(1,14)+coef_xyz(33)*pol_z(1,1,kg)
                coef_xy(2,14)=coef_xy(2,14)+coef_xyz(33)*pol_z(2,1,kg)
                coef_xy(1,16)=coef_xy(1,16)+coef_xyz(34)*pol_z(1,1,kg)
                coef_xy(2,16)=coef_xy(2,16)+coef_xyz(34)*pol_z(2,1,kg)
                coef_xy(1,17)=coef_xy(1,17)+coef_xyz(35)*pol_z(1,1,kg)
                coef_xy(2,17)=coef_xy(2,17)+coef_xyz(35)*pol_z(2,1,kg)
                coef_xy(1,19)=coef_xy(1,19)+coef_xyz(36)*pol_z(1,1,kg)
                coef_xy(2,19)=coef_xy(2,19)+coef_xyz(36)*pol_z(2,1,kg)
                coef_xy(1,1)=coef_xy(1,1)+coef_xyz(37)*pol_z(1,2,kg)
                coef_xy(2,1)=coef_xy(2,1)+coef_xyz(37)*pol_z(2,2,kg)
                coef_xy(1,2)=coef_xy(1,2)+coef_xyz(38)*pol_z(1,2,kg)
                coef_xy(2,2)=coef_xy(2,2)+coef_xyz(38)*pol_z(2,2,kg)
                coef_xy(1,3)=coef_xy(1,3)+coef_xyz(39)*pol_z(1,2,kg)
                coef_xy(2,3)=coef_xy(2,3)+coef_xyz(39)*pol_z(2,2,kg)
                coef_xy(1,4)=coef_xy(1,4)+coef_xyz(40)*pol_z(1,2,kg)
                coef_xy(2,4)=coef_xy(2,4)+coef_xyz(40)*pol_z(2,2,kg)
                coef_xy(1,7)=coef_xy(1,7)+coef_xyz(41)*pol_z(1,2,kg)
                coef_xy(2,7)=coef_xy(2,7)+coef_xyz(41)*pol_z(2,2,kg)
                coef_xy(1,8)=coef_xy(1,8)+coef_xyz(42)*pol_z(1,2,kg)
                coef_xy(2,8)=coef_xy(2,8)+coef_xyz(42)*pol_z(2,2,kg)
                coef_xy(1,9)=coef_xy(1,9)+coef_xyz(43)*pol_z(1,2,kg)
                coef_xy(2,9)=coef_xy(2,9)+coef_xyz(43)*pol_z(2,2,kg)
                coef_xy(1,12)=coef_xy(1,12)+coef_xyz(44)*pol_z(1,2,kg)
                coef_xy(2,12)=coef_xy(2,12)+coef_xyz(44)*pol_z(2,2,kg)
                coef_xy(1,13)=coef_xy(1,13)+coef_xyz(45)*pol_z(1,2,kg)
                coef_xy(2,13)=coef_xy(2,13)+coef_xyz(45)*pol_z(2,2,kg)
                coef_xy(1,16)=coef_xy(1,16)+coef_xyz(46)*pol_z(1,2,kg)
                coef_xy(2,16)=coef_xy(2,16)+coef_xyz(46)*pol_z(2,2,kg)
                coef_xy(1,1)=coef_xy(1,1)+coef_xyz(47)*pol_z(1,3,kg)
                coef_xy(2,1)=coef_xy(2,1)+coef_xyz(47)*pol_z(2,3,kg)
                coef_xy(1,2)=coef_xy(1,2)+coef_xyz(48)*pol_z(1,3,kg)
                coef_xy(2,2)=coef_xy(2,2)+coef_xyz(48)*pol_z(2,3,kg)
                coef_xy(1,3)=coef_xy(1,3)+coef_xyz(49)*pol_z(1,3,kg)
                coef_xy(2,3)=coef_xy(2,3)+coef_xyz(49)*pol_z(2,3,kg)
                coef_xy(1,7)=coef_xy(1,7)+coef_xyz(50)*pol_z(1,3,kg)
                coef_xy(2,7)=coef_xy(2,7)+coef_xyz(50)*pol_z(2,3,kg)
                coef_xy(1,8)=coef_xy(1,8)+coef_xyz(51)*pol_z(1,3,kg)
                coef_xy(2,8)=coef_xy(2,8)+coef_xyz(51)*pol_z(2,3,kg)
                coef_xy(1,12)=coef_xy(1,12)+coef_xyz(52)*pol_z(1,3,kg)
                coef_xy(2,12)=coef_xy(2,12)+coef_xyz(52)*pol_z(2,3,kg)
                coef_xy(1,1)=coef_xy(1,1)+coef_xyz(53)*pol_z(1,4,kg)
                coef_xy(2,1)=coef_xy(2,1)+coef_xyz(53)*pol_z(2,4,kg)
                coef_xy(1,2)=coef_xy(1,2)+coef_xyz(54)*pol_z(1,4,kg)
                coef_xy(2,2)=coef_xy(2,2)+coef_xyz(54)*pol_z(2,4,kg)
                coef_xy(1,7)=coef_xy(1,7)+coef_xyz(55)*pol_z(1,4,kg)
                coef_xy(2,7)=coef_xy(2,7)+coef_xyz(55)*pol_z(2,4,kg)
                coef_xy(1,1)=coef_xy(1,1)+coef_xyz(56)*pol_z(1,5,kg)
                coef_xy(2,1)=coef_xy(2,1)+coef_xyz(56)*pol_z(2,5,kg)
       jgmin=sphere_bounds(sci)
       sci=sci+1
       DO jg=jgmin,0
          jg2=1-jg
          j=map(jg,2)
          j2=map(jg2,2)
          igmin=sphere_bounds(sci)
          sci=sci+1
          igmax=1-igmin
          coef_x=0.0_dp
             coef_x(1:2,0)=coef_x(1:2,0)+coef_xy(1:2,1)*pol_y(1,0,jg)
             coef_x(3:4,0)=coef_x(3:4,0)+coef_xy(1:2,1)*pol_y(2,0,jg)
             coef_x(1:2,1)=coef_x(1:2,1)+coef_xy(1:2,2)*pol_y(1,0,jg)
             coef_x(3:4,1)=coef_x(3:4,1)+coef_xy(1:2,2)*pol_y(2,0,jg)
             coef_x(1:2,2)=coef_x(1:2,2)+coef_xy(1:2,3)*pol_y(1,0,jg)
             coef_x(3:4,2)=coef_x(3:4,2)+coef_xy(1:2,3)*pol_y(2,0,jg)
             coef_x(1:2,3)=coef_x(1:2,3)+coef_xy(1:2,4)*pol_y(1,0,jg)
             coef_x(3:4,3)=coef_x(3:4,3)+coef_xy(1:2,4)*pol_y(2,0,jg)
             coef_x(1:2,4)=coef_x(1:2,4)+coef_xy(1:2,5)*pol_y(1,0,jg)
             coef_x(3:4,4)=coef_x(3:4,4)+coef_xy(1:2,5)*pol_y(2,0,jg)
             coef_x(1:2,5)=coef_x(1:2,5)+coef_xy(1:2,6)*pol_y(1,0,jg)
             coef_x(3:4,5)=coef_x(3:4,5)+coef_xy(1:2,6)*pol_y(2,0,jg)
             coef_x(1:2,0)=coef_x(1:2,0)+coef_xy(1:2,7)*pol_y(1,1,jg)
             coef_x(3:4,0)=coef_x(3:4,0)+coef_xy(1:2,7)*pol_y(2,1,jg)
             coef_x(1:2,1)=coef_x(1:2,1)+coef_xy(1:2,8)*pol_y(1,1,jg)
             coef_x(3:4,1)=coef_x(3:4,1)+coef_xy(1:2,8)*pol_y(2,1,jg)
             coef_x(1:2,2)=coef_x(1:2,2)+coef_xy(1:2,9)*pol_y(1,1,jg)
             coef_x(3:4,2)=coef_x(3:4,2)+coef_xy(1:2,9)*pol_y(2,1,jg)
             coef_x(1:2,3)=coef_x(1:2,3)+coef_xy(1:2,10)*pol_y(1,1,jg)
             coef_x(3:4,3)=coef_x(3:4,3)+coef_xy(1:2,10)*pol_y(2,1,jg)
             coef_x(1:2,4)=coef_x(1:2,4)+coef_xy(1:2,11)*pol_y(1,1,jg)
             coef_x(3:4,4)=coef_x(3:4,4)+coef_xy(1:2,11)*pol_y(2,1,jg)
             coef_x(1:2,0)=coef_x(1:2,0)+coef_xy(1:2,12)*pol_y(1,2,jg)
             coef_x(3:4,0)=coef_x(3:4,0)+coef_xy(1:2,12)*pol_y(2,2,jg)
             coef_x(1:2,1)=coef_x(1:2,1)+coef_xy(1:2,13)*pol_y(1,2,jg)
             coef_x(3:4,1)=coef_x(3:4,1)+coef_xy(1:2,13)*pol_y(2,2,jg)
             coef_x(1:2,2)=coef_x(1:2,2)+coef_xy(1:2,14)*pol_y(1,2,jg)
             coef_x(3:4,2)=coef_x(3:4,2)+coef_xy(1:2,14)*pol_y(2,2,jg)
             coef_x(1:2,3)=coef_x(1:2,3)+coef_xy(1:2,15)*pol_y(1,2,jg)
             coef_x(3:4,3)=coef_x(3:4,3)+coef_xy(1:2,15)*pol_y(2,2,jg)
             coef_x(1:2,0)=coef_x(1:2,0)+coef_xy(1:2,16)*pol_y(1,3,jg)
             coef_x(3:4,0)=coef_x(3:4,0)+coef_xy(1:2,16)*pol_y(2,3,jg)
             coef_x(1:2,1)=coef_x(1:2,1)+coef_xy(1:2,17)*pol_y(1,3,jg)
             coef_x(3:4,1)=coef_x(3:4,1)+coef_xy(1:2,17)*pol_y(2,3,jg)
             coef_x(1:2,2)=coef_x(1:2,2)+coef_xy(1:2,18)*pol_y(1,3,jg)
             coef_x(3:4,2)=coef_x(3:4,2)+coef_xy(1:2,18)*pol_y(2,3,jg)
             coef_x(1:2,0)=coef_x(1:2,0)+coef_xy(1:2,19)*pol_y(1,4,jg)
             coef_x(3:4,0)=coef_x(3:4,0)+coef_xy(1:2,19)*pol_y(2,4,jg)
             coef_x(1:2,1)=coef_x(1:2,1)+coef_xy(1:2,20)*pol_y(1,4,jg)
             coef_x(3:4,1)=coef_x(3:4,1)+coef_xy(1:2,20)*pol_y(2,4,jg)
             coef_x(1:2,0)=coef_x(1:2,0)+coef_xy(1:2,21)*pol_y(1,5,jg)
             coef_x(3:4,0)=coef_x(3:4,0)+coef_xy(1:2,21)*pol_y(2,5,jg)
          DO ig=igmin,igmax
             i=map(ig,1)
             s01=0.0_dp
             s02=0.0_dp
             s03=0.0_dp
             s04=0.0_dp
             DO lxp=0,lp
                s01=s01+coef_x(1,lxp)*pol_x(lxp,ig)
                s02=s02+coef_x(2,lxp)*pol_x(lxp,ig)
                s03=s03+coef_x(3,lxp)*pol_x(lxp,ig)
                s04=s04+coef_x(4,lxp)*pol_x(lxp,ig)
             ENDDO
             grid(i,j,k) = grid(i,j,k)     + s01
             grid(i,j2,k) = grid(i,j2,k)   + s03
             grid(i,j,k2) = grid(i,j,k2)   + s02
             grid(i,j2,k2) = grid(i,j2,k2) + s04
          END DO
       END DO
    END DO

  END SUBROUTINE collocate_core_5
  SUBROUTINE collocate_core_6(grid,coef_xyz,pol_x,pol_y,pol_z,map,sphere_bounds,cmax,gridbounds)
    USE kinds,                        ONLY: dp
    INTEGER, INTENT(IN)                      :: sphere_bounds(*), cmax, &
                                                map(-cmax:cmax,1:3), &
                                                gridbounds(2,3)
    REAL(dp), INTENT(INOUT) :: grid(gridbounds(1,1):gridbounds(2,1), &
      gridbounds(1,2):gridbounds(2,2), gridbounds(1,3):gridbounds(2,3))
    INTEGER, PARAMETER                       :: lp = 6
    REAL(dp), INTENT(IN) :: pol_x(0:lp,-cmax:cmax), pol_y(1:2,0:lp,-cmax:0), &
      pol_z(1:2,0:lp,-cmax:0), coef_xyz(((lp+1)*(lp+2)*(lp+3))/6)

    INTEGER                                  :: i, ig, igmax, igmin, j, j2, &
                                                jg, jg2, jgmin, k, k2, kg, &
                                                kg2, kgmin, lxp, sci
    REAL(dp)                                 :: coef_x(4,0:lp), &
                                                coef_xy(2,(lp+1)*(lp+2)/2), &
                                                s01, s02, s03, s04

    sci=1

    kgmin=sphere_bounds(sci)
    sci=sci+1
    DO kg=kgmin,0
       kg2=1-kg
       k=map(kg,3)
       k2=map(kg2,3)
       coef_xy=0.0_dp
                coef_xy(:,1)=coef_xy(:,1)+coef_xyz(1)*pol_z(:,0,kg)
                coef_xy(:,2)=coef_xy(:,2)+coef_xyz(2)*pol_z(:,0,kg)
                coef_xy(:,3)=coef_xy(:,3)+coef_xyz(3)*pol_z(:,0,kg)
                coef_xy(:,4)=coef_xy(:,4)+coef_xyz(4)*pol_z(:,0,kg)
                coef_xy(:,5)=coef_xy(:,5)+coef_xyz(5)*pol_z(:,0,kg)
                coef_xy(:,6)=coef_xy(:,6)+coef_xyz(6)*pol_z(:,0,kg)
                coef_xy(:,7)=coef_xy(:,7)+coef_xyz(7)*pol_z(:,0,kg)
                coef_xy(:,8)=coef_xy(:,8)+coef_xyz(8)*pol_z(:,0,kg)
                coef_xy(:,9)=coef_xy(:,9)+coef_xyz(9)*pol_z(:,0,kg)
                coef_xy(:,10)=coef_xy(:,10)+coef_xyz(10)*pol_z(:,0,kg)
                coef_xy(:,11)=coef_xy(:,11)+coef_xyz(11)*pol_z(:,0,kg)
                coef_xy(:,12)=coef_xy(:,12)+coef_xyz(12)*pol_z(:,0,kg)
                coef_xy(:,13)=coef_xy(:,13)+coef_xyz(13)*pol_z(:,0,kg)
                coef_xy(:,14)=coef_xy(:,14)+coef_xyz(14)*pol_z(:,0,kg)
                coef_xy(:,15)=coef_xy(:,15)+coef_xyz(15)*pol_z(:,0,kg)
                coef_xy(:,16)=coef_xy(:,16)+coef_xyz(16)*pol_z(:,0,kg)
                coef_xy(:,17)=coef_xy(:,17)+coef_xyz(17)*pol_z(:,0,kg)
                coef_xy(:,18)=coef_xy(:,18)+coef_xyz(18)*pol_z(:,0,kg)
                coef_xy(:,19)=coef_xy(:,19)+coef_xyz(19)*pol_z(:,0,kg)
                coef_xy(:,20)=coef_xy(:,20)+coef_xyz(20)*pol_z(:,0,kg)
                coef_xy(:,21)=coef_xy(:,21)+coef_xyz(21)*pol_z(:,0,kg)
                coef_xy(:,22)=coef_xy(:,22)+coef_xyz(22)*pol_z(:,0,kg)
                coef_xy(:,23)=coef_xy(:,23)+coef_xyz(23)*pol_z(:,0,kg)
                coef_xy(:,24)=coef_xy(:,24)+coef_xyz(24)*pol_z(:,0,kg)
                coef_xy(:,25)=coef_xy(:,25)+coef_xyz(25)*pol_z(:,0,kg)
                coef_xy(:,26)=coef_xy(:,26)+coef_xyz(26)*pol_z(:,0,kg)
                coef_xy(:,27)=coef_xy(:,27)+coef_xyz(27)*pol_z(:,0,kg)
                coef_xy(:,28)=coef_xy(:,28)+coef_xyz(28)*pol_z(:,0,kg)
                coef_xy(:,1)=coef_xy(:,1)+coef_xyz(29)*pol_z(:,1,kg)
                coef_xy(:,2)=coef_xy(:,2)+coef_xyz(30)*pol_z(:,1,kg)
                coef_xy(:,3)=coef_xy(:,3)+coef_xyz(31)*pol_z(:,1,kg)
                coef_xy(:,4)=coef_xy(:,4)+coef_xyz(32)*pol_z(:,1,kg)
                coef_xy(:,5)=coef_xy(:,5)+coef_xyz(33)*pol_z(:,1,kg)
                coef_xy(:,6)=coef_xy(:,6)+coef_xyz(34)*pol_z(:,1,kg)
                coef_xy(:,8)=coef_xy(:,8)+coef_xyz(35)*pol_z(:,1,kg)
                coef_xy(:,9)=coef_xy(:,9)+coef_xyz(36)*pol_z(:,1,kg)
                coef_xy(:,10)=coef_xy(:,10)+coef_xyz(37)*pol_z(:,1,kg)
                coef_xy(:,11)=coef_xy(:,11)+coef_xyz(38)*pol_z(:,1,kg)
                coef_xy(:,12)=coef_xy(:,12)+coef_xyz(39)*pol_z(:,1,kg)
                coef_xy(:,14)=coef_xy(:,14)+coef_xyz(40)*pol_z(:,1,kg)
                coef_xy(:,15)=coef_xy(:,15)+coef_xyz(41)*pol_z(:,1,kg)
                coef_xy(:,16)=coef_xy(:,16)+coef_xyz(42)*pol_z(:,1,kg)
                coef_xy(:,17)=coef_xy(:,17)+coef_xyz(43)*pol_z(:,1,kg)
                coef_xy(:,19)=coef_xy(:,19)+coef_xyz(44)*pol_z(:,1,kg)
                coef_xy(:,20)=coef_xy(:,20)+coef_xyz(45)*pol_z(:,1,kg)
                coef_xy(:,21)=coef_xy(:,21)+coef_xyz(46)*pol_z(:,1,kg)
                coef_xy(:,23)=coef_xy(:,23)+coef_xyz(47)*pol_z(:,1,kg)
                coef_xy(:,24)=coef_xy(:,24)+coef_xyz(48)*pol_z(:,1,kg)
                coef_xy(:,26)=coef_xy(:,26)+coef_xyz(49)*pol_z(:,1,kg)
                coef_xy(:,1)=coef_xy(:,1)+coef_xyz(50)*pol_z(:,2,kg)
                coef_xy(:,2)=coef_xy(:,2)+coef_xyz(51)*pol_z(:,2,kg)
                coef_xy(:,3)=coef_xy(:,3)+coef_xyz(52)*pol_z(:,2,kg)
                coef_xy(:,4)=coef_xy(:,4)+coef_xyz(53)*pol_z(:,2,kg)
                coef_xy(:,5)=coef_xy(:,5)+coef_xyz(54)*pol_z(:,2,kg)
                coef_xy(:,8)=coef_xy(:,8)+coef_xyz(55)*pol_z(:,2,kg)
                coef_xy(:,9)=coef_xy(:,9)+coef_xyz(56)*pol_z(:,2,kg)
                coef_xy(:,10)=coef_xy(:,10)+coef_xyz(57)*pol_z(:,2,kg)
                coef_xy(:,11)=coef_xy(:,11)+coef_xyz(58)*pol_z(:,2,kg)
                coef_xy(:,14)=coef_xy(:,14)+coef_xyz(59)*pol_z(:,2,kg)
                coef_xy(:,15)=coef_xy(:,15)+coef_xyz(60)*pol_z(:,2,kg)
                coef_xy(:,16)=coef_xy(:,16)+coef_xyz(61)*pol_z(:,2,kg)
                coef_xy(:,19)=coef_xy(:,19)+coef_xyz(62)*pol_z(:,2,kg)
                coef_xy(:,20)=coef_xy(:,20)+coef_xyz(63)*pol_z(:,2,kg)
                coef_xy(:,23)=coef_xy(:,23)+coef_xyz(64)*pol_z(:,2,kg)
                coef_xy(:,1)=coef_xy(:,1)+coef_xyz(65)*pol_z(:,3,kg)
                coef_xy(:,2)=coef_xy(:,2)+coef_xyz(66)*pol_z(:,3,kg)
                coef_xy(:,3)=coef_xy(:,3)+coef_xyz(67)*pol_z(:,3,kg)
                coef_xy(:,4)=coef_xy(:,4)+coef_xyz(68)*pol_z(:,3,kg)
                coef_xy(:,8)=coef_xy(:,8)+coef_xyz(69)*pol_z(:,3,kg)
                coef_xy(:,9)=coef_xy(:,9)+coef_xyz(70)*pol_z(:,3,kg)
                coef_xy(:,10)=coef_xy(:,10)+coef_xyz(71)*pol_z(:,3,kg)
                coef_xy(:,14)=coef_xy(:,14)+coef_xyz(72)*pol_z(:,3,kg)
                coef_xy(:,15)=coef_xy(:,15)+coef_xyz(73)*pol_z(:,3,kg)
                coef_xy(:,19)=coef_xy(:,19)+coef_xyz(74)*pol_z(:,3,kg)
                coef_xy(:,1)=coef_xy(:,1)+coef_xyz(75)*pol_z(:,4,kg)
                coef_xy(:,2)=coef_xy(:,2)+coef_xyz(76)*pol_z(:,4,kg)
                coef_xy(:,3)=coef_xy(:,3)+coef_xyz(77)*pol_z(:,4,kg)
                coef_xy(:,8)=coef_xy(:,8)+coef_xyz(78)*pol_z(:,4,kg)
                coef_xy(:,9)=coef_xy(:,9)+coef_xyz(79)*pol_z(:,4,kg)
                coef_xy(:,14)=coef_xy(:,14)+coef_xyz(80)*pol_z(:,4,kg)
                coef_xy(:,1)=coef_xy(:,1)+coef_xyz(81)*pol_z(:,5,kg)
                coef_xy(:,2)=coef_xy(:,2)+coef_xyz(82)*pol_z(:,5,kg)
                coef_xy(:,8)=coef_xy(:,8)+coef_xyz(83)*pol_z(:,5,kg)
                coef_xy(:,1)=coef_xy(:,1)+coef_xyz(84)*pol_z(:,6,kg)
       jgmin=sphere_bounds(sci)
       sci=sci+1
       DO jg=jgmin,0
          jg2=1-jg
          j=map(jg,2)
          j2=map(jg2,2)
          igmin=sphere_bounds(sci)
          sci=sci+1
          igmax=1-igmin
          coef_x=0.0_dp
             coef_x(1,0)=coef_x(1,0)+coef_xy(1,1)*pol_y(1,0,jg)
             coef_x(2,0)=coef_x(2,0)+coef_xy(2,1)*pol_y(1,0,jg)
             coef_x(3,0)=coef_x(3,0)+coef_xy(1,1)*pol_y(2,0,jg)
             coef_x(4,0)=coef_x(4,0)+coef_xy(2,1)*pol_y(2,0,jg)
             coef_x(1,1)=coef_x(1,1)+coef_xy(1,2)*pol_y(1,0,jg)
             coef_x(2,1)=coef_x(2,1)+coef_xy(2,2)*pol_y(1,0,jg)
             coef_x(3,1)=coef_x(3,1)+coef_xy(1,2)*pol_y(2,0,jg)
             coef_x(4,1)=coef_x(4,1)+coef_xy(2,2)*pol_y(2,0,jg)
             coef_x(1,2)=coef_x(1,2)+coef_xy(1,3)*pol_y(1,0,jg)
             coef_x(2,2)=coef_x(2,2)+coef_xy(2,3)*pol_y(1,0,jg)
             coef_x(3,2)=coef_x(3,2)+coef_xy(1,3)*pol_y(2,0,jg)
             coef_x(4,2)=coef_x(4,2)+coef_xy(2,3)*pol_y(2,0,jg)
             coef_x(1,3)=coef_x(1,3)+coef_xy(1,4)*pol_y(1,0,jg)
             coef_x(2,3)=coef_x(2,3)+coef_xy(2,4)*pol_y(1,0,jg)
             coef_x(3,3)=coef_x(3,3)+coef_xy(1,4)*pol_y(2,0,jg)
             coef_x(4,3)=coef_x(4,3)+coef_xy(2,4)*pol_y(2,0,jg)
             coef_x(1,4)=coef_x(1,4)+coef_xy(1,5)*pol_y(1,0,jg)
             coef_x(2,4)=coef_x(2,4)+coef_xy(2,5)*pol_y(1,0,jg)
             coef_x(3,4)=coef_x(3,4)+coef_xy(1,5)*pol_y(2,0,jg)
             coef_x(4,4)=coef_x(4,4)+coef_xy(2,5)*pol_y(2,0,jg)
             coef_x(1,5)=coef_x(1,5)+coef_xy(1,6)*pol_y(1,0,jg)
             coef_x(2,5)=coef_x(2,5)+coef_xy(2,6)*pol_y(1,0,jg)
             coef_x(3,5)=coef_x(3,5)+coef_xy(1,6)*pol_y(2,0,jg)
             coef_x(4,5)=coef_x(4,5)+coef_xy(2,6)*pol_y(2,0,jg)
             coef_x(1,6)=coef_x(1,6)+coef_xy(1,7)*pol_y(1,0,jg)
             coef_x(2,6)=coef_x(2,6)+coef_xy(2,7)*pol_y(1,0,jg)
             coef_x(3,6)=coef_x(3,6)+coef_xy(1,7)*pol_y(2,0,jg)
             coef_x(4,6)=coef_x(4,6)+coef_xy(2,7)*pol_y(2,0,jg)
             coef_x(1,0)=coef_x(1,0)+coef_xy(1,8)*pol_y(1,1,jg)
             coef_x(2,0)=coef_x(2,0)+coef_xy(2,8)*pol_y(1,1,jg)
             coef_x(3,0)=coef_x(3,0)+coef_xy(1,8)*pol_y(2,1,jg)
             coef_x(4,0)=coef_x(4,0)+coef_xy(2,8)*pol_y(2,1,jg)
             coef_x(1,1)=coef_x(1,1)+coef_xy(1,9)*pol_y(1,1,jg)
             coef_x(2,1)=coef_x(2,1)+coef_xy(2,9)*pol_y(1,1,jg)
             coef_x(3,1)=coef_x(3,1)+coef_xy(1,9)*pol_y(2,1,jg)
             coef_x(4,1)=coef_x(4,1)+coef_xy(2,9)*pol_y(2,1,jg)
             coef_x(1,2)=coef_x(1,2)+coef_xy(1,10)*pol_y(1,1,jg)
             coef_x(2,2)=coef_x(2,2)+coef_xy(2,10)*pol_y(1,1,jg)
             coef_x(3,2)=coef_x(3,2)+coef_xy(1,10)*pol_y(2,1,jg)
             coef_x(4,2)=coef_x(4,2)+coef_xy(2,10)*pol_y(2,1,jg)
             coef_x(1,3)=coef_x(1,3)+coef_xy(1,11)*pol_y(1,1,jg)
             coef_x(2,3)=coef_x(2,3)+coef_xy(2,11)*pol_y(1,1,jg)
             coef_x(3,3)=coef_x(3,3)+coef_xy(1,11)*pol_y(2,1,jg)
             coef_x(4,3)=coef_x(4,3)+coef_xy(2,11)*pol_y(2,1,jg)
             coef_x(1,4)=coef_x(1,4)+coef_xy(1,12)*pol_y(1,1,jg)
             coef_x(2,4)=coef_x(2,4)+coef_xy(2,12)*pol_y(1,1,jg)
             coef_x(3,4)=coef_x(3,4)+coef_xy(1,12)*pol_y(2,1,jg)
             coef_x(4,4)=coef_x(4,4)+coef_xy(2,12)*pol_y(2,1,jg)
             coef_x(1,5)=coef_x(1,5)+coef_xy(1,13)*pol_y(1,1,jg)
             coef_x(2,5)=coef_x(2,5)+coef_xy(2,13)*pol_y(1,1,jg)
             coef_x(3,5)=coef_x(3,5)+coef_xy(1,13)*pol_y(2,1,jg)
             coef_x(4,5)=coef_x(4,5)+coef_xy(2,13)*pol_y(2,1,jg)
             coef_x(1,0)=coef_x(1,0)+coef_xy(1,14)*pol_y(1,2,jg)
             coef_x(2,0)=coef_x(2,0)+coef_xy(2,14)*pol_y(1,2,jg)
             coef_x(3,0)=coef_x(3,0)+coef_xy(1,14)*pol_y(2,2,jg)
             coef_x(4,0)=coef_x(4,0)+coef_xy(2,14)*pol_y(2,2,jg)
             coef_x(1,1)=coef_x(1,1)+coef_xy(1,15)*pol_y(1,2,jg)
             coef_x(2,1)=coef_x(2,1)+coef_xy(2,15)*pol_y(1,2,jg)
             coef_x(3,1)=coef_x(3,1)+coef_xy(1,15)*pol_y(2,2,jg)
             coef_x(4,1)=coef_x(4,1)+coef_xy(2,15)*pol_y(2,2,jg)
             coef_x(1,2)=coef_x(1,2)+coef_xy(1,16)*pol_y(1,2,jg)
             coef_x(2,2)=coef_x(2,2)+coef_xy(2,16)*pol_y(1,2,jg)
             coef_x(3,2)=coef_x(3,2)+coef_xy(1,16)*pol_y(2,2,jg)
             coef_x(4,2)=coef_x(4,2)+coef_xy(2,16)*pol_y(2,2,jg)
             coef_x(1,3)=coef_x(1,3)+coef_xy(1,17)*pol_y(1,2,jg)
             coef_x(2,3)=coef_x(2,3)+coef_xy(2,17)*pol_y(1,2,jg)
             coef_x(3,3)=coef_x(3,3)+coef_xy(1,17)*pol_y(2,2,jg)
             coef_x(4,3)=coef_x(4,3)+coef_xy(2,17)*pol_y(2,2,jg)
             coef_x(1,4)=coef_x(1,4)+coef_xy(1,18)*pol_y(1,2,jg)
             coef_x(2,4)=coef_x(2,4)+coef_xy(2,18)*pol_y(1,2,jg)
             coef_x(3,4)=coef_x(3,4)+coef_xy(1,18)*pol_y(2,2,jg)
             coef_x(4,4)=coef_x(4,4)+coef_xy(2,18)*pol_y(2,2,jg)
             coef_x(1,0)=coef_x(1,0)+coef_xy(1,19)*pol_y(1,3,jg)
             coef_x(2,0)=coef_x(2,0)+coef_xy(2,19)*pol_y(1,3,jg)
             coef_x(3,0)=coef_x(3,0)+coef_xy(1,19)*pol_y(2,3,jg)
             coef_x(4,0)=coef_x(4,0)+coef_xy(2,19)*pol_y(2,3,jg)
             coef_x(1,1)=coef_x(1,1)+coef_xy(1,20)*pol_y(1,3,jg)
             coef_x(2,1)=coef_x(2,1)+coef_xy(2,20)*pol_y(1,3,jg)
             coef_x(3,1)=coef_x(3,1)+coef_xy(1,20)*pol_y(2,3,jg)
             coef_x(4,1)=coef_x(4,1)+coef_xy(2,20)*pol_y(2,3,jg)
             coef_x(1,2)=coef_x(1,2)+coef_xy(1,21)*pol_y(1,3,jg)
             coef_x(2,2)=coef_x(2,2)+coef_xy(2,21)*pol_y(1,3,jg)
             coef_x(3,2)=coef_x(3,2)+coef_xy(1,21)*pol_y(2,3,jg)
             coef_x(4,2)=coef_x(4,2)+coef_xy(2,21)*pol_y(2,3,jg)
             coef_x(1,3)=coef_x(1,3)+coef_xy(1,22)*pol_y(1,3,jg)
             coef_x(2,3)=coef_x(2,3)+coef_xy(2,22)*pol_y(1,3,jg)
             coef_x(3,3)=coef_x(3,3)+coef_xy(1,22)*pol_y(2,3,jg)
             coef_x(4,3)=coef_x(4,3)+coef_xy(2,22)*pol_y(2,3,jg)
             coef_x(1,0)=coef_x(1,0)+coef_xy(1,23)*pol_y(1,4,jg)
             coef_x(2,0)=coef_x(2,0)+coef_xy(2,23)*pol_y(1,4,jg)
             coef_x(3,0)=coef_x(3,0)+coef_xy(1,23)*pol_y(2,4,jg)
             coef_x(4,0)=coef_x(4,0)+coef_xy(2,23)*pol_y(2,4,jg)
             coef_x(1,1)=coef_x(1,1)+coef_xy(1,24)*pol_y(1,4,jg)
             coef_x(2,1)=coef_x(2,1)+coef_xy(2,24)*pol_y(1,4,jg)
             coef_x(3,1)=coef_x(3,1)+coef_xy(1,24)*pol_y(2,4,jg)
             coef_x(4,1)=coef_x(4,1)+coef_xy(2,24)*pol_y(2,4,jg)
             coef_x(1,2)=coef_x(1,2)+coef_xy(1,25)*pol_y(1,4,jg)
             coef_x(2,2)=coef_x(2,2)+coef_xy(2,25)*pol_y(1,4,jg)
             coef_x(3,2)=coef_x(3,2)+coef_xy(1,25)*pol_y(2,4,jg)
             coef_x(4,2)=coef_x(4,2)+coef_xy(2,25)*pol_y(2,4,jg)
             coef_x(1,0)=coef_x(1,0)+coef_xy(1,26)*pol_y(1,5,jg)
             coef_x(2,0)=coef_x(2,0)+coef_xy(2,26)*pol_y(1,5,jg)
             coef_x(3,0)=coef_x(3,0)+coef_xy(1,26)*pol_y(2,5,jg)
             coef_x(4,0)=coef_x(4,0)+coef_xy(2,26)*pol_y(2,5,jg)
             coef_x(1,1)=coef_x(1,1)+coef_xy(1,27)*pol_y(1,5,jg)
             coef_x(2,1)=coef_x(2,1)+coef_xy(2,27)*pol_y(1,5,jg)
             coef_x(3,1)=coef_x(3,1)+coef_xy(1,27)*pol_y(2,5,jg)
             coef_x(4,1)=coef_x(4,1)+coef_xy(2,27)*pol_y(2,5,jg)
             coef_x(1,0)=coef_x(1,0)+coef_xy(1,28)*pol_y(1,6,jg)
             coef_x(2,0)=coef_x(2,0)+coef_xy(2,28)*pol_y(1,6,jg)
             coef_x(3,0)=coef_x(3,0)+coef_xy(1,28)*pol_y(2,6,jg)
             coef_x(4,0)=coef_x(4,0)+coef_xy(2,28)*pol_y(2,6,jg)
          DO ig=igmin,igmax
             i=map(ig,1)
             s01=0.0_dp
             s02=0.0_dp
             s03=0.0_dp
             s04=0.0_dp
             DO lxp=0,lp
                s01=s01+coef_x(1,lxp)*pol_x(lxp,ig)
                s02=s02+coef_x(2,lxp)*pol_x(lxp,ig)
                s03=s03+coef_x(3,lxp)*pol_x(lxp,ig)
                s04=s04+coef_x(4,lxp)*pol_x(lxp,ig)
             ENDDO
             grid(i,j,k) = grid(i,j,k)     + s01
             grid(i,j2,k) = grid(i,j2,k)   + s03
             grid(i,j,k2) = grid(i,j,k2)   + s02
             grid(i,j2,k2) = grid(i,j2,k2) + s04
          END DO
       END DO
    END DO

  END SUBROUTINE collocate_core_6
  SUBROUTINE collocate_core_7(grid,coef_xyz,pol_x,pol_y,pol_z,map,sphere_bounds,cmax,gridbounds)
    USE kinds,                        ONLY: dp
    INTEGER, INTENT(IN)                      :: sphere_bounds(*), cmax, &
                                                map(-cmax:cmax,1:3), &
                                                gridbounds(2,3)
    REAL(dp), INTENT(INOUT) :: grid(gridbounds(1,1):gridbounds(2,1), &
      gridbounds(1,2):gridbounds(2,2), gridbounds(1,3):gridbounds(2,3))
    INTEGER, PARAMETER                       :: lp = 7
    REAL(dp), INTENT(IN) :: pol_x(0:lp,-cmax:cmax), pol_y(1:2,0:lp,-cmax:0), &
      pol_z(1:2,0:lp,-cmax:0), coef_xyz(((lp+1)*(lp+2)*(lp+3))/6)

    INTEGER                                  :: i, ig, igmax, igmin, j, j2, &
                                                jg, jg2, jgmin, k, k2, kg, &
                                                kg2, kgmin, lxp, sci
    REAL(dp)                                 :: coef_x(4,0:lp), &
                                                coef_xy(2,(lp+1)*(lp+2)/2), &
                                                s01, s02, s03, s04

    sci=1

    kgmin=sphere_bounds(sci)
    sci=sci+1
    DO kg=kgmin,0
       kg2=1-kg
       k=map(kg,3)
       k2=map(kg2,3)
       coef_xy=0.0_dp
                coef_xy(:,1)=coef_xy(:,1)+coef_xyz(1)*pol_z(:,0,kg)
                coef_xy(:,2)=coef_xy(:,2)+coef_xyz(2)*pol_z(:,0,kg)
                coef_xy(:,3)=coef_xy(:,3)+coef_xyz(3)*pol_z(:,0,kg)
                coef_xy(:,4)=coef_xy(:,4)+coef_xyz(4)*pol_z(:,0,kg)
                coef_xy(:,5)=coef_xy(:,5)+coef_xyz(5)*pol_z(:,0,kg)
                coef_xy(:,6)=coef_xy(:,6)+coef_xyz(6)*pol_z(:,0,kg)
                coef_xy(:,7)=coef_xy(:,7)+coef_xyz(7)*pol_z(:,0,kg)
                coef_xy(:,8)=coef_xy(:,8)+coef_xyz(8)*pol_z(:,0,kg)
                coef_xy(:,9)=coef_xy(:,9)+coef_xyz(9)*pol_z(:,0,kg)
                coef_xy(:,10)=coef_xy(:,10)+coef_xyz(10)*pol_z(:,0,kg)
                coef_xy(:,11)=coef_xy(:,11)+coef_xyz(11)*pol_z(:,0,kg)
                coef_xy(:,12)=coef_xy(:,12)+coef_xyz(12)*pol_z(:,0,kg)
                coef_xy(:,13)=coef_xy(:,13)+coef_xyz(13)*pol_z(:,0,kg)
                coef_xy(:,14)=coef_xy(:,14)+coef_xyz(14)*pol_z(:,0,kg)
                coef_xy(:,15)=coef_xy(:,15)+coef_xyz(15)*pol_z(:,0,kg)
                coef_xy(:,16)=coef_xy(:,16)+coef_xyz(16)*pol_z(:,0,kg)
                coef_xy(:,17)=coef_xy(:,17)+coef_xyz(17)*pol_z(:,0,kg)
                coef_xy(:,18)=coef_xy(:,18)+coef_xyz(18)*pol_z(:,0,kg)
                coef_xy(:,19)=coef_xy(:,19)+coef_xyz(19)*pol_z(:,0,kg)
                coef_xy(:,20)=coef_xy(:,20)+coef_xyz(20)*pol_z(:,0,kg)
                coef_xy(:,21)=coef_xy(:,21)+coef_xyz(21)*pol_z(:,0,kg)
                coef_xy(:,22)=coef_xy(:,22)+coef_xyz(22)*pol_z(:,0,kg)
                coef_xy(:,23)=coef_xy(:,23)+coef_xyz(23)*pol_z(:,0,kg)
                coef_xy(:,24)=coef_xy(:,24)+coef_xyz(24)*pol_z(:,0,kg)
                coef_xy(:,25)=coef_xy(:,25)+coef_xyz(25)*pol_z(:,0,kg)
                coef_xy(:,26)=coef_xy(:,26)+coef_xyz(26)*pol_z(:,0,kg)
                coef_xy(:,27)=coef_xy(:,27)+coef_xyz(27)*pol_z(:,0,kg)
                coef_xy(:,28)=coef_xy(:,28)+coef_xyz(28)*pol_z(:,0,kg)
                coef_xy(:,29)=coef_xy(:,29)+coef_xyz(29)*pol_z(:,0,kg)
                coef_xy(:,30)=coef_xy(:,30)+coef_xyz(30)*pol_z(:,0,kg)
                coef_xy(:,31)=coef_xy(:,31)+coef_xyz(31)*pol_z(:,0,kg)
                coef_xy(:,32)=coef_xy(:,32)+coef_xyz(32)*pol_z(:,0,kg)
                coef_xy(:,33)=coef_xy(:,33)+coef_xyz(33)*pol_z(:,0,kg)
                coef_xy(:,34)=coef_xy(:,34)+coef_xyz(34)*pol_z(:,0,kg)
                coef_xy(:,35)=coef_xy(:,35)+coef_xyz(35)*pol_z(:,0,kg)
                coef_xy(:,36)=coef_xy(:,36)+coef_xyz(36)*pol_z(:,0,kg)
                coef_xy(:,1)=coef_xy(:,1)+coef_xyz(37)*pol_z(:,1,kg)
                coef_xy(:,2)=coef_xy(:,2)+coef_xyz(38)*pol_z(:,1,kg)
                coef_xy(:,3)=coef_xy(:,3)+coef_xyz(39)*pol_z(:,1,kg)
                coef_xy(:,4)=coef_xy(:,4)+coef_xyz(40)*pol_z(:,1,kg)
                coef_xy(:,5)=coef_xy(:,5)+coef_xyz(41)*pol_z(:,1,kg)
                coef_xy(:,6)=coef_xy(:,6)+coef_xyz(42)*pol_z(:,1,kg)
                coef_xy(:,7)=coef_xy(:,7)+coef_xyz(43)*pol_z(:,1,kg)
                coef_xy(:,9)=coef_xy(:,9)+coef_xyz(44)*pol_z(:,1,kg)
                coef_xy(:,10)=coef_xy(:,10)+coef_xyz(45)*pol_z(:,1,kg)
                coef_xy(:,11)=coef_xy(:,11)+coef_xyz(46)*pol_z(:,1,kg)
                coef_xy(:,12)=coef_xy(:,12)+coef_xyz(47)*pol_z(:,1,kg)
                coef_xy(:,13)=coef_xy(:,13)+coef_xyz(48)*pol_z(:,1,kg)
                coef_xy(:,14)=coef_xy(:,14)+coef_xyz(49)*pol_z(:,1,kg)
                coef_xy(:,16)=coef_xy(:,16)+coef_xyz(50)*pol_z(:,1,kg)
                coef_xy(:,17)=coef_xy(:,17)+coef_xyz(51)*pol_z(:,1,kg)
                coef_xy(:,18)=coef_xy(:,18)+coef_xyz(52)*pol_z(:,1,kg)
                coef_xy(:,19)=coef_xy(:,19)+coef_xyz(53)*pol_z(:,1,kg)
                coef_xy(:,20)=coef_xy(:,20)+coef_xyz(54)*pol_z(:,1,kg)
                coef_xy(:,22)=coef_xy(:,22)+coef_xyz(55)*pol_z(:,1,kg)
                coef_xy(:,23)=coef_xy(:,23)+coef_xyz(56)*pol_z(:,1,kg)
                coef_xy(:,24)=coef_xy(:,24)+coef_xyz(57)*pol_z(:,1,kg)
                coef_xy(:,25)=coef_xy(:,25)+coef_xyz(58)*pol_z(:,1,kg)
                coef_xy(:,27)=coef_xy(:,27)+coef_xyz(59)*pol_z(:,1,kg)
                coef_xy(:,28)=coef_xy(:,28)+coef_xyz(60)*pol_z(:,1,kg)
                coef_xy(:,29)=coef_xy(:,29)+coef_xyz(61)*pol_z(:,1,kg)
                coef_xy(:,31)=coef_xy(:,31)+coef_xyz(62)*pol_z(:,1,kg)
                coef_xy(:,32)=coef_xy(:,32)+coef_xyz(63)*pol_z(:,1,kg)
                coef_xy(:,34)=coef_xy(:,34)+coef_xyz(64)*pol_z(:,1,kg)
                coef_xy(:,1)=coef_xy(:,1)+coef_xyz(65)*pol_z(:,2,kg)
                coef_xy(:,2)=coef_xy(:,2)+coef_xyz(66)*pol_z(:,2,kg)
                coef_xy(:,3)=coef_xy(:,3)+coef_xyz(67)*pol_z(:,2,kg)
                coef_xy(:,4)=coef_xy(:,4)+coef_xyz(68)*pol_z(:,2,kg)
                coef_xy(:,5)=coef_xy(:,5)+coef_xyz(69)*pol_z(:,2,kg)
                coef_xy(:,6)=coef_xy(:,6)+coef_xyz(70)*pol_z(:,2,kg)
                coef_xy(:,9)=coef_xy(:,9)+coef_xyz(71)*pol_z(:,2,kg)
                coef_xy(:,10)=coef_xy(:,10)+coef_xyz(72)*pol_z(:,2,kg)
                coef_xy(:,11)=coef_xy(:,11)+coef_xyz(73)*pol_z(:,2,kg)
                coef_xy(:,12)=coef_xy(:,12)+coef_xyz(74)*pol_z(:,2,kg)
                coef_xy(:,13)=coef_xy(:,13)+coef_xyz(75)*pol_z(:,2,kg)
                coef_xy(:,16)=coef_xy(:,16)+coef_xyz(76)*pol_z(:,2,kg)
                coef_xy(:,17)=coef_xy(:,17)+coef_xyz(77)*pol_z(:,2,kg)
                coef_xy(:,18)=coef_xy(:,18)+coef_xyz(78)*pol_z(:,2,kg)
                coef_xy(:,19)=coef_xy(:,19)+coef_xyz(79)*pol_z(:,2,kg)
                coef_xy(:,22)=coef_xy(:,22)+coef_xyz(80)*pol_z(:,2,kg)
                coef_xy(:,23)=coef_xy(:,23)+coef_xyz(81)*pol_z(:,2,kg)
                coef_xy(:,24)=coef_xy(:,24)+coef_xyz(82)*pol_z(:,2,kg)
                coef_xy(:,27)=coef_xy(:,27)+coef_xyz(83)*pol_z(:,2,kg)
                coef_xy(:,28)=coef_xy(:,28)+coef_xyz(84)*pol_z(:,2,kg)
                coef_xy(:,31)=coef_xy(:,31)+coef_xyz(85)*pol_z(:,2,kg)
                coef_xy(:,1)=coef_xy(:,1)+coef_xyz(86)*pol_z(:,3,kg)
                coef_xy(:,2)=coef_xy(:,2)+coef_xyz(87)*pol_z(:,3,kg)
                coef_xy(:,3)=coef_xy(:,3)+coef_xyz(88)*pol_z(:,3,kg)
                coef_xy(:,4)=coef_xy(:,4)+coef_xyz(89)*pol_z(:,3,kg)
                coef_xy(:,5)=coef_xy(:,5)+coef_xyz(90)*pol_z(:,3,kg)
                coef_xy(:,9)=coef_xy(:,9)+coef_xyz(91)*pol_z(:,3,kg)
                coef_xy(:,10)=coef_xy(:,10)+coef_xyz(92)*pol_z(:,3,kg)
                coef_xy(:,11)=coef_xy(:,11)+coef_xyz(93)*pol_z(:,3,kg)
                coef_xy(:,12)=coef_xy(:,12)+coef_xyz(94)*pol_z(:,3,kg)
                coef_xy(:,16)=coef_xy(:,16)+coef_xyz(95)*pol_z(:,3,kg)
                coef_xy(:,17)=coef_xy(:,17)+coef_xyz(96)*pol_z(:,3,kg)
                coef_xy(:,18)=coef_xy(:,18)+coef_xyz(97)*pol_z(:,3,kg)
                coef_xy(:,22)=coef_xy(:,22)+coef_xyz(98)*pol_z(:,3,kg)
                coef_xy(:,23)=coef_xy(:,23)+coef_xyz(99)*pol_z(:,3,kg)
                coef_xy(:,27)=coef_xy(:,27)+coef_xyz(100)*pol_z(:,3,kg)
                coef_xy(:,1)=coef_xy(:,1)+coef_xyz(101)*pol_z(:,4,kg)
                coef_xy(:,2)=coef_xy(:,2)+coef_xyz(102)*pol_z(:,4,kg)
                coef_xy(:,3)=coef_xy(:,3)+coef_xyz(103)*pol_z(:,4,kg)
                coef_xy(:,4)=coef_xy(:,4)+coef_xyz(104)*pol_z(:,4,kg)
                coef_xy(:,9)=coef_xy(:,9)+coef_xyz(105)*pol_z(:,4,kg)
                coef_xy(:,10)=coef_xy(:,10)+coef_xyz(106)*pol_z(:,4,kg)
                coef_xy(:,11)=coef_xy(:,11)+coef_xyz(107)*pol_z(:,4,kg)
                coef_xy(:,16)=coef_xy(:,16)+coef_xyz(108)*pol_z(:,4,kg)
                coef_xy(:,17)=coef_xy(:,17)+coef_xyz(109)*pol_z(:,4,kg)
                coef_xy(:,22)=coef_xy(:,22)+coef_xyz(110)*pol_z(:,4,kg)
                coef_xy(:,1)=coef_xy(:,1)+coef_xyz(111)*pol_z(:,5,kg)
                coef_xy(:,2)=coef_xy(:,2)+coef_xyz(112)*pol_z(:,5,kg)
                coef_xy(:,3)=coef_xy(:,3)+coef_xyz(113)*pol_z(:,5,kg)
                coef_xy(:,9)=coef_xy(:,9)+coef_xyz(114)*pol_z(:,5,kg)
                coef_xy(:,10)=coef_xy(:,10)+coef_xyz(115)*pol_z(:,5,kg)
                coef_xy(:,16)=coef_xy(:,16)+coef_xyz(116)*pol_z(:,5,kg)
                coef_xy(:,1)=coef_xy(:,1)+coef_xyz(117)*pol_z(:,6,kg)
                coef_xy(:,2)=coef_xy(:,2)+coef_xyz(118)*pol_z(:,6,kg)
                coef_xy(:,9)=coef_xy(:,9)+coef_xyz(119)*pol_z(:,6,kg)
                coef_xy(:,1)=coef_xy(:,1)+coef_xyz(120)*pol_z(:,7,kg)
       jgmin=sphere_bounds(sci)
       sci=sci+1
       DO jg=jgmin,0
          jg2=1-jg
          j=map(jg,2)
          j2=map(jg2,2)
          igmin=sphere_bounds(sci)
          sci=sci+1
          igmax=1-igmin
          coef_x=0.0_dp
             coef_x(1,0)=coef_x(1,0)+coef_xy(1,1)*pol_y(1,0,jg)
             coef_x(2,0)=coef_x(2,0)+coef_xy(2,1)*pol_y(1,0,jg)
             coef_x(3,0)=coef_x(3,0)+coef_xy(1,1)*pol_y(2,0,jg)
             coef_x(4,0)=coef_x(4,0)+coef_xy(2,1)*pol_y(2,0,jg)
             coef_x(1,1)=coef_x(1,1)+coef_xy(1,2)*pol_y(1,0,jg)
             coef_x(2,1)=coef_x(2,1)+coef_xy(2,2)*pol_y(1,0,jg)
             coef_x(3,1)=coef_x(3,1)+coef_xy(1,2)*pol_y(2,0,jg)
             coef_x(4,1)=coef_x(4,1)+coef_xy(2,2)*pol_y(2,0,jg)
             coef_x(1,2)=coef_x(1,2)+coef_xy(1,3)*pol_y(1,0,jg)
             coef_x(2,2)=coef_x(2,2)+coef_xy(2,3)*pol_y(1,0,jg)
             coef_x(3,2)=coef_x(3,2)+coef_xy(1,3)*pol_y(2,0,jg)
             coef_x(4,2)=coef_x(4,2)+coef_xy(2,3)*pol_y(2,0,jg)
             coef_x(1,3)=coef_x(1,3)+coef_xy(1,4)*pol_y(1,0,jg)
             coef_x(2,3)=coef_x(2,3)+coef_xy(2,4)*pol_y(1,0,jg)
             coef_x(3,3)=coef_x(3,3)+coef_xy(1,4)*pol_y(2,0,jg)
             coef_x(4,3)=coef_x(4,3)+coef_xy(2,4)*pol_y(2,0,jg)
             coef_x(1,4)=coef_x(1,4)+coef_xy(1,5)*pol_y(1,0,jg)
             coef_x(2,4)=coef_x(2,4)+coef_xy(2,5)*pol_y(1,0,jg)
             coef_x(3,4)=coef_x(3,4)+coef_xy(1,5)*pol_y(2,0,jg)
             coef_x(4,4)=coef_x(4,4)+coef_xy(2,5)*pol_y(2,0,jg)
             coef_x(1,5)=coef_x(1,5)+coef_xy(1,6)*pol_y(1,0,jg)
             coef_x(2,5)=coef_x(2,5)+coef_xy(2,6)*pol_y(1,0,jg)
             coef_x(3,5)=coef_x(3,5)+coef_xy(1,6)*pol_y(2,0,jg)
             coef_x(4,5)=coef_x(4,5)+coef_xy(2,6)*pol_y(2,0,jg)
             coef_x(1,6)=coef_x(1,6)+coef_xy(1,7)*pol_y(1,0,jg)
             coef_x(2,6)=coef_x(2,6)+coef_xy(2,7)*pol_y(1,0,jg)
             coef_x(3,6)=coef_x(3,6)+coef_xy(1,7)*pol_y(2,0,jg)
             coef_x(4,6)=coef_x(4,6)+coef_xy(2,7)*pol_y(2,0,jg)
             coef_x(1,7)=coef_x(1,7)+coef_xy(1,8)*pol_y(1,0,jg)
             coef_x(2,7)=coef_x(2,7)+coef_xy(2,8)*pol_y(1,0,jg)
             coef_x(3,7)=coef_x(3,7)+coef_xy(1,8)*pol_y(2,0,jg)
             coef_x(4,7)=coef_x(4,7)+coef_xy(2,8)*pol_y(2,0,jg)
             coef_x(1,0)=coef_x(1,0)+coef_xy(1,9)*pol_y(1,1,jg)
             coef_x(2,0)=coef_x(2,0)+coef_xy(2,9)*pol_y(1,1,jg)
             coef_x(3,0)=coef_x(3,0)+coef_xy(1,9)*pol_y(2,1,jg)
             coef_x(4,0)=coef_x(4,0)+coef_xy(2,9)*pol_y(2,1,jg)
             coef_x(1,1)=coef_x(1,1)+coef_xy(1,10)*pol_y(1,1,jg)
             coef_x(2,1)=coef_x(2,1)+coef_xy(2,10)*pol_y(1,1,jg)
             coef_x(3,1)=coef_x(3,1)+coef_xy(1,10)*pol_y(2,1,jg)
             coef_x(4,1)=coef_x(4,1)+coef_xy(2,10)*pol_y(2,1,jg)
             coef_x(1,2)=coef_x(1,2)+coef_xy(1,11)*pol_y(1,1,jg)
             coef_x(2,2)=coef_x(2,2)+coef_xy(2,11)*pol_y(1,1,jg)
             coef_x(3,2)=coef_x(3,2)+coef_xy(1,11)*pol_y(2,1,jg)
             coef_x(4,2)=coef_x(4,2)+coef_xy(2,11)*pol_y(2,1,jg)
             coef_x(1,3)=coef_x(1,3)+coef_xy(1,12)*pol_y(1,1,jg)
             coef_x(2,3)=coef_x(2,3)+coef_xy(2,12)*pol_y(1,1,jg)
             coef_x(3,3)=coef_x(3,3)+coef_xy(1,12)*pol_y(2,1,jg)
             coef_x(4,3)=coef_x(4,3)+coef_xy(2,12)*pol_y(2,1,jg)
             coef_x(1,4)=coef_x(1,4)+coef_xy(1,13)*pol_y(1,1,jg)
             coef_x(2,4)=coef_x(2,4)+coef_xy(2,13)*pol_y(1,1,jg)
             coef_x(3,4)=coef_x(3,4)+coef_xy(1,13)*pol_y(2,1,jg)
             coef_x(4,4)=coef_x(4,4)+coef_xy(2,13)*pol_y(2,1,jg)
             coef_x(1,5)=coef_x(1,5)+coef_xy(1,14)*pol_y(1,1,jg)
             coef_x(2,5)=coef_x(2,5)+coef_xy(2,14)*pol_y(1,1,jg)
             coef_x(3,5)=coef_x(3,5)+coef_xy(1,14)*pol_y(2,1,jg)
             coef_x(4,5)=coef_x(4,5)+coef_xy(2,14)*pol_y(2,1,jg)
             coef_x(1,6)=coef_x(1,6)+coef_xy(1,15)*pol_y(1,1,jg)
             coef_x(2,6)=coef_x(2,6)+coef_xy(2,15)*pol_y(1,1,jg)
             coef_x(3,6)=coef_x(3,6)+coef_xy(1,15)*pol_y(2,1,jg)
             coef_x(4,6)=coef_x(4,6)+coef_xy(2,15)*pol_y(2,1,jg)
             coef_x(1,0)=coef_x(1,0)+coef_xy(1,16)*pol_y(1,2,jg)
             coef_x(2,0)=coef_x(2,0)+coef_xy(2,16)*pol_y(1,2,jg)
             coef_x(3,0)=coef_x(3,0)+coef_xy(1,16)*pol_y(2,2,jg)
             coef_x(4,0)=coef_x(4,0)+coef_xy(2,16)*pol_y(2,2,jg)
             coef_x(1,1)=coef_x(1,1)+coef_xy(1,17)*pol_y(1,2,jg)
             coef_x(2,1)=coef_x(2,1)+coef_xy(2,17)*pol_y(1,2,jg)
             coef_x(3,1)=coef_x(3,1)+coef_xy(1,17)*pol_y(2,2,jg)
             coef_x(4,1)=coef_x(4,1)+coef_xy(2,17)*pol_y(2,2,jg)
             coef_x(1,2)=coef_x(1,2)+coef_xy(1,18)*pol_y(1,2,jg)
             coef_x(2,2)=coef_x(2,2)+coef_xy(2,18)*pol_y(1,2,jg)
             coef_x(3,2)=coef_x(3,2)+coef_xy(1,18)*pol_y(2,2,jg)
             coef_x(4,2)=coef_x(4,2)+coef_xy(2,18)*pol_y(2,2,jg)
             coef_x(1,3)=coef_x(1,3)+coef_xy(1,19)*pol_y(1,2,jg)
             coef_x(2,3)=coef_x(2,3)+coef_xy(2,19)*pol_y(1,2,jg)
             coef_x(3,3)=coef_x(3,3)+coef_xy(1,19)*pol_y(2,2,jg)
             coef_x(4,3)=coef_x(4,3)+coef_xy(2,19)*pol_y(2,2,jg)
             coef_x(1,4)=coef_x(1,4)+coef_xy(1,20)*pol_y(1,2,jg)
             coef_x(2,4)=coef_x(2,4)+coef_xy(2,20)*pol_y(1,2,jg)
             coef_x(3,4)=coef_x(3,4)+coef_xy(1,20)*pol_y(2,2,jg)
             coef_x(4,4)=coef_x(4,4)+coef_xy(2,20)*pol_y(2,2,jg)
             coef_x(1,5)=coef_x(1,5)+coef_xy(1,21)*pol_y(1,2,jg)
             coef_x(2,5)=coef_x(2,5)+coef_xy(2,21)*pol_y(1,2,jg)
             coef_x(3,5)=coef_x(3,5)+coef_xy(1,21)*pol_y(2,2,jg)
             coef_x(4,5)=coef_x(4,5)+coef_xy(2,21)*pol_y(2,2,jg)
             coef_x(1,0)=coef_x(1,0)+coef_xy(1,22)*pol_y(1,3,jg)
             coef_x(2,0)=coef_x(2,0)+coef_xy(2,22)*pol_y(1,3,jg)
             coef_x(3,0)=coef_x(3,0)+coef_xy(1,22)*pol_y(2,3,jg)
             coef_x(4,0)=coef_x(4,0)+coef_xy(2,22)*pol_y(2,3,jg)
             coef_x(1,1)=coef_x(1,1)+coef_xy(1,23)*pol_y(1,3,jg)
             coef_x(2,1)=coef_x(2,1)+coef_xy(2,23)*pol_y(1,3,jg)
             coef_x(3,1)=coef_x(3,1)+coef_xy(1,23)*pol_y(2,3,jg)
             coef_x(4,1)=coef_x(4,1)+coef_xy(2,23)*pol_y(2,3,jg)
             coef_x(1,2)=coef_x(1,2)+coef_xy(1,24)*pol_y(1,3,jg)
             coef_x(2,2)=coef_x(2,2)+coef_xy(2,24)*pol_y(1,3,jg)
             coef_x(3,2)=coef_x(3,2)+coef_xy(1,24)*pol_y(2,3,jg)
             coef_x(4,2)=coef_x(4,2)+coef_xy(2,24)*pol_y(2,3,jg)
             coef_x(1,3)=coef_x(1,3)+coef_xy(1,25)*pol_y(1,3,jg)
             coef_x(2,3)=coef_x(2,3)+coef_xy(2,25)*pol_y(1,3,jg)
             coef_x(3,3)=coef_x(3,3)+coef_xy(1,25)*pol_y(2,3,jg)
             coef_x(4,3)=coef_x(4,3)+coef_xy(2,25)*pol_y(2,3,jg)
             coef_x(1,4)=coef_x(1,4)+coef_xy(1,26)*pol_y(1,3,jg)
             coef_x(2,4)=coef_x(2,4)+coef_xy(2,26)*pol_y(1,3,jg)
             coef_x(3,4)=coef_x(3,4)+coef_xy(1,26)*pol_y(2,3,jg)
             coef_x(4,4)=coef_x(4,4)+coef_xy(2,26)*pol_y(2,3,jg)
             coef_x(1,0)=coef_x(1,0)+coef_xy(1,27)*pol_y(1,4,jg)
             coef_x(2,0)=coef_x(2,0)+coef_xy(2,27)*pol_y(1,4,jg)
             coef_x(3,0)=coef_x(3,0)+coef_xy(1,27)*pol_y(2,4,jg)
             coef_x(4,0)=coef_x(4,0)+coef_xy(2,27)*pol_y(2,4,jg)
             coef_x(1,1)=coef_x(1,1)+coef_xy(1,28)*pol_y(1,4,jg)
             coef_x(2,1)=coef_x(2,1)+coef_xy(2,28)*pol_y(1,4,jg)
             coef_x(3,1)=coef_x(3,1)+coef_xy(1,28)*pol_y(2,4,jg)
             coef_x(4,1)=coef_x(4,1)+coef_xy(2,28)*pol_y(2,4,jg)
             coef_x(1,2)=coef_x(1,2)+coef_xy(1,29)*pol_y(1,4,jg)
             coef_x(2,2)=coef_x(2,2)+coef_xy(2,29)*pol_y(1,4,jg)
             coef_x(3,2)=coef_x(3,2)+coef_xy(1,29)*pol_y(2,4,jg)
             coef_x(4,2)=coef_x(4,2)+coef_xy(2,29)*pol_y(2,4,jg)
             coef_x(1,3)=coef_x(1,3)+coef_xy(1,30)*pol_y(1,4,jg)
             coef_x(2,3)=coef_x(2,3)+coef_xy(2,30)*pol_y(1,4,jg)
             coef_x(3,3)=coef_x(3,3)+coef_xy(1,30)*pol_y(2,4,jg)
             coef_x(4,3)=coef_x(4,3)+coef_xy(2,30)*pol_y(2,4,jg)
             coef_x(1,0)=coef_x(1,0)+coef_xy(1,31)*pol_y(1,5,jg)
             coef_x(2,0)=coef_x(2,0)+coef_xy(2,31)*pol_y(1,5,jg)
             coef_x(3,0)=coef_x(3,0)+coef_xy(1,31)*pol_y(2,5,jg)
             coef_x(4,0)=coef_x(4,0)+coef_xy(2,31)*pol_y(2,5,jg)
             coef_x(1,1)=coef_x(1,1)+coef_xy(1,32)*pol_y(1,5,jg)
             coef_x(2,1)=coef_x(2,1)+coef_xy(2,32)*pol_y(1,5,jg)
             coef_x(3,1)=coef_x(3,1)+coef_xy(1,32)*pol_y(2,5,jg)
             coef_x(4,1)=coef_x(4,1)+coef_xy(2,32)*pol_y(2,5,jg)
             coef_x(1,2)=coef_x(1,2)+coef_xy(1,33)*pol_y(1,5,jg)
             coef_x(2,2)=coef_x(2,2)+coef_xy(2,33)*pol_y(1,5,jg)
             coef_x(3,2)=coef_x(3,2)+coef_xy(1,33)*pol_y(2,5,jg)
             coef_x(4,2)=coef_x(4,2)+coef_xy(2,33)*pol_y(2,5,jg)
             coef_x(1,0)=coef_x(1,0)+coef_xy(1,34)*pol_y(1,6,jg)
             coef_x(2,0)=coef_x(2,0)+coef_xy(2,34)*pol_y(1,6,jg)
             coef_x(3,0)=coef_x(3,0)+coef_xy(1,34)*pol_y(2,6,jg)
             coef_x(4,0)=coef_x(4,0)+coef_xy(2,34)*pol_y(2,6,jg)
             coef_x(1,1)=coef_x(1,1)+coef_xy(1,35)*pol_y(1,6,jg)
             coef_x(2,1)=coef_x(2,1)+coef_xy(2,35)*pol_y(1,6,jg)
             coef_x(3,1)=coef_x(3,1)+coef_xy(1,35)*pol_y(2,6,jg)
             coef_x(4,1)=coef_x(4,1)+coef_xy(2,35)*pol_y(2,6,jg)
             coef_x(1,0)=coef_x(1,0)+coef_xy(1,36)*pol_y(1,7,jg)
             coef_x(2,0)=coef_x(2,0)+coef_xy(2,36)*pol_y(1,7,jg)
             coef_x(3,0)=coef_x(3,0)+coef_xy(1,36)*pol_y(2,7,jg)
             coef_x(4,0)=coef_x(4,0)+coef_xy(2,36)*pol_y(2,7,jg)
          DO ig=igmin,igmax
             i=map(ig,1)
             s01=0.0_dp
             s02=0.0_dp
             s03=0.0_dp
             s04=0.0_dp
             DO lxp=0,lp
                s01=s01+coef_x(1,lxp)*pol_x(lxp,ig)
                s02=s02+coef_x(2,lxp)*pol_x(lxp,ig)
                s03=s03+coef_x(3,lxp)*pol_x(lxp,ig)
                s04=s04+coef_x(4,lxp)*pol_x(lxp,ig)
             ENDDO
             grid(i,j,k) = grid(i,j,k)     + s01
             grid(i,j2,k) = grid(i,j2,k)   + s03
             grid(i,j,k2) = grid(i,j,k2)   + s02
             grid(i,j2,k2) = grid(i,j2,k2) + s04
          END DO
       END DO
    END DO

  END SUBROUTINE collocate_core_7
  SUBROUTINE collocate_core_8(grid,coef_xyz,pol_x,pol_y,pol_z,map,sphere_bounds,cmax,gridbounds)
    USE kinds,                        ONLY: dp
    INTEGER, INTENT(IN)                      :: sphere_bounds(*), cmax, &
                                                map(-cmax:cmax,1:3), &
                                                gridbounds(2,3)
    REAL(dp), INTENT(INOUT) :: grid(gridbounds(1,1):gridbounds(2,1), &
      gridbounds(1,2):gridbounds(2,2), gridbounds(1,3):gridbounds(2,3))
    INTEGER, PARAMETER                       :: lp = 8
    REAL(dp), INTENT(IN) :: pol_x(0:lp,-cmax:cmax), pol_y(1:2,0:lp,-cmax:0), &
      pol_z(1:2,0:lp,-cmax:0), coef_xyz(((lp+1)*(lp+2)*(lp+3))/6)

    INTEGER                                  :: i, ig, igmax, igmin, j, j2, &
                                                jg, jg2, jgmin, k, k2, kg, &
                                                kg2, kgmin, lxp, sci
    REAL(dp)                                 :: coef_x(4,0:lp), &
                                                coef_xy(2,(lp+1)*(lp+2)/2), &
                                                s01, s02, s03, s04

    sci=1

    kgmin=sphere_bounds(sci)
    sci=sci+1
    DO kg=kgmin,0
       kg2=1-kg
       k=map(kg,3)
       k2=map(kg2,3)
       coef_xy=0.0_dp
                coef_xy(:,1)=coef_xy(:,1)+coef_xyz(1)*pol_z(:,0,kg)
                coef_xy(:,2)=coef_xy(:,2)+coef_xyz(2)*pol_z(:,0,kg)
                coef_xy(:,3)=coef_xy(:,3)+coef_xyz(3)*pol_z(:,0,kg)
                coef_xy(:,4)=coef_xy(:,4)+coef_xyz(4)*pol_z(:,0,kg)
                coef_xy(:,5)=coef_xy(:,5)+coef_xyz(5)*pol_z(:,0,kg)
                coef_xy(:,6)=coef_xy(:,6)+coef_xyz(6)*pol_z(:,0,kg)
                coef_xy(:,7)=coef_xy(:,7)+coef_xyz(7)*pol_z(:,0,kg)
                coef_xy(:,8)=coef_xy(:,8)+coef_xyz(8)*pol_z(:,0,kg)
                coef_xy(:,9)=coef_xy(:,9)+coef_xyz(9)*pol_z(:,0,kg)
                coef_xy(:,10)=coef_xy(:,10)+coef_xyz(10)*pol_z(:,0,kg)
                coef_xy(:,11)=coef_xy(:,11)+coef_xyz(11)*pol_z(:,0,kg)
                coef_xy(:,12)=coef_xy(:,12)+coef_xyz(12)*pol_z(:,0,kg)
                coef_xy(:,13)=coef_xy(:,13)+coef_xyz(13)*pol_z(:,0,kg)
                coef_xy(:,14)=coef_xy(:,14)+coef_xyz(14)*pol_z(:,0,kg)
                coef_xy(:,15)=coef_xy(:,15)+coef_xyz(15)*pol_z(:,0,kg)
                coef_xy(:,16)=coef_xy(:,16)+coef_xyz(16)*pol_z(:,0,kg)
                coef_xy(:,17)=coef_xy(:,17)+coef_xyz(17)*pol_z(:,0,kg)
                coef_xy(:,18)=coef_xy(:,18)+coef_xyz(18)*pol_z(:,0,kg)
                coef_xy(:,19)=coef_xy(:,19)+coef_xyz(19)*pol_z(:,0,kg)
                coef_xy(:,20)=coef_xy(:,20)+coef_xyz(20)*pol_z(:,0,kg)
                coef_xy(:,21)=coef_xy(:,21)+coef_xyz(21)*pol_z(:,0,kg)
                coef_xy(:,22)=coef_xy(:,22)+coef_xyz(22)*pol_z(:,0,kg)
                coef_xy(:,23)=coef_xy(:,23)+coef_xyz(23)*pol_z(:,0,kg)
                coef_xy(:,24)=coef_xy(:,24)+coef_xyz(24)*pol_z(:,0,kg)
                coef_xy(:,25)=coef_xy(:,25)+coef_xyz(25)*pol_z(:,0,kg)
                coef_xy(:,26)=coef_xy(:,26)+coef_xyz(26)*pol_z(:,0,kg)
                coef_xy(:,27)=coef_xy(:,27)+coef_xyz(27)*pol_z(:,0,kg)
                coef_xy(:,28)=coef_xy(:,28)+coef_xyz(28)*pol_z(:,0,kg)
                coef_xy(:,29)=coef_xy(:,29)+coef_xyz(29)*pol_z(:,0,kg)
                coef_xy(:,30)=coef_xy(:,30)+coef_xyz(30)*pol_z(:,0,kg)
                coef_xy(:,31)=coef_xy(:,31)+coef_xyz(31)*pol_z(:,0,kg)
                coef_xy(:,32)=coef_xy(:,32)+coef_xyz(32)*pol_z(:,0,kg)
                coef_xy(:,33)=coef_xy(:,33)+coef_xyz(33)*pol_z(:,0,kg)
                coef_xy(:,34)=coef_xy(:,34)+coef_xyz(34)*pol_z(:,0,kg)
                coef_xy(:,35)=coef_xy(:,35)+coef_xyz(35)*pol_z(:,0,kg)
                coef_xy(:,36)=coef_xy(:,36)+coef_xyz(36)*pol_z(:,0,kg)
                coef_xy(:,37)=coef_xy(:,37)+coef_xyz(37)*pol_z(:,0,kg)
                coef_xy(:,38)=coef_xy(:,38)+coef_xyz(38)*pol_z(:,0,kg)
                coef_xy(:,39)=coef_xy(:,39)+coef_xyz(39)*pol_z(:,0,kg)
                coef_xy(:,40)=coef_xy(:,40)+coef_xyz(40)*pol_z(:,0,kg)
                coef_xy(:,41)=coef_xy(:,41)+coef_xyz(41)*pol_z(:,0,kg)
                coef_xy(:,42)=coef_xy(:,42)+coef_xyz(42)*pol_z(:,0,kg)
                coef_xy(:,43)=coef_xy(:,43)+coef_xyz(43)*pol_z(:,0,kg)
                coef_xy(:,44)=coef_xy(:,44)+coef_xyz(44)*pol_z(:,0,kg)
                coef_xy(:,45)=coef_xy(:,45)+coef_xyz(45)*pol_z(:,0,kg)
                coef_xy(:,1)=coef_xy(:,1)+coef_xyz(46)*pol_z(:,1,kg)
                coef_xy(:,2)=coef_xy(:,2)+coef_xyz(47)*pol_z(:,1,kg)
                coef_xy(:,3)=coef_xy(:,3)+coef_xyz(48)*pol_z(:,1,kg)
                coef_xy(:,4)=coef_xy(:,4)+coef_xyz(49)*pol_z(:,1,kg)
                coef_xy(:,5)=coef_xy(:,5)+coef_xyz(50)*pol_z(:,1,kg)
                coef_xy(:,6)=coef_xy(:,6)+coef_xyz(51)*pol_z(:,1,kg)
                coef_xy(:,7)=coef_xy(:,7)+coef_xyz(52)*pol_z(:,1,kg)
                coef_xy(:,8)=coef_xy(:,8)+coef_xyz(53)*pol_z(:,1,kg)
                coef_xy(:,10)=coef_xy(:,10)+coef_xyz(54)*pol_z(:,1,kg)
                coef_xy(:,11)=coef_xy(:,11)+coef_xyz(55)*pol_z(:,1,kg)
                coef_xy(:,12)=coef_xy(:,12)+coef_xyz(56)*pol_z(:,1,kg)
                coef_xy(:,13)=coef_xy(:,13)+coef_xyz(57)*pol_z(:,1,kg)
                coef_xy(:,14)=coef_xy(:,14)+coef_xyz(58)*pol_z(:,1,kg)
                coef_xy(:,15)=coef_xy(:,15)+coef_xyz(59)*pol_z(:,1,kg)
                coef_xy(:,16)=coef_xy(:,16)+coef_xyz(60)*pol_z(:,1,kg)
                coef_xy(:,18)=coef_xy(:,18)+coef_xyz(61)*pol_z(:,1,kg)
                coef_xy(:,19)=coef_xy(:,19)+coef_xyz(62)*pol_z(:,1,kg)
                coef_xy(:,20)=coef_xy(:,20)+coef_xyz(63)*pol_z(:,1,kg)
                coef_xy(:,21)=coef_xy(:,21)+coef_xyz(64)*pol_z(:,1,kg)
                coef_xy(:,22)=coef_xy(:,22)+coef_xyz(65)*pol_z(:,1,kg)
                coef_xy(:,23)=coef_xy(:,23)+coef_xyz(66)*pol_z(:,1,kg)
                coef_xy(:,25)=coef_xy(:,25)+coef_xyz(67)*pol_z(:,1,kg)
                coef_xy(:,26)=coef_xy(:,26)+coef_xyz(68)*pol_z(:,1,kg)
                coef_xy(:,27)=coef_xy(:,27)+coef_xyz(69)*pol_z(:,1,kg)
                coef_xy(:,28)=coef_xy(:,28)+coef_xyz(70)*pol_z(:,1,kg)
                coef_xy(:,29)=coef_xy(:,29)+coef_xyz(71)*pol_z(:,1,kg)
                coef_xy(:,31)=coef_xy(:,31)+coef_xyz(72)*pol_z(:,1,kg)
                coef_xy(:,32)=coef_xy(:,32)+coef_xyz(73)*pol_z(:,1,kg)
                coef_xy(:,33)=coef_xy(:,33)+coef_xyz(74)*pol_z(:,1,kg)
                coef_xy(:,34)=coef_xy(:,34)+coef_xyz(75)*pol_z(:,1,kg)
                coef_xy(:,36)=coef_xy(:,36)+coef_xyz(76)*pol_z(:,1,kg)
                coef_xy(:,37)=coef_xy(:,37)+coef_xyz(77)*pol_z(:,1,kg)
                coef_xy(:,38)=coef_xy(:,38)+coef_xyz(78)*pol_z(:,1,kg)
                coef_xy(:,40)=coef_xy(:,40)+coef_xyz(79)*pol_z(:,1,kg)
                coef_xy(:,41)=coef_xy(:,41)+coef_xyz(80)*pol_z(:,1,kg)
                coef_xy(:,43)=coef_xy(:,43)+coef_xyz(81)*pol_z(:,1,kg)
                coef_xy(:,1)=coef_xy(:,1)+coef_xyz(82)*pol_z(:,2,kg)
                coef_xy(:,2)=coef_xy(:,2)+coef_xyz(83)*pol_z(:,2,kg)
                coef_xy(:,3)=coef_xy(:,3)+coef_xyz(84)*pol_z(:,2,kg)
                coef_xy(:,4)=coef_xy(:,4)+coef_xyz(85)*pol_z(:,2,kg)
                coef_xy(:,5)=coef_xy(:,5)+coef_xyz(86)*pol_z(:,2,kg)
                coef_xy(:,6)=coef_xy(:,6)+coef_xyz(87)*pol_z(:,2,kg)
                coef_xy(:,7)=coef_xy(:,7)+coef_xyz(88)*pol_z(:,2,kg)
                coef_xy(:,10)=coef_xy(:,10)+coef_xyz(89)*pol_z(:,2,kg)
                coef_xy(:,11)=coef_xy(:,11)+coef_xyz(90)*pol_z(:,2,kg)
                coef_xy(:,12)=coef_xy(:,12)+coef_xyz(91)*pol_z(:,2,kg)
                coef_xy(:,13)=coef_xy(:,13)+coef_xyz(92)*pol_z(:,2,kg)
                coef_xy(:,14)=coef_xy(:,14)+coef_xyz(93)*pol_z(:,2,kg)
                coef_xy(:,15)=coef_xy(:,15)+coef_xyz(94)*pol_z(:,2,kg)
                coef_xy(:,18)=coef_xy(:,18)+coef_xyz(95)*pol_z(:,2,kg)
                coef_xy(:,19)=coef_xy(:,19)+coef_xyz(96)*pol_z(:,2,kg)
                coef_xy(:,20)=coef_xy(:,20)+coef_xyz(97)*pol_z(:,2,kg)
                coef_xy(:,21)=coef_xy(:,21)+coef_xyz(98)*pol_z(:,2,kg)
                coef_xy(:,22)=coef_xy(:,22)+coef_xyz(99)*pol_z(:,2,kg)
                coef_xy(:,25)=coef_xy(:,25)+coef_xyz(100)*pol_z(:,2,kg)
                coef_xy(:,26)=coef_xy(:,26)+coef_xyz(101)*pol_z(:,2,kg)
                coef_xy(:,27)=coef_xy(:,27)+coef_xyz(102)*pol_z(:,2,kg)
                coef_xy(:,28)=coef_xy(:,28)+coef_xyz(103)*pol_z(:,2,kg)
                coef_xy(:,31)=coef_xy(:,31)+coef_xyz(104)*pol_z(:,2,kg)
                coef_xy(:,32)=coef_xy(:,32)+coef_xyz(105)*pol_z(:,2,kg)
                coef_xy(:,33)=coef_xy(:,33)+coef_xyz(106)*pol_z(:,2,kg)
                coef_xy(:,36)=coef_xy(:,36)+coef_xyz(107)*pol_z(:,2,kg)
                coef_xy(:,37)=coef_xy(:,37)+coef_xyz(108)*pol_z(:,2,kg)
                coef_xy(:,40)=coef_xy(:,40)+coef_xyz(109)*pol_z(:,2,kg)
                coef_xy(:,1)=coef_xy(:,1)+coef_xyz(110)*pol_z(:,3,kg)
                coef_xy(:,2)=coef_xy(:,2)+coef_xyz(111)*pol_z(:,3,kg)
                coef_xy(:,3)=coef_xy(:,3)+coef_xyz(112)*pol_z(:,3,kg)
                coef_xy(:,4)=coef_xy(:,4)+coef_xyz(113)*pol_z(:,3,kg)
                coef_xy(:,5)=coef_xy(:,5)+coef_xyz(114)*pol_z(:,3,kg)
                coef_xy(:,6)=coef_xy(:,6)+coef_xyz(115)*pol_z(:,3,kg)
                coef_xy(:,10)=coef_xy(:,10)+coef_xyz(116)*pol_z(:,3,kg)
                coef_xy(:,11)=coef_xy(:,11)+coef_xyz(117)*pol_z(:,3,kg)
                coef_xy(:,12)=coef_xy(:,12)+coef_xyz(118)*pol_z(:,3,kg)
                coef_xy(:,13)=coef_xy(:,13)+coef_xyz(119)*pol_z(:,3,kg)
                coef_xy(:,14)=coef_xy(:,14)+coef_xyz(120)*pol_z(:,3,kg)
                coef_xy(:,18)=coef_xy(:,18)+coef_xyz(121)*pol_z(:,3,kg)
                coef_xy(:,19)=coef_xy(:,19)+coef_xyz(122)*pol_z(:,3,kg)
                coef_xy(:,20)=coef_xy(:,20)+coef_xyz(123)*pol_z(:,3,kg)
                coef_xy(:,21)=coef_xy(:,21)+coef_xyz(124)*pol_z(:,3,kg)
                coef_xy(:,25)=coef_xy(:,25)+coef_xyz(125)*pol_z(:,3,kg)
                coef_xy(:,26)=coef_xy(:,26)+coef_xyz(126)*pol_z(:,3,kg)
                coef_xy(:,27)=coef_xy(:,27)+coef_xyz(127)*pol_z(:,3,kg)
                coef_xy(:,31)=coef_xy(:,31)+coef_xyz(128)*pol_z(:,3,kg)
                coef_xy(:,32)=coef_xy(:,32)+coef_xyz(129)*pol_z(:,3,kg)
                coef_xy(:,36)=coef_xy(:,36)+coef_xyz(130)*pol_z(:,3,kg)
                coef_xy(:,1)=coef_xy(:,1)+coef_xyz(131)*pol_z(:,4,kg)
                coef_xy(:,2)=coef_xy(:,2)+coef_xyz(132)*pol_z(:,4,kg)
                coef_xy(:,3)=coef_xy(:,3)+coef_xyz(133)*pol_z(:,4,kg)
                coef_xy(:,4)=coef_xy(:,4)+coef_xyz(134)*pol_z(:,4,kg)
                coef_xy(:,5)=coef_xy(:,5)+coef_xyz(135)*pol_z(:,4,kg)
                coef_xy(:,10)=coef_xy(:,10)+coef_xyz(136)*pol_z(:,4,kg)
                coef_xy(:,11)=coef_xy(:,11)+coef_xyz(137)*pol_z(:,4,kg)
                coef_xy(:,12)=coef_xy(:,12)+coef_xyz(138)*pol_z(:,4,kg)
                coef_xy(:,13)=coef_xy(:,13)+coef_xyz(139)*pol_z(:,4,kg)
                coef_xy(:,18)=coef_xy(:,18)+coef_xyz(140)*pol_z(:,4,kg)
                coef_xy(:,19)=coef_xy(:,19)+coef_xyz(141)*pol_z(:,4,kg)
                coef_xy(:,20)=coef_xy(:,20)+coef_xyz(142)*pol_z(:,4,kg)
                coef_xy(:,25)=coef_xy(:,25)+coef_xyz(143)*pol_z(:,4,kg)
                coef_xy(:,26)=coef_xy(:,26)+coef_xyz(144)*pol_z(:,4,kg)
                coef_xy(:,31)=coef_xy(:,31)+coef_xyz(145)*pol_z(:,4,kg)
                coef_xy(:,1)=coef_xy(:,1)+coef_xyz(146)*pol_z(:,5,kg)
                coef_xy(:,2)=coef_xy(:,2)+coef_xyz(147)*pol_z(:,5,kg)
                coef_xy(:,3)=coef_xy(:,3)+coef_xyz(148)*pol_z(:,5,kg)
                coef_xy(:,4)=coef_xy(:,4)+coef_xyz(149)*pol_z(:,5,kg)
                coef_xy(:,10)=coef_xy(:,10)+coef_xyz(150)*pol_z(:,5,kg)
                coef_xy(:,11)=coef_xy(:,11)+coef_xyz(151)*pol_z(:,5,kg)
                coef_xy(:,12)=coef_xy(:,12)+coef_xyz(152)*pol_z(:,5,kg)
                coef_xy(:,18)=coef_xy(:,18)+coef_xyz(153)*pol_z(:,5,kg)
                coef_xy(:,19)=coef_xy(:,19)+coef_xyz(154)*pol_z(:,5,kg)
                coef_xy(:,25)=coef_xy(:,25)+coef_xyz(155)*pol_z(:,5,kg)
                coef_xy(:,1)=coef_xy(:,1)+coef_xyz(156)*pol_z(:,6,kg)
                coef_xy(:,2)=coef_xy(:,2)+coef_xyz(157)*pol_z(:,6,kg)
                coef_xy(:,3)=coef_xy(:,3)+coef_xyz(158)*pol_z(:,6,kg)
                coef_xy(:,10)=coef_xy(:,10)+coef_xyz(159)*pol_z(:,6,kg)
                coef_xy(:,11)=coef_xy(:,11)+coef_xyz(160)*pol_z(:,6,kg)
                coef_xy(:,18)=coef_xy(:,18)+coef_xyz(161)*pol_z(:,6,kg)
                coef_xy(:,1)=coef_xy(:,1)+coef_xyz(162)*pol_z(:,7,kg)
                coef_xy(:,2)=coef_xy(:,2)+coef_xyz(163)*pol_z(:,7,kg)
                coef_xy(:,10)=coef_xy(:,10)+coef_xyz(164)*pol_z(:,7,kg)
                coef_xy(:,1)=coef_xy(:,1)+coef_xyz(165)*pol_z(:,8,kg)
       jgmin=sphere_bounds(sci)
       sci=sci+1
       DO jg=jgmin,0
          jg2=1-jg
          j=map(jg,2)
          j2=map(jg2,2)
          igmin=sphere_bounds(sci)
          sci=sci+1
          igmax=1-igmin
          coef_x=0.0_dp
             coef_x(1,0)=coef_x(1,0)+coef_xy(1,1)*pol_y(1,0,jg)
             coef_x(2,0)=coef_x(2,0)+coef_xy(2,1)*pol_y(1,0,jg)
             coef_x(3,0)=coef_x(3,0)+coef_xy(1,1)*pol_y(2,0,jg)
             coef_x(4,0)=coef_x(4,0)+coef_xy(2,1)*pol_y(2,0,jg)
             coef_x(1,1)=coef_x(1,1)+coef_xy(1,2)*pol_y(1,0,jg)
             coef_x(2,1)=coef_x(2,1)+coef_xy(2,2)*pol_y(1,0,jg)
             coef_x(3,1)=coef_x(3,1)+coef_xy(1,2)*pol_y(2,0,jg)
             coef_x(4,1)=coef_x(4,1)+coef_xy(2,2)*pol_y(2,0,jg)
             coef_x(1,2)=coef_x(1,2)+coef_xy(1,3)*pol_y(1,0,jg)
             coef_x(2,2)=coef_x(2,2)+coef_xy(2,3)*pol_y(1,0,jg)
             coef_x(3,2)=coef_x(3,2)+coef_xy(1,3)*pol_y(2,0,jg)
             coef_x(4,2)=coef_x(4,2)+coef_xy(2,3)*pol_y(2,0,jg)
             coef_x(1,3)=coef_x(1,3)+coef_xy(1,4)*pol_y(1,0,jg)
             coef_x(2,3)=coef_x(2,3)+coef_xy(2,4)*pol_y(1,0,jg)
             coef_x(3,3)=coef_x(3,3)+coef_xy(1,4)*pol_y(2,0,jg)
             coef_x(4,3)=coef_x(4,3)+coef_xy(2,4)*pol_y(2,0,jg)
             coef_x(1,4)=coef_x(1,4)+coef_xy(1,5)*pol_y(1,0,jg)
             coef_x(2,4)=coef_x(2,4)+coef_xy(2,5)*pol_y(1,0,jg)
             coef_x(3,4)=coef_x(3,4)+coef_xy(1,5)*pol_y(2,0,jg)
             coef_x(4,4)=coef_x(4,4)+coef_xy(2,5)*pol_y(2,0,jg)
             coef_x(1,5)=coef_x(1,5)+coef_xy(1,6)*pol_y(1,0,jg)
             coef_x(2,5)=coef_x(2,5)+coef_xy(2,6)*pol_y(1,0,jg)
             coef_x(3,5)=coef_x(3,5)+coef_xy(1,6)*pol_y(2,0,jg)
             coef_x(4,5)=coef_x(4,5)+coef_xy(2,6)*pol_y(2,0,jg)
             coef_x(1,6)=coef_x(1,6)+coef_xy(1,7)*pol_y(1,0,jg)
             coef_x(2,6)=coef_x(2,6)+coef_xy(2,7)*pol_y(1,0,jg)
             coef_x(3,6)=coef_x(3,6)+coef_xy(1,7)*pol_y(2,0,jg)
             coef_x(4,6)=coef_x(4,6)+coef_xy(2,7)*pol_y(2,0,jg)
             coef_x(1,7)=coef_x(1,7)+coef_xy(1,8)*pol_y(1,0,jg)
             coef_x(2,7)=coef_x(2,7)+coef_xy(2,8)*pol_y(1,0,jg)
             coef_x(3,7)=coef_x(3,7)+coef_xy(1,8)*pol_y(2,0,jg)
             coef_x(4,7)=coef_x(4,7)+coef_xy(2,8)*pol_y(2,0,jg)
             coef_x(1,8)=coef_x(1,8)+coef_xy(1,9)*pol_y(1,0,jg)
             coef_x(2,8)=coef_x(2,8)+coef_xy(2,9)*pol_y(1,0,jg)
             coef_x(3,8)=coef_x(3,8)+coef_xy(1,9)*pol_y(2,0,jg)
             coef_x(4,8)=coef_x(4,8)+coef_xy(2,9)*pol_y(2,0,jg)
             coef_x(1,0)=coef_x(1,0)+coef_xy(1,10)*pol_y(1,1,jg)
             coef_x(2,0)=coef_x(2,0)+coef_xy(2,10)*pol_y(1,1,jg)
             coef_x(3,0)=coef_x(3,0)+coef_xy(1,10)*pol_y(2,1,jg)
             coef_x(4,0)=coef_x(4,0)+coef_xy(2,10)*pol_y(2,1,jg)
             coef_x(1,1)=coef_x(1,1)+coef_xy(1,11)*pol_y(1,1,jg)
             coef_x(2,1)=coef_x(2,1)+coef_xy(2,11)*pol_y(1,1,jg)
             coef_x(3,1)=coef_x(3,1)+coef_xy(1,11)*pol_y(2,1,jg)
             coef_x(4,1)=coef_x(4,1)+coef_xy(2,11)*pol_y(2,1,jg)
             coef_x(1,2)=coef_x(1,2)+coef_xy(1,12)*pol_y(1,1,jg)
             coef_x(2,2)=coef_x(2,2)+coef_xy(2,12)*pol_y(1,1,jg)
             coef_x(3,2)=coef_x(3,2)+coef_xy(1,12)*pol_y(2,1,jg)
             coef_x(4,2)=coef_x(4,2)+coef_xy(2,12)*pol_y(2,1,jg)
             coef_x(1,3)=coef_x(1,3)+coef_xy(1,13)*pol_y(1,1,jg)
             coef_x(2,3)=coef_x(2,3)+coef_xy(2,13)*pol_y(1,1,jg)
             coef_x(3,3)=coef_x(3,3)+coef_xy(1,13)*pol_y(2,1,jg)
             coef_x(4,3)=coef_x(4,3)+coef_xy(2,13)*pol_y(2,1,jg)
             coef_x(1,4)=coef_x(1,4)+coef_xy(1,14)*pol_y(1,1,jg)
             coef_x(2,4)=coef_x(2,4)+coef_xy(2,14)*pol_y(1,1,jg)
             coef_x(3,4)=coef_x(3,4)+coef_xy(1,14)*pol_y(2,1,jg)
             coef_x(4,4)=coef_x(4,4)+coef_xy(2,14)*pol_y(2,1,jg)
             coef_x(1,5)=coef_x(1,5)+coef_xy(1,15)*pol_y(1,1,jg)
             coef_x(2,5)=coef_x(2,5)+coef_xy(2,15)*pol_y(1,1,jg)
             coef_x(3,5)=coef_x(3,5)+coef_xy(1,15)*pol_y(2,1,jg)
             coef_x(4,5)=coef_x(4,5)+coef_xy(2,15)*pol_y(2,1,jg)
             coef_x(1,6)=coef_x(1,6)+coef_xy(1,16)*pol_y(1,1,jg)
             coef_x(2,6)=coef_x(2,6)+coef_xy(2,16)*pol_y(1,1,jg)
             coef_x(3,6)=coef_x(3,6)+coef_xy(1,16)*pol_y(2,1,jg)
             coef_x(4,6)=coef_x(4,6)+coef_xy(2,16)*pol_y(2,1,jg)
             coef_x(1,7)=coef_x(1,7)+coef_xy(1,17)*pol_y(1,1,jg)
             coef_x(2,7)=coef_x(2,7)+coef_xy(2,17)*pol_y(1,1,jg)
             coef_x(3,7)=coef_x(3,7)+coef_xy(1,17)*pol_y(2,1,jg)
             coef_x(4,7)=coef_x(4,7)+coef_xy(2,17)*pol_y(2,1,jg)
             coef_x(1,0)=coef_x(1,0)+coef_xy(1,18)*pol_y(1,2,jg)
             coef_x(2,0)=coef_x(2,0)+coef_xy(2,18)*pol_y(1,2,jg)
             coef_x(3,0)=coef_x(3,0)+coef_xy(1,18)*pol_y(2,2,jg)
             coef_x(4,0)=coef_x(4,0)+coef_xy(2,18)*pol_y(2,2,jg)
             coef_x(1,1)=coef_x(1,1)+coef_xy(1,19)*pol_y(1,2,jg)
             coef_x(2,1)=coef_x(2,1)+coef_xy(2,19)*pol_y(1,2,jg)
             coef_x(3,1)=coef_x(3,1)+coef_xy(1,19)*pol_y(2,2,jg)
             coef_x(4,1)=coef_x(4,1)+coef_xy(2,19)*pol_y(2,2,jg)
             coef_x(1,2)=coef_x(1,2)+coef_xy(1,20)*pol_y(1,2,jg)
             coef_x(2,2)=coef_x(2,2)+coef_xy(2,20)*pol_y(1,2,jg)
             coef_x(3,2)=coef_x(3,2)+coef_xy(1,20)*pol_y(2,2,jg)
             coef_x(4,2)=coef_x(4,2)+coef_xy(2,20)*pol_y(2,2,jg)
             coef_x(1,3)=coef_x(1,3)+coef_xy(1,21)*pol_y(1,2,jg)
             coef_x(2,3)=coef_x(2,3)+coef_xy(2,21)*pol_y(1,2,jg)
             coef_x(3,3)=coef_x(3,3)+coef_xy(1,21)*pol_y(2,2,jg)
             coef_x(4,3)=coef_x(4,3)+coef_xy(2,21)*pol_y(2,2,jg)
             coef_x(1,4)=coef_x(1,4)+coef_xy(1,22)*pol_y(1,2,jg)
             coef_x(2,4)=coef_x(2,4)+coef_xy(2,22)*pol_y(1,2,jg)
             coef_x(3,4)=coef_x(3,4)+coef_xy(1,22)*pol_y(2,2,jg)
             coef_x(4,4)=coef_x(4,4)+coef_xy(2,22)*pol_y(2,2,jg)
             coef_x(1,5)=coef_x(1,5)+coef_xy(1,23)*pol_y(1,2,jg)
             coef_x(2,5)=coef_x(2,5)+coef_xy(2,23)*pol_y(1,2,jg)
             coef_x(3,5)=coef_x(3,5)+coef_xy(1,23)*pol_y(2,2,jg)
             coef_x(4,5)=coef_x(4,5)+coef_xy(2,23)*pol_y(2,2,jg)
             coef_x(1,6)=coef_x(1,6)+coef_xy(1,24)*pol_y(1,2,jg)
             coef_x(2,6)=coef_x(2,6)+coef_xy(2,24)*pol_y(1,2,jg)
             coef_x(3,6)=coef_x(3,6)+coef_xy(1,24)*pol_y(2,2,jg)
             coef_x(4,6)=coef_x(4,6)+coef_xy(2,24)*pol_y(2,2,jg)
             coef_x(1,0)=coef_x(1,0)+coef_xy(1,25)*pol_y(1,3,jg)
             coef_x(2,0)=coef_x(2,0)+coef_xy(2,25)*pol_y(1,3,jg)
             coef_x(3,0)=coef_x(3,0)+coef_xy(1,25)*pol_y(2,3,jg)
             coef_x(4,0)=coef_x(4,0)+coef_xy(2,25)*pol_y(2,3,jg)
             coef_x(1,1)=coef_x(1,1)+coef_xy(1,26)*pol_y(1,3,jg)
             coef_x(2,1)=coef_x(2,1)+coef_xy(2,26)*pol_y(1,3,jg)
             coef_x(3,1)=coef_x(3,1)+coef_xy(1,26)*pol_y(2,3,jg)
             coef_x(4,1)=coef_x(4,1)+coef_xy(2,26)*pol_y(2,3,jg)
             coef_x(1,2)=coef_x(1,2)+coef_xy(1,27)*pol_y(1,3,jg)
             coef_x(2,2)=coef_x(2,2)+coef_xy(2,27)*pol_y(1,3,jg)
             coef_x(3,2)=coef_x(3,2)+coef_xy(1,27)*pol_y(2,3,jg)
             coef_x(4,2)=coef_x(4,2)+coef_xy(2,27)*pol_y(2,3,jg)
             coef_x(1,3)=coef_x(1,3)+coef_xy(1,28)*pol_y(1,3,jg)
             coef_x(2,3)=coef_x(2,3)+coef_xy(2,28)*pol_y(1,3,jg)
             coef_x(3,3)=coef_x(3,3)+coef_xy(1,28)*pol_y(2,3,jg)
             coef_x(4,3)=coef_x(4,3)+coef_xy(2,28)*pol_y(2,3,jg)
             coef_x(1,4)=coef_x(1,4)+coef_xy(1,29)*pol_y(1,3,jg)
             coef_x(2,4)=coef_x(2,4)+coef_xy(2,29)*pol_y(1,3,jg)
             coef_x(3,4)=coef_x(3,4)+coef_xy(1,29)*pol_y(2,3,jg)
             coef_x(4,4)=coef_x(4,4)+coef_xy(2,29)*pol_y(2,3,jg)
             coef_x(1,5)=coef_x(1,5)+coef_xy(1,30)*pol_y(1,3,jg)
             coef_x(2,5)=coef_x(2,5)+coef_xy(2,30)*pol_y(1,3,jg)
             coef_x(3,5)=coef_x(3,5)+coef_xy(1,30)*pol_y(2,3,jg)
             coef_x(4,5)=coef_x(4,5)+coef_xy(2,30)*pol_y(2,3,jg)
             coef_x(1,0)=coef_x(1,0)+coef_xy(1,31)*pol_y(1,4,jg)
             coef_x(2,0)=coef_x(2,0)+coef_xy(2,31)*pol_y(1,4,jg)
             coef_x(3,0)=coef_x(3,0)+coef_xy(1,31)*pol_y(2,4,jg)
             coef_x(4,0)=coef_x(4,0)+coef_xy(2,31)*pol_y(2,4,jg)
             coef_x(1,1)=coef_x(1,1)+coef_xy(1,32)*pol_y(1,4,jg)
             coef_x(2,1)=coef_x(2,1)+coef_xy(2,32)*pol_y(1,4,jg)
             coef_x(3,1)=coef_x(3,1)+coef_xy(1,32)*pol_y(2,4,jg)
             coef_x(4,1)=coef_x(4,1)+coef_xy(2,32)*pol_y(2,4,jg)
             coef_x(1,2)=coef_x(1,2)+coef_xy(1,33)*pol_y(1,4,jg)
             coef_x(2,2)=coef_x(2,2)+coef_xy(2,33)*pol_y(1,4,jg)
             coef_x(3,2)=coef_x(3,2)+coef_xy(1,33)*pol_y(2,4,jg)
             coef_x(4,2)=coef_x(4,2)+coef_xy(2,33)*pol_y(2,4,jg)
             coef_x(1,3)=coef_x(1,3)+coef_xy(1,34)*pol_y(1,4,jg)
             coef_x(2,3)=coef_x(2,3)+coef_xy(2,34)*pol_y(1,4,jg)
             coef_x(3,3)=coef_x(3,3)+coef_xy(1,34)*pol_y(2,4,jg)
             coef_x(4,3)=coef_x(4,3)+coef_xy(2,34)*pol_y(2,4,jg)
             coef_x(1,4)=coef_x(1,4)+coef_xy(1,35)*pol_y(1,4,jg)
             coef_x(2,4)=coef_x(2,4)+coef_xy(2,35)*pol_y(1,4,jg)
             coef_x(3,4)=coef_x(3,4)+coef_xy(1,35)*pol_y(2,4,jg)
             coef_x(4,4)=coef_x(4,4)+coef_xy(2,35)*pol_y(2,4,jg)
             coef_x(1,0)=coef_x(1,0)+coef_xy(1,36)*pol_y(1,5,jg)
             coef_x(2,0)=coef_x(2,0)+coef_xy(2,36)*pol_y(1,5,jg)
             coef_x(3,0)=coef_x(3,0)+coef_xy(1,36)*pol_y(2,5,jg)
             coef_x(4,0)=coef_x(4,0)+coef_xy(2,36)*pol_y(2,5,jg)
             coef_x(1,1)=coef_x(1,1)+coef_xy(1,37)*pol_y(1,5,jg)
             coef_x(2,1)=coef_x(2,1)+coef_xy(2,37)*pol_y(1,5,jg)
             coef_x(3,1)=coef_x(3,1)+coef_xy(1,37)*pol_y(2,5,jg)
             coef_x(4,1)=coef_x(4,1)+coef_xy(2,37)*pol_y(2,5,jg)
             coef_x(1,2)=coef_x(1,2)+coef_xy(1,38)*pol_y(1,5,jg)
             coef_x(2,2)=coef_x(2,2)+coef_xy(2,38)*pol_y(1,5,jg)
             coef_x(3,2)=coef_x(3,2)+coef_xy(1,38)*pol_y(2,5,jg)
             coef_x(4,2)=coef_x(4,2)+coef_xy(2,38)*pol_y(2,5,jg)
             coef_x(1,3)=coef_x(1,3)+coef_xy(1,39)*pol_y(1,5,jg)
             coef_x(2,3)=coef_x(2,3)+coef_xy(2,39)*pol_y(1,5,jg)
             coef_x(3,3)=coef_x(3,3)+coef_xy(1,39)*pol_y(2,5,jg)
             coef_x(4,3)=coef_x(4,3)+coef_xy(2,39)*pol_y(2,5,jg)
             coef_x(1,0)=coef_x(1,0)+coef_xy(1,40)*pol_y(1,6,jg)
             coef_x(2,0)=coef_x(2,0)+coef_xy(2,40)*pol_y(1,6,jg)
             coef_x(3,0)=coef_x(3,0)+coef_xy(1,40)*pol_y(2,6,jg)
             coef_x(4,0)=coef_x(4,0)+coef_xy(2,40)*pol_y(2,6,jg)
             coef_x(1,1)=coef_x(1,1)+coef_xy(1,41)*pol_y(1,6,jg)
             coef_x(2,1)=coef_x(2,1)+coef_xy(2,41)*pol_y(1,6,jg)
             coef_x(3,1)=coef_x(3,1)+coef_xy(1,41)*pol_y(2,6,jg)
             coef_x(4,1)=coef_x(4,1)+coef_xy(2,41)*pol_y(2,6,jg)
             coef_x(1,2)=coef_x(1,2)+coef_xy(1,42)*pol_y(1,6,jg)
             coef_x(2,2)=coef_x(2,2)+coef_xy(2,42)*pol_y(1,6,jg)
             coef_x(3,2)=coef_x(3,2)+coef_xy(1,42)*pol_y(2,6,jg)
             coef_x(4,2)=coef_x(4,2)+coef_xy(2,42)*pol_y(2,6,jg)
             coef_x(1,0)=coef_x(1,0)+coef_xy(1,43)*pol_y(1,7,jg)
             coef_x(2,0)=coef_x(2,0)+coef_xy(2,43)*pol_y(1,7,jg)
             coef_x(3,0)=coef_x(3,0)+coef_xy(1,43)*pol_y(2,7,jg)
             coef_x(4,0)=coef_x(4,0)+coef_xy(2,43)*pol_y(2,7,jg)
             coef_x(1,1)=coef_x(1,1)+coef_xy(1,44)*pol_y(1,7,jg)
             coef_x(2,1)=coef_x(2,1)+coef_xy(2,44)*pol_y(1,7,jg)
             coef_x(3,1)=coef_x(3,1)+coef_xy(1,44)*pol_y(2,7,jg)
             coef_x(4,1)=coef_x(4,1)+coef_xy(2,44)*pol_y(2,7,jg)
             coef_x(1,0)=coef_x(1,0)+coef_xy(1,45)*pol_y(1,8,jg)
             coef_x(2,0)=coef_x(2,0)+coef_xy(2,45)*pol_y(1,8,jg)
             coef_x(3,0)=coef_x(3,0)+coef_xy(1,45)*pol_y(2,8,jg)
             coef_x(4,0)=coef_x(4,0)+coef_xy(2,45)*pol_y(2,8,jg)
          DO ig=igmin,igmax
             i=map(ig,1)
             s01=0.0_dp
             s02=0.0_dp
             s03=0.0_dp
             s04=0.0_dp
             DO lxp=0,lp
                s01=s01+coef_x(1,lxp)*pol_x(lxp,ig)
                s02=s02+coef_x(2,lxp)*pol_x(lxp,ig)
                s03=s03+coef_x(3,lxp)*pol_x(lxp,ig)
                s04=s04+coef_x(4,lxp)*pol_x(lxp,ig)
             ENDDO
             grid(i,j,k) = grid(i,j,k)     + s01
             grid(i,j2,k) = grid(i,j2,k)   + s03
             grid(i,j,k2) = grid(i,j,k2)   + s02
             grid(i,j2,k2) = grid(i,j2,k2) + s04
          END DO
       END DO
    END DO

  END SUBROUTINE collocate_core_8
  SUBROUTINE collocate_core_9(grid,coef_xyz,pol_x,pol_y,pol_z,map,sphere_bounds,cmax,gridbounds)
    USE kinds,                        ONLY: dp
    INTEGER, INTENT(IN)                      :: sphere_bounds(*), cmax, &
                                                map(-cmax:cmax,1:3), &
                                                gridbounds(2,3)
    REAL(dp), INTENT(INOUT) :: grid(gridbounds(1,1):gridbounds(2,1), &
      gridbounds(1,2):gridbounds(2,2), gridbounds(1,3):gridbounds(2,3))
    INTEGER, PARAMETER                       :: lp = 9
    REAL(dp), INTENT(IN) :: pol_x(0:lp,-cmax:cmax), pol_y(1:2,0:lp,-cmax:0), &
      pol_z(1:2,0:lp,-cmax:0), coef_xyz(((lp+1)*(lp+2)*(lp+3))/6)

    INTEGER                                  :: i, ig, igmax, igmin, j, j2, &
                                                jg, jg2, jgmin, k, k2, kg, &
                                                kg2, kgmin, lxp, sci
    REAL(dp)                                 :: coef_x(4,0:lp), &
                                                coef_xy(2,(lp+1)*(lp+2)/2), &
                                                s01, s02, s03, s04

    sci=1

    kgmin=sphere_bounds(sci)
    sci=sci+1
    DO kg=kgmin,0
       kg2=1-kg
       k=map(kg,3)
       k2=map(kg2,3)
       coef_xy=0.0_dp
                coef_xy(:,1)=coef_xy(:,1)+coef_xyz(1)*pol_z(:,0,kg)
                coef_xy(:,2)=coef_xy(:,2)+coef_xyz(2)*pol_z(:,0,kg)
                coef_xy(:,3)=coef_xy(:,3)+coef_xyz(3)*pol_z(:,0,kg)
                coef_xy(:,4)=coef_xy(:,4)+coef_xyz(4)*pol_z(:,0,kg)
                coef_xy(:,5)=coef_xy(:,5)+coef_xyz(5)*pol_z(:,0,kg)
                coef_xy(:,6)=coef_xy(:,6)+coef_xyz(6)*pol_z(:,0,kg)
                coef_xy(:,7)=coef_xy(:,7)+coef_xyz(7)*pol_z(:,0,kg)
                coef_xy(:,8)=coef_xy(:,8)+coef_xyz(8)*pol_z(:,0,kg)
                coef_xy(:,9)=coef_xy(:,9)+coef_xyz(9)*pol_z(:,0,kg)
                coef_xy(:,10)=coef_xy(:,10)+coef_xyz(10)*pol_z(:,0,kg)
                coef_xy(:,11)=coef_xy(:,11)+coef_xyz(11)*pol_z(:,0,kg)
                coef_xy(:,12)=coef_xy(:,12)+coef_xyz(12)*pol_z(:,0,kg)
                coef_xy(:,13)=coef_xy(:,13)+coef_xyz(13)*pol_z(:,0,kg)
                coef_xy(:,14)=coef_xy(:,14)+coef_xyz(14)*pol_z(:,0,kg)
                coef_xy(:,15)=coef_xy(:,15)+coef_xyz(15)*pol_z(:,0,kg)
                coef_xy(:,16)=coef_xy(:,16)+coef_xyz(16)*pol_z(:,0,kg)
                coef_xy(:,17)=coef_xy(:,17)+coef_xyz(17)*pol_z(:,0,kg)
                coef_xy(:,18)=coef_xy(:,18)+coef_xyz(18)*pol_z(:,0,kg)
                coef_xy(:,19)=coef_xy(:,19)+coef_xyz(19)*pol_z(:,0,kg)
                coef_xy(:,20)=coef_xy(:,20)+coef_xyz(20)*pol_z(:,0,kg)
                coef_xy(:,21)=coef_xy(:,21)+coef_xyz(21)*pol_z(:,0,kg)
                coef_xy(:,22)=coef_xy(:,22)+coef_xyz(22)*pol_z(:,0,kg)
                coef_xy(:,23)=coef_xy(:,23)+coef_xyz(23)*pol_z(:,0,kg)
                coef_xy(:,24)=coef_xy(:,24)+coef_xyz(24)*pol_z(:,0,kg)
                coef_xy(:,25)=coef_xy(:,25)+coef_xyz(25)*pol_z(:,0,kg)
                coef_xy(:,26)=coef_xy(:,26)+coef_xyz(26)*pol_z(:,0,kg)
                coef_xy(:,27)=coef_xy(:,27)+coef_xyz(27)*pol_z(:,0,kg)
                coef_xy(:,28)=coef_xy(:,28)+coef_xyz(28)*pol_z(:,0,kg)
                coef_xy(:,29)=coef_xy(:,29)+coef_xyz(29)*pol_z(:,0,kg)
                coef_xy(:,30)=coef_xy(:,30)+coef_xyz(30)*pol_z(:,0,kg)
                coef_xy(:,31)=coef_xy(:,31)+coef_xyz(31)*pol_z(:,0,kg)
                coef_xy(:,32)=coef_xy(:,32)+coef_xyz(32)*pol_z(:,0,kg)
                coef_xy(:,33)=coef_xy(:,33)+coef_xyz(33)*pol_z(:,0,kg)
                coef_xy(:,34)=coef_xy(:,34)+coef_xyz(34)*pol_z(:,0,kg)
                coef_xy(:,35)=coef_xy(:,35)+coef_xyz(35)*pol_z(:,0,kg)
                coef_xy(:,36)=coef_xy(:,36)+coef_xyz(36)*pol_z(:,0,kg)
                coef_xy(:,37)=coef_xy(:,37)+coef_xyz(37)*pol_z(:,0,kg)
                coef_xy(:,38)=coef_xy(:,38)+coef_xyz(38)*pol_z(:,0,kg)
                coef_xy(:,39)=coef_xy(:,39)+coef_xyz(39)*pol_z(:,0,kg)
                coef_xy(:,40)=coef_xy(:,40)+coef_xyz(40)*pol_z(:,0,kg)
                coef_xy(:,41)=coef_xy(:,41)+coef_xyz(41)*pol_z(:,0,kg)
                coef_xy(:,42)=coef_xy(:,42)+coef_xyz(42)*pol_z(:,0,kg)
                coef_xy(:,43)=coef_xy(:,43)+coef_xyz(43)*pol_z(:,0,kg)
                coef_xy(:,44)=coef_xy(:,44)+coef_xyz(44)*pol_z(:,0,kg)
                coef_xy(:,45)=coef_xy(:,45)+coef_xyz(45)*pol_z(:,0,kg)
                coef_xy(:,46)=coef_xy(:,46)+coef_xyz(46)*pol_z(:,0,kg)
                coef_xy(:,47)=coef_xy(:,47)+coef_xyz(47)*pol_z(:,0,kg)
                coef_xy(:,48)=coef_xy(:,48)+coef_xyz(48)*pol_z(:,0,kg)
                coef_xy(:,49)=coef_xy(:,49)+coef_xyz(49)*pol_z(:,0,kg)
                coef_xy(:,50)=coef_xy(:,50)+coef_xyz(50)*pol_z(:,0,kg)
                coef_xy(:,51)=coef_xy(:,51)+coef_xyz(51)*pol_z(:,0,kg)
                coef_xy(:,52)=coef_xy(:,52)+coef_xyz(52)*pol_z(:,0,kg)
                coef_xy(:,53)=coef_xy(:,53)+coef_xyz(53)*pol_z(:,0,kg)
                coef_xy(:,54)=coef_xy(:,54)+coef_xyz(54)*pol_z(:,0,kg)
                coef_xy(:,55)=coef_xy(:,55)+coef_xyz(55)*pol_z(:,0,kg)
                coef_xy(:,1)=coef_xy(:,1)+coef_xyz(56)*pol_z(:,1,kg)
                coef_xy(:,2)=coef_xy(:,2)+coef_xyz(57)*pol_z(:,1,kg)
                coef_xy(:,3)=coef_xy(:,3)+coef_xyz(58)*pol_z(:,1,kg)
                coef_xy(:,4)=coef_xy(:,4)+coef_xyz(59)*pol_z(:,1,kg)
                coef_xy(:,5)=coef_xy(:,5)+coef_xyz(60)*pol_z(:,1,kg)
                coef_xy(:,6)=coef_xy(:,6)+coef_xyz(61)*pol_z(:,1,kg)
                coef_xy(:,7)=coef_xy(:,7)+coef_xyz(62)*pol_z(:,1,kg)
                coef_xy(:,8)=coef_xy(:,8)+coef_xyz(63)*pol_z(:,1,kg)
                coef_xy(:,9)=coef_xy(:,9)+coef_xyz(64)*pol_z(:,1,kg)
                coef_xy(:,11)=coef_xy(:,11)+coef_xyz(65)*pol_z(:,1,kg)
                coef_xy(:,12)=coef_xy(:,12)+coef_xyz(66)*pol_z(:,1,kg)
                coef_xy(:,13)=coef_xy(:,13)+coef_xyz(67)*pol_z(:,1,kg)
                coef_xy(:,14)=coef_xy(:,14)+coef_xyz(68)*pol_z(:,1,kg)
                coef_xy(:,15)=coef_xy(:,15)+coef_xyz(69)*pol_z(:,1,kg)
                coef_xy(:,16)=coef_xy(:,16)+coef_xyz(70)*pol_z(:,1,kg)
                coef_xy(:,17)=coef_xy(:,17)+coef_xyz(71)*pol_z(:,1,kg)
                coef_xy(:,18)=coef_xy(:,18)+coef_xyz(72)*pol_z(:,1,kg)
                coef_xy(:,20)=coef_xy(:,20)+coef_xyz(73)*pol_z(:,1,kg)
                coef_xy(:,21)=coef_xy(:,21)+coef_xyz(74)*pol_z(:,1,kg)
                coef_xy(:,22)=coef_xy(:,22)+coef_xyz(75)*pol_z(:,1,kg)
                coef_xy(:,23)=coef_xy(:,23)+coef_xyz(76)*pol_z(:,1,kg)
                coef_xy(:,24)=coef_xy(:,24)+coef_xyz(77)*pol_z(:,1,kg)
                coef_xy(:,25)=coef_xy(:,25)+coef_xyz(78)*pol_z(:,1,kg)
                coef_xy(:,26)=coef_xy(:,26)+coef_xyz(79)*pol_z(:,1,kg)
                coef_xy(:,28)=coef_xy(:,28)+coef_xyz(80)*pol_z(:,1,kg)
                coef_xy(:,29)=coef_xy(:,29)+coef_xyz(81)*pol_z(:,1,kg)
                coef_xy(:,30)=coef_xy(:,30)+coef_xyz(82)*pol_z(:,1,kg)
                coef_xy(:,31)=coef_xy(:,31)+coef_xyz(83)*pol_z(:,1,kg)
                coef_xy(:,32)=coef_xy(:,32)+coef_xyz(84)*pol_z(:,1,kg)
                coef_xy(:,33)=coef_xy(:,33)+coef_xyz(85)*pol_z(:,1,kg)
                coef_xy(:,35)=coef_xy(:,35)+coef_xyz(86)*pol_z(:,1,kg)
                coef_xy(:,36)=coef_xy(:,36)+coef_xyz(87)*pol_z(:,1,kg)
                coef_xy(:,37)=coef_xy(:,37)+coef_xyz(88)*pol_z(:,1,kg)
                coef_xy(:,38)=coef_xy(:,38)+coef_xyz(89)*pol_z(:,1,kg)
                coef_xy(:,39)=coef_xy(:,39)+coef_xyz(90)*pol_z(:,1,kg)
                coef_xy(:,41)=coef_xy(:,41)+coef_xyz(91)*pol_z(:,1,kg)
                coef_xy(:,42)=coef_xy(:,42)+coef_xyz(92)*pol_z(:,1,kg)
                coef_xy(:,43)=coef_xy(:,43)+coef_xyz(93)*pol_z(:,1,kg)
                coef_xy(:,44)=coef_xy(:,44)+coef_xyz(94)*pol_z(:,1,kg)
                coef_xy(:,46)=coef_xy(:,46)+coef_xyz(95)*pol_z(:,1,kg)
                coef_xy(:,47)=coef_xy(:,47)+coef_xyz(96)*pol_z(:,1,kg)
                coef_xy(:,48)=coef_xy(:,48)+coef_xyz(97)*pol_z(:,1,kg)
                coef_xy(:,50)=coef_xy(:,50)+coef_xyz(98)*pol_z(:,1,kg)
                coef_xy(:,51)=coef_xy(:,51)+coef_xyz(99)*pol_z(:,1,kg)
                coef_xy(:,53)=coef_xy(:,53)+coef_xyz(100)*pol_z(:,1,kg)
                coef_xy(:,1)=coef_xy(:,1)+coef_xyz(101)*pol_z(:,2,kg)
                coef_xy(:,2)=coef_xy(:,2)+coef_xyz(102)*pol_z(:,2,kg)
                coef_xy(:,3)=coef_xy(:,3)+coef_xyz(103)*pol_z(:,2,kg)
                coef_xy(:,4)=coef_xy(:,4)+coef_xyz(104)*pol_z(:,2,kg)
                coef_xy(:,5)=coef_xy(:,5)+coef_xyz(105)*pol_z(:,2,kg)
                coef_xy(:,6)=coef_xy(:,6)+coef_xyz(106)*pol_z(:,2,kg)
                coef_xy(:,7)=coef_xy(:,7)+coef_xyz(107)*pol_z(:,2,kg)
                coef_xy(:,8)=coef_xy(:,8)+coef_xyz(108)*pol_z(:,2,kg)
                coef_xy(:,11)=coef_xy(:,11)+coef_xyz(109)*pol_z(:,2,kg)
                coef_xy(:,12)=coef_xy(:,12)+coef_xyz(110)*pol_z(:,2,kg)
                coef_xy(:,13)=coef_xy(:,13)+coef_xyz(111)*pol_z(:,2,kg)
                coef_xy(:,14)=coef_xy(:,14)+coef_xyz(112)*pol_z(:,2,kg)
                coef_xy(:,15)=coef_xy(:,15)+coef_xyz(113)*pol_z(:,2,kg)
                coef_xy(:,16)=coef_xy(:,16)+coef_xyz(114)*pol_z(:,2,kg)
                coef_xy(:,17)=coef_xy(:,17)+coef_xyz(115)*pol_z(:,2,kg)
                coef_xy(:,20)=coef_xy(:,20)+coef_xyz(116)*pol_z(:,2,kg)
                coef_xy(:,21)=coef_xy(:,21)+coef_xyz(117)*pol_z(:,2,kg)
                coef_xy(:,22)=coef_xy(:,22)+coef_xyz(118)*pol_z(:,2,kg)
                coef_xy(:,23)=coef_xy(:,23)+coef_xyz(119)*pol_z(:,2,kg)
                coef_xy(:,24)=coef_xy(:,24)+coef_xyz(120)*pol_z(:,2,kg)
                coef_xy(:,25)=coef_xy(:,25)+coef_xyz(121)*pol_z(:,2,kg)
                coef_xy(:,28)=coef_xy(:,28)+coef_xyz(122)*pol_z(:,2,kg)
                coef_xy(:,29)=coef_xy(:,29)+coef_xyz(123)*pol_z(:,2,kg)
                coef_xy(:,30)=coef_xy(:,30)+coef_xyz(124)*pol_z(:,2,kg)
                coef_xy(:,31)=coef_xy(:,31)+coef_xyz(125)*pol_z(:,2,kg)
                coef_xy(:,32)=coef_xy(:,32)+coef_xyz(126)*pol_z(:,2,kg)
                coef_xy(:,35)=coef_xy(:,35)+coef_xyz(127)*pol_z(:,2,kg)
                coef_xy(:,36)=coef_xy(:,36)+coef_xyz(128)*pol_z(:,2,kg)
                coef_xy(:,37)=coef_xy(:,37)+coef_xyz(129)*pol_z(:,2,kg)
                coef_xy(:,38)=coef_xy(:,38)+coef_xyz(130)*pol_z(:,2,kg)
                coef_xy(:,41)=coef_xy(:,41)+coef_xyz(131)*pol_z(:,2,kg)
                coef_xy(:,42)=coef_xy(:,42)+coef_xyz(132)*pol_z(:,2,kg)
                coef_xy(:,43)=coef_xy(:,43)+coef_xyz(133)*pol_z(:,2,kg)
                coef_xy(:,46)=coef_xy(:,46)+coef_xyz(134)*pol_z(:,2,kg)
                coef_xy(:,47)=coef_xy(:,47)+coef_xyz(135)*pol_z(:,2,kg)
                coef_xy(:,50)=coef_xy(:,50)+coef_xyz(136)*pol_z(:,2,kg)
                coef_xy(:,1)=coef_xy(:,1)+coef_xyz(137)*pol_z(:,3,kg)
                coef_xy(:,2)=coef_xy(:,2)+coef_xyz(138)*pol_z(:,3,kg)
                coef_xy(:,3)=coef_xy(:,3)+coef_xyz(139)*pol_z(:,3,kg)
                coef_xy(:,4)=coef_xy(:,4)+coef_xyz(140)*pol_z(:,3,kg)
                coef_xy(:,5)=coef_xy(:,5)+coef_xyz(141)*pol_z(:,3,kg)
                coef_xy(:,6)=coef_xy(:,6)+coef_xyz(142)*pol_z(:,3,kg)
                coef_xy(:,7)=coef_xy(:,7)+coef_xyz(143)*pol_z(:,3,kg)
                coef_xy(:,11)=coef_xy(:,11)+coef_xyz(144)*pol_z(:,3,kg)
                coef_xy(:,12)=coef_xy(:,12)+coef_xyz(145)*pol_z(:,3,kg)
                coef_xy(:,13)=coef_xy(:,13)+coef_xyz(146)*pol_z(:,3,kg)
                coef_xy(:,14)=coef_xy(:,14)+coef_xyz(147)*pol_z(:,3,kg)
                coef_xy(:,15)=coef_xy(:,15)+coef_xyz(148)*pol_z(:,3,kg)
                coef_xy(:,16)=coef_xy(:,16)+coef_xyz(149)*pol_z(:,3,kg)
                coef_xy(:,20)=coef_xy(:,20)+coef_xyz(150)*pol_z(:,3,kg)
                coef_xy(:,21)=coef_xy(:,21)+coef_xyz(151)*pol_z(:,3,kg)
                coef_xy(:,22)=coef_xy(:,22)+coef_xyz(152)*pol_z(:,3,kg)
                coef_xy(:,23)=coef_xy(:,23)+coef_xyz(153)*pol_z(:,3,kg)
                coef_xy(:,24)=coef_xy(:,24)+coef_xyz(154)*pol_z(:,3,kg)
                coef_xy(:,28)=coef_xy(:,28)+coef_xyz(155)*pol_z(:,3,kg)
                coef_xy(:,29)=coef_xy(:,29)+coef_xyz(156)*pol_z(:,3,kg)
                coef_xy(:,30)=coef_xy(:,30)+coef_xyz(157)*pol_z(:,3,kg)
                coef_xy(:,31)=coef_xy(:,31)+coef_xyz(158)*pol_z(:,3,kg)
                coef_xy(:,35)=coef_xy(:,35)+coef_xyz(159)*pol_z(:,3,kg)
                coef_xy(:,36)=coef_xy(:,36)+coef_xyz(160)*pol_z(:,3,kg)
                coef_xy(:,37)=coef_xy(:,37)+coef_xyz(161)*pol_z(:,3,kg)
                coef_xy(:,41)=coef_xy(:,41)+coef_xyz(162)*pol_z(:,3,kg)
                coef_xy(:,42)=coef_xy(:,42)+coef_xyz(163)*pol_z(:,3,kg)
                coef_xy(:,46)=coef_xy(:,46)+coef_xyz(164)*pol_z(:,3,kg)
                coef_xy(:,1)=coef_xy(:,1)+coef_xyz(165)*pol_z(:,4,kg)
                coef_xy(:,2)=coef_xy(:,2)+coef_xyz(166)*pol_z(:,4,kg)
                coef_xy(:,3)=coef_xy(:,3)+coef_xyz(167)*pol_z(:,4,kg)
                coef_xy(:,4)=coef_xy(:,4)+coef_xyz(168)*pol_z(:,4,kg)
                coef_xy(:,5)=coef_xy(:,5)+coef_xyz(169)*pol_z(:,4,kg)
                coef_xy(:,6)=coef_xy(:,6)+coef_xyz(170)*pol_z(:,4,kg)
                coef_xy(:,11)=coef_xy(:,11)+coef_xyz(171)*pol_z(:,4,kg)
                coef_xy(:,12)=coef_xy(:,12)+coef_xyz(172)*pol_z(:,4,kg)
                coef_xy(:,13)=coef_xy(:,13)+coef_xyz(173)*pol_z(:,4,kg)
                coef_xy(:,14)=coef_xy(:,14)+coef_xyz(174)*pol_z(:,4,kg)
                coef_xy(:,15)=coef_xy(:,15)+coef_xyz(175)*pol_z(:,4,kg)
                coef_xy(:,20)=coef_xy(:,20)+coef_xyz(176)*pol_z(:,4,kg)
                coef_xy(:,21)=coef_xy(:,21)+coef_xyz(177)*pol_z(:,4,kg)
                coef_xy(:,22)=coef_xy(:,22)+coef_xyz(178)*pol_z(:,4,kg)
                coef_xy(:,23)=coef_xy(:,23)+coef_xyz(179)*pol_z(:,4,kg)
                coef_xy(:,28)=coef_xy(:,28)+coef_xyz(180)*pol_z(:,4,kg)
                coef_xy(:,29)=coef_xy(:,29)+coef_xyz(181)*pol_z(:,4,kg)
                coef_xy(:,30)=coef_xy(:,30)+coef_xyz(182)*pol_z(:,4,kg)
                coef_xy(:,35)=coef_xy(:,35)+coef_xyz(183)*pol_z(:,4,kg)
                coef_xy(:,36)=coef_xy(:,36)+coef_xyz(184)*pol_z(:,4,kg)
                coef_xy(:,41)=coef_xy(:,41)+coef_xyz(185)*pol_z(:,4,kg)
                coef_xy(:,1)=coef_xy(:,1)+coef_xyz(186)*pol_z(:,5,kg)
                coef_xy(:,2)=coef_xy(:,2)+coef_xyz(187)*pol_z(:,5,kg)
                coef_xy(:,3)=coef_xy(:,3)+coef_xyz(188)*pol_z(:,5,kg)
                coef_xy(:,4)=coef_xy(:,4)+coef_xyz(189)*pol_z(:,5,kg)
                coef_xy(:,5)=coef_xy(:,5)+coef_xyz(190)*pol_z(:,5,kg)
                coef_xy(:,11)=coef_xy(:,11)+coef_xyz(191)*pol_z(:,5,kg)
                coef_xy(:,12)=coef_xy(:,12)+coef_xyz(192)*pol_z(:,5,kg)
                coef_xy(:,13)=coef_xy(:,13)+coef_xyz(193)*pol_z(:,5,kg)
                coef_xy(:,14)=coef_xy(:,14)+coef_xyz(194)*pol_z(:,5,kg)
                coef_xy(:,20)=coef_xy(:,20)+coef_xyz(195)*pol_z(:,5,kg)
                coef_xy(:,21)=coef_xy(:,21)+coef_xyz(196)*pol_z(:,5,kg)
                coef_xy(:,22)=coef_xy(:,22)+coef_xyz(197)*pol_z(:,5,kg)
                coef_xy(:,28)=coef_xy(:,28)+coef_xyz(198)*pol_z(:,5,kg)
                coef_xy(:,29)=coef_xy(:,29)+coef_xyz(199)*pol_z(:,5,kg)
                coef_xy(:,35)=coef_xy(:,35)+coef_xyz(200)*pol_z(:,5,kg)
                coef_xy(:,1)=coef_xy(:,1)+coef_xyz(201)*pol_z(:,6,kg)
                coef_xy(:,2)=coef_xy(:,2)+coef_xyz(202)*pol_z(:,6,kg)
                coef_xy(:,3)=coef_xy(:,3)+coef_xyz(203)*pol_z(:,6,kg)
                coef_xy(:,4)=coef_xy(:,4)+coef_xyz(204)*pol_z(:,6,kg)
                coef_xy(:,11)=coef_xy(:,11)+coef_xyz(205)*pol_z(:,6,kg)
                coef_xy(:,12)=coef_xy(:,12)+coef_xyz(206)*pol_z(:,6,kg)
                coef_xy(:,13)=coef_xy(:,13)+coef_xyz(207)*pol_z(:,6,kg)
                coef_xy(:,20)=coef_xy(:,20)+coef_xyz(208)*pol_z(:,6,kg)
                coef_xy(:,21)=coef_xy(:,21)+coef_xyz(209)*pol_z(:,6,kg)
                coef_xy(:,28)=coef_xy(:,28)+coef_xyz(210)*pol_z(:,6,kg)
                coef_xy(:,1)=coef_xy(:,1)+coef_xyz(211)*pol_z(:,7,kg)
                coef_xy(:,2)=coef_xy(:,2)+coef_xyz(212)*pol_z(:,7,kg)
                coef_xy(:,3)=coef_xy(:,3)+coef_xyz(213)*pol_z(:,7,kg)
                coef_xy(:,11)=coef_xy(:,11)+coef_xyz(214)*pol_z(:,7,kg)
                coef_xy(:,12)=coef_xy(:,12)+coef_xyz(215)*pol_z(:,7,kg)
                coef_xy(:,20)=coef_xy(:,20)+coef_xyz(216)*pol_z(:,7,kg)
                coef_xy(:,1)=coef_xy(:,1)+coef_xyz(217)*pol_z(:,8,kg)
                coef_xy(:,2)=coef_xy(:,2)+coef_xyz(218)*pol_z(:,8,kg)
                coef_xy(:,11)=coef_xy(:,11)+coef_xyz(219)*pol_z(:,8,kg)
                coef_xy(:,1)=coef_xy(:,1)+coef_xyz(220)*pol_z(:,9,kg)
       jgmin=sphere_bounds(sci)
       sci=sci+1
       DO jg=jgmin,0
          jg2=1-jg
          j=map(jg,2)
          j2=map(jg2,2)
          igmin=sphere_bounds(sci)
          sci=sci+1
          igmax=1-igmin
          coef_x=0.0_dp
             coef_x(1,0)=coef_x(1,0)+coef_xy(1,1)*pol_y(1,0,jg)
             coef_x(2,0)=coef_x(2,0)+coef_xy(2,1)*pol_y(1,0,jg)
             coef_x(3,0)=coef_x(3,0)+coef_xy(1,1)*pol_y(2,0,jg)
             coef_x(4,0)=coef_x(4,0)+coef_xy(2,1)*pol_y(2,0,jg)
             coef_x(1,1)=coef_x(1,1)+coef_xy(1,2)*pol_y(1,0,jg)
             coef_x(2,1)=coef_x(2,1)+coef_xy(2,2)*pol_y(1,0,jg)
             coef_x(3,1)=coef_x(3,1)+coef_xy(1,2)*pol_y(2,0,jg)
             coef_x(4,1)=coef_x(4,1)+coef_xy(2,2)*pol_y(2,0,jg)
             coef_x(1,2)=coef_x(1,2)+coef_xy(1,3)*pol_y(1,0,jg)
             coef_x(2,2)=coef_x(2,2)+coef_xy(2,3)*pol_y(1,0,jg)
             coef_x(3,2)=coef_x(3,2)+coef_xy(1,3)*pol_y(2,0,jg)
             coef_x(4,2)=coef_x(4,2)+coef_xy(2,3)*pol_y(2,0,jg)
             coef_x(1,3)=coef_x(1,3)+coef_xy(1,4)*pol_y(1,0,jg)
             coef_x(2,3)=coef_x(2,3)+coef_xy(2,4)*pol_y(1,0,jg)
             coef_x(3,3)=coef_x(3,3)+coef_xy(1,4)*pol_y(2,0,jg)
             coef_x(4,3)=coef_x(4,3)+coef_xy(2,4)*pol_y(2,0,jg)
             coef_x(1,4)=coef_x(1,4)+coef_xy(1,5)*pol_y(1,0,jg)
             coef_x(2,4)=coef_x(2,4)+coef_xy(2,5)*pol_y(1,0,jg)
             coef_x(3,4)=coef_x(3,4)+coef_xy(1,5)*pol_y(2,0,jg)
             coef_x(4,4)=coef_x(4,4)+coef_xy(2,5)*pol_y(2,0,jg)
             coef_x(1,5)=coef_x(1,5)+coef_xy(1,6)*pol_y(1,0,jg)
             coef_x(2,5)=coef_x(2,5)+coef_xy(2,6)*pol_y(1,0,jg)
             coef_x(3,5)=coef_x(3,5)+coef_xy(1,6)*pol_y(2,0,jg)
             coef_x(4,5)=coef_x(4,5)+coef_xy(2,6)*pol_y(2,0,jg)
             coef_x(1,6)=coef_x(1,6)+coef_xy(1,7)*pol_y(1,0,jg)
             coef_x(2,6)=coef_x(2,6)+coef_xy(2,7)*pol_y(1,0,jg)
             coef_x(3,6)=coef_x(3,6)+coef_xy(1,7)*pol_y(2,0,jg)
             coef_x(4,6)=coef_x(4,6)+coef_xy(2,7)*pol_y(2,0,jg)
             coef_x(1,7)=coef_x(1,7)+coef_xy(1,8)*pol_y(1,0,jg)
             coef_x(2,7)=coef_x(2,7)+coef_xy(2,8)*pol_y(1,0,jg)
             coef_x(3,7)=coef_x(3,7)+coef_xy(1,8)*pol_y(2,0,jg)
             coef_x(4,7)=coef_x(4,7)+coef_xy(2,8)*pol_y(2,0,jg)
             coef_x(1,8)=coef_x(1,8)+coef_xy(1,9)*pol_y(1,0,jg)
             coef_x(2,8)=coef_x(2,8)+coef_xy(2,9)*pol_y(1,0,jg)
             coef_x(3,8)=coef_x(3,8)+coef_xy(1,9)*pol_y(2,0,jg)
             coef_x(4,8)=coef_x(4,8)+coef_xy(2,9)*pol_y(2,0,jg)
             coef_x(1,9)=coef_x(1,9)+coef_xy(1,10)*pol_y(1,0,jg)
             coef_x(2,9)=coef_x(2,9)+coef_xy(2,10)*pol_y(1,0,jg)
             coef_x(3,9)=coef_x(3,9)+coef_xy(1,10)*pol_y(2,0,jg)
             coef_x(4,9)=coef_x(4,9)+coef_xy(2,10)*pol_y(2,0,jg)
             coef_x(1,0)=coef_x(1,0)+coef_xy(1,11)*pol_y(1,1,jg)
             coef_x(2,0)=coef_x(2,0)+coef_xy(2,11)*pol_y(1,1,jg)
             coef_x(3,0)=coef_x(3,0)+coef_xy(1,11)*pol_y(2,1,jg)
             coef_x(4,0)=coef_x(4,0)+coef_xy(2,11)*pol_y(2,1,jg)
             coef_x(1,1)=coef_x(1,1)+coef_xy(1,12)*pol_y(1,1,jg)
             coef_x(2,1)=coef_x(2,1)+coef_xy(2,12)*pol_y(1,1,jg)
             coef_x(3,1)=coef_x(3,1)+coef_xy(1,12)*pol_y(2,1,jg)
             coef_x(4,1)=coef_x(4,1)+coef_xy(2,12)*pol_y(2,1,jg)
             coef_x(1,2)=coef_x(1,2)+coef_xy(1,13)*pol_y(1,1,jg)
             coef_x(2,2)=coef_x(2,2)+coef_xy(2,13)*pol_y(1,1,jg)
             coef_x(3,2)=coef_x(3,2)+coef_xy(1,13)*pol_y(2,1,jg)
             coef_x(4,2)=coef_x(4,2)+coef_xy(2,13)*pol_y(2,1,jg)
             coef_x(1,3)=coef_x(1,3)+coef_xy(1,14)*pol_y(1,1,jg)
             coef_x(2,3)=coef_x(2,3)+coef_xy(2,14)*pol_y(1,1,jg)
             coef_x(3,3)=coef_x(3,3)+coef_xy(1,14)*pol_y(2,1,jg)
             coef_x(4,3)=coef_x(4,3)+coef_xy(2,14)*pol_y(2,1,jg)
             coef_x(1,4)=coef_x(1,4)+coef_xy(1,15)*pol_y(1,1,jg)
             coef_x(2,4)=coef_x(2,4)+coef_xy(2,15)*pol_y(1,1,jg)
             coef_x(3,4)=coef_x(3,4)+coef_xy(1,15)*pol_y(2,1,jg)
             coef_x(4,4)=coef_x(4,4)+coef_xy(2,15)*pol_y(2,1,jg)
             coef_x(1,5)=coef_x(1,5)+coef_xy(1,16)*pol_y(1,1,jg)
             coef_x(2,5)=coef_x(2,5)+coef_xy(2,16)*pol_y(1,1,jg)
             coef_x(3,5)=coef_x(3,5)+coef_xy(1,16)*pol_y(2,1,jg)
             coef_x(4,5)=coef_x(4,5)+coef_xy(2,16)*pol_y(2,1,jg)
             coef_x(1,6)=coef_x(1,6)+coef_xy(1,17)*pol_y(1,1,jg)
             coef_x(2,6)=coef_x(2,6)+coef_xy(2,17)*pol_y(1,1,jg)
             coef_x(3,6)=coef_x(3,6)+coef_xy(1,17)*pol_y(2,1,jg)
             coef_x(4,6)=coef_x(4,6)+coef_xy(2,17)*pol_y(2,1,jg)
             coef_x(1,7)=coef_x(1,7)+coef_xy(1,18)*pol_y(1,1,jg)
             coef_x(2,7)=coef_x(2,7)+coef_xy(2,18)*pol_y(1,1,jg)
             coef_x(3,7)=coef_x(3,7)+coef_xy(1,18)*pol_y(2,1,jg)
             coef_x(4,7)=coef_x(4,7)+coef_xy(2,18)*pol_y(2,1,jg)
             coef_x(1,8)=coef_x(1,8)+coef_xy(1,19)*pol_y(1,1,jg)
             coef_x(2,8)=coef_x(2,8)+coef_xy(2,19)*pol_y(1,1,jg)
             coef_x(3,8)=coef_x(3,8)+coef_xy(1,19)*pol_y(2,1,jg)
             coef_x(4,8)=coef_x(4,8)+coef_xy(2,19)*pol_y(2,1,jg)
             coef_x(1,0)=coef_x(1,0)+coef_xy(1,20)*pol_y(1,2,jg)
             coef_x(2,0)=coef_x(2,0)+coef_xy(2,20)*pol_y(1,2,jg)
             coef_x(3,0)=coef_x(3,0)+coef_xy(1,20)*pol_y(2,2,jg)
             coef_x(4,0)=coef_x(4,0)+coef_xy(2,20)*pol_y(2,2,jg)
             coef_x(1,1)=coef_x(1,1)+coef_xy(1,21)*pol_y(1,2,jg)
             coef_x(2,1)=coef_x(2,1)+coef_xy(2,21)*pol_y(1,2,jg)
             coef_x(3,1)=coef_x(3,1)+coef_xy(1,21)*pol_y(2,2,jg)
             coef_x(4,1)=coef_x(4,1)+coef_xy(2,21)*pol_y(2,2,jg)
             coef_x(1,2)=coef_x(1,2)+coef_xy(1,22)*pol_y(1,2,jg)
             coef_x(2,2)=coef_x(2,2)+coef_xy(2,22)*pol_y(1,2,jg)
             coef_x(3,2)=coef_x(3,2)+coef_xy(1,22)*pol_y(2,2,jg)
             coef_x(4,2)=coef_x(4,2)+coef_xy(2,22)*pol_y(2,2,jg)
             coef_x(1,3)=coef_x(1,3)+coef_xy(1,23)*pol_y(1,2,jg)
             coef_x(2,3)=coef_x(2,3)+coef_xy(2,23)*pol_y(1,2,jg)
             coef_x(3,3)=coef_x(3,3)+coef_xy(1,23)*pol_y(2,2,jg)
             coef_x(4,3)=coef_x(4,3)+coef_xy(2,23)*pol_y(2,2,jg)
             coef_x(1,4)=coef_x(1,4)+coef_xy(1,24)*pol_y(1,2,jg)
             coef_x(2,4)=coef_x(2,4)+coef_xy(2,24)*pol_y(1,2,jg)
             coef_x(3,4)=coef_x(3,4)+coef_xy(1,24)*pol_y(2,2,jg)
             coef_x(4,4)=coef_x(4,4)+coef_xy(2,24)*pol_y(2,2,jg)
             coef_x(1,5)=coef_x(1,5)+coef_xy(1,25)*pol_y(1,2,jg)
             coef_x(2,5)=coef_x(2,5)+coef_xy(2,25)*pol_y(1,2,jg)
             coef_x(3,5)=coef_x(3,5)+coef_xy(1,25)*pol_y(2,2,jg)
             coef_x(4,5)=coef_x(4,5)+coef_xy(2,25)*pol_y(2,2,jg)
             coef_x(1,6)=coef_x(1,6)+coef_xy(1,26)*pol_y(1,2,jg)
             coef_x(2,6)=coef_x(2,6)+coef_xy(2,26)*pol_y(1,2,jg)
             coef_x(3,6)=coef_x(3,6)+coef_xy(1,26)*pol_y(2,2,jg)
             coef_x(4,6)=coef_x(4,6)+coef_xy(2,26)*pol_y(2,2,jg)
             coef_x(1,7)=coef_x(1,7)+coef_xy(1,27)*pol_y(1,2,jg)
             coef_x(2,7)=coef_x(2,7)+coef_xy(2,27)*pol_y(1,2,jg)
             coef_x(3,7)=coef_x(3,7)+coef_xy(1,27)*pol_y(2,2,jg)
             coef_x(4,7)=coef_x(4,7)+coef_xy(2,27)*pol_y(2,2,jg)
             coef_x(1,0)=coef_x(1,0)+coef_xy(1,28)*pol_y(1,3,jg)
             coef_x(2,0)=coef_x(2,0)+coef_xy(2,28)*pol_y(1,3,jg)
             coef_x(3,0)=coef_x(3,0)+coef_xy(1,28)*pol_y(2,3,jg)
             coef_x(4,0)=coef_x(4,0)+coef_xy(2,28)*pol_y(2,3,jg)
             coef_x(1,1)=coef_x(1,1)+coef_xy(1,29)*pol_y(1,3,jg)
             coef_x(2,1)=coef_x(2,1)+coef_xy(2,29)*pol_y(1,3,jg)
             coef_x(3,1)=coef_x(3,1)+coef_xy(1,29)*pol_y(2,3,jg)
             coef_x(4,1)=coef_x(4,1)+coef_xy(2,29)*pol_y(2,3,jg)
             coef_x(1,2)=coef_x(1,2)+coef_xy(1,30)*pol_y(1,3,jg)
             coef_x(2,2)=coef_x(2,2)+coef_xy(2,30)*pol_y(1,3,jg)
             coef_x(3,2)=coef_x(3,2)+coef_xy(1,30)*pol_y(2,3,jg)
             coef_x(4,2)=coef_x(4,2)+coef_xy(2,30)*pol_y(2,3,jg)
             coef_x(1,3)=coef_x(1,3)+coef_xy(1,31)*pol_y(1,3,jg)
             coef_x(2,3)=coef_x(2,3)+coef_xy(2,31)*pol_y(1,3,jg)
             coef_x(3,3)=coef_x(3,3)+coef_xy(1,31)*pol_y(2,3,jg)
             coef_x(4,3)=coef_x(4,3)+coef_xy(2,31)*pol_y(2,3,jg)
             coef_x(1,4)=coef_x(1,4)+coef_xy(1,32)*pol_y(1,3,jg)
             coef_x(2,4)=coef_x(2,4)+coef_xy(2,32)*pol_y(1,3,jg)
             coef_x(3,4)=coef_x(3,4)+coef_xy(1,32)*pol_y(2,3,jg)
             coef_x(4,4)=coef_x(4,4)+coef_xy(2,32)*pol_y(2,3,jg)
             coef_x(1,5)=coef_x(1,5)+coef_xy(1,33)*pol_y(1,3,jg)
             coef_x(2,5)=coef_x(2,5)+coef_xy(2,33)*pol_y(1,3,jg)
             coef_x(3,5)=coef_x(3,5)+coef_xy(1,33)*pol_y(2,3,jg)
             coef_x(4,5)=coef_x(4,5)+coef_xy(2,33)*pol_y(2,3,jg)
             coef_x(1,6)=coef_x(1,6)+coef_xy(1,34)*pol_y(1,3,jg)
             coef_x(2,6)=coef_x(2,6)+coef_xy(2,34)*pol_y(1,3,jg)
             coef_x(3,6)=coef_x(3,6)+coef_xy(1,34)*pol_y(2,3,jg)
             coef_x(4,6)=coef_x(4,6)+coef_xy(2,34)*pol_y(2,3,jg)
             coef_x(1,0)=coef_x(1,0)+coef_xy(1,35)*pol_y(1,4,jg)
             coef_x(2,0)=coef_x(2,0)+coef_xy(2,35)*pol_y(1,4,jg)
             coef_x(3,0)=coef_x(3,0)+coef_xy(1,35)*pol_y(2,4,jg)
             coef_x(4,0)=coef_x(4,0)+coef_xy(2,35)*pol_y(2,4,jg)
             coef_x(1,1)=coef_x(1,1)+coef_xy(1,36)*pol_y(1,4,jg)
             coef_x(2,1)=coef_x(2,1)+coef_xy(2,36)*pol_y(1,4,jg)
             coef_x(3,1)=coef_x(3,1)+coef_xy(1,36)*pol_y(2,4,jg)
             coef_x(4,1)=coef_x(4,1)+coef_xy(2,36)*pol_y(2,4,jg)
             coef_x(1,2)=coef_x(1,2)+coef_xy(1,37)*pol_y(1,4,jg)
             coef_x(2,2)=coef_x(2,2)+coef_xy(2,37)*pol_y(1,4,jg)
             coef_x(3,2)=coef_x(3,2)+coef_xy(1,37)*pol_y(2,4,jg)
             coef_x(4,2)=coef_x(4,2)+coef_xy(2,37)*pol_y(2,4,jg)
             coef_x(1,3)=coef_x(1,3)+coef_xy(1,38)*pol_y(1,4,jg)
             coef_x(2,3)=coef_x(2,3)+coef_xy(2,38)*pol_y(1,4,jg)
             coef_x(3,3)=coef_x(3,3)+coef_xy(1,38)*pol_y(2,4,jg)
             coef_x(4,3)=coef_x(4,3)+coef_xy(2,38)*pol_y(2,4,jg)
             coef_x(1,4)=coef_x(1,4)+coef_xy(1,39)*pol_y(1,4,jg)
             coef_x(2,4)=coef_x(2,4)+coef_xy(2,39)*pol_y(1,4,jg)
             coef_x(3,4)=coef_x(3,4)+coef_xy(1,39)*pol_y(2,4,jg)
             coef_x(4,4)=coef_x(4,4)+coef_xy(2,39)*pol_y(2,4,jg)
             coef_x(1,5)=coef_x(1,5)+coef_xy(1,40)*pol_y(1,4,jg)
             coef_x(2,5)=coef_x(2,5)+coef_xy(2,40)*pol_y(1,4,jg)
             coef_x(3,5)=coef_x(3,5)+coef_xy(1,40)*pol_y(2,4,jg)
             coef_x(4,5)=coef_x(4,5)+coef_xy(2,40)*pol_y(2,4,jg)
             coef_x(1,0)=coef_x(1,0)+coef_xy(1,41)*pol_y(1,5,jg)
             coef_x(2,0)=coef_x(2,0)+coef_xy(2,41)*pol_y(1,5,jg)
             coef_x(3,0)=coef_x(3,0)+coef_xy(1,41)*pol_y(2,5,jg)
             coef_x(4,0)=coef_x(4,0)+coef_xy(2,41)*pol_y(2,5,jg)
             coef_x(1,1)=coef_x(1,1)+coef_xy(1,42)*pol_y(1,5,jg)
             coef_x(2,1)=coef_x(2,1)+coef_xy(2,42)*pol_y(1,5,jg)
             coef_x(3,1)=coef_x(3,1)+coef_xy(1,42)*pol_y(2,5,jg)
             coef_x(4,1)=coef_x(4,1)+coef_xy(2,42)*pol_y(2,5,jg)
             coef_x(1,2)=coef_x(1,2)+coef_xy(1,43)*pol_y(1,5,jg)
             coef_x(2,2)=coef_x(2,2)+coef_xy(2,43)*pol_y(1,5,jg)
             coef_x(3,2)=coef_x(3,2)+coef_xy(1,43)*pol_y(2,5,jg)
             coef_x(4,2)=coef_x(4,2)+coef_xy(2,43)*pol_y(2,5,jg)
             coef_x(1,3)=coef_x(1,3)+coef_xy(1,44)*pol_y(1,5,jg)
             coef_x(2,3)=coef_x(2,3)+coef_xy(2,44)*pol_y(1,5,jg)
             coef_x(3,3)=coef_x(3,3)+coef_xy(1,44)*pol_y(2,5,jg)
             coef_x(4,3)=coef_x(4,3)+coef_xy(2,44)*pol_y(2,5,jg)
             coef_x(1,4)=coef_x(1,4)+coef_xy(1,45)*pol_y(1,5,jg)
             coef_x(2,4)=coef_x(2,4)+coef_xy(2,45)*pol_y(1,5,jg)
             coef_x(3,4)=coef_x(3,4)+coef_xy(1,45)*pol_y(2,5,jg)
             coef_x(4,4)=coef_x(4,4)+coef_xy(2,45)*pol_y(2,5,jg)
             coef_x(1,0)=coef_x(1,0)+coef_xy(1,46)*pol_y(1,6,jg)
             coef_x(2,0)=coef_x(2,0)+coef_xy(2,46)*pol_y(1,6,jg)
             coef_x(3,0)=coef_x(3,0)+coef_xy(1,46)*pol_y(2,6,jg)
             coef_x(4,0)=coef_x(4,0)+coef_xy(2,46)*pol_y(2,6,jg)
             coef_x(1,1)=coef_x(1,1)+coef_xy(1,47)*pol_y(1,6,jg)
             coef_x(2,1)=coef_x(2,1)+coef_xy(2,47)*pol_y(1,6,jg)
             coef_x(3,1)=coef_x(3,1)+coef_xy(1,47)*pol_y(2,6,jg)
             coef_x(4,1)=coef_x(4,1)+coef_xy(2,47)*pol_y(2,6,jg)
             coef_x(1,2)=coef_x(1,2)+coef_xy(1,48)*pol_y(1,6,jg)
             coef_x(2,2)=coef_x(2,2)+coef_xy(2,48)*pol_y(1,6,jg)
             coef_x(3,2)=coef_x(3,2)+coef_xy(1,48)*pol_y(2,6,jg)
             coef_x(4,2)=coef_x(4,2)+coef_xy(2,48)*pol_y(2,6,jg)
             coef_x(1,3)=coef_x(1,3)+coef_xy(1,49)*pol_y(1,6,jg)
             coef_x(2,3)=coef_x(2,3)+coef_xy(2,49)*pol_y(1,6,jg)
             coef_x(3,3)=coef_x(3,3)+coef_xy(1,49)*pol_y(2,6,jg)
             coef_x(4,3)=coef_x(4,3)+coef_xy(2,49)*pol_y(2,6,jg)
             coef_x(1,0)=coef_x(1,0)+coef_xy(1,50)*pol_y(1,7,jg)
             coef_x(2,0)=coef_x(2,0)+coef_xy(2,50)*pol_y(1,7,jg)
             coef_x(3,0)=coef_x(3,0)+coef_xy(1,50)*pol_y(2,7,jg)
             coef_x(4,0)=coef_x(4,0)+coef_xy(2,50)*pol_y(2,7,jg)
             coef_x(1,1)=coef_x(1,1)+coef_xy(1,51)*pol_y(1,7,jg)
             coef_x(2,1)=coef_x(2,1)+coef_xy(2,51)*pol_y(1,7,jg)
             coef_x(3,1)=coef_x(3,1)+coef_xy(1,51)*pol_y(2,7,jg)
             coef_x(4,1)=coef_x(4,1)+coef_xy(2,51)*pol_y(2,7,jg)
             coef_x(1,2)=coef_x(1,2)+coef_xy(1,52)*pol_y(1,7,jg)
             coef_x(2,2)=coef_x(2,2)+coef_xy(2,52)*pol_y(1,7,jg)
             coef_x(3,2)=coef_x(3,2)+coef_xy(1,52)*pol_y(2,7,jg)
             coef_x(4,2)=coef_x(4,2)+coef_xy(2,52)*pol_y(2,7,jg)
             coef_x(1,0)=coef_x(1,0)+coef_xy(1,53)*pol_y(1,8,jg)
             coef_x(2,0)=coef_x(2,0)+coef_xy(2,53)*pol_y(1,8,jg)
             coef_x(3,0)=coef_x(3,0)+coef_xy(1,53)*pol_y(2,8,jg)
             coef_x(4,0)=coef_x(4,0)+coef_xy(2,53)*pol_y(2,8,jg)
             coef_x(1,1)=coef_x(1,1)+coef_xy(1,54)*pol_y(1,8,jg)
             coef_x(2,1)=coef_x(2,1)+coef_xy(2,54)*pol_y(1,8,jg)
             coef_x(3,1)=coef_x(3,1)+coef_xy(1,54)*pol_y(2,8,jg)
             coef_x(4,1)=coef_x(4,1)+coef_xy(2,54)*pol_y(2,8,jg)
             coef_x(1,0)=coef_x(1,0)+coef_xy(1,55)*pol_y(1,9,jg)
             coef_x(2,0)=coef_x(2,0)+coef_xy(2,55)*pol_y(1,9,jg)
             coef_x(3,0)=coef_x(3,0)+coef_xy(1,55)*pol_y(2,9,jg)
             coef_x(4,0)=coef_x(4,0)+coef_xy(2,55)*pol_y(2,9,jg)
          DO ig=igmin,igmax
             i=map(ig,1)
             s01=0.0_dp
             s02=0.0_dp
             s03=0.0_dp
             s04=0.0_dp
             DO lxp=0,lp
                s01=s01+coef_x(1,lxp)*pol_x(lxp,ig)
                s02=s02+coef_x(2,lxp)*pol_x(lxp,ig)
                s03=s03+coef_x(3,lxp)*pol_x(lxp,ig)
                s04=s04+coef_x(4,lxp)*pol_x(lxp,ig)
             ENDDO
             grid(i,j,k) = grid(i,j,k)     + s01
             grid(i,j2,k) = grid(i,j2,k)   + s03
             grid(i,j,k2) = grid(i,j,k2)   + s02
             grid(i,j2,k2) = grid(i,j2,k2) + s04
          END DO
       END DO
    END DO

  END SUBROUTINE collocate_core_9



! ideally users test for the optional 4 macro definition
! otherwise, we provide some reasonable defaults
! the included fortran files represent the same kernel, with a fortran representation that optimizes best for a given compiler




  SUBROUTINE integrate_core_default(grid,coef_xyz,pol_x,pol_y,pol_z,map,sphere_bounds,lp,cmax,gridbounds)
    USE kinds,                        ONLY: dp
    INTEGER, INTENT(IN)                      :: sphere_bounds(*), lp
    REAL(dp), INTENT(OUT) :: coef_xyz(((lp+1)*(lp+2)*(lp+3))/6)
    INTEGER, INTENT(IN)                      :: cmax
    REAL(dp), INTENT(IN)                     :: pol_x(0:lp,-cmax:cmax), &
                                                pol_y(1:2,0:lp,-cmax:0), &
                                                pol_z(1:2,0:lp,-cmax:0)
    INTEGER, INTENT(IN)                      :: map(-cmax:cmax,1:3), &
                                                gridbounds(2,3)
    REAL(dp), INTENT(IN) :: grid(gridbounds(1,1):gridbounds(2,1), gridbounds(1&
      ,2):gridbounds(2,2), gridbounds(1,3):gridbounds(2,3))

    INTEGER                                  :: i, ig, igmax, igmin, j, j2, &
                                                jg, jg2, jgmin, k, k2, kg, &
                                                kg2, kgmin, lxp, lxy, lxyz, &
                                                lyp, lzp, sci
    REAL(dp)                                 :: coef_x(4,0:lp), &
                                                coef_xy(2,((lp+1)*(lp+2))/2), &
                                                s01, s02, s03, s04

    sci=1

    coef_xyz=0.0_dp

    kgmin=sphere_bounds(sci)
    sci=sci+1
    DO kg=kgmin,0
       kg2=1-kg
       k=map(kg,3)
       k2=map(kg2,3)

       coef_xy=0.0_dp

       jgmin=sphere_bounds(sci)
       sci=sci+1
       DO jg=jgmin,0
          jg2=1-jg
          j=map(jg,2)
          j2=map(jg2,2)
          igmin=sphere_bounds(sci)
          sci=sci+1
          igmax=1-igmin

          coef_x=0.0_dp

          DO ig=igmin,igmax
             i=map(ig,1)
             s01=grid(i,j,k)
             s02=grid(i,j,k2)
             s03=grid(i,j2,k)
             s04=grid(i,j2,k2)
             DO lxp=0,lp
                coef_x(1,lxp)=coef_x(1,lxp)+s01*pol_x(lxp,ig)
                coef_x(2,lxp)=coef_x(2,lxp)+s02*pol_x(lxp,ig)
                coef_x(3,lxp)=coef_x(3,lxp)+s03*pol_x(lxp,ig)
                coef_x(4,lxp)=coef_x(4,lxp)+s04*pol_x(lxp,ig)
             ENDDO
          END DO

          lxy=0
          DO lyp=0,lp
          DO lxp=0,lp-lyp
             lxy=lxy+1
             coef_xy(1,lxy)=coef_xy(1,lxy)+coef_x(1,lxp)*pol_y(1,lyp,jg)
             coef_xy(2,lxy)=coef_xy(2,lxy)+coef_x(2,lxp)*pol_y(1,lyp,jg)
             coef_xy(1,lxy)=coef_xy(1,lxy)+coef_x(3,lxp)*pol_y(2,lyp,jg)
             coef_xy(2,lxy)=coef_xy(2,lxy)+coef_x(4,lxp)*pol_y(2,lyp,jg)
          ENDDO
          ENDDO

       END DO

       lxyz = 0
       DO lzp=0,lp
          lxy=0
          DO lyp=0,lp-lzp
             DO lxp=0,lp-lzp-lyp
                lxyz=lxyz+1 ; lxy=lxy+1
                coef_xyz(lxyz)=coef_xyz(lxyz)+coef_xy(1,lxy)*pol_z(1,lzp,kg)
                coef_xyz(lxyz)=coef_xyz(lxyz)+coef_xy(2,lxy)*pol_z(2,lzp,kg)
             ENDDO
             lxy=lxy+lzp
          ENDDO
       ENDDO

    END DO

  END SUBROUTINE integrate_core_default
  SUBROUTINE integrate_core_0(grid,coef_xyz,pol_x,pol_y,pol_z,map,sphere_bounds,cmax,gridbounds)
    USE kinds,                        ONLY: dp
    INTEGER, INTENT(IN)                      :: sphere_bounds(*), cmax, &
                                                map(-cmax:cmax,1:3), &
                                                gridbounds(2,3)
    REAL(dp), INTENT(IN) :: grid(gridbounds(1,1):gridbounds(2,1), gridbounds(1&
      ,2):gridbounds(2,2), gridbounds(1,3):gridbounds(2,3))
    INTEGER, PARAMETER                       :: lp = 0
    REAL(dp), INTENT(IN)                     :: pol_x(0:lp,-cmax:cmax), &
                                                pol_y(1:2,0:lp,-cmax:0), &
                                                pol_z(1:2,0:lp,-cmax:0)
    REAL(dp), INTENT(OUT) :: coef_xyz(((lp+1)*(lp+2)*(lp+3))/6)

    INTEGER                                  :: i, ig, igmax, igmin, j, j2, &
                                                jg, jg2, jgmin, k, k2, kg, &
                                                kg2, kgmin, lxp, lxy, lxyz, &
                                                lyp, lzp, sci
    REAL(dp)                                 :: coef_x(4,0:lp), &
                                                coef_xy(2,((lp+1)*(lp+2))/2), &
                                                s(4)


    sci=1

    coef_xyz=0.0_dp

    kgmin=sphere_bounds(sci)
    sci=sci+1
    DO kg=kgmin,0
       kg2=1-kg
       k=map(kg,3)
       k2=map(kg2,3)

       coef_xy=0.0_dp

       jgmin=sphere_bounds(sci)
       sci=sci+1
       DO jg=jgmin,0
          jg2=1-jg
          j=map(jg,2)
          j2=map(jg2,2)
          igmin=sphere_bounds(sci)
          sci=sci+1
          igmax=1-igmin

          coef_x=0.0_dp
          DO ig=igmin,igmax
             i=map(ig,1)
             s(1)=grid(i,j,k)
             s(2)=grid(i,j,k2)
             s(3)=grid(i,j2,k)
             s(4)=grid(i,j2,k2)
             DO lxp=0,lp
                coef_x(:,lxp)=coef_x(:,lxp)+s(:)*pol_x(lxp,ig)
             ENDDO
          END DO
             coef_xy(:,1)=coef_xy(:,1)+coef_x(1:2,0)*pol_y(1,0,jg)
             coef_xy(:,1)=coef_xy(:,1)+coef_x(3:4,0)*pol_y(2,0,jg)
       END DO
       lxyz = 0
       DO lzp=0,lp
          lxy=0
          DO lyp=0,lp-lzp
             DO lxp=0,lp-lzp-lyp
                lxyz=lxyz+1 ; lxy=lxy+1
                coef_xyz(lxyz)=coef_xyz(lxyz)+coef_xy(1,lxy)*pol_z(1,lzp,kg)
                coef_xyz(lxyz)=coef_xyz(lxyz)+coef_xy(2,lxy)*pol_z(2,lzp,kg)
             ENDDO
             lxy=lxy+lzp
          ENDDO
       ENDDO
    END DO

  END SUBROUTINE integrate_core_0
  SUBROUTINE integrate_core_1(grid,coef_xyz,pol_x,pol_y,pol_z,map,sphere_bounds,cmax,gridbounds)
    USE kinds,                        ONLY: dp
    INTEGER, INTENT(IN)                      :: sphere_bounds(*), cmax, &
                                                map(-cmax:cmax,1:3), &
                                                gridbounds(2,3)
    REAL(dp), INTENT(IN) :: grid(gridbounds(1,1):gridbounds(2,1), gridbounds(1&
      ,2):gridbounds(2,2), gridbounds(1,3):gridbounds(2,3))
    INTEGER, PARAMETER                       :: lp = 1
    REAL(dp), INTENT(IN)                     :: pol_x(0:lp,-cmax:cmax), &
                                                pol_y(1:2,0:lp,-cmax:0), &
                                                pol_z(1:2,0:lp,-cmax:0)
    REAL(dp), INTENT(OUT) :: coef_xyz(((lp+1)*(lp+2)*(lp+3))/6)

    INTEGER                                  :: i, ig, igmax, igmin, j, j2, &
                                                jg, jg2, jgmin, k, k2, kg, &
                                                kg2, kgmin, lxp, sci
    REAL(dp)                                 :: coef_x(4,0:lp), &
                                                coef_xy(2,((lp+1)*(lp+2))/2), &
                                                s(4)

    sci=1

    coef_xyz=0.0_dp

    kgmin=sphere_bounds(sci)
    sci=sci+1
    DO kg=kgmin,0
       kg2=1-kg
       k=map(kg,3)
       k2=map(kg2,3)

       coef_xy=0.0_dp

       jgmin=sphere_bounds(sci)
       sci=sci+1
       DO jg=jgmin,0
          jg2=1-jg
          j=map(jg,2)
          j2=map(jg2,2)
          igmin=sphere_bounds(sci)
          sci=sci+1
          igmax=1-igmin

          coef_x=0.0_dp
          DO ig=igmin,igmax
             i=map(ig,1)
             s(1)=grid(i,j,k)
             s(2)=grid(i,j,k2)
             s(3)=grid(i,j2,k)
             s(4)=grid(i,j2,k2)
             DO lxp=0,lp
                coef_x(:,lxp)=coef_x(:,lxp)+s(:)*pol_x(lxp,ig)
             ENDDO
          END DO
             coef_xy(:,1)=coef_xy(:,1)+coef_x(1:2,0)*pol_y(1,0,jg)
             coef_xy(:,1)=coef_xy(:,1)+coef_x(3:4,0)*pol_y(2,0,jg)
             coef_xy(:,2)=coef_xy(:,2)+coef_x(1:2,1)*pol_y(1,0,jg)
             coef_xy(:,2)=coef_xy(:,2)+coef_x(3:4,1)*pol_y(2,0,jg)
             coef_xy(:,3)=coef_xy(:,3)+coef_x(1:2,0)*pol_y(1,1,jg)
             coef_xy(:,3)=coef_xy(:,3)+coef_x(3:4,0)*pol_y(2,1,jg)
       END DO
                coef_xyz(1)=coef_xyz(1)+coef_xy(1,1)*pol_z(1,0,kg)
                coef_xyz(1)=coef_xyz(1)+coef_xy(2,1)*pol_z(2,0,kg)
                coef_xyz(2)=coef_xyz(2)+coef_xy(1,2)*pol_z(1,0,kg)
                coef_xyz(2)=coef_xyz(2)+coef_xy(2,2)*pol_z(2,0,kg)
                coef_xyz(3)=coef_xyz(3)+coef_xy(1,3)*pol_z(1,0,kg)
                coef_xyz(3)=coef_xyz(3)+coef_xy(2,3)*pol_z(2,0,kg)
                coef_xyz(4)=coef_xyz(4)+coef_xy(1,1)*pol_z(1,1,kg)
                coef_xyz(4)=coef_xyz(4)+coef_xy(2,1)*pol_z(2,1,kg)
    END DO

  END SUBROUTINE integrate_core_1
  SUBROUTINE integrate_core_2(grid,coef_xyz,pol_x,pol_y,pol_z,map,sphere_bounds,cmax,gridbounds)
    USE kinds,                        ONLY: dp
    INTEGER, INTENT(IN)                      :: sphere_bounds(*), cmax, &
                                                map(-cmax:cmax,1:3), &
                                                gridbounds(2,3)
    REAL(dp), INTENT(IN) :: grid(gridbounds(1,1):gridbounds(2,1), gridbounds(1&
      ,2):gridbounds(2,2), gridbounds(1,3):gridbounds(2,3))
    INTEGER, PARAMETER                       :: lp = 2
    REAL(dp), INTENT(IN)                     :: pol_x(0:lp,-cmax:cmax), &
                                                pol_y(1:2,0:lp,-cmax:0), &
                                                pol_z(1:2,0:lp,-cmax:0)
    REAL(dp), INTENT(OUT) :: coef_xyz(((lp+1)*(lp+2)*(lp+3))/6)

    INTEGER                                  :: i, ig, igmax, igmin, j, j2, &
                                                jg, jg2, jgmin, k, k2, kg, &
                                                kg2, kgmin, lxp, sci
    REAL(dp)                                 :: coef_x(4,0:lp), &
                                                coef_xy(2,((lp+1)*(lp+2))/2), &
                                                s01, s02, s03, s04

    sci=1

    coef_xyz=0.0_dp

    kgmin=sphere_bounds(sci)
    sci=sci+1
    DO kg=kgmin,0
       kg2=1-kg
       k=map(kg,3)
       k2=map(kg2,3)

       coef_xy=0.0_dp

       jgmin=sphere_bounds(sci)
       sci=sci+1
       DO jg=jgmin,0
          jg2=1-jg
          j=map(jg,2)
          j2=map(jg2,2)
          igmin=sphere_bounds(sci)
          sci=sci+1
          igmax=1-igmin

          coef_x=0.0_dp
          DO ig=igmin,igmax
             i=map(ig,1)
             s01=grid(i,j,k)
             s02=grid(i,j,k2)
             s03=grid(i,j2,k)
             s04=grid(i,j2,k2)
             DO lxp=0,lp
                coef_x(1,lxp)=coef_x(1,lxp)+s01*pol_x(lxp,ig)
                coef_x(2,lxp)=coef_x(2,lxp)+s02*pol_x(lxp,ig)
                coef_x(3,lxp)=coef_x(3,lxp)+s03*pol_x(lxp,ig)
                coef_x(4,lxp)=coef_x(4,lxp)+s04*pol_x(lxp,ig)
             ENDDO
          END DO
             coef_xy(:,1)=coef_xy(:,1)+coef_x(1:2,0)*pol_y(1,0,jg)
             coef_xy(:,1)=coef_xy(:,1)+coef_x(3:4,0)*pol_y(2,0,jg)
             coef_xy(:,2)=coef_xy(:,2)+coef_x(1:2,1)*pol_y(1,0,jg)
             coef_xy(:,2)=coef_xy(:,2)+coef_x(3:4,1)*pol_y(2,0,jg)
             coef_xy(:,3)=coef_xy(:,3)+coef_x(1:2,2)*pol_y(1,0,jg)
             coef_xy(:,3)=coef_xy(:,3)+coef_x(3:4,2)*pol_y(2,0,jg)
             coef_xy(:,4)=coef_xy(:,4)+coef_x(1:2,0)*pol_y(1,1,jg)
             coef_xy(:,4)=coef_xy(:,4)+coef_x(3:4,0)*pol_y(2,1,jg)
             coef_xy(:,5)=coef_xy(:,5)+coef_x(1:2,1)*pol_y(1,1,jg)
             coef_xy(:,5)=coef_xy(:,5)+coef_x(3:4,1)*pol_y(2,1,jg)
             coef_xy(:,6)=coef_xy(:,6)+coef_x(1:2,0)*pol_y(1,2,jg)
             coef_xy(:,6)=coef_xy(:,6)+coef_x(3:4,0)*pol_y(2,2,jg)
       END DO
                coef_xyz(1)=coef_xyz(1)+coef_xy(1,1)*pol_z(1,0,kg)
                coef_xyz(1)=coef_xyz(1)+coef_xy(2,1)*pol_z(2,0,kg)
                coef_xyz(2)=coef_xyz(2)+coef_xy(1,2)*pol_z(1,0,kg)
                coef_xyz(2)=coef_xyz(2)+coef_xy(2,2)*pol_z(2,0,kg)
                coef_xyz(3)=coef_xyz(3)+coef_xy(1,3)*pol_z(1,0,kg)
                coef_xyz(3)=coef_xyz(3)+coef_xy(2,3)*pol_z(2,0,kg)
                coef_xyz(4)=coef_xyz(4)+coef_xy(1,4)*pol_z(1,0,kg)
                coef_xyz(4)=coef_xyz(4)+coef_xy(2,4)*pol_z(2,0,kg)
                coef_xyz(5)=coef_xyz(5)+coef_xy(1,5)*pol_z(1,0,kg)
                coef_xyz(5)=coef_xyz(5)+coef_xy(2,5)*pol_z(2,0,kg)
                coef_xyz(6)=coef_xyz(6)+coef_xy(1,6)*pol_z(1,0,kg)
                coef_xyz(6)=coef_xyz(6)+coef_xy(2,6)*pol_z(2,0,kg)
                coef_xyz(7)=coef_xyz(7)+coef_xy(1,1)*pol_z(1,1,kg)
                coef_xyz(7)=coef_xyz(7)+coef_xy(2,1)*pol_z(2,1,kg)
                coef_xyz(8)=coef_xyz(8)+coef_xy(1,2)*pol_z(1,1,kg)
                coef_xyz(8)=coef_xyz(8)+coef_xy(2,2)*pol_z(2,1,kg)
                coef_xyz(9)=coef_xyz(9)+coef_xy(1,4)*pol_z(1,1,kg)
                coef_xyz(9)=coef_xyz(9)+coef_xy(2,4)*pol_z(2,1,kg)
                coef_xyz(10)=coef_xyz(10)+coef_xy(1,1)*pol_z(1,2,kg)
                coef_xyz(10)=coef_xyz(10)+coef_xy(2,1)*pol_z(2,2,kg)
    END DO

  END SUBROUTINE integrate_core_2
  SUBROUTINE integrate_core_3(grid,coef_xyz,pol_x,pol_y,pol_z,map,sphere_bounds,cmax,gridbounds)
    USE kinds,                        ONLY: dp
    INTEGER, INTENT(IN)                      :: sphere_bounds(*), cmax, &
                                                map(-cmax:cmax,1:3), &
                                                gridbounds(2,3)
    REAL(dp), INTENT(IN) :: grid(gridbounds(1,1):gridbounds(2,1), gridbounds(1&
      ,2):gridbounds(2,2), gridbounds(1,3):gridbounds(2,3))
    INTEGER, PARAMETER                       :: lp = 3
    REAL(dp), INTENT(IN)                     :: pol_x(0:lp,-cmax:cmax), &
                                                pol_y(1:2,0:lp,-cmax:0), &
                                                pol_z(1:2,0:lp,-cmax:0)
    REAL(dp), INTENT(OUT) :: coef_xyz(((lp+1)*(lp+2)*(lp+3))/6)

    INTEGER                                  :: i, ig, igmax, igmin, j, j2, &
                                                jg, jg2, jgmin, k, k2, kg, &
                                                kg2, kgmin, lxp, sci
    REAL(dp)                                 :: coef_x(4,0:lp), &
                                                coef_xy(2,((lp+1)*(lp+2))/2), &
                                                s01, s02, s03, s04

    sci=1

    coef_xyz=0.0_dp

    kgmin=sphere_bounds(sci)
    sci=sci+1
    DO kg=kgmin,0
       kg2=1-kg
       k=map(kg,3)
       k2=map(kg2,3)

       coef_xy=0.0_dp

       jgmin=sphere_bounds(sci)
       sci=sci+1
       DO jg=jgmin,0
          jg2=1-jg
          j=map(jg,2)
          j2=map(jg2,2)
          igmin=sphere_bounds(sci)
          sci=sci+1
          igmax=1-igmin

          coef_x=0.0_dp
          DO ig=igmin,igmax
             i=map(ig,1)
             s01=grid(i,j,k)
             s02=grid(i,j,k2)
             s03=grid(i,j2,k)
             s04=grid(i,j2,k2)
             DO lxp=0,lp
                coef_x(1,lxp)=coef_x(1,lxp)+s01*pol_x(lxp,ig)
                coef_x(2,lxp)=coef_x(2,lxp)+s02*pol_x(lxp,ig)
                coef_x(3,lxp)=coef_x(3,lxp)+s03*pol_x(lxp,ig)
                coef_x(4,lxp)=coef_x(4,lxp)+s04*pol_x(lxp,ig)
             ENDDO
          END DO
             coef_xy(:,1)=coef_xy(:,1)+coef_x(1:2,0)*pol_y(1,0,jg)
             coef_xy(:,1)=coef_xy(:,1)+coef_x(3:4,0)*pol_y(2,0,jg)
             coef_xy(:,2)=coef_xy(:,2)+coef_x(1:2,1)*pol_y(1,0,jg)
             coef_xy(:,2)=coef_xy(:,2)+coef_x(3:4,1)*pol_y(2,0,jg)
             coef_xy(:,3)=coef_xy(:,3)+coef_x(1:2,2)*pol_y(1,0,jg)
             coef_xy(:,3)=coef_xy(:,3)+coef_x(3:4,2)*pol_y(2,0,jg)
             coef_xy(:,4)=coef_xy(:,4)+coef_x(1:2,3)*pol_y(1,0,jg)
             coef_xy(:,4)=coef_xy(:,4)+coef_x(3:4,3)*pol_y(2,0,jg)
             coef_xy(:,5)=coef_xy(:,5)+coef_x(1:2,0)*pol_y(1,1,jg)
             coef_xy(:,5)=coef_xy(:,5)+coef_x(3:4,0)*pol_y(2,1,jg)
             coef_xy(:,6)=coef_xy(:,6)+coef_x(1:2,1)*pol_y(1,1,jg)
             coef_xy(:,6)=coef_xy(:,6)+coef_x(3:4,1)*pol_y(2,1,jg)
             coef_xy(:,7)=coef_xy(:,7)+coef_x(1:2,2)*pol_y(1,1,jg)
             coef_xy(:,7)=coef_xy(:,7)+coef_x(3:4,2)*pol_y(2,1,jg)
             coef_xy(:,8)=coef_xy(:,8)+coef_x(1:2,0)*pol_y(1,2,jg)
             coef_xy(:,8)=coef_xy(:,8)+coef_x(3:4,0)*pol_y(2,2,jg)
             coef_xy(:,9)=coef_xy(:,9)+coef_x(1:2,1)*pol_y(1,2,jg)
             coef_xy(:,9)=coef_xy(:,9)+coef_x(3:4,1)*pol_y(2,2,jg)
             coef_xy(:,10)=coef_xy(:,10)+coef_x(1:2,0)*pol_y(1,3,jg)
             coef_xy(:,10)=coef_xy(:,10)+coef_x(3:4,0)*pol_y(2,3,jg)
       END DO
                coef_xyz(1)=coef_xyz(1)+coef_xy(1,1)*pol_z(1,0,kg)
                coef_xyz(1)=coef_xyz(1)+coef_xy(2,1)*pol_z(2,0,kg)
                coef_xyz(2)=coef_xyz(2)+coef_xy(1,2)*pol_z(1,0,kg)
                coef_xyz(2)=coef_xyz(2)+coef_xy(2,2)*pol_z(2,0,kg)
                coef_xyz(3)=coef_xyz(3)+coef_xy(1,3)*pol_z(1,0,kg)
                coef_xyz(3)=coef_xyz(3)+coef_xy(2,3)*pol_z(2,0,kg)
                coef_xyz(4)=coef_xyz(4)+coef_xy(1,4)*pol_z(1,0,kg)
                coef_xyz(4)=coef_xyz(4)+coef_xy(2,4)*pol_z(2,0,kg)
                coef_xyz(5)=coef_xyz(5)+coef_xy(1,5)*pol_z(1,0,kg)
                coef_xyz(5)=coef_xyz(5)+coef_xy(2,5)*pol_z(2,0,kg)
                coef_xyz(6)=coef_xyz(6)+coef_xy(1,6)*pol_z(1,0,kg)
                coef_xyz(6)=coef_xyz(6)+coef_xy(2,6)*pol_z(2,0,kg)
                coef_xyz(7)=coef_xyz(7)+coef_xy(1,7)*pol_z(1,0,kg)
                coef_xyz(7)=coef_xyz(7)+coef_xy(2,7)*pol_z(2,0,kg)
                coef_xyz(8)=coef_xyz(8)+coef_xy(1,8)*pol_z(1,0,kg)
                coef_xyz(8)=coef_xyz(8)+coef_xy(2,8)*pol_z(2,0,kg)
                coef_xyz(9)=coef_xyz(9)+coef_xy(1,9)*pol_z(1,0,kg)
                coef_xyz(9)=coef_xyz(9)+coef_xy(2,9)*pol_z(2,0,kg)
                coef_xyz(10)=coef_xyz(10)+coef_xy(1,10)*pol_z(1,0,kg)
                coef_xyz(10)=coef_xyz(10)+coef_xy(2,10)*pol_z(2,0,kg)
                coef_xyz(11)=coef_xyz(11)+coef_xy(1,1)*pol_z(1,1,kg)
                coef_xyz(11)=coef_xyz(11)+coef_xy(2,1)*pol_z(2,1,kg)
                coef_xyz(12)=coef_xyz(12)+coef_xy(1,2)*pol_z(1,1,kg)
                coef_xyz(12)=coef_xyz(12)+coef_xy(2,2)*pol_z(2,1,kg)
                coef_xyz(13)=coef_xyz(13)+coef_xy(1,3)*pol_z(1,1,kg)
                coef_xyz(13)=coef_xyz(13)+coef_xy(2,3)*pol_z(2,1,kg)
                coef_xyz(14)=coef_xyz(14)+coef_xy(1,5)*pol_z(1,1,kg)
                coef_xyz(14)=coef_xyz(14)+coef_xy(2,5)*pol_z(2,1,kg)
                coef_xyz(15)=coef_xyz(15)+coef_xy(1,6)*pol_z(1,1,kg)
                coef_xyz(15)=coef_xyz(15)+coef_xy(2,6)*pol_z(2,1,kg)
                coef_xyz(16)=coef_xyz(16)+coef_xy(1,8)*pol_z(1,1,kg)
                coef_xyz(16)=coef_xyz(16)+coef_xy(2,8)*pol_z(2,1,kg)
                coef_xyz(17)=coef_xyz(17)+coef_xy(1,1)*pol_z(1,2,kg)
                coef_xyz(17)=coef_xyz(17)+coef_xy(2,1)*pol_z(2,2,kg)
                coef_xyz(18)=coef_xyz(18)+coef_xy(1,2)*pol_z(1,2,kg)
                coef_xyz(18)=coef_xyz(18)+coef_xy(2,2)*pol_z(2,2,kg)
                coef_xyz(19)=coef_xyz(19)+coef_xy(1,5)*pol_z(1,2,kg)
                coef_xyz(19)=coef_xyz(19)+coef_xy(2,5)*pol_z(2,2,kg)
                coef_xyz(20)=coef_xyz(20)+coef_xy(1,1)*pol_z(1,3,kg)
                coef_xyz(20)=coef_xyz(20)+coef_xy(2,1)*pol_z(2,3,kg)
    END DO

  END SUBROUTINE integrate_core_3
  SUBROUTINE integrate_core_4(grid,coef_xyz,pol_x,pol_y,pol_z,map,sphere_bounds,cmax,gridbounds)
    USE kinds,                        ONLY: dp
    INTEGER, INTENT(IN)                      :: sphere_bounds(*), cmax, &
                                                map(-cmax:cmax,1:3), &
                                                gridbounds(2,3)
    REAL(dp), INTENT(IN) :: grid(gridbounds(1,1):gridbounds(2,1), gridbounds(1&
      ,2):gridbounds(2,2), gridbounds(1,3):gridbounds(2,3))
    INTEGER, PARAMETER                       :: lp = 4
    REAL(dp), INTENT(IN)                     :: pol_x(0:lp,-cmax:cmax), &
                                                pol_y(1:2,0:lp,-cmax:0), &
                                                pol_z(1:2,0:lp,-cmax:0)
    REAL(dp), INTENT(OUT) :: coef_xyz(((lp+1)*(lp+2)*(lp+3))/6)

    INTEGER                                  :: i, ig, igmax, igmin, j, j2, &
                                                jg, jg2, jgmin, k, k2, kg, &
                                                kg2, kgmin, lxp, sci
    REAL(dp)                                 :: coef_x(4,0:lp), &
                                                coef_xy(2,((lp+1)*(lp+2))/2), &
                                                s01, s02, s03, s04

    sci=1

    coef_xyz=0.0_dp

    kgmin=sphere_bounds(sci)
    sci=sci+1
    DO kg=kgmin,0
       kg2=1-kg
       k=map(kg,3)
       k2=map(kg2,3)

       coef_xy=0.0_dp

       jgmin=sphere_bounds(sci)
       sci=sci+1
       DO jg=jgmin,0
          jg2=1-jg
          j=map(jg,2)
          j2=map(jg2,2)
          igmin=sphere_bounds(sci)
          sci=sci+1
          igmax=1-igmin

          coef_x=0.0_dp
          DO ig=igmin,igmax
             i=map(ig,1)
             s01=grid(i,j,k)
             s02=grid(i,j,k2)
             s03=grid(i,j2,k)
             s04=grid(i,j2,k2)
             DO lxp=0,lp
                coef_x(1,lxp)=coef_x(1,lxp)+s01*pol_x(lxp,ig)
                coef_x(2,lxp)=coef_x(2,lxp)+s02*pol_x(lxp,ig)
                coef_x(3,lxp)=coef_x(3,lxp)+s03*pol_x(lxp,ig)
                coef_x(4,lxp)=coef_x(4,lxp)+s04*pol_x(lxp,ig)
             ENDDO
          END DO
             coef_xy(1,1)=coef_xy(1,1)+coef_x(1,0)*pol_y(1,0,jg)
             coef_xy(2,1)=coef_xy(2,1)+coef_x(2,0)*pol_y(1,0,jg)
             coef_xy(1,1)=coef_xy(1,1)+coef_x(3,0)*pol_y(2,0,jg)
             coef_xy(2,1)=coef_xy(2,1)+coef_x(4,0)*pol_y(2,0,jg)
             coef_xy(1,2)=coef_xy(1,2)+coef_x(1,1)*pol_y(1,0,jg)
             coef_xy(2,2)=coef_xy(2,2)+coef_x(2,1)*pol_y(1,0,jg)
             coef_xy(1,2)=coef_xy(1,2)+coef_x(3,1)*pol_y(2,0,jg)
             coef_xy(2,2)=coef_xy(2,2)+coef_x(4,1)*pol_y(2,0,jg)
             coef_xy(1,3)=coef_xy(1,3)+coef_x(1,2)*pol_y(1,0,jg)
             coef_xy(2,3)=coef_xy(2,3)+coef_x(2,2)*pol_y(1,0,jg)
             coef_xy(1,3)=coef_xy(1,3)+coef_x(3,2)*pol_y(2,0,jg)
             coef_xy(2,3)=coef_xy(2,3)+coef_x(4,2)*pol_y(2,0,jg)
             coef_xy(1,4)=coef_xy(1,4)+coef_x(1,3)*pol_y(1,0,jg)
             coef_xy(2,4)=coef_xy(2,4)+coef_x(2,3)*pol_y(1,0,jg)
             coef_xy(1,4)=coef_xy(1,4)+coef_x(3,3)*pol_y(2,0,jg)
             coef_xy(2,4)=coef_xy(2,4)+coef_x(4,3)*pol_y(2,0,jg)
             coef_xy(1,5)=coef_xy(1,5)+coef_x(1,4)*pol_y(1,0,jg)
             coef_xy(2,5)=coef_xy(2,5)+coef_x(2,4)*pol_y(1,0,jg)
             coef_xy(1,5)=coef_xy(1,5)+coef_x(3,4)*pol_y(2,0,jg)
             coef_xy(2,5)=coef_xy(2,5)+coef_x(4,4)*pol_y(2,0,jg)
             coef_xy(1,6)=coef_xy(1,6)+coef_x(1,0)*pol_y(1,1,jg)
             coef_xy(2,6)=coef_xy(2,6)+coef_x(2,0)*pol_y(1,1,jg)
             coef_xy(1,6)=coef_xy(1,6)+coef_x(3,0)*pol_y(2,1,jg)
             coef_xy(2,6)=coef_xy(2,6)+coef_x(4,0)*pol_y(2,1,jg)
             coef_xy(1,7)=coef_xy(1,7)+coef_x(1,1)*pol_y(1,1,jg)
             coef_xy(2,7)=coef_xy(2,7)+coef_x(2,1)*pol_y(1,1,jg)
             coef_xy(1,7)=coef_xy(1,7)+coef_x(3,1)*pol_y(2,1,jg)
             coef_xy(2,7)=coef_xy(2,7)+coef_x(4,1)*pol_y(2,1,jg)
             coef_xy(1,8)=coef_xy(1,8)+coef_x(1,2)*pol_y(1,1,jg)
             coef_xy(2,8)=coef_xy(2,8)+coef_x(2,2)*pol_y(1,1,jg)
             coef_xy(1,8)=coef_xy(1,8)+coef_x(3,2)*pol_y(2,1,jg)
             coef_xy(2,8)=coef_xy(2,8)+coef_x(4,2)*pol_y(2,1,jg)
             coef_xy(1,9)=coef_xy(1,9)+coef_x(1,3)*pol_y(1,1,jg)
             coef_xy(2,9)=coef_xy(2,9)+coef_x(2,3)*pol_y(1,1,jg)
             coef_xy(1,9)=coef_xy(1,9)+coef_x(3,3)*pol_y(2,1,jg)
             coef_xy(2,9)=coef_xy(2,9)+coef_x(4,3)*pol_y(2,1,jg)
             coef_xy(1,10)=coef_xy(1,10)+coef_x(1,0)*pol_y(1,2,jg)
             coef_xy(2,10)=coef_xy(2,10)+coef_x(2,0)*pol_y(1,2,jg)
             coef_xy(1,10)=coef_xy(1,10)+coef_x(3,0)*pol_y(2,2,jg)
             coef_xy(2,10)=coef_xy(2,10)+coef_x(4,0)*pol_y(2,2,jg)
             coef_xy(1,11)=coef_xy(1,11)+coef_x(1,1)*pol_y(1,2,jg)
             coef_xy(2,11)=coef_xy(2,11)+coef_x(2,1)*pol_y(1,2,jg)
             coef_xy(1,11)=coef_xy(1,11)+coef_x(3,1)*pol_y(2,2,jg)
             coef_xy(2,11)=coef_xy(2,11)+coef_x(4,1)*pol_y(2,2,jg)
             coef_xy(1,12)=coef_xy(1,12)+coef_x(1,2)*pol_y(1,2,jg)
             coef_xy(2,12)=coef_xy(2,12)+coef_x(2,2)*pol_y(1,2,jg)
             coef_xy(1,12)=coef_xy(1,12)+coef_x(3,2)*pol_y(2,2,jg)
             coef_xy(2,12)=coef_xy(2,12)+coef_x(4,2)*pol_y(2,2,jg)
             coef_xy(1,13)=coef_xy(1,13)+coef_x(1,0)*pol_y(1,3,jg)
             coef_xy(2,13)=coef_xy(2,13)+coef_x(2,0)*pol_y(1,3,jg)
             coef_xy(1,13)=coef_xy(1,13)+coef_x(3,0)*pol_y(2,3,jg)
             coef_xy(2,13)=coef_xy(2,13)+coef_x(4,0)*pol_y(2,3,jg)
             coef_xy(1,14)=coef_xy(1,14)+coef_x(1,1)*pol_y(1,3,jg)
             coef_xy(2,14)=coef_xy(2,14)+coef_x(2,1)*pol_y(1,3,jg)
             coef_xy(1,14)=coef_xy(1,14)+coef_x(3,1)*pol_y(2,3,jg)
             coef_xy(2,14)=coef_xy(2,14)+coef_x(4,1)*pol_y(2,3,jg)
             coef_xy(1,15)=coef_xy(1,15)+coef_x(1,0)*pol_y(1,4,jg)
             coef_xy(2,15)=coef_xy(2,15)+coef_x(2,0)*pol_y(1,4,jg)
             coef_xy(1,15)=coef_xy(1,15)+coef_x(3,0)*pol_y(2,4,jg)
             coef_xy(2,15)=coef_xy(2,15)+coef_x(4,0)*pol_y(2,4,jg)
       END DO
                coef_xyz(1)=coef_xyz(1)+coef_xy(1,1)*pol_z(1,0,kg)
                coef_xyz(1)=coef_xyz(1)+coef_xy(2,1)*pol_z(2,0,kg)
                coef_xyz(2)=coef_xyz(2)+coef_xy(1,2)*pol_z(1,0,kg)
                coef_xyz(2)=coef_xyz(2)+coef_xy(2,2)*pol_z(2,0,kg)
                coef_xyz(3)=coef_xyz(3)+coef_xy(1,3)*pol_z(1,0,kg)
                coef_xyz(3)=coef_xyz(3)+coef_xy(2,3)*pol_z(2,0,kg)
                coef_xyz(4)=coef_xyz(4)+coef_xy(1,4)*pol_z(1,0,kg)
                coef_xyz(4)=coef_xyz(4)+coef_xy(2,4)*pol_z(2,0,kg)
                coef_xyz(5)=coef_xyz(5)+coef_xy(1,5)*pol_z(1,0,kg)
                coef_xyz(5)=coef_xyz(5)+coef_xy(2,5)*pol_z(2,0,kg)
                coef_xyz(6)=coef_xyz(6)+coef_xy(1,6)*pol_z(1,0,kg)
                coef_xyz(6)=coef_xyz(6)+coef_xy(2,6)*pol_z(2,0,kg)
                coef_xyz(7)=coef_xyz(7)+coef_xy(1,7)*pol_z(1,0,kg)
                coef_xyz(7)=coef_xyz(7)+coef_xy(2,7)*pol_z(2,0,kg)
                coef_xyz(8)=coef_xyz(8)+coef_xy(1,8)*pol_z(1,0,kg)
                coef_xyz(8)=coef_xyz(8)+coef_xy(2,8)*pol_z(2,0,kg)
                coef_xyz(9)=coef_xyz(9)+coef_xy(1,9)*pol_z(1,0,kg)
                coef_xyz(9)=coef_xyz(9)+coef_xy(2,9)*pol_z(2,0,kg)
                coef_xyz(10)=coef_xyz(10)+coef_xy(1,10)*pol_z(1,0,kg)
                coef_xyz(10)=coef_xyz(10)+coef_xy(2,10)*pol_z(2,0,kg)
                coef_xyz(11)=coef_xyz(11)+coef_xy(1,11)*pol_z(1,0,kg)
                coef_xyz(11)=coef_xyz(11)+coef_xy(2,11)*pol_z(2,0,kg)
                coef_xyz(12)=coef_xyz(12)+coef_xy(1,12)*pol_z(1,0,kg)
                coef_xyz(12)=coef_xyz(12)+coef_xy(2,12)*pol_z(2,0,kg)
                coef_xyz(13)=coef_xyz(13)+coef_xy(1,13)*pol_z(1,0,kg)
                coef_xyz(13)=coef_xyz(13)+coef_xy(2,13)*pol_z(2,0,kg)
                coef_xyz(14)=coef_xyz(14)+coef_xy(1,14)*pol_z(1,0,kg)
                coef_xyz(14)=coef_xyz(14)+coef_xy(2,14)*pol_z(2,0,kg)
                coef_xyz(15)=coef_xyz(15)+coef_xy(1,15)*pol_z(1,0,kg)
                coef_xyz(15)=coef_xyz(15)+coef_xy(2,15)*pol_z(2,0,kg)
                coef_xyz(16)=coef_xyz(16)+coef_xy(1,1)*pol_z(1,1,kg)
                coef_xyz(16)=coef_xyz(16)+coef_xy(2,1)*pol_z(2,1,kg)
                coef_xyz(17)=coef_xyz(17)+coef_xy(1,2)*pol_z(1,1,kg)
                coef_xyz(17)=coef_xyz(17)+coef_xy(2,2)*pol_z(2,1,kg)
                coef_xyz(18)=coef_xyz(18)+coef_xy(1,3)*pol_z(1,1,kg)
                coef_xyz(18)=coef_xyz(18)+coef_xy(2,3)*pol_z(2,1,kg)
                coef_xyz(19)=coef_xyz(19)+coef_xy(1,4)*pol_z(1,1,kg)
                coef_xyz(19)=coef_xyz(19)+coef_xy(2,4)*pol_z(2,1,kg)
                coef_xyz(20)=coef_xyz(20)+coef_xy(1,6)*pol_z(1,1,kg)
                coef_xyz(20)=coef_xyz(20)+coef_xy(2,6)*pol_z(2,1,kg)
                coef_xyz(21)=coef_xyz(21)+coef_xy(1,7)*pol_z(1,1,kg)
                coef_xyz(21)=coef_xyz(21)+coef_xy(2,7)*pol_z(2,1,kg)
                coef_xyz(22)=coef_xyz(22)+coef_xy(1,8)*pol_z(1,1,kg)
                coef_xyz(22)=coef_xyz(22)+coef_xy(2,8)*pol_z(2,1,kg)
                coef_xyz(23)=coef_xyz(23)+coef_xy(1,10)*pol_z(1,1,kg)
                coef_xyz(23)=coef_xyz(23)+coef_xy(2,10)*pol_z(2,1,kg)
                coef_xyz(24)=coef_xyz(24)+coef_xy(1,11)*pol_z(1,1,kg)
                coef_xyz(24)=coef_xyz(24)+coef_xy(2,11)*pol_z(2,1,kg)
                coef_xyz(25)=coef_xyz(25)+coef_xy(1,13)*pol_z(1,1,kg)
                coef_xyz(25)=coef_xyz(25)+coef_xy(2,13)*pol_z(2,1,kg)
                coef_xyz(26)=coef_xyz(26)+coef_xy(1,1)*pol_z(1,2,kg)
                coef_xyz(26)=coef_xyz(26)+coef_xy(2,1)*pol_z(2,2,kg)
                coef_xyz(27)=coef_xyz(27)+coef_xy(1,2)*pol_z(1,2,kg)
                coef_xyz(27)=coef_xyz(27)+coef_xy(2,2)*pol_z(2,2,kg)
                coef_xyz(28)=coef_xyz(28)+coef_xy(1,3)*pol_z(1,2,kg)
                coef_xyz(28)=coef_xyz(28)+coef_xy(2,3)*pol_z(2,2,kg)
                coef_xyz(29)=coef_xyz(29)+coef_xy(1,6)*pol_z(1,2,kg)
                coef_xyz(29)=coef_xyz(29)+coef_xy(2,6)*pol_z(2,2,kg)
                coef_xyz(30)=coef_xyz(30)+coef_xy(1,7)*pol_z(1,2,kg)
                coef_xyz(30)=coef_xyz(30)+coef_xy(2,7)*pol_z(2,2,kg)
                coef_xyz(31)=coef_xyz(31)+coef_xy(1,10)*pol_z(1,2,kg)
                coef_xyz(31)=coef_xyz(31)+coef_xy(2,10)*pol_z(2,2,kg)
                coef_xyz(32)=coef_xyz(32)+coef_xy(1,1)*pol_z(1,3,kg)
                coef_xyz(32)=coef_xyz(32)+coef_xy(2,1)*pol_z(2,3,kg)
                coef_xyz(33)=coef_xyz(33)+coef_xy(1,2)*pol_z(1,3,kg)
                coef_xyz(33)=coef_xyz(33)+coef_xy(2,2)*pol_z(2,3,kg)
                coef_xyz(34)=coef_xyz(34)+coef_xy(1,6)*pol_z(1,3,kg)
                coef_xyz(34)=coef_xyz(34)+coef_xy(2,6)*pol_z(2,3,kg)
                coef_xyz(35)=coef_xyz(35)+coef_xy(1,1)*pol_z(1,4,kg)
                coef_xyz(35)=coef_xyz(35)+coef_xy(2,1)*pol_z(2,4,kg)
    END DO

  END SUBROUTINE integrate_core_4
  SUBROUTINE integrate_core_5(grid,coef_xyz,pol_x,pol_y,pol_z,map,sphere_bounds,cmax,gridbounds)
    USE kinds,                        ONLY: dp
    INTEGER, INTENT(IN)                      :: sphere_bounds(*), cmax, &
                                                map(-cmax:cmax,1:3), &
                                                gridbounds(2,3)
    REAL(dp), INTENT(IN) :: grid(gridbounds(1,1):gridbounds(2,1), gridbounds(1&
      ,2):gridbounds(2,2), gridbounds(1,3):gridbounds(2,3))
    INTEGER, PARAMETER                       :: lp = 5
    REAL(dp), INTENT(IN)                     :: pol_x(0:lp,-cmax:cmax), &
                                                pol_y(1:2,0:lp,-cmax:0), &
                                                pol_z(1:2,0:lp,-cmax:0)
    REAL(dp), INTENT(OUT) :: coef_xyz(((lp+1)*(lp+2)*(lp+3))/6)

    INTEGER                                  :: i, ig, igmax, igmin, j, j2, &
                                                jg, jg2, jgmin, k, k2, kg, &
                                                kg2, kgmin, sci
    REAL(dp)                                 :: coef_x(4,0:lp), &
                                                coef_xy(2,((lp+1)*(lp+2))/2), &
                                                s01, s02, s03, s04

    sci=1

    coef_xyz=0.0_dp

    kgmin=sphere_bounds(sci)
    sci=sci+1
    DO kg=kgmin,0
       kg2=1-kg
       k=map(kg,3)
       k2=map(kg2,3)

       coef_xy=0.0_dp

       jgmin=sphere_bounds(sci)
       sci=sci+1
       DO jg=jgmin,0
          jg2=1-jg
          j=map(jg,2)
          j2=map(jg2,2)
          igmin=sphere_bounds(sci)
          sci=sci+1
          igmax=1-igmin

          coef_x=0.0_dp
          DO ig=igmin,igmax
             i=map(ig,1)
             s01=grid(i,j,k)
             s02=grid(i,j,k2)
             s03=grid(i,j2,k)
             s04=grid(i,j2,k2)
                coef_x(1,0)=coef_x(1,0)+s01*pol_x(0,ig)
                coef_x(2,0)=coef_x(2,0)+s02*pol_x(0,ig)
                coef_x(3,0)=coef_x(3,0)+s03*pol_x(0,ig)
                coef_x(4,0)=coef_x(4,0)+s04*pol_x(0,ig)
                coef_x(1,1)=coef_x(1,1)+s01*pol_x(1,ig)
                coef_x(2,1)=coef_x(2,1)+s02*pol_x(1,ig)
                coef_x(3,1)=coef_x(3,1)+s03*pol_x(1,ig)
                coef_x(4,1)=coef_x(4,1)+s04*pol_x(1,ig)
                coef_x(1,2)=coef_x(1,2)+s01*pol_x(2,ig)
                coef_x(2,2)=coef_x(2,2)+s02*pol_x(2,ig)
                coef_x(3,2)=coef_x(3,2)+s03*pol_x(2,ig)
                coef_x(4,2)=coef_x(4,2)+s04*pol_x(2,ig)
                coef_x(1,3)=coef_x(1,3)+s01*pol_x(3,ig)
                coef_x(2,3)=coef_x(2,3)+s02*pol_x(3,ig)
                coef_x(3,3)=coef_x(3,3)+s03*pol_x(3,ig)
                coef_x(4,3)=coef_x(4,3)+s04*pol_x(3,ig)
                coef_x(1,4)=coef_x(1,4)+s01*pol_x(4,ig)
                coef_x(2,4)=coef_x(2,4)+s02*pol_x(4,ig)
                coef_x(3,4)=coef_x(3,4)+s03*pol_x(4,ig)
                coef_x(4,4)=coef_x(4,4)+s04*pol_x(4,ig)
                coef_x(1,5)=coef_x(1,5)+s01*pol_x(5,ig)
                coef_x(2,5)=coef_x(2,5)+s02*pol_x(5,ig)
                coef_x(3,5)=coef_x(3,5)+s03*pol_x(5,ig)
                coef_x(4,5)=coef_x(4,5)+s04*pol_x(5,ig)
          END DO
             coef_xy(1,1)=coef_xy(1,1)+coef_x(1,0)*pol_y(1,0,jg)
             coef_xy(2,1)=coef_xy(2,1)+coef_x(2,0)*pol_y(1,0,jg)
             coef_xy(1,1)=coef_xy(1,1)+coef_x(3,0)*pol_y(2,0,jg)
             coef_xy(2,1)=coef_xy(2,1)+coef_x(4,0)*pol_y(2,0,jg)
             coef_xy(1,2)=coef_xy(1,2)+coef_x(1,1)*pol_y(1,0,jg)
             coef_xy(2,2)=coef_xy(2,2)+coef_x(2,1)*pol_y(1,0,jg)
             coef_xy(1,2)=coef_xy(1,2)+coef_x(3,1)*pol_y(2,0,jg)
             coef_xy(2,2)=coef_xy(2,2)+coef_x(4,1)*pol_y(2,0,jg)
             coef_xy(1,3)=coef_xy(1,3)+coef_x(1,2)*pol_y(1,0,jg)
             coef_xy(2,3)=coef_xy(2,3)+coef_x(2,2)*pol_y(1,0,jg)
             coef_xy(1,3)=coef_xy(1,3)+coef_x(3,2)*pol_y(2,0,jg)
             coef_xy(2,3)=coef_xy(2,3)+coef_x(4,2)*pol_y(2,0,jg)
             coef_xy(1,4)=coef_xy(1,4)+coef_x(1,3)*pol_y(1,0,jg)
             coef_xy(2,4)=coef_xy(2,4)+coef_x(2,3)*pol_y(1,0,jg)
             coef_xy(1,4)=coef_xy(1,4)+coef_x(3,3)*pol_y(2,0,jg)
             coef_xy(2,4)=coef_xy(2,4)+coef_x(4,3)*pol_y(2,0,jg)
             coef_xy(1,5)=coef_xy(1,5)+coef_x(1,4)*pol_y(1,0,jg)
             coef_xy(2,5)=coef_xy(2,5)+coef_x(2,4)*pol_y(1,0,jg)
             coef_xy(1,5)=coef_xy(1,5)+coef_x(3,4)*pol_y(2,0,jg)
             coef_xy(2,5)=coef_xy(2,5)+coef_x(4,4)*pol_y(2,0,jg)
             coef_xy(1,6)=coef_xy(1,6)+coef_x(1,5)*pol_y(1,0,jg)
             coef_xy(2,6)=coef_xy(2,6)+coef_x(2,5)*pol_y(1,0,jg)
             coef_xy(1,6)=coef_xy(1,6)+coef_x(3,5)*pol_y(2,0,jg)
             coef_xy(2,6)=coef_xy(2,6)+coef_x(4,5)*pol_y(2,0,jg)
             coef_xy(1,7)=coef_xy(1,7)+coef_x(1,0)*pol_y(1,1,jg)
             coef_xy(2,7)=coef_xy(2,7)+coef_x(2,0)*pol_y(1,1,jg)
             coef_xy(1,7)=coef_xy(1,7)+coef_x(3,0)*pol_y(2,1,jg)
             coef_xy(2,7)=coef_xy(2,7)+coef_x(4,0)*pol_y(2,1,jg)
             coef_xy(1,8)=coef_xy(1,8)+coef_x(1,1)*pol_y(1,1,jg)
             coef_xy(2,8)=coef_xy(2,8)+coef_x(2,1)*pol_y(1,1,jg)
             coef_xy(1,8)=coef_xy(1,8)+coef_x(3,1)*pol_y(2,1,jg)
             coef_xy(2,8)=coef_xy(2,8)+coef_x(4,1)*pol_y(2,1,jg)
             coef_xy(1,9)=coef_xy(1,9)+coef_x(1,2)*pol_y(1,1,jg)
             coef_xy(2,9)=coef_xy(2,9)+coef_x(2,2)*pol_y(1,1,jg)
             coef_xy(1,9)=coef_xy(1,9)+coef_x(3,2)*pol_y(2,1,jg)
             coef_xy(2,9)=coef_xy(2,9)+coef_x(4,2)*pol_y(2,1,jg)
             coef_xy(1,10)=coef_xy(1,10)+coef_x(1,3)*pol_y(1,1,jg)
             coef_xy(2,10)=coef_xy(2,10)+coef_x(2,3)*pol_y(1,1,jg)
             coef_xy(1,10)=coef_xy(1,10)+coef_x(3,3)*pol_y(2,1,jg)
             coef_xy(2,10)=coef_xy(2,10)+coef_x(4,3)*pol_y(2,1,jg)
             coef_xy(1,11)=coef_xy(1,11)+coef_x(1,4)*pol_y(1,1,jg)
             coef_xy(2,11)=coef_xy(2,11)+coef_x(2,4)*pol_y(1,1,jg)
             coef_xy(1,11)=coef_xy(1,11)+coef_x(3,4)*pol_y(2,1,jg)
             coef_xy(2,11)=coef_xy(2,11)+coef_x(4,4)*pol_y(2,1,jg)
             coef_xy(1,12)=coef_xy(1,12)+coef_x(1,0)*pol_y(1,2,jg)
             coef_xy(2,12)=coef_xy(2,12)+coef_x(2,0)*pol_y(1,2,jg)
             coef_xy(1,12)=coef_xy(1,12)+coef_x(3,0)*pol_y(2,2,jg)
             coef_xy(2,12)=coef_xy(2,12)+coef_x(4,0)*pol_y(2,2,jg)
             coef_xy(1,13)=coef_xy(1,13)+coef_x(1,1)*pol_y(1,2,jg)
             coef_xy(2,13)=coef_xy(2,13)+coef_x(2,1)*pol_y(1,2,jg)
             coef_xy(1,13)=coef_xy(1,13)+coef_x(3,1)*pol_y(2,2,jg)
             coef_xy(2,13)=coef_xy(2,13)+coef_x(4,1)*pol_y(2,2,jg)
             coef_xy(1,14)=coef_xy(1,14)+coef_x(1,2)*pol_y(1,2,jg)
             coef_xy(2,14)=coef_xy(2,14)+coef_x(2,2)*pol_y(1,2,jg)
             coef_xy(1,14)=coef_xy(1,14)+coef_x(3,2)*pol_y(2,2,jg)
             coef_xy(2,14)=coef_xy(2,14)+coef_x(4,2)*pol_y(2,2,jg)
             coef_xy(1,15)=coef_xy(1,15)+coef_x(1,3)*pol_y(1,2,jg)
             coef_xy(2,15)=coef_xy(2,15)+coef_x(2,3)*pol_y(1,2,jg)
             coef_xy(1,15)=coef_xy(1,15)+coef_x(3,3)*pol_y(2,2,jg)
             coef_xy(2,15)=coef_xy(2,15)+coef_x(4,3)*pol_y(2,2,jg)
             coef_xy(1,16)=coef_xy(1,16)+coef_x(1,0)*pol_y(1,3,jg)
             coef_xy(2,16)=coef_xy(2,16)+coef_x(2,0)*pol_y(1,3,jg)
             coef_xy(1,16)=coef_xy(1,16)+coef_x(3,0)*pol_y(2,3,jg)
             coef_xy(2,16)=coef_xy(2,16)+coef_x(4,0)*pol_y(2,3,jg)
             coef_xy(1,17)=coef_xy(1,17)+coef_x(1,1)*pol_y(1,3,jg)
             coef_xy(2,17)=coef_xy(2,17)+coef_x(2,1)*pol_y(1,3,jg)
             coef_xy(1,17)=coef_xy(1,17)+coef_x(3,1)*pol_y(2,3,jg)
             coef_xy(2,17)=coef_xy(2,17)+coef_x(4,1)*pol_y(2,3,jg)
             coef_xy(1,18)=coef_xy(1,18)+coef_x(1,2)*pol_y(1,3,jg)
             coef_xy(2,18)=coef_xy(2,18)+coef_x(2,2)*pol_y(1,3,jg)
             coef_xy(1,18)=coef_xy(1,18)+coef_x(3,2)*pol_y(2,3,jg)
             coef_xy(2,18)=coef_xy(2,18)+coef_x(4,2)*pol_y(2,3,jg)
             coef_xy(1,19)=coef_xy(1,19)+coef_x(1,0)*pol_y(1,4,jg)
             coef_xy(2,19)=coef_xy(2,19)+coef_x(2,0)*pol_y(1,4,jg)
             coef_xy(1,19)=coef_xy(1,19)+coef_x(3,0)*pol_y(2,4,jg)
             coef_xy(2,19)=coef_xy(2,19)+coef_x(4,0)*pol_y(2,4,jg)
             coef_xy(1,20)=coef_xy(1,20)+coef_x(1,1)*pol_y(1,4,jg)
             coef_xy(2,20)=coef_xy(2,20)+coef_x(2,1)*pol_y(1,4,jg)
             coef_xy(1,20)=coef_xy(1,20)+coef_x(3,1)*pol_y(2,4,jg)
             coef_xy(2,20)=coef_xy(2,20)+coef_x(4,1)*pol_y(2,4,jg)
             coef_xy(1,21)=coef_xy(1,21)+coef_x(1,0)*pol_y(1,5,jg)
             coef_xy(2,21)=coef_xy(2,21)+coef_x(2,0)*pol_y(1,5,jg)
             coef_xy(1,21)=coef_xy(1,21)+coef_x(3,0)*pol_y(2,5,jg)
             coef_xy(2,21)=coef_xy(2,21)+coef_x(4,0)*pol_y(2,5,jg)
       END DO
                coef_xyz(1)=coef_xyz(1)+SUM(coef_xy(:,1)*pol_z(:,0,kg))
                coef_xyz(2)=coef_xyz(2)+SUM(coef_xy(:,2)*pol_z(:,0,kg))
                coef_xyz(3)=coef_xyz(3)+SUM(coef_xy(:,3)*pol_z(:,0,kg))
                coef_xyz(4)=coef_xyz(4)+SUM(coef_xy(:,4)*pol_z(:,0,kg))
                coef_xyz(5)=coef_xyz(5)+SUM(coef_xy(:,5)*pol_z(:,0,kg))
                coef_xyz(6)=coef_xyz(6)+SUM(coef_xy(:,6)*pol_z(:,0,kg))
                coef_xyz(7)=coef_xyz(7)+SUM(coef_xy(:,7)*pol_z(:,0,kg))
                coef_xyz(8)=coef_xyz(8)+SUM(coef_xy(:,8)*pol_z(:,0,kg))
                coef_xyz(9)=coef_xyz(9)+SUM(coef_xy(:,9)*pol_z(:,0,kg))
                coef_xyz(10)=coef_xyz(10)+SUM(coef_xy(:,10)*pol_z(:,0,kg))
                coef_xyz(11)=coef_xyz(11)+SUM(coef_xy(:,11)*pol_z(:,0,kg))
                coef_xyz(12)=coef_xyz(12)+SUM(coef_xy(:,12)*pol_z(:,0,kg))
                coef_xyz(13)=coef_xyz(13)+SUM(coef_xy(:,13)*pol_z(:,0,kg))
                coef_xyz(14)=coef_xyz(14)+SUM(coef_xy(:,14)*pol_z(:,0,kg))
                coef_xyz(15)=coef_xyz(15)+SUM(coef_xy(:,15)*pol_z(:,0,kg))
                coef_xyz(16)=coef_xyz(16)+SUM(coef_xy(:,16)*pol_z(:,0,kg))
                coef_xyz(17)=coef_xyz(17)+SUM(coef_xy(:,17)*pol_z(:,0,kg))
                coef_xyz(18)=coef_xyz(18)+SUM(coef_xy(:,18)*pol_z(:,0,kg))
                coef_xyz(19)=coef_xyz(19)+SUM(coef_xy(:,19)*pol_z(:,0,kg))
                coef_xyz(20)=coef_xyz(20)+SUM(coef_xy(:,20)*pol_z(:,0,kg))
                coef_xyz(21)=coef_xyz(21)+SUM(coef_xy(:,21)*pol_z(:,0,kg))
                coef_xyz(22)=coef_xyz(22)+SUM(coef_xy(:,1)*pol_z(:,1,kg))
                coef_xyz(23)=coef_xyz(23)+SUM(coef_xy(:,2)*pol_z(:,1,kg))
                coef_xyz(24)=coef_xyz(24)+SUM(coef_xy(:,3)*pol_z(:,1,kg))
                coef_xyz(25)=coef_xyz(25)+SUM(coef_xy(:,4)*pol_z(:,1,kg))
                coef_xyz(26)=coef_xyz(26)+SUM(coef_xy(:,5)*pol_z(:,1,kg))
                coef_xyz(27)=coef_xyz(27)+SUM(coef_xy(:,7)*pol_z(:,1,kg))
                coef_xyz(28)=coef_xyz(28)+SUM(coef_xy(:,8)*pol_z(:,1,kg))
                coef_xyz(29)=coef_xyz(29)+SUM(coef_xy(:,9)*pol_z(:,1,kg))
                coef_xyz(30)=coef_xyz(30)+SUM(coef_xy(:,10)*pol_z(:,1,kg))
                coef_xyz(31)=coef_xyz(31)+SUM(coef_xy(:,12)*pol_z(:,1,kg))
                coef_xyz(32)=coef_xyz(32)+SUM(coef_xy(:,13)*pol_z(:,1,kg))
                coef_xyz(33)=coef_xyz(33)+SUM(coef_xy(:,14)*pol_z(:,1,kg))
                coef_xyz(34)=coef_xyz(34)+SUM(coef_xy(:,16)*pol_z(:,1,kg))
                coef_xyz(35)=coef_xyz(35)+SUM(coef_xy(:,17)*pol_z(:,1,kg))
                coef_xyz(36)=coef_xyz(36)+SUM(coef_xy(:,19)*pol_z(:,1,kg))
                coef_xyz(37)=coef_xyz(37)+SUM(coef_xy(:,1)*pol_z(:,2,kg))
                coef_xyz(38)=coef_xyz(38)+SUM(coef_xy(:,2)*pol_z(:,2,kg))
                coef_xyz(39)=coef_xyz(39)+SUM(coef_xy(:,3)*pol_z(:,2,kg))
                coef_xyz(40)=coef_xyz(40)+SUM(coef_xy(:,4)*pol_z(:,2,kg))
                coef_xyz(41)=coef_xyz(41)+SUM(coef_xy(:,7)*pol_z(:,2,kg))
                coef_xyz(42)=coef_xyz(42)+SUM(coef_xy(:,8)*pol_z(:,2,kg))
                coef_xyz(43)=coef_xyz(43)+SUM(coef_xy(:,9)*pol_z(:,2,kg))
                coef_xyz(44)=coef_xyz(44)+SUM(coef_xy(:,12)*pol_z(:,2,kg))
                coef_xyz(45)=coef_xyz(45)+SUM(coef_xy(:,13)*pol_z(:,2,kg))
                coef_xyz(46)=coef_xyz(46)+SUM(coef_xy(:,16)*pol_z(:,2,kg))
                coef_xyz(47)=coef_xyz(47)+SUM(coef_xy(:,1)*pol_z(:,3,kg))
                coef_xyz(48)=coef_xyz(48)+SUM(coef_xy(:,2)*pol_z(:,3,kg))
                coef_xyz(49)=coef_xyz(49)+SUM(coef_xy(:,3)*pol_z(:,3,kg))
                coef_xyz(50)=coef_xyz(50)+SUM(coef_xy(:,7)*pol_z(:,3,kg))
                coef_xyz(51)=coef_xyz(51)+SUM(coef_xy(:,8)*pol_z(:,3,kg))
                coef_xyz(52)=coef_xyz(52)+SUM(coef_xy(:,12)*pol_z(:,3,kg))
                coef_xyz(53)=coef_xyz(53)+SUM(coef_xy(:,1)*pol_z(:,4,kg))
                coef_xyz(54)=coef_xyz(54)+SUM(coef_xy(:,2)*pol_z(:,4,kg))
                coef_xyz(55)=coef_xyz(55)+SUM(coef_xy(:,7)*pol_z(:,4,kg))
                coef_xyz(56)=coef_xyz(56)+SUM(coef_xy(:,1)*pol_z(:,5,kg))
    END DO

  END SUBROUTINE integrate_core_5
  SUBROUTINE integrate_core_6(grid,coef_xyz,pol_x,pol_y,pol_z,map,sphere_bounds,cmax,gridbounds)
    USE kinds,                        ONLY: dp
    INTEGER, INTENT(IN)                      :: sphere_bounds(*), cmax, &
                                                map(-cmax:cmax,1:3), &
                                                gridbounds(2,3)
    REAL(dp), INTENT(IN) :: grid(gridbounds(1,1):gridbounds(2,1), gridbounds(1&
      ,2):gridbounds(2,2), gridbounds(1,3):gridbounds(2,3))
    INTEGER, PARAMETER                       :: lp = 6
    REAL(dp), INTENT(IN)                     :: pol_x(0:lp,-cmax:cmax), &
                                                pol_y(1:2,0:lp,-cmax:0), &
                                                pol_z(1:2,0:lp,-cmax:0)
    REAL(dp), INTENT(OUT) :: coef_xyz(((lp+1)*(lp+2)*(lp+3))/6)

    INTEGER                                  :: i, ig, igmax, igmin, j, j2, &
                                                jg, jg2, jgmin, k, k2, kg, &
                                                kg2, kgmin, sci
    REAL(dp)                                 :: coef_x(4,0:lp), &
                                                coef_xy(2,((lp+1)*(lp+2))/2), &
                                                s01, s02, s03, s04

    sci=1

    coef_xyz=0.0_dp

    kgmin=sphere_bounds(sci)
    sci=sci+1
    DO kg=kgmin,0
       kg2=1-kg
       k=map(kg,3)
       k2=map(kg2,3)

       coef_xy=0.0_dp

       jgmin=sphere_bounds(sci)
       sci=sci+1
       DO jg=jgmin,0
          jg2=1-jg
          j=map(jg,2)
          j2=map(jg2,2)
          igmin=sphere_bounds(sci)
          sci=sci+1
          igmax=1-igmin

          coef_x=0.0_dp
          DO ig=igmin,igmax
             i=map(ig,1)
             s01=grid(i,j,k)
             s02=grid(i,j,k2)
             s03=grid(i,j2,k)
             s04=grid(i,j2,k2)
                coef_x(1,0)=coef_x(1,0)+s01*pol_x(0,ig)
                coef_x(2,0)=coef_x(2,0)+s02*pol_x(0,ig)
                coef_x(3,0)=coef_x(3,0)+s03*pol_x(0,ig)
                coef_x(4,0)=coef_x(4,0)+s04*pol_x(0,ig)
                coef_x(1,1)=coef_x(1,1)+s01*pol_x(1,ig)
                coef_x(2,1)=coef_x(2,1)+s02*pol_x(1,ig)
                coef_x(3,1)=coef_x(3,1)+s03*pol_x(1,ig)
                coef_x(4,1)=coef_x(4,1)+s04*pol_x(1,ig)
                coef_x(1,2)=coef_x(1,2)+s01*pol_x(2,ig)
                coef_x(2,2)=coef_x(2,2)+s02*pol_x(2,ig)
                coef_x(3,2)=coef_x(3,2)+s03*pol_x(2,ig)
                coef_x(4,2)=coef_x(4,2)+s04*pol_x(2,ig)
                coef_x(1,3)=coef_x(1,3)+s01*pol_x(3,ig)
                coef_x(2,3)=coef_x(2,3)+s02*pol_x(3,ig)
                coef_x(3,3)=coef_x(3,3)+s03*pol_x(3,ig)
                coef_x(4,3)=coef_x(4,3)+s04*pol_x(3,ig)
                coef_x(1,4)=coef_x(1,4)+s01*pol_x(4,ig)
                coef_x(2,4)=coef_x(2,4)+s02*pol_x(4,ig)
                coef_x(3,4)=coef_x(3,4)+s03*pol_x(4,ig)
                coef_x(4,4)=coef_x(4,4)+s04*pol_x(4,ig)
                coef_x(1,5)=coef_x(1,5)+s01*pol_x(5,ig)
                coef_x(2,5)=coef_x(2,5)+s02*pol_x(5,ig)
                coef_x(3,5)=coef_x(3,5)+s03*pol_x(5,ig)
                coef_x(4,5)=coef_x(4,5)+s04*pol_x(5,ig)
                coef_x(1,6)=coef_x(1,6)+s01*pol_x(6,ig)
                coef_x(2,6)=coef_x(2,6)+s02*pol_x(6,ig)
                coef_x(3,6)=coef_x(3,6)+s03*pol_x(6,ig)
                coef_x(4,6)=coef_x(4,6)+s04*pol_x(6,ig)
          END DO
             coef_xy(1,1)=coef_xy(1,1)+coef_x(1,0)*pol_y(1,0,jg)
             coef_xy(2,1)=coef_xy(2,1)+coef_x(2,0)*pol_y(1,0,jg)
             coef_xy(1,1)=coef_xy(1,1)+coef_x(3,0)*pol_y(2,0,jg)
             coef_xy(2,1)=coef_xy(2,1)+coef_x(4,0)*pol_y(2,0,jg)
             coef_xy(1,2)=coef_xy(1,2)+coef_x(1,1)*pol_y(1,0,jg)
             coef_xy(2,2)=coef_xy(2,2)+coef_x(2,1)*pol_y(1,0,jg)
             coef_xy(1,2)=coef_xy(1,2)+coef_x(3,1)*pol_y(2,0,jg)
             coef_xy(2,2)=coef_xy(2,2)+coef_x(4,1)*pol_y(2,0,jg)
             coef_xy(1,3)=coef_xy(1,3)+coef_x(1,2)*pol_y(1,0,jg)
             coef_xy(2,3)=coef_xy(2,3)+coef_x(2,2)*pol_y(1,0,jg)
             coef_xy(1,3)=coef_xy(1,3)+coef_x(3,2)*pol_y(2,0,jg)
             coef_xy(2,3)=coef_xy(2,3)+coef_x(4,2)*pol_y(2,0,jg)
             coef_xy(1,4)=coef_xy(1,4)+coef_x(1,3)*pol_y(1,0,jg)
             coef_xy(2,4)=coef_xy(2,4)+coef_x(2,3)*pol_y(1,0,jg)
             coef_xy(1,4)=coef_xy(1,4)+coef_x(3,3)*pol_y(2,0,jg)
             coef_xy(2,4)=coef_xy(2,4)+coef_x(4,3)*pol_y(2,0,jg)
             coef_xy(1,5)=coef_xy(1,5)+coef_x(1,4)*pol_y(1,0,jg)
             coef_xy(2,5)=coef_xy(2,5)+coef_x(2,4)*pol_y(1,0,jg)
             coef_xy(1,5)=coef_xy(1,5)+coef_x(3,4)*pol_y(2,0,jg)
             coef_xy(2,5)=coef_xy(2,5)+coef_x(4,4)*pol_y(2,0,jg)
             coef_xy(1,6)=coef_xy(1,6)+coef_x(1,5)*pol_y(1,0,jg)
             coef_xy(2,6)=coef_xy(2,6)+coef_x(2,5)*pol_y(1,0,jg)
             coef_xy(1,6)=coef_xy(1,6)+coef_x(3,5)*pol_y(2,0,jg)
             coef_xy(2,6)=coef_xy(2,6)+coef_x(4,5)*pol_y(2,0,jg)
             coef_xy(1,7)=coef_xy(1,7)+coef_x(1,6)*pol_y(1,0,jg)
             coef_xy(2,7)=coef_xy(2,7)+coef_x(2,6)*pol_y(1,0,jg)
             coef_xy(1,7)=coef_xy(1,7)+coef_x(3,6)*pol_y(2,0,jg)
             coef_xy(2,7)=coef_xy(2,7)+coef_x(4,6)*pol_y(2,0,jg)
             coef_xy(1,8)=coef_xy(1,8)+coef_x(1,0)*pol_y(1,1,jg)
             coef_xy(2,8)=coef_xy(2,8)+coef_x(2,0)*pol_y(1,1,jg)
             coef_xy(1,8)=coef_xy(1,8)+coef_x(3,0)*pol_y(2,1,jg)
             coef_xy(2,8)=coef_xy(2,8)+coef_x(4,0)*pol_y(2,1,jg)
             coef_xy(1,9)=coef_xy(1,9)+coef_x(1,1)*pol_y(1,1,jg)
             coef_xy(2,9)=coef_xy(2,9)+coef_x(2,1)*pol_y(1,1,jg)
             coef_xy(1,9)=coef_xy(1,9)+coef_x(3,1)*pol_y(2,1,jg)
             coef_xy(2,9)=coef_xy(2,9)+coef_x(4,1)*pol_y(2,1,jg)
             coef_xy(1,10)=coef_xy(1,10)+coef_x(1,2)*pol_y(1,1,jg)
             coef_xy(2,10)=coef_xy(2,10)+coef_x(2,2)*pol_y(1,1,jg)
             coef_xy(1,10)=coef_xy(1,10)+coef_x(3,2)*pol_y(2,1,jg)
             coef_xy(2,10)=coef_xy(2,10)+coef_x(4,2)*pol_y(2,1,jg)
             coef_xy(1,11)=coef_xy(1,11)+coef_x(1,3)*pol_y(1,1,jg)
             coef_xy(2,11)=coef_xy(2,11)+coef_x(2,3)*pol_y(1,1,jg)
             coef_xy(1,11)=coef_xy(1,11)+coef_x(3,3)*pol_y(2,1,jg)
             coef_xy(2,11)=coef_xy(2,11)+coef_x(4,3)*pol_y(2,1,jg)
             coef_xy(1,12)=coef_xy(1,12)+coef_x(1,4)*pol_y(1,1,jg)
             coef_xy(2,12)=coef_xy(2,12)+coef_x(2,4)*pol_y(1,1,jg)
             coef_xy(1,12)=coef_xy(1,12)+coef_x(3,4)*pol_y(2,1,jg)
             coef_xy(2,12)=coef_xy(2,12)+coef_x(4,4)*pol_y(2,1,jg)
             coef_xy(1,13)=coef_xy(1,13)+coef_x(1,5)*pol_y(1,1,jg)
             coef_xy(2,13)=coef_xy(2,13)+coef_x(2,5)*pol_y(1,1,jg)
             coef_xy(1,13)=coef_xy(1,13)+coef_x(3,5)*pol_y(2,1,jg)
             coef_xy(2,13)=coef_xy(2,13)+coef_x(4,5)*pol_y(2,1,jg)
             coef_xy(1,14)=coef_xy(1,14)+coef_x(1,0)*pol_y(1,2,jg)
             coef_xy(2,14)=coef_xy(2,14)+coef_x(2,0)*pol_y(1,2,jg)
             coef_xy(1,14)=coef_xy(1,14)+coef_x(3,0)*pol_y(2,2,jg)
             coef_xy(2,14)=coef_xy(2,14)+coef_x(4,0)*pol_y(2,2,jg)
             coef_xy(1,15)=coef_xy(1,15)+coef_x(1,1)*pol_y(1,2,jg)
             coef_xy(2,15)=coef_xy(2,15)+coef_x(2,1)*pol_y(1,2,jg)
             coef_xy(1,15)=coef_xy(1,15)+coef_x(3,1)*pol_y(2,2,jg)
             coef_xy(2,15)=coef_xy(2,15)+coef_x(4,1)*pol_y(2,2,jg)
             coef_xy(1,16)=coef_xy(1,16)+coef_x(1,2)*pol_y(1,2,jg)
             coef_xy(2,16)=coef_xy(2,16)+coef_x(2,2)*pol_y(1,2,jg)
             coef_xy(1,16)=coef_xy(1,16)+coef_x(3,2)*pol_y(2,2,jg)
             coef_xy(2,16)=coef_xy(2,16)+coef_x(4,2)*pol_y(2,2,jg)
             coef_xy(1,17)=coef_xy(1,17)+coef_x(1,3)*pol_y(1,2,jg)
             coef_xy(2,17)=coef_xy(2,17)+coef_x(2,3)*pol_y(1,2,jg)
             coef_xy(1,17)=coef_xy(1,17)+coef_x(3,3)*pol_y(2,2,jg)
             coef_xy(2,17)=coef_xy(2,17)+coef_x(4,3)*pol_y(2,2,jg)
             coef_xy(1,18)=coef_xy(1,18)+coef_x(1,4)*pol_y(1,2,jg)
             coef_xy(2,18)=coef_xy(2,18)+coef_x(2,4)*pol_y(1,2,jg)
             coef_xy(1,18)=coef_xy(1,18)+coef_x(3,4)*pol_y(2,2,jg)
             coef_xy(2,18)=coef_xy(2,18)+coef_x(4,4)*pol_y(2,2,jg)
             coef_xy(1,19)=coef_xy(1,19)+coef_x(1,0)*pol_y(1,3,jg)
             coef_xy(2,19)=coef_xy(2,19)+coef_x(2,0)*pol_y(1,3,jg)
             coef_xy(1,19)=coef_xy(1,19)+coef_x(3,0)*pol_y(2,3,jg)
             coef_xy(2,19)=coef_xy(2,19)+coef_x(4,0)*pol_y(2,3,jg)
             coef_xy(1,20)=coef_xy(1,20)+coef_x(1,1)*pol_y(1,3,jg)
             coef_xy(2,20)=coef_xy(2,20)+coef_x(2,1)*pol_y(1,3,jg)
             coef_xy(1,20)=coef_xy(1,20)+coef_x(3,1)*pol_y(2,3,jg)
             coef_xy(2,20)=coef_xy(2,20)+coef_x(4,1)*pol_y(2,3,jg)
             coef_xy(1,21)=coef_xy(1,21)+coef_x(1,2)*pol_y(1,3,jg)
             coef_xy(2,21)=coef_xy(2,21)+coef_x(2,2)*pol_y(1,3,jg)
             coef_xy(1,21)=coef_xy(1,21)+coef_x(3,2)*pol_y(2,3,jg)
             coef_xy(2,21)=coef_xy(2,21)+coef_x(4,2)*pol_y(2,3,jg)
             coef_xy(1,22)=coef_xy(1,22)+coef_x(1,3)*pol_y(1,3,jg)
             coef_xy(2,22)=coef_xy(2,22)+coef_x(2,3)*pol_y(1,3,jg)
             coef_xy(1,22)=coef_xy(1,22)+coef_x(3,3)*pol_y(2,3,jg)
             coef_xy(2,22)=coef_xy(2,22)+coef_x(4,3)*pol_y(2,3,jg)
             coef_xy(1,23)=coef_xy(1,23)+coef_x(1,0)*pol_y(1,4,jg)
             coef_xy(2,23)=coef_xy(2,23)+coef_x(2,0)*pol_y(1,4,jg)
             coef_xy(1,23)=coef_xy(1,23)+coef_x(3,0)*pol_y(2,4,jg)
             coef_xy(2,23)=coef_xy(2,23)+coef_x(4,0)*pol_y(2,4,jg)
             coef_xy(1,24)=coef_xy(1,24)+coef_x(1,1)*pol_y(1,4,jg)
             coef_xy(2,24)=coef_xy(2,24)+coef_x(2,1)*pol_y(1,4,jg)
             coef_xy(1,24)=coef_xy(1,24)+coef_x(3,1)*pol_y(2,4,jg)
             coef_xy(2,24)=coef_xy(2,24)+coef_x(4,1)*pol_y(2,4,jg)
             coef_xy(1,25)=coef_xy(1,25)+coef_x(1,2)*pol_y(1,4,jg)
             coef_xy(2,25)=coef_xy(2,25)+coef_x(2,2)*pol_y(1,4,jg)
             coef_xy(1,25)=coef_xy(1,25)+coef_x(3,2)*pol_y(2,4,jg)
             coef_xy(2,25)=coef_xy(2,25)+coef_x(4,2)*pol_y(2,4,jg)
             coef_xy(1,26)=coef_xy(1,26)+coef_x(1,0)*pol_y(1,5,jg)
             coef_xy(2,26)=coef_xy(2,26)+coef_x(2,0)*pol_y(1,5,jg)
             coef_xy(1,26)=coef_xy(1,26)+coef_x(3,0)*pol_y(2,5,jg)
             coef_xy(2,26)=coef_xy(2,26)+coef_x(4,0)*pol_y(2,5,jg)
             coef_xy(1,27)=coef_xy(1,27)+coef_x(1,1)*pol_y(1,5,jg)
             coef_xy(2,27)=coef_xy(2,27)+coef_x(2,1)*pol_y(1,5,jg)
             coef_xy(1,27)=coef_xy(1,27)+coef_x(3,1)*pol_y(2,5,jg)
             coef_xy(2,27)=coef_xy(2,27)+coef_x(4,1)*pol_y(2,5,jg)
             coef_xy(1,28)=coef_xy(1,28)+coef_x(1,0)*pol_y(1,6,jg)
             coef_xy(2,28)=coef_xy(2,28)+coef_x(2,0)*pol_y(1,6,jg)
             coef_xy(1,28)=coef_xy(1,28)+coef_x(3,0)*pol_y(2,6,jg)
             coef_xy(2,28)=coef_xy(2,28)+coef_x(4,0)*pol_y(2,6,jg)
       END DO
                coef_xyz(1)=coef_xyz(1)+coef_xy(1,1)*pol_z(1,0,kg)
                coef_xyz(1)=coef_xyz(1)+coef_xy(2,1)*pol_z(2,0,kg)
                coef_xyz(2)=coef_xyz(2)+coef_xy(1,2)*pol_z(1,0,kg)
                coef_xyz(2)=coef_xyz(2)+coef_xy(2,2)*pol_z(2,0,kg)
                coef_xyz(3)=coef_xyz(3)+coef_xy(1,3)*pol_z(1,0,kg)
                coef_xyz(3)=coef_xyz(3)+coef_xy(2,3)*pol_z(2,0,kg)
                coef_xyz(4)=coef_xyz(4)+coef_xy(1,4)*pol_z(1,0,kg)
                coef_xyz(4)=coef_xyz(4)+coef_xy(2,4)*pol_z(2,0,kg)
                coef_xyz(5)=coef_xyz(5)+coef_xy(1,5)*pol_z(1,0,kg)
                coef_xyz(5)=coef_xyz(5)+coef_xy(2,5)*pol_z(2,0,kg)
                coef_xyz(6)=coef_xyz(6)+coef_xy(1,6)*pol_z(1,0,kg)
                coef_xyz(6)=coef_xyz(6)+coef_xy(2,6)*pol_z(2,0,kg)
                coef_xyz(7)=coef_xyz(7)+coef_xy(1,7)*pol_z(1,0,kg)
                coef_xyz(7)=coef_xyz(7)+coef_xy(2,7)*pol_z(2,0,kg)
                coef_xyz(8)=coef_xyz(8)+coef_xy(1,8)*pol_z(1,0,kg)
                coef_xyz(8)=coef_xyz(8)+coef_xy(2,8)*pol_z(2,0,kg)
                coef_xyz(9)=coef_xyz(9)+coef_xy(1,9)*pol_z(1,0,kg)
                coef_xyz(9)=coef_xyz(9)+coef_xy(2,9)*pol_z(2,0,kg)
                coef_xyz(10)=coef_xyz(10)+coef_xy(1,10)*pol_z(1,0,kg)
                coef_xyz(10)=coef_xyz(10)+coef_xy(2,10)*pol_z(2,0,kg)
                coef_xyz(11)=coef_xyz(11)+coef_xy(1,11)*pol_z(1,0,kg)
                coef_xyz(11)=coef_xyz(11)+coef_xy(2,11)*pol_z(2,0,kg)
                coef_xyz(12)=coef_xyz(12)+coef_xy(1,12)*pol_z(1,0,kg)
                coef_xyz(12)=coef_xyz(12)+coef_xy(2,12)*pol_z(2,0,kg)
                coef_xyz(13)=coef_xyz(13)+coef_xy(1,13)*pol_z(1,0,kg)
                coef_xyz(13)=coef_xyz(13)+coef_xy(2,13)*pol_z(2,0,kg)
                coef_xyz(14)=coef_xyz(14)+coef_xy(1,14)*pol_z(1,0,kg)
                coef_xyz(14)=coef_xyz(14)+coef_xy(2,14)*pol_z(2,0,kg)
                coef_xyz(15)=coef_xyz(15)+coef_xy(1,15)*pol_z(1,0,kg)
                coef_xyz(15)=coef_xyz(15)+coef_xy(2,15)*pol_z(2,0,kg)
                coef_xyz(16)=coef_xyz(16)+coef_xy(1,16)*pol_z(1,0,kg)
                coef_xyz(16)=coef_xyz(16)+coef_xy(2,16)*pol_z(2,0,kg)
                coef_xyz(17)=coef_xyz(17)+coef_xy(1,17)*pol_z(1,0,kg)
                coef_xyz(17)=coef_xyz(17)+coef_xy(2,17)*pol_z(2,0,kg)
                coef_xyz(18)=coef_xyz(18)+coef_xy(1,18)*pol_z(1,0,kg)
                coef_xyz(18)=coef_xyz(18)+coef_xy(2,18)*pol_z(2,0,kg)
                coef_xyz(19)=coef_xyz(19)+coef_xy(1,19)*pol_z(1,0,kg)
                coef_xyz(19)=coef_xyz(19)+coef_xy(2,19)*pol_z(2,0,kg)
                coef_xyz(20)=coef_xyz(20)+coef_xy(1,20)*pol_z(1,0,kg)
                coef_xyz(20)=coef_xyz(20)+coef_xy(2,20)*pol_z(2,0,kg)
                coef_xyz(21)=coef_xyz(21)+coef_xy(1,21)*pol_z(1,0,kg)
                coef_xyz(21)=coef_xyz(21)+coef_xy(2,21)*pol_z(2,0,kg)
                coef_xyz(22)=coef_xyz(22)+coef_xy(1,22)*pol_z(1,0,kg)
                coef_xyz(22)=coef_xyz(22)+coef_xy(2,22)*pol_z(2,0,kg)
                coef_xyz(23)=coef_xyz(23)+coef_xy(1,23)*pol_z(1,0,kg)
                coef_xyz(23)=coef_xyz(23)+coef_xy(2,23)*pol_z(2,0,kg)
                coef_xyz(24)=coef_xyz(24)+coef_xy(1,24)*pol_z(1,0,kg)
                coef_xyz(24)=coef_xyz(24)+coef_xy(2,24)*pol_z(2,0,kg)
                coef_xyz(25)=coef_xyz(25)+coef_xy(1,25)*pol_z(1,0,kg)
                coef_xyz(25)=coef_xyz(25)+coef_xy(2,25)*pol_z(2,0,kg)
                coef_xyz(26)=coef_xyz(26)+coef_xy(1,26)*pol_z(1,0,kg)
                coef_xyz(26)=coef_xyz(26)+coef_xy(2,26)*pol_z(2,0,kg)
                coef_xyz(27)=coef_xyz(27)+coef_xy(1,27)*pol_z(1,0,kg)
                coef_xyz(27)=coef_xyz(27)+coef_xy(2,27)*pol_z(2,0,kg)
                coef_xyz(28)=coef_xyz(28)+coef_xy(1,28)*pol_z(1,0,kg)
                coef_xyz(28)=coef_xyz(28)+coef_xy(2,28)*pol_z(2,0,kg)
                coef_xyz(29)=coef_xyz(29)+coef_xy(1,1)*pol_z(1,1,kg)
                coef_xyz(29)=coef_xyz(29)+coef_xy(2,1)*pol_z(2,1,kg)
                coef_xyz(30)=coef_xyz(30)+coef_xy(1,2)*pol_z(1,1,kg)
                coef_xyz(30)=coef_xyz(30)+coef_xy(2,2)*pol_z(2,1,kg)
                coef_xyz(31)=coef_xyz(31)+coef_xy(1,3)*pol_z(1,1,kg)
                coef_xyz(31)=coef_xyz(31)+coef_xy(2,3)*pol_z(2,1,kg)
                coef_xyz(32)=coef_xyz(32)+coef_xy(1,4)*pol_z(1,1,kg)
                coef_xyz(32)=coef_xyz(32)+coef_xy(2,4)*pol_z(2,1,kg)
                coef_xyz(33)=coef_xyz(33)+coef_xy(1,5)*pol_z(1,1,kg)
                coef_xyz(33)=coef_xyz(33)+coef_xy(2,5)*pol_z(2,1,kg)
                coef_xyz(34)=coef_xyz(34)+coef_xy(1,6)*pol_z(1,1,kg)
                coef_xyz(34)=coef_xyz(34)+coef_xy(2,6)*pol_z(2,1,kg)
                coef_xyz(35)=coef_xyz(35)+coef_xy(1,8)*pol_z(1,1,kg)
                coef_xyz(35)=coef_xyz(35)+coef_xy(2,8)*pol_z(2,1,kg)
                coef_xyz(36)=coef_xyz(36)+coef_xy(1,9)*pol_z(1,1,kg)
                coef_xyz(36)=coef_xyz(36)+coef_xy(2,9)*pol_z(2,1,kg)
                coef_xyz(37)=coef_xyz(37)+coef_xy(1,10)*pol_z(1,1,kg)
                coef_xyz(37)=coef_xyz(37)+coef_xy(2,10)*pol_z(2,1,kg)
                coef_xyz(38)=coef_xyz(38)+coef_xy(1,11)*pol_z(1,1,kg)
                coef_xyz(38)=coef_xyz(38)+coef_xy(2,11)*pol_z(2,1,kg)
                coef_xyz(39)=coef_xyz(39)+coef_xy(1,12)*pol_z(1,1,kg)
                coef_xyz(39)=coef_xyz(39)+coef_xy(2,12)*pol_z(2,1,kg)
                coef_xyz(40)=coef_xyz(40)+coef_xy(1,14)*pol_z(1,1,kg)
                coef_xyz(40)=coef_xyz(40)+coef_xy(2,14)*pol_z(2,1,kg)
                coef_xyz(41)=coef_xyz(41)+coef_xy(1,15)*pol_z(1,1,kg)
                coef_xyz(41)=coef_xyz(41)+coef_xy(2,15)*pol_z(2,1,kg)
                coef_xyz(42)=coef_xyz(42)+coef_xy(1,16)*pol_z(1,1,kg)
                coef_xyz(42)=coef_xyz(42)+coef_xy(2,16)*pol_z(2,1,kg)
                coef_xyz(43)=coef_xyz(43)+coef_xy(1,17)*pol_z(1,1,kg)
                coef_xyz(43)=coef_xyz(43)+coef_xy(2,17)*pol_z(2,1,kg)
                coef_xyz(44)=coef_xyz(44)+coef_xy(1,19)*pol_z(1,1,kg)
                coef_xyz(44)=coef_xyz(44)+coef_xy(2,19)*pol_z(2,1,kg)
                coef_xyz(45)=coef_xyz(45)+coef_xy(1,20)*pol_z(1,1,kg)
                coef_xyz(45)=coef_xyz(45)+coef_xy(2,20)*pol_z(2,1,kg)
                coef_xyz(46)=coef_xyz(46)+coef_xy(1,21)*pol_z(1,1,kg)
                coef_xyz(46)=coef_xyz(46)+coef_xy(2,21)*pol_z(2,1,kg)
                coef_xyz(47)=coef_xyz(47)+coef_xy(1,23)*pol_z(1,1,kg)
                coef_xyz(47)=coef_xyz(47)+coef_xy(2,23)*pol_z(2,1,kg)
                coef_xyz(48)=coef_xyz(48)+coef_xy(1,24)*pol_z(1,1,kg)
                coef_xyz(48)=coef_xyz(48)+coef_xy(2,24)*pol_z(2,1,kg)
                coef_xyz(49)=coef_xyz(49)+coef_xy(1,26)*pol_z(1,1,kg)
                coef_xyz(49)=coef_xyz(49)+coef_xy(2,26)*pol_z(2,1,kg)
                coef_xyz(50)=coef_xyz(50)+coef_xy(1,1)*pol_z(1,2,kg)
                coef_xyz(50)=coef_xyz(50)+coef_xy(2,1)*pol_z(2,2,kg)
                coef_xyz(51)=coef_xyz(51)+coef_xy(1,2)*pol_z(1,2,kg)
                coef_xyz(51)=coef_xyz(51)+coef_xy(2,2)*pol_z(2,2,kg)
                coef_xyz(52)=coef_xyz(52)+coef_xy(1,3)*pol_z(1,2,kg)
                coef_xyz(52)=coef_xyz(52)+coef_xy(2,3)*pol_z(2,2,kg)
                coef_xyz(53)=coef_xyz(53)+coef_xy(1,4)*pol_z(1,2,kg)
                coef_xyz(53)=coef_xyz(53)+coef_xy(2,4)*pol_z(2,2,kg)
                coef_xyz(54)=coef_xyz(54)+coef_xy(1,5)*pol_z(1,2,kg)
                coef_xyz(54)=coef_xyz(54)+coef_xy(2,5)*pol_z(2,2,kg)
                coef_xyz(55)=coef_xyz(55)+coef_xy(1,8)*pol_z(1,2,kg)
                coef_xyz(55)=coef_xyz(55)+coef_xy(2,8)*pol_z(2,2,kg)
                coef_xyz(56)=coef_xyz(56)+coef_xy(1,9)*pol_z(1,2,kg)
                coef_xyz(56)=coef_xyz(56)+coef_xy(2,9)*pol_z(2,2,kg)
                coef_xyz(57)=coef_xyz(57)+coef_xy(1,10)*pol_z(1,2,kg)
                coef_xyz(57)=coef_xyz(57)+coef_xy(2,10)*pol_z(2,2,kg)
                coef_xyz(58)=coef_xyz(58)+coef_xy(1,11)*pol_z(1,2,kg)
                coef_xyz(58)=coef_xyz(58)+coef_xy(2,11)*pol_z(2,2,kg)
                coef_xyz(59)=coef_xyz(59)+coef_xy(1,14)*pol_z(1,2,kg)
                coef_xyz(59)=coef_xyz(59)+coef_xy(2,14)*pol_z(2,2,kg)
                coef_xyz(60)=coef_xyz(60)+coef_xy(1,15)*pol_z(1,2,kg)
                coef_xyz(60)=coef_xyz(60)+coef_xy(2,15)*pol_z(2,2,kg)
                coef_xyz(61)=coef_xyz(61)+coef_xy(1,16)*pol_z(1,2,kg)
                coef_xyz(61)=coef_xyz(61)+coef_xy(2,16)*pol_z(2,2,kg)
                coef_xyz(62)=coef_xyz(62)+coef_xy(1,19)*pol_z(1,2,kg)
                coef_xyz(62)=coef_xyz(62)+coef_xy(2,19)*pol_z(2,2,kg)
                coef_xyz(63)=coef_xyz(63)+coef_xy(1,20)*pol_z(1,2,kg)
                coef_xyz(63)=coef_xyz(63)+coef_xy(2,20)*pol_z(2,2,kg)
                coef_xyz(64)=coef_xyz(64)+coef_xy(1,23)*pol_z(1,2,kg)
                coef_xyz(64)=coef_xyz(64)+coef_xy(2,23)*pol_z(2,2,kg)
                coef_xyz(65)=coef_xyz(65)+coef_xy(1,1)*pol_z(1,3,kg)
                coef_xyz(65)=coef_xyz(65)+coef_xy(2,1)*pol_z(2,3,kg)
                coef_xyz(66)=coef_xyz(66)+coef_xy(1,2)*pol_z(1,3,kg)
                coef_xyz(66)=coef_xyz(66)+coef_xy(2,2)*pol_z(2,3,kg)
                coef_xyz(67)=coef_xyz(67)+coef_xy(1,3)*pol_z(1,3,kg)
                coef_xyz(67)=coef_xyz(67)+coef_xy(2,3)*pol_z(2,3,kg)
                coef_xyz(68)=coef_xyz(68)+coef_xy(1,4)*pol_z(1,3,kg)
                coef_xyz(68)=coef_xyz(68)+coef_xy(2,4)*pol_z(2,3,kg)
                coef_xyz(69)=coef_xyz(69)+coef_xy(1,8)*pol_z(1,3,kg)
                coef_xyz(69)=coef_xyz(69)+coef_xy(2,8)*pol_z(2,3,kg)
                coef_xyz(70)=coef_xyz(70)+coef_xy(1,9)*pol_z(1,3,kg)
                coef_xyz(70)=coef_xyz(70)+coef_xy(2,9)*pol_z(2,3,kg)
                coef_xyz(71)=coef_xyz(71)+coef_xy(1,10)*pol_z(1,3,kg)
                coef_xyz(71)=coef_xyz(71)+coef_xy(2,10)*pol_z(2,3,kg)
                coef_xyz(72)=coef_xyz(72)+coef_xy(1,14)*pol_z(1,3,kg)
                coef_xyz(72)=coef_xyz(72)+coef_xy(2,14)*pol_z(2,3,kg)
                coef_xyz(73)=coef_xyz(73)+coef_xy(1,15)*pol_z(1,3,kg)
                coef_xyz(73)=coef_xyz(73)+coef_xy(2,15)*pol_z(2,3,kg)
                coef_xyz(74)=coef_xyz(74)+coef_xy(1,19)*pol_z(1,3,kg)
                coef_xyz(74)=coef_xyz(74)+coef_xy(2,19)*pol_z(2,3,kg)
                coef_xyz(75)=coef_xyz(75)+coef_xy(1,1)*pol_z(1,4,kg)
                coef_xyz(75)=coef_xyz(75)+coef_xy(2,1)*pol_z(2,4,kg)
                coef_xyz(76)=coef_xyz(76)+coef_xy(1,2)*pol_z(1,4,kg)
                coef_xyz(76)=coef_xyz(76)+coef_xy(2,2)*pol_z(2,4,kg)
                coef_xyz(77)=coef_xyz(77)+coef_xy(1,3)*pol_z(1,4,kg)
                coef_xyz(77)=coef_xyz(77)+coef_xy(2,3)*pol_z(2,4,kg)
                coef_xyz(78)=coef_xyz(78)+coef_xy(1,8)*pol_z(1,4,kg)
                coef_xyz(78)=coef_xyz(78)+coef_xy(2,8)*pol_z(2,4,kg)
                coef_xyz(79)=coef_xyz(79)+coef_xy(1,9)*pol_z(1,4,kg)
                coef_xyz(79)=coef_xyz(79)+coef_xy(2,9)*pol_z(2,4,kg)
                coef_xyz(80)=coef_xyz(80)+coef_xy(1,14)*pol_z(1,4,kg)
                coef_xyz(80)=coef_xyz(80)+coef_xy(2,14)*pol_z(2,4,kg)
                coef_xyz(81)=coef_xyz(81)+coef_xy(1,1)*pol_z(1,5,kg)
                coef_xyz(81)=coef_xyz(81)+coef_xy(2,1)*pol_z(2,5,kg)
                coef_xyz(82)=coef_xyz(82)+coef_xy(1,2)*pol_z(1,5,kg)
                coef_xyz(82)=coef_xyz(82)+coef_xy(2,2)*pol_z(2,5,kg)
                coef_xyz(83)=coef_xyz(83)+coef_xy(1,8)*pol_z(1,5,kg)
                coef_xyz(83)=coef_xyz(83)+coef_xy(2,8)*pol_z(2,5,kg)
                coef_xyz(84)=coef_xyz(84)+coef_xy(1,1)*pol_z(1,6,kg)
                coef_xyz(84)=coef_xyz(84)+coef_xy(2,1)*pol_z(2,6,kg)
    END DO

  END SUBROUTINE integrate_core_6
  SUBROUTINE integrate_core_7(grid,coef_xyz,pol_x,pol_y,pol_z,map,sphere_bounds,cmax,gridbounds)
    USE kinds,                        ONLY: dp
    INTEGER, INTENT(IN)                      :: sphere_bounds(*), cmax, &
                                                map(-cmax:cmax,1:3), &
                                                gridbounds(2,3)
    REAL(dp), INTENT(IN) :: grid(gridbounds(1,1):gridbounds(2,1), gridbounds(1&
      ,2):gridbounds(2,2), gridbounds(1,3):gridbounds(2,3))
    INTEGER, PARAMETER                       :: lp = 7
    REAL(dp), INTENT(IN)                     :: pol_x(0:lp,-cmax:cmax), &
                                                pol_y(1:2,0:lp,-cmax:0), &
                                                pol_z(1:2,0:lp,-cmax:0)
    REAL(dp), INTENT(OUT) :: coef_xyz(((lp+1)*(lp+2)*(lp+3))/6)

    INTEGER                                  :: i, ig, igmax, igmin, j, j2, &
                                                jg, jg2, jgmin, k, k2, kg, &
                                                kg2, kgmin, lxp, sci
    REAL(dp)                                 :: coef_x(4,0:lp), &
                                                coef_xy(2,((lp+1)*(lp+2))/2), &
                                                s01, s02, s03, s04

    sci=1

    coef_xyz=0.0_dp

    kgmin=sphere_bounds(sci)
    sci=sci+1
    DO kg=kgmin,0
       kg2=1-kg
       k=map(kg,3)
       k2=map(kg2,3)

       coef_xy=0.0_dp

       jgmin=sphere_bounds(sci)
       sci=sci+1
       DO jg=jgmin,0
          jg2=1-jg
          j=map(jg,2)
          j2=map(jg2,2)
          igmin=sphere_bounds(sci)
          sci=sci+1
          igmax=1-igmin

          coef_x=0.0_dp
          DO ig=igmin,igmax
             i=map(ig,1)
             s01=grid(i,j,k)
             s02=grid(i,j,k2)
             s03=grid(i,j2,k)
             s04=grid(i,j2,k2)
             DO lxp=0,lp
                coef_x(1,lxp)=coef_x(1,lxp)+s01*pol_x(lxp,ig)
                coef_x(2,lxp)=coef_x(2,lxp)+s02*pol_x(lxp,ig)
                coef_x(3,lxp)=coef_x(3,lxp)+s03*pol_x(lxp,ig)
                coef_x(4,lxp)=coef_x(4,lxp)+s04*pol_x(lxp,ig)
             ENDDO
          END DO
             coef_xy(1,1)=coef_xy(1,1)+coef_x(1,0)*pol_y(1,0,jg)
             coef_xy(2,1)=coef_xy(2,1)+coef_x(2,0)*pol_y(1,0,jg)
             coef_xy(1,1)=coef_xy(1,1)+coef_x(3,0)*pol_y(2,0,jg)
             coef_xy(2,1)=coef_xy(2,1)+coef_x(4,0)*pol_y(2,0,jg)
             coef_xy(1,2)=coef_xy(1,2)+coef_x(1,1)*pol_y(1,0,jg)
             coef_xy(2,2)=coef_xy(2,2)+coef_x(2,1)*pol_y(1,0,jg)
             coef_xy(1,2)=coef_xy(1,2)+coef_x(3,1)*pol_y(2,0,jg)
             coef_xy(2,2)=coef_xy(2,2)+coef_x(4,1)*pol_y(2,0,jg)
             coef_xy(1,3)=coef_xy(1,3)+coef_x(1,2)*pol_y(1,0,jg)
             coef_xy(2,3)=coef_xy(2,3)+coef_x(2,2)*pol_y(1,0,jg)
             coef_xy(1,3)=coef_xy(1,3)+coef_x(3,2)*pol_y(2,0,jg)
             coef_xy(2,3)=coef_xy(2,3)+coef_x(4,2)*pol_y(2,0,jg)
             coef_xy(1,4)=coef_xy(1,4)+coef_x(1,3)*pol_y(1,0,jg)
             coef_xy(2,4)=coef_xy(2,4)+coef_x(2,3)*pol_y(1,0,jg)
             coef_xy(1,4)=coef_xy(1,4)+coef_x(3,3)*pol_y(2,0,jg)
             coef_xy(2,4)=coef_xy(2,4)+coef_x(4,3)*pol_y(2,0,jg)
             coef_xy(1,5)=coef_xy(1,5)+coef_x(1,4)*pol_y(1,0,jg)
             coef_xy(2,5)=coef_xy(2,5)+coef_x(2,4)*pol_y(1,0,jg)
             coef_xy(1,5)=coef_xy(1,5)+coef_x(3,4)*pol_y(2,0,jg)
             coef_xy(2,5)=coef_xy(2,5)+coef_x(4,4)*pol_y(2,0,jg)
             coef_xy(1,6)=coef_xy(1,6)+coef_x(1,5)*pol_y(1,0,jg)
             coef_xy(2,6)=coef_xy(2,6)+coef_x(2,5)*pol_y(1,0,jg)
             coef_xy(1,6)=coef_xy(1,6)+coef_x(3,5)*pol_y(2,0,jg)
             coef_xy(2,6)=coef_xy(2,6)+coef_x(4,5)*pol_y(2,0,jg)
             coef_xy(1,7)=coef_xy(1,7)+coef_x(1,6)*pol_y(1,0,jg)
             coef_xy(2,7)=coef_xy(2,7)+coef_x(2,6)*pol_y(1,0,jg)
             coef_xy(1,7)=coef_xy(1,7)+coef_x(3,6)*pol_y(2,0,jg)
             coef_xy(2,7)=coef_xy(2,7)+coef_x(4,6)*pol_y(2,0,jg)
             coef_xy(1,8)=coef_xy(1,8)+coef_x(1,7)*pol_y(1,0,jg)
             coef_xy(2,8)=coef_xy(2,8)+coef_x(2,7)*pol_y(1,0,jg)
             coef_xy(1,8)=coef_xy(1,8)+coef_x(3,7)*pol_y(2,0,jg)
             coef_xy(2,8)=coef_xy(2,8)+coef_x(4,7)*pol_y(2,0,jg)
             coef_xy(1,9)=coef_xy(1,9)+coef_x(1,0)*pol_y(1,1,jg)
             coef_xy(2,9)=coef_xy(2,9)+coef_x(2,0)*pol_y(1,1,jg)
             coef_xy(1,9)=coef_xy(1,9)+coef_x(3,0)*pol_y(2,1,jg)
             coef_xy(2,9)=coef_xy(2,9)+coef_x(4,0)*pol_y(2,1,jg)
             coef_xy(1,10)=coef_xy(1,10)+coef_x(1,1)*pol_y(1,1,jg)
             coef_xy(2,10)=coef_xy(2,10)+coef_x(2,1)*pol_y(1,1,jg)
             coef_xy(1,10)=coef_xy(1,10)+coef_x(3,1)*pol_y(2,1,jg)
             coef_xy(2,10)=coef_xy(2,10)+coef_x(4,1)*pol_y(2,1,jg)
             coef_xy(1,11)=coef_xy(1,11)+coef_x(1,2)*pol_y(1,1,jg)
             coef_xy(2,11)=coef_xy(2,11)+coef_x(2,2)*pol_y(1,1,jg)
             coef_xy(1,11)=coef_xy(1,11)+coef_x(3,2)*pol_y(2,1,jg)
             coef_xy(2,11)=coef_xy(2,11)+coef_x(4,2)*pol_y(2,1,jg)
             coef_xy(1,12)=coef_xy(1,12)+coef_x(1,3)*pol_y(1,1,jg)
             coef_xy(2,12)=coef_xy(2,12)+coef_x(2,3)*pol_y(1,1,jg)
             coef_xy(1,12)=coef_xy(1,12)+coef_x(3,3)*pol_y(2,1,jg)
             coef_xy(2,12)=coef_xy(2,12)+coef_x(4,3)*pol_y(2,1,jg)
             coef_xy(1,13)=coef_xy(1,13)+coef_x(1,4)*pol_y(1,1,jg)
             coef_xy(2,13)=coef_xy(2,13)+coef_x(2,4)*pol_y(1,1,jg)
             coef_xy(1,13)=coef_xy(1,13)+coef_x(3,4)*pol_y(2,1,jg)
             coef_xy(2,13)=coef_xy(2,13)+coef_x(4,4)*pol_y(2,1,jg)
             coef_xy(1,14)=coef_xy(1,14)+coef_x(1,5)*pol_y(1,1,jg)
             coef_xy(2,14)=coef_xy(2,14)+coef_x(2,5)*pol_y(1,1,jg)
             coef_xy(1,14)=coef_xy(1,14)+coef_x(3,5)*pol_y(2,1,jg)
             coef_xy(2,14)=coef_xy(2,14)+coef_x(4,5)*pol_y(2,1,jg)
             coef_xy(1,15)=coef_xy(1,15)+coef_x(1,6)*pol_y(1,1,jg)
             coef_xy(2,15)=coef_xy(2,15)+coef_x(2,6)*pol_y(1,1,jg)
             coef_xy(1,15)=coef_xy(1,15)+coef_x(3,6)*pol_y(2,1,jg)
             coef_xy(2,15)=coef_xy(2,15)+coef_x(4,6)*pol_y(2,1,jg)
             coef_xy(1,16)=coef_xy(1,16)+coef_x(1,0)*pol_y(1,2,jg)
             coef_xy(2,16)=coef_xy(2,16)+coef_x(2,0)*pol_y(1,2,jg)
             coef_xy(1,16)=coef_xy(1,16)+coef_x(3,0)*pol_y(2,2,jg)
             coef_xy(2,16)=coef_xy(2,16)+coef_x(4,0)*pol_y(2,2,jg)
             coef_xy(1,17)=coef_xy(1,17)+coef_x(1,1)*pol_y(1,2,jg)
             coef_xy(2,17)=coef_xy(2,17)+coef_x(2,1)*pol_y(1,2,jg)
             coef_xy(1,17)=coef_xy(1,17)+coef_x(3,1)*pol_y(2,2,jg)
             coef_xy(2,17)=coef_xy(2,17)+coef_x(4,1)*pol_y(2,2,jg)
             coef_xy(1,18)=coef_xy(1,18)+coef_x(1,2)*pol_y(1,2,jg)
             coef_xy(2,18)=coef_xy(2,18)+coef_x(2,2)*pol_y(1,2,jg)
             coef_xy(1,18)=coef_xy(1,18)+coef_x(3,2)*pol_y(2,2,jg)
             coef_xy(2,18)=coef_xy(2,18)+coef_x(4,2)*pol_y(2,2,jg)
             coef_xy(1,19)=coef_xy(1,19)+coef_x(1,3)*pol_y(1,2,jg)
             coef_xy(2,19)=coef_xy(2,19)+coef_x(2,3)*pol_y(1,2,jg)
             coef_xy(1,19)=coef_xy(1,19)+coef_x(3,3)*pol_y(2,2,jg)
             coef_xy(2,19)=coef_xy(2,19)+coef_x(4,3)*pol_y(2,2,jg)
             coef_xy(1,20)=coef_xy(1,20)+coef_x(1,4)*pol_y(1,2,jg)
             coef_xy(2,20)=coef_xy(2,20)+coef_x(2,4)*pol_y(1,2,jg)
             coef_xy(1,20)=coef_xy(1,20)+coef_x(3,4)*pol_y(2,2,jg)
             coef_xy(2,20)=coef_xy(2,20)+coef_x(4,4)*pol_y(2,2,jg)
             coef_xy(1,21)=coef_xy(1,21)+coef_x(1,5)*pol_y(1,2,jg)
             coef_xy(2,21)=coef_xy(2,21)+coef_x(2,5)*pol_y(1,2,jg)
             coef_xy(1,21)=coef_xy(1,21)+coef_x(3,5)*pol_y(2,2,jg)
             coef_xy(2,21)=coef_xy(2,21)+coef_x(4,5)*pol_y(2,2,jg)
             coef_xy(1,22)=coef_xy(1,22)+coef_x(1,0)*pol_y(1,3,jg)
             coef_xy(2,22)=coef_xy(2,22)+coef_x(2,0)*pol_y(1,3,jg)
             coef_xy(1,22)=coef_xy(1,22)+coef_x(3,0)*pol_y(2,3,jg)
             coef_xy(2,22)=coef_xy(2,22)+coef_x(4,0)*pol_y(2,3,jg)
             coef_xy(1,23)=coef_xy(1,23)+coef_x(1,1)*pol_y(1,3,jg)
             coef_xy(2,23)=coef_xy(2,23)+coef_x(2,1)*pol_y(1,3,jg)
             coef_xy(1,23)=coef_xy(1,23)+coef_x(3,1)*pol_y(2,3,jg)
             coef_xy(2,23)=coef_xy(2,23)+coef_x(4,1)*pol_y(2,3,jg)
             coef_xy(1,24)=coef_xy(1,24)+coef_x(1,2)*pol_y(1,3,jg)
             coef_xy(2,24)=coef_xy(2,24)+coef_x(2,2)*pol_y(1,3,jg)
             coef_xy(1,24)=coef_xy(1,24)+coef_x(3,2)*pol_y(2,3,jg)
             coef_xy(2,24)=coef_xy(2,24)+coef_x(4,2)*pol_y(2,3,jg)
             coef_xy(1,25)=coef_xy(1,25)+coef_x(1,3)*pol_y(1,3,jg)
             coef_xy(2,25)=coef_xy(2,25)+coef_x(2,3)*pol_y(1,3,jg)
             coef_xy(1,25)=coef_xy(1,25)+coef_x(3,3)*pol_y(2,3,jg)
             coef_xy(2,25)=coef_xy(2,25)+coef_x(4,3)*pol_y(2,3,jg)
             coef_xy(1,26)=coef_xy(1,26)+coef_x(1,4)*pol_y(1,3,jg)
             coef_xy(2,26)=coef_xy(2,26)+coef_x(2,4)*pol_y(1,3,jg)
             coef_xy(1,26)=coef_xy(1,26)+coef_x(3,4)*pol_y(2,3,jg)
             coef_xy(2,26)=coef_xy(2,26)+coef_x(4,4)*pol_y(2,3,jg)
             coef_xy(1,27)=coef_xy(1,27)+coef_x(1,0)*pol_y(1,4,jg)
             coef_xy(2,27)=coef_xy(2,27)+coef_x(2,0)*pol_y(1,4,jg)
             coef_xy(1,27)=coef_xy(1,27)+coef_x(3,0)*pol_y(2,4,jg)
             coef_xy(2,27)=coef_xy(2,27)+coef_x(4,0)*pol_y(2,4,jg)
             coef_xy(1,28)=coef_xy(1,28)+coef_x(1,1)*pol_y(1,4,jg)
             coef_xy(2,28)=coef_xy(2,28)+coef_x(2,1)*pol_y(1,4,jg)
             coef_xy(1,28)=coef_xy(1,28)+coef_x(3,1)*pol_y(2,4,jg)
             coef_xy(2,28)=coef_xy(2,28)+coef_x(4,1)*pol_y(2,4,jg)
             coef_xy(1,29)=coef_xy(1,29)+coef_x(1,2)*pol_y(1,4,jg)
             coef_xy(2,29)=coef_xy(2,29)+coef_x(2,2)*pol_y(1,4,jg)
             coef_xy(1,29)=coef_xy(1,29)+coef_x(3,2)*pol_y(2,4,jg)
             coef_xy(2,29)=coef_xy(2,29)+coef_x(4,2)*pol_y(2,4,jg)
             coef_xy(1,30)=coef_xy(1,30)+coef_x(1,3)*pol_y(1,4,jg)
             coef_xy(2,30)=coef_xy(2,30)+coef_x(2,3)*pol_y(1,4,jg)
             coef_xy(1,30)=coef_xy(1,30)+coef_x(3,3)*pol_y(2,4,jg)
             coef_xy(2,30)=coef_xy(2,30)+coef_x(4,3)*pol_y(2,4,jg)
             coef_xy(1,31)=coef_xy(1,31)+coef_x(1,0)*pol_y(1,5,jg)
             coef_xy(2,31)=coef_xy(2,31)+coef_x(2,0)*pol_y(1,5,jg)
             coef_xy(1,31)=coef_xy(1,31)+coef_x(3,0)*pol_y(2,5,jg)
             coef_xy(2,31)=coef_xy(2,31)+coef_x(4,0)*pol_y(2,5,jg)
             coef_xy(1,32)=coef_xy(1,32)+coef_x(1,1)*pol_y(1,5,jg)
             coef_xy(2,32)=coef_xy(2,32)+coef_x(2,1)*pol_y(1,5,jg)
             coef_xy(1,32)=coef_xy(1,32)+coef_x(3,1)*pol_y(2,5,jg)
             coef_xy(2,32)=coef_xy(2,32)+coef_x(4,1)*pol_y(2,5,jg)
             coef_xy(1,33)=coef_xy(1,33)+coef_x(1,2)*pol_y(1,5,jg)
             coef_xy(2,33)=coef_xy(2,33)+coef_x(2,2)*pol_y(1,5,jg)
             coef_xy(1,33)=coef_xy(1,33)+coef_x(3,2)*pol_y(2,5,jg)
             coef_xy(2,33)=coef_xy(2,33)+coef_x(4,2)*pol_y(2,5,jg)
             coef_xy(1,34)=coef_xy(1,34)+coef_x(1,0)*pol_y(1,6,jg)
             coef_xy(2,34)=coef_xy(2,34)+coef_x(2,0)*pol_y(1,6,jg)
             coef_xy(1,34)=coef_xy(1,34)+coef_x(3,0)*pol_y(2,6,jg)
             coef_xy(2,34)=coef_xy(2,34)+coef_x(4,0)*pol_y(2,6,jg)
             coef_xy(1,35)=coef_xy(1,35)+coef_x(1,1)*pol_y(1,6,jg)
             coef_xy(2,35)=coef_xy(2,35)+coef_x(2,1)*pol_y(1,6,jg)
             coef_xy(1,35)=coef_xy(1,35)+coef_x(3,1)*pol_y(2,6,jg)
             coef_xy(2,35)=coef_xy(2,35)+coef_x(4,1)*pol_y(2,6,jg)
             coef_xy(1,36)=coef_xy(1,36)+coef_x(1,0)*pol_y(1,7,jg)
             coef_xy(2,36)=coef_xy(2,36)+coef_x(2,0)*pol_y(1,7,jg)
             coef_xy(1,36)=coef_xy(1,36)+coef_x(3,0)*pol_y(2,7,jg)
             coef_xy(2,36)=coef_xy(2,36)+coef_x(4,0)*pol_y(2,7,jg)
       END DO
                coef_xyz(1)=coef_xyz(1)+coef_xy(1,1)*pol_z(1,0,kg)
                coef_xyz(1)=coef_xyz(1)+coef_xy(2,1)*pol_z(2,0,kg)
                coef_xyz(2)=coef_xyz(2)+coef_xy(1,2)*pol_z(1,0,kg)
                coef_xyz(2)=coef_xyz(2)+coef_xy(2,2)*pol_z(2,0,kg)
                coef_xyz(3)=coef_xyz(3)+coef_xy(1,3)*pol_z(1,0,kg)
                coef_xyz(3)=coef_xyz(3)+coef_xy(2,3)*pol_z(2,0,kg)
                coef_xyz(4)=coef_xyz(4)+coef_xy(1,4)*pol_z(1,0,kg)
                coef_xyz(4)=coef_xyz(4)+coef_xy(2,4)*pol_z(2,0,kg)
                coef_xyz(5)=coef_xyz(5)+coef_xy(1,5)*pol_z(1,0,kg)
                coef_xyz(5)=coef_xyz(5)+coef_xy(2,5)*pol_z(2,0,kg)
                coef_xyz(6)=coef_xyz(6)+coef_xy(1,6)*pol_z(1,0,kg)
                coef_xyz(6)=coef_xyz(6)+coef_xy(2,6)*pol_z(2,0,kg)
                coef_xyz(7)=coef_xyz(7)+coef_xy(1,7)*pol_z(1,0,kg)
                coef_xyz(7)=coef_xyz(7)+coef_xy(2,7)*pol_z(2,0,kg)
                coef_xyz(8)=coef_xyz(8)+coef_xy(1,8)*pol_z(1,0,kg)
                coef_xyz(8)=coef_xyz(8)+coef_xy(2,8)*pol_z(2,0,kg)
                coef_xyz(9)=coef_xyz(9)+coef_xy(1,9)*pol_z(1,0,kg)
                coef_xyz(9)=coef_xyz(9)+coef_xy(2,9)*pol_z(2,0,kg)
                coef_xyz(10)=coef_xyz(10)+coef_xy(1,10)*pol_z(1,0,kg)
                coef_xyz(10)=coef_xyz(10)+coef_xy(2,10)*pol_z(2,0,kg)
                coef_xyz(11)=coef_xyz(11)+coef_xy(1,11)*pol_z(1,0,kg)
                coef_xyz(11)=coef_xyz(11)+coef_xy(2,11)*pol_z(2,0,kg)
                coef_xyz(12)=coef_xyz(12)+coef_xy(1,12)*pol_z(1,0,kg)
                coef_xyz(12)=coef_xyz(12)+coef_xy(2,12)*pol_z(2,0,kg)
                coef_xyz(13)=coef_xyz(13)+coef_xy(1,13)*pol_z(1,0,kg)
                coef_xyz(13)=coef_xyz(13)+coef_xy(2,13)*pol_z(2,0,kg)
                coef_xyz(14)=coef_xyz(14)+coef_xy(1,14)*pol_z(1,0,kg)
                coef_xyz(14)=coef_xyz(14)+coef_xy(2,14)*pol_z(2,0,kg)
                coef_xyz(15)=coef_xyz(15)+coef_xy(1,15)*pol_z(1,0,kg)
                coef_xyz(15)=coef_xyz(15)+coef_xy(2,15)*pol_z(2,0,kg)
                coef_xyz(16)=coef_xyz(16)+coef_xy(1,16)*pol_z(1,0,kg)
                coef_xyz(16)=coef_xyz(16)+coef_xy(2,16)*pol_z(2,0,kg)
                coef_xyz(17)=coef_xyz(17)+coef_xy(1,17)*pol_z(1,0,kg)
                coef_xyz(17)=coef_xyz(17)+coef_xy(2,17)*pol_z(2,0,kg)
                coef_xyz(18)=coef_xyz(18)+coef_xy(1,18)*pol_z(1,0,kg)
                coef_xyz(18)=coef_xyz(18)+coef_xy(2,18)*pol_z(2,0,kg)
                coef_xyz(19)=coef_xyz(19)+coef_xy(1,19)*pol_z(1,0,kg)
                coef_xyz(19)=coef_xyz(19)+coef_xy(2,19)*pol_z(2,0,kg)
                coef_xyz(20)=coef_xyz(20)+coef_xy(1,20)*pol_z(1,0,kg)
                coef_xyz(20)=coef_xyz(20)+coef_xy(2,20)*pol_z(2,0,kg)
                coef_xyz(21)=coef_xyz(21)+coef_xy(1,21)*pol_z(1,0,kg)
                coef_xyz(21)=coef_xyz(21)+coef_xy(2,21)*pol_z(2,0,kg)
                coef_xyz(22)=coef_xyz(22)+coef_xy(1,22)*pol_z(1,0,kg)
                coef_xyz(22)=coef_xyz(22)+coef_xy(2,22)*pol_z(2,0,kg)
                coef_xyz(23)=coef_xyz(23)+coef_xy(1,23)*pol_z(1,0,kg)
                coef_xyz(23)=coef_xyz(23)+coef_xy(2,23)*pol_z(2,0,kg)
                coef_xyz(24)=coef_xyz(24)+coef_xy(1,24)*pol_z(1,0,kg)
                coef_xyz(24)=coef_xyz(24)+coef_xy(2,24)*pol_z(2,0,kg)
                coef_xyz(25)=coef_xyz(25)+coef_xy(1,25)*pol_z(1,0,kg)
                coef_xyz(25)=coef_xyz(25)+coef_xy(2,25)*pol_z(2,0,kg)
                coef_xyz(26)=coef_xyz(26)+coef_xy(1,26)*pol_z(1,0,kg)
                coef_xyz(26)=coef_xyz(26)+coef_xy(2,26)*pol_z(2,0,kg)
                coef_xyz(27)=coef_xyz(27)+coef_xy(1,27)*pol_z(1,0,kg)
                coef_xyz(27)=coef_xyz(27)+coef_xy(2,27)*pol_z(2,0,kg)
                coef_xyz(28)=coef_xyz(28)+coef_xy(1,28)*pol_z(1,0,kg)
                coef_xyz(28)=coef_xyz(28)+coef_xy(2,28)*pol_z(2,0,kg)
                coef_xyz(29)=coef_xyz(29)+coef_xy(1,29)*pol_z(1,0,kg)
                coef_xyz(29)=coef_xyz(29)+coef_xy(2,29)*pol_z(2,0,kg)
                coef_xyz(30)=coef_xyz(30)+coef_xy(1,30)*pol_z(1,0,kg)
                coef_xyz(30)=coef_xyz(30)+coef_xy(2,30)*pol_z(2,0,kg)
                coef_xyz(31)=coef_xyz(31)+coef_xy(1,31)*pol_z(1,0,kg)
                coef_xyz(31)=coef_xyz(31)+coef_xy(2,31)*pol_z(2,0,kg)
                coef_xyz(32)=coef_xyz(32)+coef_xy(1,32)*pol_z(1,0,kg)
                coef_xyz(32)=coef_xyz(32)+coef_xy(2,32)*pol_z(2,0,kg)
                coef_xyz(33)=coef_xyz(33)+coef_xy(1,33)*pol_z(1,0,kg)
                coef_xyz(33)=coef_xyz(33)+coef_xy(2,33)*pol_z(2,0,kg)
                coef_xyz(34)=coef_xyz(34)+coef_xy(1,34)*pol_z(1,0,kg)
                coef_xyz(34)=coef_xyz(34)+coef_xy(2,34)*pol_z(2,0,kg)
                coef_xyz(35)=coef_xyz(35)+coef_xy(1,35)*pol_z(1,0,kg)
                coef_xyz(35)=coef_xyz(35)+coef_xy(2,35)*pol_z(2,0,kg)
                coef_xyz(36)=coef_xyz(36)+coef_xy(1,36)*pol_z(1,0,kg)
                coef_xyz(36)=coef_xyz(36)+coef_xy(2,36)*pol_z(2,0,kg)
                coef_xyz(37)=coef_xyz(37)+coef_xy(1,1)*pol_z(1,1,kg)
                coef_xyz(37)=coef_xyz(37)+coef_xy(2,1)*pol_z(2,1,kg)
                coef_xyz(38)=coef_xyz(38)+coef_xy(1,2)*pol_z(1,1,kg)
                coef_xyz(38)=coef_xyz(38)+coef_xy(2,2)*pol_z(2,1,kg)
                coef_xyz(39)=coef_xyz(39)+coef_xy(1,3)*pol_z(1,1,kg)
                coef_xyz(39)=coef_xyz(39)+coef_xy(2,3)*pol_z(2,1,kg)
                coef_xyz(40)=coef_xyz(40)+coef_xy(1,4)*pol_z(1,1,kg)
                coef_xyz(40)=coef_xyz(40)+coef_xy(2,4)*pol_z(2,1,kg)
                coef_xyz(41)=coef_xyz(41)+coef_xy(1,5)*pol_z(1,1,kg)
                coef_xyz(41)=coef_xyz(41)+coef_xy(2,5)*pol_z(2,1,kg)
                coef_xyz(42)=coef_xyz(42)+coef_xy(1,6)*pol_z(1,1,kg)
                coef_xyz(42)=coef_xyz(42)+coef_xy(2,6)*pol_z(2,1,kg)
                coef_xyz(43)=coef_xyz(43)+coef_xy(1,7)*pol_z(1,1,kg)
                coef_xyz(43)=coef_xyz(43)+coef_xy(2,7)*pol_z(2,1,kg)
                coef_xyz(44)=coef_xyz(44)+coef_xy(1,9)*pol_z(1,1,kg)
                coef_xyz(44)=coef_xyz(44)+coef_xy(2,9)*pol_z(2,1,kg)
                coef_xyz(45)=coef_xyz(45)+coef_xy(1,10)*pol_z(1,1,kg)
                coef_xyz(45)=coef_xyz(45)+coef_xy(2,10)*pol_z(2,1,kg)
                coef_xyz(46)=coef_xyz(46)+coef_xy(1,11)*pol_z(1,1,kg)
                coef_xyz(46)=coef_xyz(46)+coef_xy(2,11)*pol_z(2,1,kg)
                coef_xyz(47)=coef_xyz(47)+coef_xy(1,12)*pol_z(1,1,kg)
                coef_xyz(47)=coef_xyz(47)+coef_xy(2,12)*pol_z(2,1,kg)
                coef_xyz(48)=coef_xyz(48)+coef_xy(1,13)*pol_z(1,1,kg)
                coef_xyz(48)=coef_xyz(48)+coef_xy(2,13)*pol_z(2,1,kg)
                coef_xyz(49)=coef_xyz(49)+coef_xy(1,14)*pol_z(1,1,kg)
                coef_xyz(49)=coef_xyz(49)+coef_xy(2,14)*pol_z(2,1,kg)
                coef_xyz(50)=coef_xyz(50)+coef_xy(1,16)*pol_z(1,1,kg)
                coef_xyz(50)=coef_xyz(50)+coef_xy(2,16)*pol_z(2,1,kg)
                coef_xyz(51)=coef_xyz(51)+coef_xy(1,17)*pol_z(1,1,kg)
                coef_xyz(51)=coef_xyz(51)+coef_xy(2,17)*pol_z(2,1,kg)
                coef_xyz(52)=coef_xyz(52)+coef_xy(1,18)*pol_z(1,1,kg)
                coef_xyz(52)=coef_xyz(52)+coef_xy(2,18)*pol_z(2,1,kg)
                coef_xyz(53)=coef_xyz(53)+coef_xy(1,19)*pol_z(1,1,kg)
                coef_xyz(53)=coef_xyz(53)+coef_xy(2,19)*pol_z(2,1,kg)
                coef_xyz(54)=coef_xyz(54)+coef_xy(1,20)*pol_z(1,1,kg)
                coef_xyz(54)=coef_xyz(54)+coef_xy(2,20)*pol_z(2,1,kg)
                coef_xyz(55)=coef_xyz(55)+coef_xy(1,22)*pol_z(1,1,kg)
                coef_xyz(55)=coef_xyz(55)+coef_xy(2,22)*pol_z(2,1,kg)
                coef_xyz(56)=coef_xyz(56)+coef_xy(1,23)*pol_z(1,1,kg)
                coef_xyz(56)=coef_xyz(56)+coef_xy(2,23)*pol_z(2,1,kg)
                coef_xyz(57)=coef_xyz(57)+coef_xy(1,24)*pol_z(1,1,kg)
                coef_xyz(57)=coef_xyz(57)+coef_xy(2,24)*pol_z(2,1,kg)
                coef_xyz(58)=coef_xyz(58)+coef_xy(1,25)*pol_z(1,1,kg)
                coef_xyz(58)=coef_xyz(58)+coef_xy(2,25)*pol_z(2,1,kg)
                coef_xyz(59)=coef_xyz(59)+coef_xy(1,27)*pol_z(1,1,kg)
                coef_xyz(59)=coef_xyz(59)+coef_xy(2,27)*pol_z(2,1,kg)
                coef_xyz(60)=coef_xyz(60)+coef_xy(1,28)*pol_z(1,1,kg)
                coef_xyz(60)=coef_xyz(60)+coef_xy(2,28)*pol_z(2,1,kg)
                coef_xyz(61)=coef_xyz(61)+coef_xy(1,29)*pol_z(1,1,kg)
                coef_xyz(61)=coef_xyz(61)+coef_xy(2,29)*pol_z(2,1,kg)
                coef_xyz(62)=coef_xyz(62)+coef_xy(1,31)*pol_z(1,1,kg)
                coef_xyz(62)=coef_xyz(62)+coef_xy(2,31)*pol_z(2,1,kg)
                coef_xyz(63)=coef_xyz(63)+coef_xy(1,32)*pol_z(1,1,kg)
                coef_xyz(63)=coef_xyz(63)+coef_xy(2,32)*pol_z(2,1,kg)
                coef_xyz(64)=coef_xyz(64)+coef_xy(1,34)*pol_z(1,1,kg)
                coef_xyz(64)=coef_xyz(64)+coef_xy(2,34)*pol_z(2,1,kg)
                coef_xyz(65)=coef_xyz(65)+coef_xy(1,1)*pol_z(1,2,kg)
                coef_xyz(65)=coef_xyz(65)+coef_xy(2,1)*pol_z(2,2,kg)
                coef_xyz(66)=coef_xyz(66)+coef_xy(1,2)*pol_z(1,2,kg)
                coef_xyz(66)=coef_xyz(66)+coef_xy(2,2)*pol_z(2,2,kg)
                coef_xyz(67)=coef_xyz(67)+coef_xy(1,3)*pol_z(1,2,kg)
                coef_xyz(67)=coef_xyz(67)+coef_xy(2,3)*pol_z(2,2,kg)
                coef_xyz(68)=coef_xyz(68)+coef_xy(1,4)*pol_z(1,2,kg)
                coef_xyz(68)=coef_xyz(68)+coef_xy(2,4)*pol_z(2,2,kg)
                coef_xyz(69)=coef_xyz(69)+coef_xy(1,5)*pol_z(1,2,kg)
                coef_xyz(69)=coef_xyz(69)+coef_xy(2,5)*pol_z(2,2,kg)
                coef_xyz(70)=coef_xyz(70)+coef_xy(1,6)*pol_z(1,2,kg)
                coef_xyz(70)=coef_xyz(70)+coef_xy(2,6)*pol_z(2,2,kg)
                coef_xyz(71)=coef_xyz(71)+coef_xy(1,9)*pol_z(1,2,kg)
                coef_xyz(71)=coef_xyz(71)+coef_xy(2,9)*pol_z(2,2,kg)
                coef_xyz(72)=coef_xyz(72)+coef_xy(1,10)*pol_z(1,2,kg)
                coef_xyz(72)=coef_xyz(72)+coef_xy(2,10)*pol_z(2,2,kg)
                coef_xyz(73)=coef_xyz(73)+coef_xy(1,11)*pol_z(1,2,kg)
                coef_xyz(73)=coef_xyz(73)+coef_xy(2,11)*pol_z(2,2,kg)
                coef_xyz(74)=coef_xyz(74)+coef_xy(1,12)*pol_z(1,2,kg)
                coef_xyz(74)=coef_xyz(74)+coef_xy(2,12)*pol_z(2,2,kg)
                coef_xyz(75)=coef_xyz(75)+coef_xy(1,13)*pol_z(1,2,kg)
                coef_xyz(75)=coef_xyz(75)+coef_xy(2,13)*pol_z(2,2,kg)
                coef_xyz(76)=coef_xyz(76)+coef_xy(1,16)*pol_z(1,2,kg)
                coef_xyz(76)=coef_xyz(76)+coef_xy(2,16)*pol_z(2,2,kg)
                coef_xyz(77)=coef_xyz(77)+coef_xy(1,17)*pol_z(1,2,kg)
                coef_xyz(77)=coef_xyz(77)+coef_xy(2,17)*pol_z(2,2,kg)
                coef_xyz(78)=coef_xyz(78)+coef_xy(1,18)*pol_z(1,2,kg)
                coef_xyz(78)=coef_xyz(78)+coef_xy(2,18)*pol_z(2,2,kg)
                coef_xyz(79)=coef_xyz(79)+coef_xy(1,19)*pol_z(1,2,kg)
                coef_xyz(79)=coef_xyz(79)+coef_xy(2,19)*pol_z(2,2,kg)
                coef_xyz(80)=coef_xyz(80)+coef_xy(1,22)*pol_z(1,2,kg)
                coef_xyz(80)=coef_xyz(80)+coef_xy(2,22)*pol_z(2,2,kg)
                coef_xyz(81)=coef_xyz(81)+coef_xy(1,23)*pol_z(1,2,kg)
                coef_xyz(81)=coef_xyz(81)+coef_xy(2,23)*pol_z(2,2,kg)
                coef_xyz(82)=coef_xyz(82)+coef_xy(1,24)*pol_z(1,2,kg)
                coef_xyz(82)=coef_xyz(82)+coef_xy(2,24)*pol_z(2,2,kg)
                coef_xyz(83)=coef_xyz(83)+coef_xy(1,27)*pol_z(1,2,kg)
                coef_xyz(83)=coef_xyz(83)+coef_xy(2,27)*pol_z(2,2,kg)
                coef_xyz(84)=coef_xyz(84)+coef_xy(1,28)*pol_z(1,2,kg)
                coef_xyz(84)=coef_xyz(84)+coef_xy(2,28)*pol_z(2,2,kg)
                coef_xyz(85)=coef_xyz(85)+coef_xy(1,31)*pol_z(1,2,kg)
                coef_xyz(85)=coef_xyz(85)+coef_xy(2,31)*pol_z(2,2,kg)
                coef_xyz(86)=coef_xyz(86)+coef_xy(1,1)*pol_z(1,3,kg)
                coef_xyz(86)=coef_xyz(86)+coef_xy(2,1)*pol_z(2,3,kg)
                coef_xyz(87)=coef_xyz(87)+coef_xy(1,2)*pol_z(1,3,kg)
                coef_xyz(87)=coef_xyz(87)+coef_xy(2,2)*pol_z(2,3,kg)
                coef_xyz(88)=coef_xyz(88)+coef_xy(1,3)*pol_z(1,3,kg)
                coef_xyz(88)=coef_xyz(88)+coef_xy(2,3)*pol_z(2,3,kg)
                coef_xyz(89)=coef_xyz(89)+coef_xy(1,4)*pol_z(1,3,kg)
                coef_xyz(89)=coef_xyz(89)+coef_xy(2,4)*pol_z(2,3,kg)
                coef_xyz(90)=coef_xyz(90)+coef_xy(1,5)*pol_z(1,3,kg)
                coef_xyz(90)=coef_xyz(90)+coef_xy(2,5)*pol_z(2,3,kg)
                coef_xyz(91)=coef_xyz(91)+coef_xy(1,9)*pol_z(1,3,kg)
                coef_xyz(91)=coef_xyz(91)+coef_xy(2,9)*pol_z(2,3,kg)
                coef_xyz(92)=coef_xyz(92)+coef_xy(1,10)*pol_z(1,3,kg)
                coef_xyz(92)=coef_xyz(92)+coef_xy(2,10)*pol_z(2,3,kg)
                coef_xyz(93)=coef_xyz(93)+coef_xy(1,11)*pol_z(1,3,kg)
                coef_xyz(93)=coef_xyz(93)+coef_xy(2,11)*pol_z(2,3,kg)
                coef_xyz(94)=coef_xyz(94)+coef_xy(1,12)*pol_z(1,3,kg)
                coef_xyz(94)=coef_xyz(94)+coef_xy(2,12)*pol_z(2,3,kg)
                coef_xyz(95)=coef_xyz(95)+coef_xy(1,16)*pol_z(1,3,kg)
                coef_xyz(95)=coef_xyz(95)+coef_xy(2,16)*pol_z(2,3,kg)
                coef_xyz(96)=coef_xyz(96)+coef_xy(1,17)*pol_z(1,3,kg)
                coef_xyz(96)=coef_xyz(96)+coef_xy(2,17)*pol_z(2,3,kg)
                coef_xyz(97)=coef_xyz(97)+coef_xy(1,18)*pol_z(1,3,kg)
                coef_xyz(97)=coef_xyz(97)+coef_xy(2,18)*pol_z(2,3,kg)
                coef_xyz(98)=coef_xyz(98)+coef_xy(1,22)*pol_z(1,3,kg)
                coef_xyz(98)=coef_xyz(98)+coef_xy(2,22)*pol_z(2,3,kg)
                coef_xyz(99)=coef_xyz(99)+coef_xy(1,23)*pol_z(1,3,kg)
                coef_xyz(99)=coef_xyz(99)+coef_xy(2,23)*pol_z(2,3,kg)
                coef_xyz(100)=coef_xyz(100)+coef_xy(1,27)*pol_z(1,3,kg)
                coef_xyz(100)=coef_xyz(100)+coef_xy(2,27)*pol_z(2,3,kg)
                coef_xyz(101)=coef_xyz(101)+coef_xy(1,1)*pol_z(1,4,kg)
                coef_xyz(101)=coef_xyz(101)+coef_xy(2,1)*pol_z(2,4,kg)
                coef_xyz(102)=coef_xyz(102)+coef_xy(1,2)*pol_z(1,4,kg)
                coef_xyz(102)=coef_xyz(102)+coef_xy(2,2)*pol_z(2,4,kg)
                coef_xyz(103)=coef_xyz(103)+coef_xy(1,3)*pol_z(1,4,kg)
                coef_xyz(103)=coef_xyz(103)+coef_xy(2,3)*pol_z(2,4,kg)
                coef_xyz(104)=coef_xyz(104)+coef_xy(1,4)*pol_z(1,4,kg)
                coef_xyz(104)=coef_xyz(104)+coef_xy(2,4)*pol_z(2,4,kg)
                coef_xyz(105)=coef_xyz(105)+coef_xy(1,9)*pol_z(1,4,kg)
                coef_xyz(105)=coef_xyz(105)+coef_xy(2,9)*pol_z(2,4,kg)
                coef_xyz(106)=coef_xyz(106)+coef_xy(1,10)*pol_z(1,4,kg)
                coef_xyz(106)=coef_xyz(106)+coef_xy(2,10)*pol_z(2,4,kg)
                coef_xyz(107)=coef_xyz(107)+coef_xy(1,11)*pol_z(1,4,kg)
                coef_xyz(107)=coef_xyz(107)+coef_xy(2,11)*pol_z(2,4,kg)
                coef_xyz(108)=coef_xyz(108)+coef_xy(1,16)*pol_z(1,4,kg)
                coef_xyz(108)=coef_xyz(108)+coef_xy(2,16)*pol_z(2,4,kg)
                coef_xyz(109)=coef_xyz(109)+coef_xy(1,17)*pol_z(1,4,kg)
                coef_xyz(109)=coef_xyz(109)+coef_xy(2,17)*pol_z(2,4,kg)
                coef_xyz(110)=coef_xyz(110)+coef_xy(1,22)*pol_z(1,4,kg)
                coef_xyz(110)=coef_xyz(110)+coef_xy(2,22)*pol_z(2,4,kg)
                coef_xyz(111)=coef_xyz(111)+coef_xy(1,1)*pol_z(1,5,kg)
                coef_xyz(111)=coef_xyz(111)+coef_xy(2,1)*pol_z(2,5,kg)
                coef_xyz(112)=coef_xyz(112)+coef_xy(1,2)*pol_z(1,5,kg)
                coef_xyz(112)=coef_xyz(112)+coef_xy(2,2)*pol_z(2,5,kg)
                coef_xyz(113)=coef_xyz(113)+coef_xy(1,3)*pol_z(1,5,kg)
                coef_xyz(113)=coef_xyz(113)+coef_xy(2,3)*pol_z(2,5,kg)
                coef_xyz(114)=coef_xyz(114)+coef_xy(1,9)*pol_z(1,5,kg)
                coef_xyz(114)=coef_xyz(114)+coef_xy(2,9)*pol_z(2,5,kg)
                coef_xyz(115)=coef_xyz(115)+coef_xy(1,10)*pol_z(1,5,kg)
                coef_xyz(115)=coef_xyz(115)+coef_xy(2,10)*pol_z(2,5,kg)
                coef_xyz(116)=coef_xyz(116)+coef_xy(1,16)*pol_z(1,5,kg)
                coef_xyz(116)=coef_xyz(116)+coef_xy(2,16)*pol_z(2,5,kg)
                coef_xyz(117)=coef_xyz(117)+coef_xy(1,1)*pol_z(1,6,kg)
                coef_xyz(117)=coef_xyz(117)+coef_xy(2,1)*pol_z(2,6,kg)
                coef_xyz(118)=coef_xyz(118)+coef_xy(1,2)*pol_z(1,6,kg)
                coef_xyz(118)=coef_xyz(118)+coef_xy(2,2)*pol_z(2,6,kg)
                coef_xyz(119)=coef_xyz(119)+coef_xy(1,9)*pol_z(1,6,kg)
                coef_xyz(119)=coef_xyz(119)+coef_xy(2,9)*pol_z(2,6,kg)
                coef_xyz(120)=coef_xyz(120)+coef_xy(1,1)*pol_z(1,7,kg)
                coef_xyz(120)=coef_xyz(120)+coef_xy(2,1)*pol_z(2,7,kg)
    END DO

  END SUBROUTINE integrate_core_7
  SUBROUTINE integrate_core_8(grid,coef_xyz,pol_x,pol_y,pol_z,map,sphere_bounds,cmax,gridbounds)
    USE kinds,                        ONLY: dp
    INTEGER, INTENT(IN)                      :: sphere_bounds(*), cmax, &
                                                map(-cmax:cmax,1:3), &
                                                gridbounds(2,3)
    REAL(dp), INTENT(IN) :: grid(gridbounds(1,1):gridbounds(2,1), gridbounds(1&
      ,2):gridbounds(2,2), gridbounds(1,3):gridbounds(2,3))
    INTEGER, PARAMETER                       :: lp = 8
    REAL(dp), INTENT(IN)                     :: pol_x(0:lp,-cmax:cmax), &
                                                pol_y(1:2,0:lp,-cmax:0), &
                                                pol_z(1:2,0:lp,-cmax:0)
    REAL(dp), INTENT(OUT) :: coef_xyz(((lp+1)*(lp+2)*(lp+3))/6)

    INTEGER                                  :: i, ig, igmax, igmin, j, j2, &
                                                jg, jg2, jgmin, k, k2, kg, &
                                                kg2, kgmin, lxp, sci
    REAL(dp)                                 :: coef_x(4,0:lp), &
                                                coef_xy(2,((lp+1)*(lp+2))/2), &
                                                s01, s02, s03, s04

    sci=1

    coef_xyz=0.0_dp

    kgmin=sphere_bounds(sci)
    sci=sci+1
    DO kg=kgmin,0
       kg2=1-kg
       k=map(kg,3)
       k2=map(kg2,3)

       coef_xy=0.0_dp

       jgmin=sphere_bounds(sci)
       sci=sci+1
       DO jg=jgmin,0
          jg2=1-jg
          j=map(jg,2)
          j2=map(jg2,2)
          igmin=sphere_bounds(sci)
          sci=sci+1
          igmax=1-igmin

          coef_x=0.0_dp
          DO ig=igmin,igmax
             i=map(ig,1)
             s01=grid(i,j,k)
             s02=grid(i,j,k2)
             s03=grid(i,j2,k)
             s04=grid(i,j2,k2)
             DO lxp=0,lp
                coef_x(1,lxp)=coef_x(1,lxp)+s01*pol_x(lxp,ig)
                coef_x(2,lxp)=coef_x(2,lxp)+s02*pol_x(lxp,ig)
                coef_x(3,lxp)=coef_x(3,lxp)+s03*pol_x(lxp,ig)
                coef_x(4,lxp)=coef_x(4,lxp)+s04*pol_x(lxp,ig)
             ENDDO
          END DO
             coef_xy(1,1)=coef_xy(1,1)+coef_x(1,0)*pol_y(1,0,jg)
             coef_xy(2,1)=coef_xy(2,1)+coef_x(2,0)*pol_y(1,0,jg)
             coef_xy(1,1)=coef_xy(1,1)+coef_x(3,0)*pol_y(2,0,jg)
             coef_xy(2,1)=coef_xy(2,1)+coef_x(4,0)*pol_y(2,0,jg)
             coef_xy(1,2)=coef_xy(1,2)+coef_x(1,1)*pol_y(1,0,jg)
             coef_xy(2,2)=coef_xy(2,2)+coef_x(2,1)*pol_y(1,0,jg)
             coef_xy(1,2)=coef_xy(1,2)+coef_x(3,1)*pol_y(2,0,jg)
             coef_xy(2,2)=coef_xy(2,2)+coef_x(4,1)*pol_y(2,0,jg)
             coef_xy(1,3)=coef_xy(1,3)+coef_x(1,2)*pol_y(1,0,jg)
             coef_xy(2,3)=coef_xy(2,3)+coef_x(2,2)*pol_y(1,0,jg)
             coef_xy(1,3)=coef_xy(1,3)+coef_x(3,2)*pol_y(2,0,jg)
             coef_xy(2,3)=coef_xy(2,3)+coef_x(4,2)*pol_y(2,0,jg)
             coef_xy(1,4)=coef_xy(1,4)+coef_x(1,3)*pol_y(1,0,jg)
             coef_xy(2,4)=coef_xy(2,4)+coef_x(2,3)*pol_y(1,0,jg)
             coef_xy(1,4)=coef_xy(1,4)+coef_x(3,3)*pol_y(2,0,jg)
             coef_xy(2,4)=coef_xy(2,4)+coef_x(4,3)*pol_y(2,0,jg)
             coef_xy(1,5)=coef_xy(1,5)+coef_x(1,4)*pol_y(1,0,jg)
             coef_xy(2,5)=coef_xy(2,5)+coef_x(2,4)*pol_y(1,0,jg)
             coef_xy(1,5)=coef_xy(1,5)+coef_x(3,4)*pol_y(2,0,jg)
             coef_xy(2,5)=coef_xy(2,5)+coef_x(4,4)*pol_y(2,0,jg)
             coef_xy(1,6)=coef_xy(1,6)+coef_x(1,5)*pol_y(1,0,jg)
             coef_xy(2,6)=coef_xy(2,6)+coef_x(2,5)*pol_y(1,0,jg)
             coef_xy(1,6)=coef_xy(1,6)+coef_x(3,5)*pol_y(2,0,jg)
             coef_xy(2,6)=coef_xy(2,6)+coef_x(4,5)*pol_y(2,0,jg)
             coef_xy(1,7)=coef_xy(1,7)+coef_x(1,6)*pol_y(1,0,jg)
             coef_xy(2,7)=coef_xy(2,7)+coef_x(2,6)*pol_y(1,0,jg)
             coef_xy(1,7)=coef_xy(1,7)+coef_x(3,6)*pol_y(2,0,jg)
             coef_xy(2,7)=coef_xy(2,7)+coef_x(4,6)*pol_y(2,0,jg)
             coef_xy(1,8)=coef_xy(1,8)+coef_x(1,7)*pol_y(1,0,jg)
             coef_xy(2,8)=coef_xy(2,8)+coef_x(2,7)*pol_y(1,0,jg)
             coef_xy(1,8)=coef_xy(1,8)+coef_x(3,7)*pol_y(2,0,jg)
             coef_xy(2,8)=coef_xy(2,8)+coef_x(4,7)*pol_y(2,0,jg)
             coef_xy(1,9)=coef_xy(1,9)+coef_x(1,8)*pol_y(1,0,jg)
             coef_xy(2,9)=coef_xy(2,9)+coef_x(2,8)*pol_y(1,0,jg)
             coef_xy(1,9)=coef_xy(1,9)+coef_x(3,8)*pol_y(2,0,jg)
             coef_xy(2,9)=coef_xy(2,9)+coef_x(4,8)*pol_y(2,0,jg)
             coef_xy(1,10)=coef_xy(1,10)+coef_x(1,0)*pol_y(1,1,jg)
             coef_xy(2,10)=coef_xy(2,10)+coef_x(2,0)*pol_y(1,1,jg)
             coef_xy(1,10)=coef_xy(1,10)+coef_x(3,0)*pol_y(2,1,jg)
             coef_xy(2,10)=coef_xy(2,10)+coef_x(4,0)*pol_y(2,1,jg)
             coef_xy(1,11)=coef_xy(1,11)+coef_x(1,1)*pol_y(1,1,jg)
             coef_xy(2,11)=coef_xy(2,11)+coef_x(2,1)*pol_y(1,1,jg)
             coef_xy(1,11)=coef_xy(1,11)+coef_x(3,1)*pol_y(2,1,jg)
             coef_xy(2,11)=coef_xy(2,11)+coef_x(4,1)*pol_y(2,1,jg)
             coef_xy(1,12)=coef_xy(1,12)+coef_x(1,2)*pol_y(1,1,jg)
             coef_xy(2,12)=coef_xy(2,12)+coef_x(2,2)*pol_y(1,1,jg)
             coef_xy(1,12)=coef_xy(1,12)+coef_x(3,2)*pol_y(2,1,jg)
             coef_xy(2,12)=coef_xy(2,12)+coef_x(4,2)*pol_y(2,1,jg)
             coef_xy(1,13)=coef_xy(1,13)+coef_x(1,3)*pol_y(1,1,jg)
             coef_xy(2,13)=coef_xy(2,13)+coef_x(2,3)*pol_y(1,1,jg)
             coef_xy(1,13)=coef_xy(1,13)+coef_x(3,3)*pol_y(2,1,jg)
             coef_xy(2,13)=coef_xy(2,13)+coef_x(4,3)*pol_y(2,1,jg)
             coef_xy(1,14)=coef_xy(1,14)+coef_x(1,4)*pol_y(1,1,jg)
             coef_xy(2,14)=coef_xy(2,14)+coef_x(2,4)*pol_y(1,1,jg)
             coef_xy(1,14)=coef_xy(1,14)+coef_x(3,4)*pol_y(2,1,jg)
             coef_xy(2,14)=coef_xy(2,14)+coef_x(4,4)*pol_y(2,1,jg)
             coef_xy(1,15)=coef_xy(1,15)+coef_x(1,5)*pol_y(1,1,jg)
             coef_xy(2,15)=coef_xy(2,15)+coef_x(2,5)*pol_y(1,1,jg)
             coef_xy(1,15)=coef_xy(1,15)+coef_x(3,5)*pol_y(2,1,jg)
             coef_xy(2,15)=coef_xy(2,15)+coef_x(4,5)*pol_y(2,1,jg)
             coef_xy(1,16)=coef_xy(1,16)+coef_x(1,6)*pol_y(1,1,jg)
             coef_xy(2,16)=coef_xy(2,16)+coef_x(2,6)*pol_y(1,1,jg)
             coef_xy(1,16)=coef_xy(1,16)+coef_x(3,6)*pol_y(2,1,jg)
             coef_xy(2,16)=coef_xy(2,16)+coef_x(4,6)*pol_y(2,1,jg)
             coef_xy(1,17)=coef_xy(1,17)+coef_x(1,7)*pol_y(1,1,jg)
             coef_xy(2,17)=coef_xy(2,17)+coef_x(2,7)*pol_y(1,1,jg)
             coef_xy(1,17)=coef_xy(1,17)+coef_x(3,7)*pol_y(2,1,jg)
             coef_xy(2,17)=coef_xy(2,17)+coef_x(4,7)*pol_y(2,1,jg)
             coef_xy(1,18)=coef_xy(1,18)+coef_x(1,0)*pol_y(1,2,jg)
             coef_xy(2,18)=coef_xy(2,18)+coef_x(2,0)*pol_y(1,2,jg)
             coef_xy(1,18)=coef_xy(1,18)+coef_x(3,0)*pol_y(2,2,jg)
             coef_xy(2,18)=coef_xy(2,18)+coef_x(4,0)*pol_y(2,2,jg)
             coef_xy(1,19)=coef_xy(1,19)+coef_x(1,1)*pol_y(1,2,jg)
             coef_xy(2,19)=coef_xy(2,19)+coef_x(2,1)*pol_y(1,2,jg)
             coef_xy(1,19)=coef_xy(1,19)+coef_x(3,1)*pol_y(2,2,jg)
             coef_xy(2,19)=coef_xy(2,19)+coef_x(4,1)*pol_y(2,2,jg)
             coef_xy(1,20)=coef_xy(1,20)+coef_x(1,2)*pol_y(1,2,jg)
             coef_xy(2,20)=coef_xy(2,20)+coef_x(2,2)*pol_y(1,2,jg)
             coef_xy(1,20)=coef_xy(1,20)+coef_x(3,2)*pol_y(2,2,jg)
             coef_xy(2,20)=coef_xy(2,20)+coef_x(4,2)*pol_y(2,2,jg)
             coef_xy(1,21)=coef_xy(1,21)+coef_x(1,3)*pol_y(1,2,jg)
             coef_xy(2,21)=coef_xy(2,21)+coef_x(2,3)*pol_y(1,2,jg)
             coef_xy(1,21)=coef_xy(1,21)+coef_x(3,3)*pol_y(2,2,jg)
             coef_xy(2,21)=coef_xy(2,21)+coef_x(4,3)*pol_y(2,2,jg)
             coef_xy(1,22)=coef_xy(1,22)+coef_x(1,4)*pol_y(1,2,jg)
             coef_xy(2,22)=coef_xy(2,22)+coef_x(2,4)*pol_y(1,2,jg)
             coef_xy(1,22)=coef_xy(1,22)+coef_x(3,4)*pol_y(2,2,jg)
             coef_xy(2,22)=coef_xy(2,22)+coef_x(4,4)*pol_y(2,2,jg)
             coef_xy(1,23)=coef_xy(1,23)+coef_x(1,5)*pol_y(1,2,jg)
             coef_xy(2,23)=coef_xy(2,23)+coef_x(2,5)*pol_y(1,2,jg)
             coef_xy(1,23)=coef_xy(1,23)+coef_x(3,5)*pol_y(2,2,jg)
             coef_xy(2,23)=coef_xy(2,23)+coef_x(4,5)*pol_y(2,2,jg)
             coef_xy(1,24)=coef_xy(1,24)+coef_x(1,6)*pol_y(1,2,jg)
             coef_xy(2,24)=coef_xy(2,24)+coef_x(2,6)*pol_y(1,2,jg)
             coef_xy(1,24)=coef_xy(1,24)+coef_x(3,6)*pol_y(2,2,jg)
             coef_xy(2,24)=coef_xy(2,24)+coef_x(4,6)*pol_y(2,2,jg)
             coef_xy(1,25)=coef_xy(1,25)+coef_x(1,0)*pol_y(1,3,jg)
             coef_xy(2,25)=coef_xy(2,25)+coef_x(2,0)*pol_y(1,3,jg)
             coef_xy(1,25)=coef_xy(1,25)+coef_x(3,0)*pol_y(2,3,jg)
             coef_xy(2,25)=coef_xy(2,25)+coef_x(4,0)*pol_y(2,3,jg)
             coef_xy(1,26)=coef_xy(1,26)+coef_x(1,1)*pol_y(1,3,jg)
             coef_xy(2,26)=coef_xy(2,26)+coef_x(2,1)*pol_y(1,3,jg)
             coef_xy(1,26)=coef_xy(1,26)+coef_x(3,1)*pol_y(2,3,jg)
             coef_xy(2,26)=coef_xy(2,26)+coef_x(4,1)*pol_y(2,3,jg)
             coef_xy(1,27)=coef_xy(1,27)+coef_x(1,2)*pol_y(1,3,jg)
             coef_xy(2,27)=coef_xy(2,27)+coef_x(2,2)*pol_y(1,3,jg)
             coef_xy(1,27)=coef_xy(1,27)+coef_x(3,2)*pol_y(2,3,jg)
             coef_xy(2,27)=coef_xy(2,27)+coef_x(4,2)*pol_y(2,3,jg)
             coef_xy(1,28)=coef_xy(1,28)+coef_x(1,3)*pol_y(1,3,jg)
             coef_xy(2,28)=coef_xy(2,28)+coef_x(2,3)*pol_y(1,3,jg)
             coef_xy(1,28)=coef_xy(1,28)+coef_x(3,3)*pol_y(2,3,jg)
             coef_xy(2,28)=coef_xy(2,28)+coef_x(4,3)*pol_y(2,3,jg)
             coef_xy(1,29)=coef_xy(1,29)+coef_x(1,4)*pol_y(1,3,jg)
             coef_xy(2,29)=coef_xy(2,29)+coef_x(2,4)*pol_y(1,3,jg)
             coef_xy(1,29)=coef_xy(1,29)+coef_x(3,4)*pol_y(2,3,jg)
             coef_xy(2,29)=coef_xy(2,29)+coef_x(4,4)*pol_y(2,3,jg)
             coef_xy(1,30)=coef_xy(1,30)+coef_x(1,5)*pol_y(1,3,jg)
             coef_xy(2,30)=coef_xy(2,30)+coef_x(2,5)*pol_y(1,3,jg)
             coef_xy(1,30)=coef_xy(1,30)+coef_x(3,5)*pol_y(2,3,jg)
             coef_xy(2,30)=coef_xy(2,30)+coef_x(4,5)*pol_y(2,3,jg)
             coef_xy(1,31)=coef_xy(1,31)+coef_x(1,0)*pol_y(1,4,jg)
             coef_xy(2,31)=coef_xy(2,31)+coef_x(2,0)*pol_y(1,4,jg)
             coef_xy(1,31)=coef_xy(1,31)+coef_x(3,0)*pol_y(2,4,jg)
             coef_xy(2,31)=coef_xy(2,31)+coef_x(4,0)*pol_y(2,4,jg)
             coef_xy(1,32)=coef_xy(1,32)+coef_x(1,1)*pol_y(1,4,jg)
             coef_xy(2,32)=coef_xy(2,32)+coef_x(2,1)*pol_y(1,4,jg)
             coef_xy(1,32)=coef_xy(1,32)+coef_x(3,1)*pol_y(2,4,jg)
             coef_xy(2,32)=coef_xy(2,32)+coef_x(4,1)*pol_y(2,4,jg)
             coef_xy(1,33)=coef_xy(1,33)+coef_x(1,2)*pol_y(1,4,jg)
             coef_xy(2,33)=coef_xy(2,33)+coef_x(2,2)*pol_y(1,4,jg)
             coef_xy(1,33)=coef_xy(1,33)+coef_x(3,2)*pol_y(2,4,jg)
             coef_xy(2,33)=coef_xy(2,33)+coef_x(4,2)*pol_y(2,4,jg)
             coef_xy(1,34)=coef_xy(1,34)+coef_x(1,3)*pol_y(1,4,jg)
             coef_xy(2,34)=coef_xy(2,34)+coef_x(2,3)*pol_y(1,4,jg)
             coef_xy(1,34)=coef_xy(1,34)+coef_x(3,3)*pol_y(2,4,jg)
             coef_xy(2,34)=coef_xy(2,34)+coef_x(4,3)*pol_y(2,4,jg)
             coef_xy(1,35)=coef_xy(1,35)+coef_x(1,4)*pol_y(1,4,jg)
             coef_xy(2,35)=coef_xy(2,35)+coef_x(2,4)*pol_y(1,4,jg)
             coef_xy(1,35)=coef_xy(1,35)+coef_x(3,4)*pol_y(2,4,jg)
             coef_xy(2,35)=coef_xy(2,35)+coef_x(4,4)*pol_y(2,4,jg)
             coef_xy(1,36)=coef_xy(1,36)+coef_x(1,0)*pol_y(1,5,jg)
             coef_xy(2,36)=coef_xy(2,36)+coef_x(2,0)*pol_y(1,5,jg)
             coef_xy(1,36)=coef_xy(1,36)+coef_x(3,0)*pol_y(2,5,jg)
             coef_xy(2,36)=coef_xy(2,36)+coef_x(4,0)*pol_y(2,5,jg)
             coef_xy(1,37)=coef_xy(1,37)+coef_x(1,1)*pol_y(1,5,jg)
             coef_xy(2,37)=coef_xy(2,37)+coef_x(2,1)*pol_y(1,5,jg)
             coef_xy(1,37)=coef_xy(1,37)+coef_x(3,1)*pol_y(2,5,jg)
             coef_xy(2,37)=coef_xy(2,37)+coef_x(4,1)*pol_y(2,5,jg)
             coef_xy(1,38)=coef_xy(1,38)+coef_x(1,2)*pol_y(1,5,jg)
             coef_xy(2,38)=coef_xy(2,38)+coef_x(2,2)*pol_y(1,5,jg)
             coef_xy(1,38)=coef_xy(1,38)+coef_x(3,2)*pol_y(2,5,jg)
             coef_xy(2,38)=coef_xy(2,38)+coef_x(4,2)*pol_y(2,5,jg)
             coef_xy(1,39)=coef_xy(1,39)+coef_x(1,3)*pol_y(1,5,jg)
             coef_xy(2,39)=coef_xy(2,39)+coef_x(2,3)*pol_y(1,5,jg)
             coef_xy(1,39)=coef_xy(1,39)+coef_x(3,3)*pol_y(2,5,jg)
             coef_xy(2,39)=coef_xy(2,39)+coef_x(4,3)*pol_y(2,5,jg)
             coef_xy(1,40)=coef_xy(1,40)+coef_x(1,0)*pol_y(1,6,jg)
             coef_xy(2,40)=coef_xy(2,40)+coef_x(2,0)*pol_y(1,6,jg)
             coef_xy(1,40)=coef_xy(1,40)+coef_x(3,0)*pol_y(2,6,jg)
             coef_xy(2,40)=coef_xy(2,40)+coef_x(4,0)*pol_y(2,6,jg)
             coef_xy(1,41)=coef_xy(1,41)+coef_x(1,1)*pol_y(1,6,jg)
             coef_xy(2,41)=coef_xy(2,41)+coef_x(2,1)*pol_y(1,6,jg)
             coef_xy(1,41)=coef_xy(1,41)+coef_x(3,1)*pol_y(2,6,jg)
             coef_xy(2,41)=coef_xy(2,41)+coef_x(4,1)*pol_y(2,6,jg)
             coef_xy(1,42)=coef_xy(1,42)+coef_x(1,2)*pol_y(1,6,jg)
             coef_xy(2,42)=coef_xy(2,42)+coef_x(2,2)*pol_y(1,6,jg)
             coef_xy(1,42)=coef_xy(1,42)+coef_x(3,2)*pol_y(2,6,jg)
             coef_xy(2,42)=coef_xy(2,42)+coef_x(4,2)*pol_y(2,6,jg)
             coef_xy(1,43)=coef_xy(1,43)+coef_x(1,0)*pol_y(1,7,jg)
             coef_xy(2,43)=coef_xy(2,43)+coef_x(2,0)*pol_y(1,7,jg)
             coef_xy(1,43)=coef_xy(1,43)+coef_x(3,0)*pol_y(2,7,jg)
             coef_xy(2,43)=coef_xy(2,43)+coef_x(4,0)*pol_y(2,7,jg)
             coef_xy(1,44)=coef_xy(1,44)+coef_x(1,1)*pol_y(1,7,jg)
             coef_xy(2,44)=coef_xy(2,44)+coef_x(2,1)*pol_y(1,7,jg)
             coef_xy(1,44)=coef_xy(1,44)+coef_x(3,1)*pol_y(2,7,jg)
             coef_xy(2,44)=coef_xy(2,44)+coef_x(4,1)*pol_y(2,7,jg)
             coef_xy(1,45)=coef_xy(1,45)+coef_x(1,0)*pol_y(1,8,jg)
             coef_xy(2,45)=coef_xy(2,45)+coef_x(2,0)*pol_y(1,8,jg)
             coef_xy(1,45)=coef_xy(1,45)+coef_x(3,0)*pol_y(2,8,jg)
             coef_xy(2,45)=coef_xy(2,45)+coef_x(4,0)*pol_y(2,8,jg)
       END DO
                coef_xyz(1)=coef_xyz(1)+coef_xy(1,1)*pol_z(1,0,kg)
                coef_xyz(1)=coef_xyz(1)+coef_xy(2,1)*pol_z(2,0,kg)
                coef_xyz(2)=coef_xyz(2)+coef_xy(1,2)*pol_z(1,0,kg)
                coef_xyz(2)=coef_xyz(2)+coef_xy(2,2)*pol_z(2,0,kg)
                coef_xyz(3)=coef_xyz(3)+coef_xy(1,3)*pol_z(1,0,kg)
                coef_xyz(3)=coef_xyz(3)+coef_xy(2,3)*pol_z(2,0,kg)
                coef_xyz(4)=coef_xyz(4)+coef_xy(1,4)*pol_z(1,0,kg)
                coef_xyz(4)=coef_xyz(4)+coef_xy(2,4)*pol_z(2,0,kg)
                coef_xyz(5)=coef_xyz(5)+coef_xy(1,5)*pol_z(1,0,kg)
                coef_xyz(5)=coef_xyz(5)+coef_xy(2,5)*pol_z(2,0,kg)
                coef_xyz(6)=coef_xyz(6)+coef_xy(1,6)*pol_z(1,0,kg)
                coef_xyz(6)=coef_xyz(6)+coef_xy(2,6)*pol_z(2,0,kg)
                coef_xyz(7)=coef_xyz(7)+coef_xy(1,7)*pol_z(1,0,kg)
                coef_xyz(7)=coef_xyz(7)+coef_xy(2,7)*pol_z(2,0,kg)
                coef_xyz(8)=coef_xyz(8)+coef_xy(1,8)*pol_z(1,0,kg)
                coef_xyz(8)=coef_xyz(8)+coef_xy(2,8)*pol_z(2,0,kg)
                coef_xyz(9)=coef_xyz(9)+coef_xy(1,9)*pol_z(1,0,kg)
                coef_xyz(9)=coef_xyz(9)+coef_xy(2,9)*pol_z(2,0,kg)
                coef_xyz(10)=coef_xyz(10)+coef_xy(1,10)*pol_z(1,0,kg)
                coef_xyz(10)=coef_xyz(10)+coef_xy(2,10)*pol_z(2,0,kg)
                coef_xyz(11)=coef_xyz(11)+coef_xy(1,11)*pol_z(1,0,kg)
                coef_xyz(11)=coef_xyz(11)+coef_xy(2,11)*pol_z(2,0,kg)
                coef_xyz(12)=coef_xyz(12)+coef_xy(1,12)*pol_z(1,0,kg)
                coef_xyz(12)=coef_xyz(12)+coef_xy(2,12)*pol_z(2,0,kg)
                coef_xyz(13)=coef_xyz(13)+coef_xy(1,13)*pol_z(1,0,kg)
                coef_xyz(13)=coef_xyz(13)+coef_xy(2,13)*pol_z(2,0,kg)
                coef_xyz(14)=coef_xyz(14)+coef_xy(1,14)*pol_z(1,0,kg)
                coef_xyz(14)=coef_xyz(14)+coef_xy(2,14)*pol_z(2,0,kg)
                coef_xyz(15)=coef_xyz(15)+coef_xy(1,15)*pol_z(1,0,kg)
                coef_xyz(15)=coef_xyz(15)+coef_xy(2,15)*pol_z(2,0,kg)
                coef_xyz(16)=coef_xyz(16)+coef_xy(1,16)*pol_z(1,0,kg)
                coef_xyz(16)=coef_xyz(16)+coef_xy(2,16)*pol_z(2,0,kg)
                coef_xyz(17)=coef_xyz(17)+coef_xy(1,17)*pol_z(1,0,kg)
                coef_xyz(17)=coef_xyz(17)+coef_xy(2,17)*pol_z(2,0,kg)
                coef_xyz(18)=coef_xyz(18)+coef_xy(1,18)*pol_z(1,0,kg)
                coef_xyz(18)=coef_xyz(18)+coef_xy(2,18)*pol_z(2,0,kg)
                coef_xyz(19)=coef_xyz(19)+coef_xy(1,19)*pol_z(1,0,kg)
                coef_xyz(19)=coef_xyz(19)+coef_xy(2,19)*pol_z(2,0,kg)
                coef_xyz(20)=coef_xyz(20)+coef_xy(1,20)*pol_z(1,0,kg)
                coef_xyz(20)=coef_xyz(20)+coef_xy(2,20)*pol_z(2,0,kg)
                coef_xyz(21)=coef_xyz(21)+coef_xy(1,21)*pol_z(1,0,kg)
                coef_xyz(21)=coef_xyz(21)+coef_xy(2,21)*pol_z(2,0,kg)
                coef_xyz(22)=coef_xyz(22)+coef_xy(1,22)*pol_z(1,0,kg)
                coef_xyz(22)=coef_xyz(22)+coef_xy(2,22)*pol_z(2,0,kg)
                coef_xyz(23)=coef_xyz(23)+coef_xy(1,23)*pol_z(1,0,kg)
                coef_xyz(23)=coef_xyz(23)+coef_xy(2,23)*pol_z(2,0,kg)
                coef_xyz(24)=coef_xyz(24)+coef_xy(1,24)*pol_z(1,0,kg)
                coef_xyz(24)=coef_xyz(24)+coef_xy(2,24)*pol_z(2,0,kg)
                coef_xyz(25)=coef_xyz(25)+coef_xy(1,25)*pol_z(1,0,kg)
                coef_xyz(25)=coef_xyz(25)+coef_xy(2,25)*pol_z(2,0,kg)
                coef_xyz(26)=coef_xyz(26)+coef_xy(1,26)*pol_z(1,0,kg)
                coef_xyz(26)=coef_xyz(26)+coef_xy(2,26)*pol_z(2,0,kg)
                coef_xyz(27)=coef_xyz(27)+coef_xy(1,27)*pol_z(1,0,kg)
                coef_xyz(27)=coef_xyz(27)+coef_xy(2,27)*pol_z(2,0,kg)
                coef_xyz(28)=coef_xyz(28)+coef_xy(1,28)*pol_z(1,0,kg)
                coef_xyz(28)=coef_xyz(28)+coef_xy(2,28)*pol_z(2,0,kg)
                coef_xyz(29)=coef_xyz(29)+coef_xy(1,29)*pol_z(1,0,kg)
                coef_xyz(29)=coef_xyz(29)+coef_xy(2,29)*pol_z(2,0,kg)
                coef_xyz(30)=coef_xyz(30)+coef_xy(1,30)*pol_z(1,0,kg)
                coef_xyz(30)=coef_xyz(30)+coef_xy(2,30)*pol_z(2,0,kg)
                coef_xyz(31)=coef_xyz(31)+coef_xy(1,31)*pol_z(1,0,kg)
                coef_xyz(31)=coef_xyz(31)+coef_xy(2,31)*pol_z(2,0,kg)
                coef_xyz(32)=coef_xyz(32)+coef_xy(1,32)*pol_z(1,0,kg)
                coef_xyz(32)=coef_xyz(32)+coef_xy(2,32)*pol_z(2,0,kg)
                coef_xyz(33)=coef_xyz(33)+coef_xy(1,33)*pol_z(1,0,kg)
                coef_xyz(33)=coef_xyz(33)+coef_xy(2,33)*pol_z(2,0,kg)
                coef_xyz(34)=coef_xyz(34)+coef_xy(1,34)*pol_z(1,0,kg)
                coef_xyz(34)=coef_xyz(34)+coef_xy(2,34)*pol_z(2,0,kg)
                coef_xyz(35)=coef_xyz(35)+coef_xy(1,35)*pol_z(1,0,kg)
                coef_xyz(35)=coef_xyz(35)+coef_xy(2,35)*pol_z(2,0,kg)
                coef_xyz(36)=coef_xyz(36)+coef_xy(1,36)*pol_z(1,0,kg)
                coef_xyz(36)=coef_xyz(36)+coef_xy(2,36)*pol_z(2,0,kg)
                coef_xyz(37)=coef_xyz(37)+coef_xy(1,37)*pol_z(1,0,kg)
                coef_xyz(37)=coef_xyz(37)+coef_xy(2,37)*pol_z(2,0,kg)
                coef_xyz(38)=coef_xyz(38)+coef_xy(1,38)*pol_z(1,0,kg)
                coef_xyz(38)=coef_xyz(38)+coef_xy(2,38)*pol_z(2,0,kg)
                coef_xyz(39)=coef_xyz(39)+coef_xy(1,39)*pol_z(1,0,kg)
                coef_xyz(39)=coef_xyz(39)+coef_xy(2,39)*pol_z(2,0,kg)
                coef_xyz(40)=coef_xyz(40)+coef_xy(1,40)*pol_z(1,0,kg)
                coef_xyz(40)=coef_xyz(40)+coef_xy(2,40)*pol_z(2,0,kg)
                coef_xyz(41)=coef_xyz(41)+coef_xy(1,41)*pol_z(1,0,kg)
                coef_xyz(41)=coef_xyz(41)+coef_xy(2,41)*pol_z(2,0,kg)
                coef_xyz(42)=coef_xyz(42)+coef_xy(1,42)*pol_z(1,0,kg)
                coef_xyz(42)=coef_xyz(42)+coef_xy(2,42)*pol_z(2,0,kg)
                coef_xyz(43)=coef_xyz(43)+coef_xy(1,43)*pol_z(1,0,kg)
                coef_xyz(43)=coef_xyz(43)+coef_xy(2,43)*pol_z(2,0,kg)
                coef_xyz(44)=coef_xyz(44)+coef_xy(1,44)*pol_z(1,0,kg)
                coef_xyz(44)=coef_xyz(44)+coef_xy(2,44)*pol_z(2,0,kg)
                coef_xyz(45)=coef_xyz(45)+coef_xy(1,45)*pol_z(1,0,kg)
                coef_xyz(45)=coef_xyz(45)+coef_xy(2,45)*pol_z(2,0,kg)
                coef_xyz(46)=coef_xyz(46)+coef_xy(1,1)*pol_z(1,1,kg)
                coef_xyz(46)=coef_xyz(46)+coef_xy(2,1)*pol_z(2,1,kg)
                coef_xyz(47)=coef_xyz(47)+coef_xy(1,2)*pol_z(1,1,kg)
                coef_xyz(47)=coef_xyz(47)+coef_xy(2,2)*pol_z(2,1,kg)
                coef_xyz(48)=coef_xyz(48)+coef_xy(1,3)*pol_z(1,1,kg)
                coef_xyz(48)=coef_xyz(48)+coef_xy(2,3)*pol_z(2,1,kg)
                coef_xyz(49)=coef_xyz(49)+coef_xy(1,4)*pol_z(1,1,kg)
                coef_xyz(49)=coef_xyz(49)+coef_xy(2,4)*pol_z(2,1,kg)
                coef_xyz(50)=coef_xyz(50)+coef_xy(1,5)*pol_z(1,1,kg)
                coef_xyz(50)=coef_xyz(50)+coef_xy(2,5)*pol_z(2,1,kg)
                coef_xyz(51)=coef_xyz(51)+coef_xy(1,6)*pol_z(1,1,kg)
                coef_xyz(51)=coef_xyz(51)+coef_xy(2,6)*pol_z(2,1,kg)
                coef_xyz(52)=coef_xyz(52)+coef_xy(1,7)*pol_z(1,1,kg)
                coef_xyz(52)=coef_xyz(52)+coef_xy(2,7)*pol_z(2,1,kg)
                coef_xyz(53)=coef_xyz(53)+coef_xy(1,8)*pol_z(1,1,kg)
                coef_xyz(53)=coef_xyz(53)+coef_xy(2,8)*pol_z(2,1,kg)
                coef_xyz(54)=coef_xyz(54)+coef_xy(1,10)*pol_z(1,1,kg)
                coef_xyz(54)=coef_xyz(54)+coef_xy(2,10)*pol_z(2,1,kg)
                coef_xyz(55)=coef_xyz(55)+coef_xy(1,11)*pol_z(1,1,kg)
                coef_xyz(55)=coef_xyz(55)+coef_xy(2,11)*pol_z(2,1,kg)
                coef_xyz(56)=coef_xyz(56)+coef_xy(1,12)*pol_z(1,1,kg)
                coef_xyz(56)=coef_xyz(56)+coef_xy(2,12)*pol_z(2,1,kg)
                coef_xyz(57)=coef_xyz(57)+coef_xy(1,13)*pol_z(1,1,kg)
                coef_xyz(57)=coef_xyz(57)+coef_xy(2,13)*pol_z(2,1,kg)
                coef_xyz(58)=coef_xyz(58)+coef_xy(1,14)*pol_z(1,1,kg)
                coef_xyz(58)=coef_xyz(58)+coef_xy(2,14)*pol_z(2,1,kg)
                coef_xyz(59)=coef_xyz(59)+coef_xy(1,15)*pol_z(1,1,kg)
                coef_xyz(59)=coef_xyz(59)+coef_xy(2,15)*pol_z(2,1,kg)
                coef_xyz(60)=coef_xyz(60)+coef_xy(1,16)*pol_z(1,1,kg)
                coef_xyz(60)=coef_xyz(60)+coef_xy(2,16)*pol_z(2,1,kg)
                coef_xyz(61)=coef_xyz(61)+coef_xy(1,18)*pol_z(1,1,kg)
                coef_xyz(61)=coef_xyz(61)+coef_xy(2,18)*pol_z(2,1,kg)
                coef_xyz(62)=coef_xyz(62)+coef_xy(1,19)*pol_z(1,1,kg)
                coef_xyz(62)=coef_xyz(62)+coef_xy(2,19)*pol_z(2,1,kg)
                coef_xyz(63)=coef_xyz(63)+coef_xy(1,20)*pol_z(1,1,kg)
                coef_xyz(63)=coef_xyz(63)+coef_xy(2,20)*pol_z(2,1,kg)
                coef_xyz(64)=coef_xyz(64)+coef_xy(1,21)*pol_z(1,1,kg)
                coef_xyz(64)=coef_xyz(64)+coef_xy(2,21)*pol_z(2,1,kg)
                coef_xyz(65)=coef_xyz(65)+coef_xy(1,22)*pol_z(1,1,kg)
                coef_xyz(65)=coef_xyz(65)+coef_xy(2,22)*pol_z(2,1,kg)
                coef_xyz(66)=coef_xyz(66)+coef_xy(1,23)*pol_z(1,1,kg)
                coef_xyz(66)=coef_xyz(66)+coef_xy(2,23)*pol_z(2,1,kg)
                coef_xyz(67)=coef_xyz(67)+coef_xy(1,25)*pol_z(1,1,kg)
                coef_xyz(67)=coef_xyz(67)+coef_xy(2,25)*pol_z(2,1,kg)
                coef_xyz(68)=coef_xyz(68)+coef_xy(1,26)*pol_z(1,1,kg)
                coef_xyz(68)=coef_xyz(68)+coef_xy(2,26)*pol_z(2,1,kg)
                coef_xyz(69)=coef_xyz(69)+coef_xy(1,27)*pol_z(1,1,kg)
                coef_xyz(69)=coef_xyz(69)+coef_xy(2,27)*pol_z(2,1,kg)
                coef_xyz(70)=coef_xyz(70)+coef_xy(1,28)*pol_z(1,1,kg)
                coef_xyz(70)=coef_xyz(70)+coef_xy(2,28)*pol_z(2,1,kg)
                coef_xyz(71)=coef_xyz(71)+coef_xy(1,29)*pol_z(1,1,kg)
                coef_xyz(71)=coef_xyz(71)+coef_xy(2,29)*pol_z(2,1,kg)
                coef_xyz(72)=coef_xyz(72)+coef_xy(1,31)*pol_z(1,1,kg)
                coef_xyz(72)=coef_xyz(72)+coef_xy(2,31)*pol_z(2,1,kg)
                coef_xyz(73)=coef_xyz(73)+coef_xy(1,32)*pol_z(1,1,kg)
                coef_xyz(73)=coef_xyz(73)+coef_xy(2,32)*pol_z(2,1,kg)
                coef_xyz(74)=coef_xyz(74)+coef_xy(1,33)*pol_z(1,1,kg)
                coef_xyz(74)=coef_xyz(74)+coef_xy(2,33)*pol_z(2,1,kg)
                coef_xyz(75)=coef_xyz(75)+coef_xy(1,34)*pol_z(1,1,kg)
                coef_xyz(75)=coef_xyz(75)+coef_xy(2,34)*pol_z(2,1,kg)
                coef_xyz(76)=coef_xyz(76)+coef_xy(1,36)*pol_z(1,1,kg)
                coef_xyz(76)=coef_xyz(76)+coef_xy(2,36)*pol_z(2,1,kg)
                coef_xyz(77)=coef_xyz(77)+coef_xy(1,37)*pol_z(1,1,kg)
                coef_xyz(77)=coef_xyz(77)+coef_xy(2,37)*pol_z(2,1,kg)
                coef_xyz(78)=coef_xyz(78)+coef_xy(1,38)*pol_z(1,1,kg)
                coef_xyz(78)=coef_xyz(78)+coef_xy(2,38)*pol_z(2,1,kg)
                coef_xyz(79)=coef_xyz(79)+coef_xy(1,40)*pol_z(1,1,kg)
                coef_xyz(79)=coef_xyz(79)+coef_xy(2,40)*pol_z(2,1,kg)
                coef_xyz(80)=coef_xyz(80)+coef_xy(1,41)*pol_z(1,1,kg)
                coef_xyz(80)=coef_xyz(80)+coef_xy(2,41)*pol_z(2,1,kg)
                coef_xyz(81)=coef_xyz(81)+coef_xy(1,43)*pol_z(1,1,kg)
                coef_xyz(81)=coef_xyz(81)+coef_xy(2,43)*pol_z(2,1,kg)
                coef_xyz(82)=coef_xyz(82)+coef_xy(1,1)*pol_z(1,2,kg)
                coef_xyz(82)=coef_xyz(82)+coef_xy(2,1)*pol_z(2,2,kg)
                coef_xyz(83)=coef_xyz(83)+coef_xy(1,2)*pol_z(1,2,kg)
                coef_xyz(83)=coef_xyz(83)+coef_xy(2,2)*pol_z(2,2,kg)
                coef_xyz(84)=coef_xyz(84)+coef_xy(1,3)*pol_z(1,2,kg)
                coef_xyz(84)=coef_xyz(84)+coef_xy(2,3)*pol_z(2,2,kg)
                coef_xyz(85)=coef_xyz(85)+coef_xy(1,4)*pol_z(1,2,kg)
                coef_xyz(85)=coef_xyz(85)+coef_xy(2,4)*pol_z(2,2,kg)
                coef_xyz(86)=coef_xyz(86)+coef_xy(1,5)*pol_z(1,2,kg)
                coef_xyz(86)=coef_xyz(86)+coef_xy(2,5)*pol_z(2,2,kg)
                coef_xyz(87)=coef_xyz(87)+coef_xy(1,6)*pol_z(1,2,kg)
                coef_xyz(87)=coef_xyz(87)+coef_xy(2,6)*pol_z(2,2,kg)
                coef_xyz(88)=coef_xyz(88)+coef_xy(1,7)*pol_z(1,2,kg)
                coef_xyz(88)=coef_xyz(88)+coef_xy(2,7)*pol_z(2,2,kg)
                coef_xyz(89)=coef_xyz(89)+coef_xy(1,10)*pol_z(1,2,kg)
                coef_xyz(89)=coef_xyz(89)+coef_xy(2,10)*pol_z(2,2,kg)
                coef_xyz(90)=coef_xyz(90)+coef_xy(1,11)*pol_z(1,2,kg)
                coef_xyz(90)=coef_xyz(90)+coef_xy(2,11)*pol_z(2,2,kg)
                coef_xyz(91)=coef_xyz(91)+coef_xy(1,12)*pol_z(1,2,kg)
                coef_xyz(91)=coef_xyz(91)+coef_xy(2,12)*pol_z(2,2,kg)
                coef_xyz(92)=coef_xyz(92)+coef_xy(1,13)*pol_z(1,2,kg)
                coef_xyz(92)=coef_xyz(92)+coef_xy(2,13)*pol_z(2,2,kg)
                coef_xyz(93)=coef_xyz(93)+coef_xy(1,14)*pol_z(1,2,kg)
                coef_xyz(93)=coef_xyz(93)+coef_xy(2,14)*pol_z(2,2,kg)
                coef_xyz(94)=coef_xyz(94)+coef_xy(1,15)*pol_z(1,2,kg)
                coef_xyz(94)=coef_xyz(94)+coef_xy(2,15)*pol_z(2,2,kg)
                coef_xyz(95)=coef_xyz(95)+coef_xy(1,18)*pol_z(1,2,kg)
                coef_xyz(95)=coef_xyz(95)+coef_xy(2,18)*pol_z(2,2,kg)
                coef_xyz(96)=coef_xyz(96)+coef_xy(1,19)*pol_z(1,2,kg)
                coef_xyz(96)=coef_xyz(96)+coef_xy(2,19)*pol_z(2,2,kg)
                coef_xyz(97)=coef_xyz(97)+coef_xy(1,20)*pol_z(1,2,kg)
                coef_xyz(97)=coef_xyz(97)+coef_xy(2,20)*pol_z(2,2,kg)
                coef_xyz(98)=coef_xyz(98)+coef_xy(1,21)*pol_z(1,2,kg)
                coef_xyz(98)=coef_xyz(98)+coef_xy(2,21)*pol_z(2,2,kg)
                coef_xyz(99)=coef_xyz(99)+coef_xy(1,22)*pol_z(1,2,kg)
                coef_xyz(99)=coef_xyz(99)+coef_xy(2,22)*pol_z(2,2,kg)
                coef_xyz(100)=coef_xyz(100)+coef_xy(1,25)*pol_z(1,2,kg)
                coef_xyz(100)=coef_xyz(100)+coef_xy(2,25)*pol_z(2,2,kg)
                coef_xyz(101)=coef_xyz(101)+coef_xy(1,26)*pol_z(1,2,kg)
                coef_xyz(101)=coef_xyz(101)+coef_xy(2,26)*pol_z(2,2,kg)
                coef_xyz(102)=coef_xyz(102)+coef_xy(1,27)*pol_z(1,2,kg)
                coef_xyz(102)=coef_xyz(102)+coef_xy(2,27)*pol_z(2,2,kg)
                coef_xyz(103)=coef_xyz(103)+coef_xy(1,28)*pol_z(1,2,kg)
                coef_xyz(103)=coef_xyz(103)+coef_xy(2,28)*pol_z(2,2,kg)
                coef_xyz(104)=coef_xyz(104)+coef_xy(1,31)*pol_z(1,2,kg)
                coef_xyz(104)=coef_xyz(104)+coef_xy(2,31)*pol_z(2,2,kg)
                coef_xyz(105)=coef_xyz(105)+coef_xy(1,32)*pol_z(1,2,kg)
                coef_xyz(105)=coef_xyz(105)+coef_xy(2,32)*pol_z(2,2,kg)
                coef_xyz(106)=coef_xyz(106)+coef_xy(1,33)*pol_z(1,2,kg)
                coef_xyz(106)=coef_xyz(106)+coef_xy(2,33)*pol_z(2,2,kg)
                coef_xyz(107)=coef_xyz(107)+coef_xy(1,36)*pol_z(1,2,kg)
                coef_xyz(107)=coef_xyz(107)+coef_xy(2,36)*pol_z(2,2,kg)
                coef_xyz(108)=coef_xyz(108)+coef_xy(1,37)*pol_z(1,2,kg)
                coef_xyz(108)=coef_xyz(108)+coef_xy(2,37)*pol_z(2,2,kg)
                coef_xyz(109)=coef_xyz(109)+coef_xy(1,40)*pol_z(1,2,kg)
                coef_xyz(109)=coef_xyz(109)+coef_xy(2,40)*pol_z(2,2,kg)
                coef_xyz(110)=coef_xyz(110)+coef_xy(1,1)*pol_z(1,3,kg)
                coef_xyz(110)=coef_xyz(110)+coef_xy(2,1)*pol_z(2,3,kg)
                coef_xyz(111)=coef_xyz(111)+coef_xy(1,2)*pol_z(1,3,kg)
                coef_xyz(111)=coef_xyz(111)+coef_xy(2,2)*pol_z(2,3,kg)
                coef_xyz(112)=coef_xyz(112)+coef_xy(1,3)*pol_z(1,3,kg)
                coef_xyz(112)=coef_xyz(112)+coef_xy(2,3)*pol_z(2,3,kg)
                coef_xyz(113)=coef_xyz(113)+coef_xy(1,4)*pol_z(1,3,kg)
                coef_xyz(113)=coef_xyz(113)+coef_xy(2,4)*pol_z(2,3,kg)
                coef_xyz(114)=coef_xyz(114)+coef_xy(1,5)*pol_z(1,3,kg)
                coef_xyz(114)=coef_xyz(114)+coef_xy(2,5)*pol_z(2,3,kg)
                coef_xyz(115)=coef_xyz(115)+coef_xy(1,6)*pol_z(1,3,kg)
                coef_xyz(115)=coef_xyz(115)+coef_xy(2,6)*pol_z(2,3,kg)
                coef_xyz(116)=coef_xyz(116)+coef_xy(1,10)*pol_z(1,3,kg)
                coef_xyz(116)=coef_xyz(116)+coef_xy(2,10)*pol_z(2,3,kg)
                coef_xyz(117)=coef_xyz(117)+coef_xy(1,11)*pol_z(1,3,kg)
                coef_xyz(117)=coef_xyz(117)+coef_xy(2,11)*pol_z(2,3,kg)
                coef_xyz(118)=coef_xyz(118)+coef_xy(1,12)*pol_z(1,3,kg)
                coef_xyz(118)=coef_xyz(118)+coef_xy(2,12)*pol_z(2,3,kg)
                coef_xyz(119)=coef_xyz(119)+coef_xy(1,13)*pol_z(1,3,kg)
                coef_xyz(119)=coef_xyz(119)+coef_xy(2,13)*pol_z(2,3,kg)
                coef_xyz(120)=coef_xyz(120)+coef_xy(1,14)*pol_z(1,3,kg)
                coef_xyz(120)=coef_xyz(120)+coef_xy(2,14)*pol_z(2,3,kg)
                coef_xyz(121)=coef_xyz(121)+coef_xy(1,18)*pol_z(1,3,kg)
                coef_xyz(121)=coef_xyz(121)+coef_xy(2,18)*pol_z(2,3,kg)
                coef_xyz(122)=coef_xyz(122)+coef_xy(1,19)*pol_z(1,3,kg)
                coef_xyz(122)=coef_xyz(122)+coef_xy(2,19)*pol_z(2,3,kg)
                coef_xyz(123)=coef_xyz(123)+coef_xy(1,20)*pol_z(1,3,kg)
                coef_xyz(123)=coef_xyz(123)+coef_xy(2,20)*pol_z(2,3,kg)
                coef_xyz(124)=coef_xyz(124)+coef_xy(1,21)*pol_z(1,3,kg)
                coef_xyz(124)=coef_xyz(124)+coef_xy(2,21)*pol_z(2,3,kg)
                coef_xyz(125)=coef_xyz(125)+coef_xy(1,25)*pol_z(1,3,kg)
                coef_xyz(125)=coef_xyz(125)+coef_xy(2,25)*pol_z(2,3,kg)
                coef_xyz(126)=coef_xyz(126)+coef_xy(1,26)*pol_z(1,3,kg)
                coef_xyz(126)=coef_xyz(126)+coef_xy(2,26)*pol_z(2,3,kg)
                coef_xyz(127)=coef_xyz(127)+coef_xy(1,27)*pol_z(1,3,kg)
                coef_xyz(127)=coef_xyz(127)+coef_xy(2,27)*pol_z(2,3,kg)
                coef_xyz(128)=coef_xyz(128)+coef_xy(1,31)*pol_z(1,3,kg)
                coef_xyz(128)=coef_xyz(128)+coef_xy(2,31)*pol_z(2,3,kg)
                coef_xyz(129)=coef_xyz(129)+coef_xy(1,32)*pol_z(1,3,kg)
                coef_xyz(129)=coef_xyz(129)+coef_xy(2,32)*pol_z(2,3,kg)
                coef_xyz(130)=coef_xyz(130)+coef_xy(1,36)*pol_z(1,3,kg)
                coef_xyz(130)=coef_xyz(130)+coef_xy(2,36)*pol_z(2,3,kg)
                coef_xyz(131)=coef_xyz(131)+coef_xy(1,1)*pol_z(1,4,kg)
                coef_xyz(131)=coef_xyz(131)+coef_xy(2,1)*pol_z(2,4,kg)
                coef_xyz(132)=coef_xyz(132)+coef_xy(1,2)*pol_z(1,4,kg)
                coef_xyz(132)=coef_xyz(132)+coef_xy(2,2)*pol_z(2,4,kg)
                coef_xyz(133)=coef_xyz(133)+coef_xy(1,3)*pol_z(1,4,kg)
                coef_xyz(133)=coef_xyz(133)+coef_xy(2,3)*pol_z(2,4,kg)
                coef_xyz(134)=coef_xyz(134)+coef_xy(1,4)*pol_z(1,4,kg)
                coef_xyz(134)=coef_xyz(134)+coef_xy(2,4)*pol_z(2,4,kg)
                coef_xyz(135)=coef_xyz(135)+coef_xy(1,5)*pol_z(1,4,kg)
                coef_xyz(135)=coef_xyz(135)+coef_xy(2,5)*pol_z(2,4,kg)
                coef_xyz(136)=coef_xyz(136)+coef_xy(1,10)*pol_z(1,4,kg)
                coef_xyz(136)=coef_xyz(136)+coef_xy(2,10)*pol_z(2,4,kg)
                coef_xyz(137)=coef_xyz(137)+coef_xy(1,11)*pol_z(1,4,kg)
                coef_xyz(137)=coef_xyz(137)+coef_xy(2,11)*pol_z(2,4,kg)
                coef_xyz(138)=coef_xyz(138)+coef_xy(1,12)*pol_z(1,4,kg)
                coef_xyz(138)=coef_xyz(138)+coef_xy(2,12)*pol_z(2,4,kg)
                coef_xyz(139)=coef_xyz(139)+coef_xy(1,13)*pol_z(1,4,kg)
                coef_xyz(139)=coef_xyz(139)+coef_xy(2,13)*pol_z(2,4,kg)
                coef_xyz(140)=coef_xyz(140)+coef_xy(1,18)*pol_z(1,4,kg)
                coef_xyz(140)=coef_xyz(140)+coef_xy(2,18)*pol_z(2,4,kg)
                coef_xyz(141)=coef_xyz(141)+coef_xy(1,19)*pol_z(1,4,kg)
                coef_xyz(141)=coef_xyz(141)+coef_xy(2,19)*pol_z(2,4,kg)
                coef_xyz(142)=coef_xyz(142)+coef_xy(1,20)*pol_z(1,4,kg)
                coef_xyz(142)=coef_xyz(142)+coef_xy(2,20)*pol_z(2,4,kg)
                coef_xyz(143)=coef_xyz(143)+coef_xy(1,25)*pol_z(1,4,kg)
                coef_xyz(143)=coef_xyz(143)+coef_xy(2,25)*pol_z(2,4,kg)
                coef_xyz(144)=coef_xyz(144)+coef_xy(1,26)*pol_z(1,4,kg)
                coef_xyz(144)=coef_xyz(144)+coef_xy(2,26)*pol_z(2,4,kg)
                coef_xyz(145)=coef_xyz(145)+coef_xy(1,31)*pol_z(1,4,kg)
                coef_xyz(145)=coef_xyz(145)+coef_xy(2,31)*pol_z(2,4,kg)
                coef_xyz(146)=coef_xyz(146)+coef_xy(1,1)*pol_z(1,5,kg)
                coef_xyz(146)=coef_xyz(146)+coef_xy(2,1)*pol_z(2,5,kg)
                coef_xyz(147)=coef_xyz(147)+coef_xy(1,2)*pol_z(1,5,kg)
                coef_xyz(147)=coef_xyz(147)+coef_xy(2,2)*pol_z(2,5,kg)
                coef_xyz(148)=coef_xyz(148)+coef_xy(1,3)*pol_z(1,5,kg)
                coef_xyz(148)=coef_xyz(148)+coef_xy(2,3)*pol_z(2,5,kg)
                coef_xyz(149)=coef_xyz(149)+coef_xy(1,4)*pol_z(1,5,kg)
                coef_xyz(149)=coef_xyz(149)+coef_xy(2,4)*pol_z(2,5,kg)
                coef_xyz(150)=coef_xyz(150)+coef_xy(1,10)*pol_z(1,5,kg)
                coef_xyz(150)=coef_xyz(150)+coef_xy(2,10)*pol_z(2,5,kg)
                coef_xyz(151)=coef_xyz(151)+coef_xy(1,11)*pol_z(1,5,kg)
                coef_xyz(151)=coef_xyz(151)+coef_xy(2,11)*pol_z(2,5,kg)
                coef_xyz(152)=coef_xyz(152)+coef_xy(1,12)*pol_z(1,5,kg)
                coef_xyz(152)=coef_xyz(152)+coef_xy(2,12)*pol_z(2,5,kg)
                coef_xyz(153)=coef_xyz(153)+coef_xy(1,18)*pol_z(1,5,kg)
                coef_xyz(153)=coef_xyz(153)+coef_xy(2,18)*pol_z(2,5,kg)
                coef_xyz(154)=coef_xyz(154)+coef_xy(1,19)*pol_z(1,5,kg)
                coef_xyz(154)=coef_xyz(154)+coef_xy(2,19)*pol_z(2,5,kg)
                coef_xyz(155)=coef_xyz(155)+coef_xy(1,25)*pol_z(1,5,kg)
                coef_xyz(155)=coef_xyz(155)+coef_xy(2,25)*pol_z(2,5,kg)
                coef_xyz(156)=coef_xyz(156)+coef_xy(1,1)*pol_z(1,6,kg)
                coef_xyz(156)=coef_xyz(156)+coef_xy(2,1)*pol_z(2,6,kg)
                coef_xyz(157)=coef_xyz(157)+coef_xy(1,2)*pol_z(1,6,kg)
                coef_xyz(157)=coef_xyz(157)+coef_xy(2,2)*pol_z(2,6,kg)
                coef_xyz(158)=coef_xyz(158)+coef_xy(1,3)*pol_z(1,6,kg)
                coef_xyz(158)=coef_xyz(158)+coef_xy(2,3)*pol_z(2,6,kg)
                coef_xyz(159)=coef_xyz(159)+coef_xy(1,10)*pol_z(1,6,kg)
                coef_xyz(159)=coef_xyz(159)+coef_xy(2,10)*pol_z(2,6,kg)
                coef_xyz(160)=coef_xyz(160)+coef_xy(1,11)*pol_z(1,6,kg)
                coef_xyz(160)=coef_xyz(160)+coef_xy(2,11)*pol_z(2,6,kg)
                coef_xyz(161)=coef_xyz(161)+coef_xy(1,18)*pol_z(1,6,kg)
                coef_xyz(161)=coef_xyz(161)+coef_xy(2,18)*pol_z(2,6,kg)
                coef_xyz(162)=coef_xyz(162)+coef_xy(1,1)*pol_z(1,7,kg)
                coef_xyz(162)=coef_xyz(162)+coef_xy(2,1)*pol_z(2,7,kg)
                coef_xyz(163)=coef_xyz(163)+coef_xy(1,2)*pol_z(1,7,kg)
                coef_xyz(163)=coef_xyz(163)+coef_xy(2,2)*pol_z(2,7,kg)
                coef_xyz(164)=coef_xyz(164)+coef_xy(1,10)*pol_z(1,7,kg)
                coef_xyz(164)=coef_xyz(164)+coef_xy(2,10)*pol_z(2,7,kg)
                coef_xyz(165)=coef_xyz(165)+coef_xy(1,1)*pol_z(1,8,kg)
                coef_xyz(165)=coef_xyz(165)+coef_xy(2,1)*pol_z(2,8,kg)
    END DO

  END SUBROUTINE integrate_core_8
  SUBROUTINE integrate_core_9(grid,coef_xyz,pol_x,pol_y,pol_z,map,sphere_bounds,cmax,gridbounds)
    USE kinds,                        ONLY: dp
    INTEGER, INTENT(IN)                      :: sphere_bounds(*), cmax, &
                                                map(-cmax:cmax,1:3), &
                                                gridbounds(2,3)
    REAL(dp), INTENT(IN) :: grid(gridbounds(1,1):gridbounds(2,1), gridbounds(1&
      ,2):gridbounds(2,2), gridbounds(1,3):gridbounds(2,3))
    INTEGER, PARAMETER                       :: lp = 9
    REAL(dp), INTENT(IN)                     :: pol_x(0:lp,-cmax:cmax), &
                                                pol_y(1:2,0:lp,-cmax:0), &
                                                pol_z(1:2,0:lp,-cmax:0)
    REAL(dp), INTENT(OUT) :: coef_xyz(((lp+1)*(lp+2)*(lp+3))/6)

    INTEGER                                  :: i, ig, igmax, igmin, j, j2, &
                                                jg, jg2, jgmin, k, k2, kg, &
                                                kg2, kgmin, lxp, sci
    REAL(dp)                                 :: coef_x(4,0:lp), &
                                                coef_xy(2,((lp+1)*(lp+2))/2), &
                                                s01, s02, s03, s04

    sci=1

    coef_xyz=0.0_dp

    kgmin=sphere_bounds(sci)
    sci=sci+1
    DO kg=kgmin,0
       kg2=1-kg
       k=map(kg,3)
       k2=map(kg2,3)

       coef_xy=0.0_dp

       jgmin=sphere_bounds(sci)
       sci=sci+1
       DO jg=jgmin,0
          jg2=1-jg
          j=map(jg,2)
          j2=map(jg2,2)
          igmin=sphere_bounds(sci)
          sci=sci+1
          igmax=1-igmin

          coef_x=0.0_dp
          DO ig=igmin,igmax
             i=map(ig,1)
             s01=grid(i,j,k)
             s02=grid(i,j,k2)
             s03=grid(i,j2,k)
             s04=grid(i,j2,k2)
             DO lxp=0,lp
                coef_x(1,lxp)=coef_x(1,lxp)+s01*pol_x(lxp,ig)
                coef_x(2,lxp)=coef_x(2,lxp)+s02*pol_x(lxp,ig)
                coef_x(3,lxp)=coef_x(3,lxp)+s03*pol_x(lxp,ig)
                coef_x(4,lxp)=coef_x(4,lxp)+s04*pol_x(lxp,ig)
             ENDDO
          END DO
             coef_xy(1,1)=coef_xy(1,1)+coef_x(1,0)*pol_y(1,0,jg)
             coef_xy(2,1)=coef_xy(2,1)+coef_x(2,0)*pol_y(1,0,jg)
             coef_xy(1,1)=coef_xy(1,1)+coef_x(3,0)*pol_y(2,0,jg)
             coef_xy(2,1)=coef_xy(2,1)+coef_x(4,0)*pol_y(2,0,jg)
             coef_xy(1,2)=coef_xy(1,2)+coef_x(1,1)*pol_y(1,0,jg)
             coef_xy(2,2)=coef_xy(2,2)+coef_x(2,1)*pol_y(1,0,jg)
             coef_xy(1,2)=coef_xy(1,2)+coef_x(3,1)*pol_y(2,0,jg)
             coef_xy(2,2)=coef_xy(2,2)+coef_x(4,1)*pol_y(2,0,jg)
             coef_xy(1,3)=coef_xy(1,3)+coef_x(1,2)*pol_y(1,0,jg)
             coef_xy(2,3)=coef_xy(2,3)+coef_x(2,2)*pol_y(1,0,jg)
             coef_xy(1,3)=coef_xy(1,3)+coef_x(3,2)*pol_y(2,0,jg)
             coef_xy(2,3)=coef_xy(2,3)+coef_x(4,2)*pol_y(2,0,jg)
             coef_xy(1,4)=coef_xy(1,4)+coef_x(1,3)*pol_y(1,0,jg)
             coef_xy(2,4)=coef_xy(2,4)+coef_x(2,3)*pol_y(1,0,jg)
             coef_xy(1,4)=coef_xy(1,4)+coef_x(3,3)*pol_y(2,0,jg)
             coef_xy(2,4)=coef_xy(2,4)+coef_x(4,3)*pol_y(2,0,jg)
             coef_xy(1,5)=coef_xy(1,5)+coef_x(1,4)*pol_y(1,0,jg)
             coef_xy(2,5)=coef_xy(2,5)+coef_x(2,4)*pol_y(1,0,jg)
             coef_xy(1,5)=coef_xy(1,5)+coef_x(3,4)*pol_y(2,0,jg)
             coef_xy(2,5)=coef_xy(2,5)+coef_x(4,4)*pol_y(2,0,jg)
             coef_xy(1,6)=coef_xy(1,6)+coef_x(1,5)*pol_y(1,0,jg)
             coef_xy(2,6)=coef_xy(2,6)+coef_x(2,5)*pol_y(1,0,jg)
             coef_xy(1,6)=coef_xy(1,6)+coef_x(3,5)*pol_y(2,0,jg)
             coef_xy(2,6)=coef_xy(2,6)+coef_x(4,5)*pol_y(2,0,jg)
             coef_xy(1,7)=coef_xy(1,7)+coef_x(1,6)*pol_y(1,0,jg)
             coef_xy(2,7)=coef_xy(2,7)+coef_x(2,6)*pol_y(1,0,jg)
             coef_xy(1,7)=coef_xy(1,7)+coef_x(3,6)*pol_y(2,0,jg)
             coef_xy(2,7)=coef_xy(2,7)+coef_x(4,6)*pol_y(2,0,jg)
             coef_xy(1,8)=coef_xy(1,8)+coef_x(1,7)*pol_y(1,0,jg)
             coef_xy(2,8)=coef_xy(2,8)+coef_x(2,7)*pol_y(1,0,jg)
             coef_xy(1,8)=coef_xy(1,8)+coef_x(3,7)*pol_y(2,0,jg)
             coef_xy(2,8)=coef_xy(2,8)+coef_x(4,7)*pol_y(2,0,jg)
             coef_xy(1,9)=coef_xy(1,9)+coef_x(1,8)*pol_y(1,0,jg)
             coef_xy(2,9)=coef_xy(2,9)+coef_x(2,8)*pol_y(1,0,jg)
             coef_xy(1,9)=coef_xy(1,9)+coef_x(3,8)*pol_y(2,0,jg)
             coef_xy(2,9)=coef_xy(2,9)+coef_x(4,8)*pol_y(2,0,jg)
             coef_xy(1,10)=coef_xy(1,10)+coef_x(1,9)*pol_y(1,0,jg)
             coef_xy(2,10)=coef_xy(2,10)+coef_x(2,9)*pol_y(1,0,jg)
             coef_xy(1,10)=coef_xy(1,10)+coef_x(3,9)*pol_y(2,0,jg)
             coef_xy(2,10)=coef_xy(2,10)+coef_x(4,9)*pol_y(2,0,jg)
             coef_xy(1,11)=coef_xy(1,11)+coef_x(1,0)*pol_y(1,1,jg)
             coef_xy(2,11)=coef_xy(2,11)+coef_x(2,0)*pol_y(1,1,jg)
             coef_xy(1,11)=coef_xy(1,11)+coef_x(3,0)*pol_y(2,1,jg)
             coef_xy(2,11)=coef_xy(2,11)+coef_x(4,0)*pol_y(2,1,jg)
             coef_xy(1,12)=coef_xy(1,12)+coef_x(1,1)*pol_y(1,1,jg)
             coef_xy(2,12)=coef_xy(2,12)+coef_x(2,1)*pol_y(1,1,jg)
             coef_xy(1,12)=coef_xy(1,12)+coef_x(3,1)*pol_y(2,1,jg)
             coef_xy(2,12)=coef_xy(2,12)+coef_x(4,1)*pol_y(2,1,jg)
             coef_xy(1,13)=coef_xy(1,13)+coef_x(1,2)*pol_y(1,1,jg)
             coef_xy(2,13)=coef_xy(2,13)+coef_x(2,2)*pol_y(1,1,jg)
             coef_xy(1,13)=coef_xy(1,13)+coef_x(3,2)*pol_y(2,1,jg)
             coef_xy(2,13)=coef_xy(2,13)+coef_x(4,2)*pol_y(2,1,jg)
             coef_xy(1,14)=coef_xy(1,14)+coef_x(1,3)*pol_y(1,1,jg)
             coef_xy(2,14)=coef_xy(2,14)+coef_x(2,3)*pol_y(1,1,jg)
             coef_xy(1,14)=coef_xy(1,14)+coef_x(3,3)*pol_y(2,1,jg)
             coef_xy(2,14)=coef_xy(2,14)+coef_x(4,3)*pol_y(2,1,jg)
             coef_xy(1,15)=coef_xy(1,15)+coef_x(1,4)*pol_y(1,1,jg)
             coef_xy(2,15)=coef_xy(2,15)+coef_x(2,4)*pol_y(1,1,jg)
             coef_xy(1,15)=coef_xy(1,15)+coef_x(3,4)*pol_y(2,1,jg)
             coef_xy(2,15)=coef_xy(2,15)+coef_x(4,4)*pol_y(2,1,jg)
             coef_xy(1,16)=coef_xy(1,16)+coef_x(1,5)*pol_y(1,1,jg)
             coef_xy(2,16)=coef_xy(2,16)+coef_x(2,5)*pol_y(1,1,jg)
             coef_xy(1,16)=coef_xy(1,16)+coef_x(3,5)*pol_y(2,1,jg)
             coef_xy(2,16)=coef_xy(2,16)+coef_x(4,5)*pol_y(2,1,jg)
             coef_xy(1,17)=coef_xy(1,17)+coef_x(1,6)*pol_y(1,1,jg)
             coef_xy(2,17)=coef_xy(2,17)+coef_x(2,6)*pol_y(1,1,jg)
             coef_xy(1,17)=coef_xy(1,17)+coef_x(3,6)*pol_y(2,1,jg)
             coef_xy(2,17)=coef_xy(2,17)+coef_x(4,6)*pol_y(2,1,jg)
             coef_xy(1,18)=coef_xy(1,18)+coef_x(1,7)*pol_y(1,1,jg)
             coef_xy(2,18)=coef_xy(2,18)+coef_x(2,7)*pol_y(1,1,jg)
             coef_xy(1,18)=coef_xy(1,18)+coef_x(3,7)*pol_y(2,1,jg)
             coef_xy(2,18)=coef_xy(2,18)+coef_x(4,7)*pol_y(2,1,jg)
             coef_xy(1,19)=coef_xy(1,19)+coef_x(1,8)*pol_y(1,1,jg)
             coef_xy(2,19)=coef_xy(2,19)+coef_x(2,8)*pol_y(1,1,jg)
             coef_xy(1,19)=coef_xy(1,19)+coef_x(3,8)*pol_y(2,1,jg)
             coef_xy(2,19)=coef_xy(2,19)+coef_x(4,8)*pol_y(2,1,jg)
             coef_xy(1,20)=coef_xy(1,20)+coef_x(1,0)*pol_y(1,2,jg)
             coef_xy(2,20)=coef_xy(2,20)+coef_x(2,0)*pol_y(1,2,jg)
             coef_xy(1,20)=coef_xy(1,20)+coef_x(3,0)*pol_y(2,2,jg)
             coef_xy(2,20)=coef_xy(2,20)+coef_x(4,0)*pol_y(2,2,jg)
             coef_xy(1,21)=coef_xy(1,21)+coef_x(1,1)*pol_y(1,2,jg)
             coef_xy(2,21)=coef_xy(2,21)+coef_x(2,1)*pol_y(1,2,jg)
             coef_xy(1,21)=coef_xy(1,21)+coef_x(3,1)*pol_y(2,2,jg)
             coef_xy(2,21)=coef_xy(2,21)+coef_x(4,1)*pol_y(2,2,jg)
             coef_xy(1,22)=coef_xy(1,22)+coef_x(1,2)*pol_y(1,2,jg)
             coef_xy(2,22)=coef_xy(2,22)+coef_x(2,2)*pol_y(1,2,jg)
             coef_xy(1,22)=coef_xy(1,22)+coef_x(3,2)*pol_y(2,2,jg)
             coef_xy(2,22)=coef_xy(2,22)+coef_x(4,2)*pol_y(2,2,jg)
             coef_xy(1,23)=coef_xy(1,23)+coef_x(1,3)*pol_y(1,2,jg)
             coef_xy(2,23)=coef_xy(2,23)+coef_x(2,3)*pol_y(1,2,jg)
             coef_xy(1,23)=coef_xy(1,23)+coef_x(3,3)*pol_y(2,2,jg)
             coef_xy(2,23)=coef_xy(2,23)+coef_x(4,3)*pol_y(2,2,jg)
             coef_xy(1,24)=coef_xy(1,24)+coef_x(1,4)*pol_y(1,2,jg)
             coef_xy(2,24)=coef_xy(2,24)+coef_x(2,4)*pol_y(1,2,jg)
             coef_xy(1,24)=coef_xy(1,24)+coef_x(3,4)*pol_y(2,2,jg)
             coef_xy(2,24)=coef_xy(2,24)+coef_x(4,4)*pol_y(2,2,jg)
             coef_xy(1,25)=coef_xy(1,25)+coef_x(1,5)*pol_y(1,2,jg)
             coef_xy(2,25)=coef_xy(2,25)+coef_x(2,5)*pol_y(1,2,jg)
             coef_xy(1,25)=coef_xy(1,25)+coef_x(3,5)*pol_y(2,2,jg)
             coef_xy(2,25)=coef_xy(2,25)+coef_x(4,5)*pol_y(2,2,jg)
             coef_xy(1,26)=coef_xy(1,26)+coef_x(1,6)*pol_y(1,2,jg)
             coef_xy(2,26)=coef_xy(2,26)+coef_x(2,6)*pol_y(1,2,jg)
             coef_xy(1,26)=coef_xy(1,26)+coef_x(3,6)*pol_y(2,2,jg)
             coef_xy(2,26)=coef_xy(2,26)+coef_x(4,6)*pol_y(2,2,jg)
             coef_xy(1,27)=coef_xy(1,27)+coef_x(1,7)*pol_y(1,2,jg)
             coef_xy(2,27)=coef_xy(2,27)+coef_x(2,7)*pol_y(1,2,jg)
             coef_xy(1,27)=coef_xy(1,27)+coef_x(3,7)*pol_y(2,2,jg)
             coef_xy(2,27)=coef_xy(2,27)+coef_x(4,7)*pol_y(2,2,jg)
             coef_xy(1,28)=coef_xy(1,28)+coef_x(1,0)*pol_y(1,3,jg)
             coef_xy(2,28)=coef_xy(2,28)+coef_x(2,0)*pol_y(1,3,jg)
             coef_xy(1,28)=coef_xy(1,28)+coef_x(3,0)*pol_y(2,3,jg)
             coef_xy(2,28)=coef_xy(2,28)+coef_x(4,0)*pol_y(2,3,jg)
             coef_xy(1,29)=coef_xy(1,29)+coef_x(1,1)*pol_y(1,3,jg)
             coef_xy(2,29)=coef_xy(2,29)+coef_x(2,1)*pol_y(1,3,jg)
             coef_xy(1,29)=coef_xy(1,29)+coef_x(3,1)*pol_y(2,3,jg)
             coef_xy(2,29)=coef_xy(2,29)+coef_x(4,1)*pol_y(2,3,jg)
             coef_xy(1,30)=coef_xy(1,30)+coef_x(1,2)*pol_y(1,3,jg)
             coef_xy(2,30)=coef_xy(2,30)+coef_x(2,2)*pol_y(1,3,jg)
             coef_xy(1,30)=coef_xy(1,30)+coef_x(3,2)*pol_y(2,3,jg)
             coef_xy(2,30)=coef_xy(2,30)+coef_x(4,2)*pol_y(2,3,jg)
             coef_xy(1,31)=coef_xy(1,31)+coef_x(1,3)*pol_y(1,3,jg)
             coef_xy(2,31)=coef_xy(2,31)+coef_x(2,3)*pol_y(1,3,jg)
             coef_xy(1,31)=coef_xy(1,31)+coef_x(3,3)*pol_y(2,3,jg)
             coef_xy(2,31)=coef_xy(2,31)+coef_x(4,3)*pol_y(2,3,jg)
             coef_xy(1,32)=coef_xy(1,32)+coef_x(1,4)*pol_y(1,3,jg)
             coef_xy(2,32)=coef_xy(2,32)+coef_x(2,4)*pol_y(1,3,jg)
             coef_xy(1,32)=coef_xy(1,32)+coef_x(3,4)*pol_y(2,3,jg)
             coef_xy(2,32)=coef_xy(2,32)+coef_x(4,4)*pol_y(2,3,jg)
             coef_xy(1,33)=coef_xy(1,33)+coef_x(1,5)*pol_y(1,3,jg)
             coef_xy(2,33)=coef_xy(2,33)+coef_x(2,5)*pol_y(1,3,jg)
             coef_xy(1,33)=coef_xy(1,33)+coef_x(3,5)*pol_y(2,3,jg)
             coef_xy(2,33)=coef_xy(2,33)+coef_x(4,5)*pol_y(2,3,jg)
             coef_xy(1,34)=coef_xy(1,34)+coef_x(1,6)*pol_y(1,3,jg)
             coef_xy(2,34)=coef_xy(2,34)+coef_x(2,6)*pol_y(1,3,jg)
             coef_xy(1,34)=coef_xy(1,34)+coef_x(3,6)*pol_y(2,3,jg)
             coef_xy(2,34)=coef_xy(2,34)+coef_x(4,6)*pol_y(2,3,jg)
             coef_xy(1,35)=coef_xy(1,35)+coef_x(1,0)*pol_y(1,4,jg)
             coef_xy(2,35)=coef_xy(2,35)+coef_x(2,0)*pol_y(1,4,jg)
             coef_xy(1,35)=coef_xy(1,35)+coef_x(3,0)*pol_y(2,4,jg)
             coef_xy(2,35)=coef_xy(2,35)+coef_x(4,0)*pol_y(2,4,jg)
             coef_xy(1,36)=coef_xy(1,36)+coef_x(1,1)*pol_y(1,4,jg)
             coef_xy(2,36)=coef_xy(2,36)+coef_x(2,1)*pol_y(1,4,jg)
             coef_xy(1,36)=coef_xy(1,36)+coef_x(3,1)*pol_y(2,4,jg)
             coef_xy(2,36)=coef_xy(2,36)+coef_x(4,1)*pol_y(2,4,jg)
             coef_xy(1,37)=coef_xy(1,37)+coef_x(1,2)*pol_y(1,4,jg)
             coef_xy(2,37)=coef_xy(2,37)+coef_x(2,2)*pol_y(1,4,jg)
             coef_xy(1,37)=coef_xy(1,37)+coef_x(3,2)*pol_y(2,4,jg)
             coef_xy(2,37)=coef_xy(2,37)+coef_x(4,2)*pol_y(2,4,jg)
             coef_xy(1,38)=coef_xy(1,38)+coef_x(1,3)*pol_y(1,4,jg)
             coef_xy(2,38)=coef_xy(2,38)+coef_x(2,3)*pol_y(1,4,jg)
             coef_xy(1,38)=coef_xy(1,38)+coef_x(3,3)*pol_y(2,4,jg)
             coef_xy(2,38)=coef_xy(2,38)+coef_x(4,3)*pol_y(2,4,jg)
             coef_xy(1,39)=coef_xy(1,39)+coef_x(1,4)*pol_y(1,4,jg)
             coef_xy(2,39)=coef_xy(2,39)+coef_x(2,4)*pol_y(1,4,jg)
             coef_xy(1,39)=coef_xy(1,39)+coef_x(3,4)*pol_y(2,4,jg)
             coef_xy(2,39)=coef_xy(2,39)+coef_x(4,4)*pol_y(2,4,jg)
             coef_xy(1,40)=coef_xy(1,40)+coef_x(1,5)*pol_y(1,4,jg)
             coef_xy(2,40)=coef_xy(2,40)+coef_x(2,5)*pol_y(1,4,jg)
             coef_xy(1,40)=coef_xy(1,40)+coef_x(3,5)*pol_y(2,4,jg)
             coef_xy(2,40)=coef_xy(2,40)+coef_x(4,5)*pol_y(2,4,jg)
             coef_xy(1,41)=coef_xy(1,41)+coef_x(1,0)*pol_y(1,5,jg)
             coef_xy(2,41)=coef_xy(2,41)+coef_x(2,0)*pol_y(1,5,jg)
             coef_xy(1,41)=coef_xy(1,41)+coef_x(3,0)*pol_y(2,5,jg)
             coef_xy(2,41)=coef_xy(2,41)+coef_x(4,0)*pol_y(2,5,jg)
             coef_xy(1,42)=coef_xy(1,42)+coef_x(1,1)*pol_y(1,5,jg)
             coef_xy(2,42)=coef_xy(2,42)+coef_x(2,1)*pol_y(1,5,jg)
             coef_xy(1,42)=coef_xy(1,42)+coef_x(3,1)*pol_y(2,5,jg)
             coef_xy(2,42)=coef_xy(2,42)+coef_x(4,1)*pol_y(2,5,jg)
             coef_xy(1,43)=coef_xy(1,43)+coef_x(1,2)*pol_y(1,5,jg)
             coef_xy(2,43)=coef_xy(2,43)+coef_x(2,2)*pol_y(1,5,jg)
             coef_xy(1,43)=coef_xy(1,43)+coef_x(3,2)*pol_y(2,5,jg)
             coef_xy(2,43)=coef_xy(2,43)+coef_x(4,2)*pol_y(2,5,jg)
             coef_xy(1,44)=coef_xy(1,44)+coef_x(1,3)*pol_y(1,5,jg)
             coef_xy(2,44)=coef_xy(2,44)+coef_x(2,3)*pol_y(1,5,jg)
             coef_xy(1,44)=coef_xy(1,44)+coef_x(3,3)*pol_y(2,5,jg)
             coef_xy(2,44)=coef_xy(2,44)+coef_x(4,3)*pol_y(2,5,jg)
             coef_xy(1,45)=coef_xy(1,45)+coef_x(1,4)*pol_y(1,5,jg)
             coef_xy(2,45)=coef_xy(2,45)+coef_x(2,4)*pol_y(1,5,jg)
             coef_xy(1,45)=coef_xy(1,45)+coef_x(3,4)*pol_y(2,5,jg)
             coef_xy(2,45)=coef_xy(2,45)+coef_x(4,4)*pol_y(2,5,jg)
             coef_xy(1,46)=coef_xy(1,46)+coef_x(1,0)*pol_y(1,6,jg)
             coef_xy(2,46)=coef_xy(2,46)+coef_x(2,0)*pol_y(1,6,jg)
             coef_xy(1,46)=coef_xy(1,46)+coef_x(3,0)*pol_y(2,6,jg)
             coef_xy(2,46)=coef_xy(2,46)+coef_x(4,0)*pol_y(2,6,jg)
             coef_xy(1,47)=coef_xy(1,47)+coef_x(1,1)*pol_y(1,6,jg)
             coef_xy(2,47)=coef_xy(2,47)+coef_x(2,1)*pol_y(1,6,jg)
             coef_xy(1,47)=coef_xy(1,47)+coef_x(3,1)*pol_y(2,6,jg)
             coef_xy(2,47)=coef_xy(2,47)+coef_x(4,1)*pol_y(2,6,jg)
             coef_xy(1,48)=coef_xy(1,48)+coef_x(1,2)*pol_y(1,6,jg)
             coef_xy(2,48)=coef_xy(2,48)+coef_x(2,2)*pol_y(1,6,jg)
             coef_xy(1,48)=coef_xy(1,48)+coef_x(3,2)*pol_y(2,6,jg)
             coef_xy(2,48)=coef_xy(2,48)+coef_x(4,2)*pol_y(2,6,jg)
             coef_xy(1,49)=coef_xy(1,49)+coef_x(1,3)*pol_y(1,6,jg)
             coef_xy(2,49)=coef_xy(2,49)+coef_x(2,3)*pol_y(1,6,jg)
             coef_xy(1,49)=coef_xy(1,49)+coef_x(3,3)*pol_y(2,6,jg)
             coef_xy(2,49)=coef_xy(2,49)+coef_x(4,3)*pol_y(2,6,jg)
             coef_xy(1,50)=coef_xy(1,50)+coef_x(1,0)*pol_y(1,7,jg)
             coef_xy(2,50)=coef_xy(2,50)+coef_x(2,0)*pol_y(1,7,jg)
             coef_xy(1,50)=coef_xy(1,50)+coef_x(3,0)*pol_y(2,7,jg)
             coef_xy(2,50)=coef_xy(2,50)+coef_x(4,0)*pol_y(2,7,jg)
             coef_xy(1,51)=coef_xy(1,51)+coef_x(1,1)*pol_y(1,7,jg)
             coef_xy(2,51)=coef_xy(2,51)+coef_x(2,1)*pol_y(1,7,jg)
             coef_xy(1,51)=coef_xy(1,51)+coef_x(3,1)*pol_y(2,7,jg)
             coef_xy(2,51)=coef_xy(2,51)+coef_x(4,1)*pol_y(2,7,jg)
             coef_xy(1,52)=coef_xy(1,52)+coef_x(1,2)*pol_y(1,7,jg)
             coef_xy(2,52)=coef_xy(2,52)+coef_x(2,2)*pol_y(1,7,jg)
             coef_xy(1,52)=coef_xy(1,52)+coef_x(3,2)*pol_y(2,7,jg)
             coef_xy(2,52)=coef_xy(2,52)+coef_x(4,2)*pol_y(2,7,jg)
             coef_xy(1,53)=coef_xy(1,53)+coef_x(1,0)*pol_y(1,8,jg)
             coef_xy(2,53)=coef_xy(2,53)+coef_x(2,0)*pol_y(1,8,jg)
             coef_xy(1,53)=coef_xy(1,53)+coef_x(3,0)*pol_y(2,8,jg)
             coef_xy(2,53)=coef_xy(2,53)+coef_x(4,0)*pol_y(2,8,jg)
             coef_xy(1,54)=coef_xy(1,54)+coef_x(1,1)*pol_y(1,8,jg)
             coef_xy(2,54)=coef_xy(2,54)+coef_x(2,1)*pol_y(1,8,jg)
             coef_xy(1,54)=coef_xy(1,54)+coef_x(3,1)*pol_y(2,8,jg)
             coef_xy(2,54)=coef_xy(2,54)+coef_x(4,1)*pol_y(2,8,jg)
             coef_xy(1,55)=coef_xy(1,55)+coef_x(1,0)*pol_y(1,9,jg)
             coef_xy(2,55)=coef_xy(2,55)+coef_x(2,0)*pol_y(1,9,jg)
             coef_xy(1,55)=coef_xy(1,55)+coef_x(3,0)*pol_y(2,9,jg)
             coef_xy(2,55)=coef_xy(2,55)+coef_x(4,0)*pol_y(2,9,jg)
       END DO
                coef_xyz(1)=coef_xyz(1)+coef_xy(1,1)*pol_z(1,0,kg)
                coef_xyz(1)=coef_xyz(1)+coef_xy(2,1)*pol_z(2,0,kg)
                coef_xyz(2)=coef_xyz(2)+coef_xy(1,2)*pol_z(1,0,kg)
                coef_xyz(2)=coef_xyz(2)+coef_xy(2,2)*pol_z(2,0,kg)
                coef_xyz(3)=coef_xyz(3)+coef_xy(1,3)*pol_z(1,0,kg)
                coef_xyz(3)=coef_xyz(3)+coef_xy(2,3)*pol_z(2,0,kg)
                coef_xyz(4)=coef_xyz(4)+coef_xy(1,4)*pol_z(1,0,kg)
                coef_xyz(4)=coef_xyz(4)+coef_xy(2,4)*pol_z(2,0,kg)
                coef_xyz(5)=coef_xyz(5)+coef_xy(1,5)*pol_z(1,0,kg)
                coef_xyz(5)=coef_xyz(5)+coef_xy(2,5)*pol_z(2,0,kg)
                coef_xyz(6)=coef_xyz(6)+coef_xy(1,6)*pol_z(1,0,kg)
                coef_xyz(6)=coef_xyz(6)+coef_xy(2,6)*pol_z(2,0,kg)
                coef_xyz(7)=coef_xyz(7)+coef_xy(1,7)*pol_z(1,0,kg)
                coef_xyz(7)=coef_xyz(7)+coef_xy(2,7)*pol_z(2,0,kg)
                coef_xyz(8)=coef_xyz(8)+coef_xy(1,8)*pol_z(1,0,kg)
                coef_xyz(8)=coef_xyz(8)+coef_xy(2,8)*pol_z(2,0,kg)
                coef_xyz(9)=coef_xyz(9)+coef_xy(1,9)*pol_z(1,0,kg)
                coef_xyz(9)=coef_xyz(9)+coef_xy(2,9)*pol_z(2,0,kg)
                coef_xyz(10)=coef_xyz(10)+coef_xy(1,10)*pol_z(1,0,kg)
                coef_xyz(10)=coef_xyz(10)+coef_xy(2,10)*pol_z(2,0,kg)
                coef_xyz(11)=coef_xyz(11)+coef_xy(1,11)*pol_z(1,0,kg)
                coef_xyz(11)=coef_xyz(11)+coef_xy(2,11)*pol_z(2,0,kg)
                coef_xyz(12)=coef_xyz(12)+coef_xy(1,12)*pol_z(1,0,kg)
                coef_xyz(12)=coef_xyz(12)+coef_xy(2,12)*pol_z(2,0,kg)
                coef_xyz(13)=coef_xyz(13)+coef_xy(1,13)*pol_z(1,0,kg)
                coef_xyz(13)=coef_xyz(13)+coef_xy(2,13)*pol_z(2,0,kg)
                coef_xyz(14)=coef_xyz(14)+coef_xy(1,14)*pol_z(1,0,kg)
                coef_xyz(14)=coef_xyz(14)+coef_xy(2,14)*pol_z(2,0,kg)
                coef_xyz(15)=coef_xyz(15)+coef_xy(1,15)*pol_z(1,0,kg)
                coef_xyz(15)=coef_xyz(15)+coef_xy(2,15)*pol_z(2,0,kg)
                coef_xyz(16)=coef_xyz(16)+coef_xy(1,16)*pol_z(1,0,kg)
                coef_xyz(16)=coef_xyz(16)+coef_xy(2,16)*pol_z(2,0,kg)
                coef_xyz(17)=coef_xyz(17)+coef_xy(1,17)*pol_z(1,0,kg)
                coef_xyz(17)=coef_xyz(17)+coef_xy(2,17)*pol_z(2,0,kg)
                coef_xyz(18)=coef_xyz(18)+coef_xy(1,18)*pol_z(1,0,kg)
                coef_xyz(18)=coef_xyz(18)+coef_xy(2,18)*pol_z(2,0,kg)
                coef_xyz(19)=coef_xyz(19)+coef_xy(1,19)*pol_z(1,0,kg)
                coef_xyz(19)=coef_xyz(19)+coef_xy(2,19)*pol_z(2,0,kg)
                coef_xyz(20)=coef_xyz(20)+coef_xy(1,20)*pol_z(1,0,kg)
                coef_xyz(20)=coef_xyz(20)+coef_xy(2,20)*pol_z(2,0,kg)
                coef_xyz(21)=coef_xyz(21)+coef_xy(1,21)*pol_z(1,0,kg)
                coef_xyz(21)=coef_xyz(21)+coef_xy(2,21)*pol_z(2,0,kg)
                coef_xyz(22)=coef_xyz(22)+coef_xy(1,22)*pol_z(1,0,kg)
                coef_xyz(22)=coef_xyz(22)+coef_xy(2,22)*pol_z(2,0,kg)
                coef_xyz(23)=coef_xyz(23)+coef_xy(1,23)*pol_z(1,0,kg)
                coef_xyz(23)=coef_xyz(23)+coef_xy(2,23)*pol_z(2,0,kg)
                coef_xyz(24)=coef_xyz(24)+coef_xy(1,24)*pol_z(1,0,kg)
                coef_xyz(24)=coef_xyz(24)+coef_xy(2,24)*pol_z(2,0,kg)
                coef_xyz(25)=coef_xyz(25)+coef_xy(1,25)*pol_z(1,0,kg)
                coef_xyz(25)=coef_xyz(25)+coef_xy(2,25)*pol_z(2,0,kg)
                coef_xyz(26)=coef_xyz(26)+coef_xy(1,26)*pol_z(1,0,kg)
                coef_xyz(26)=coef_xyz(26)+coef_xy(2,26)*pol_z(2,0,kg)
                coef_xyz(27)=coef_xyz(27)+coef_xy(1,27)*pol_z(1,0,kg)
                coef_xyz(27)=coef_xyz(27)+coef_xy(2,27)*pol_z(2,0,kg)
                coef_xyz(28)=coef_xyz(28)+coef_xy(1,28)*pol_z(1,0,kg)
                coef_xyz(28)=coef_xyz(28)+coef_xy(2,28)*pol_z(2,0,kg)
                coef_xyz(29)=coef_xyz(29)+coef_xy(1,29)*pol_z(1,0,kg)
                coef_xyz(29)=coef_xyz(29)+coef_xy(2,29)*pol_z(2,0,kg)
                coef_xyz(30)=coef_xyz(30)+coef_xy(1,30)*pol_z(1,0,kg)
                coef_xyz(30)=coef_xyz(30)+coef_xy(2,30)*pol_z(2,0,kg)
                coef_xyz(31)=coef_xyz(31)+coef_xy(1,31)*pol_z(1,0,kg)
                coef_xyz(31)=coef_xyz(31)+coef_xy(2,31)*pol_z(2,0,kg)
                coef_xyz(32)=coef_xyz(32)+coef_xy(1,32)*pol_z(1,0,kg)
                coef_xyz(32)=coef_xyz(32)+coef_xy(2,32)*pol_z(2,0,kg)
                coef_xyz(33)=coef_xyz(33)+coef_xy(1,33)*pol_z(1,0,kg)
                coef_xyz(33)=coef_xyz(33)+coef_xy(2,33)*pol_z(2,0,kg)
                coef_xyz(34)=coef_xyz(34)+coef_xy(1,34)*pol_z(1,0,kg)
                coef_xyz(34)=coef_xyz(34)+coef_xy(2,34)*pol_z(2,0,kg)
                coef_xyz(35)=coef_xyz(35)+coef_xy(1,35)*pol_z(1,0,kg)
                coef_xyz(35)=coef_xyz(35)+coef_xy(2,35)*pol_z(2,0,kg)
                coef_xyz(36)=coef_xyz(36)+coef_xy(1,36)*pol_z(1,0,kg)
                coef_xyz(36)=coef_xyz(36)+coef_xy(2,36)*pol_z(2,0,kg)
                coef_xyz(37)=coef_xyz(37)+coef_xy(1,37)*pol_z(1,0,kg)
                coef_xyz(37)=coef_xyz(37)+coef_xy(2,37)*pol_z(2,0,kg)
                coef_xyz(38)=coef_xyz(38)+coef_xy(1,38)*pol_z(1,0,kg)
                coef_xyz(38)=coef_xyz(38)+coef_xy(2,38)*pol_z(2,0,kg)
                coef_xyz(39)=coef_xyz(39)+coef_xy(1,39)*pol_z(1,0,kg)
                coef_xyz(39)=coef_xyz(39)+coef_xy(2,39)*pol_z(2,0,kg)
                coef_xyz(40)=coef_xyz(40)+coef_xy(1,40)*pol_z(1,0,kg)
                coef_xyz(40)=coef_xyz(40)+coef_xy(2,40)*pol_z(2,0,kg)
                coef_xyz(41)=coef_xyz(41)+coef_xy(1,41)*pol_z(1,0,kg)
                coef_xyz(41)=coef_xyz(41)+coef_xy(2,41)*pol_z(2,0,kg)
                coef_xyz(42)=coef_xyz(42)+coef_xy(1,42)*pol_z(1,0,kg)
                coef_xyz(42)=coef_xyz(42)+coef_xy(2,42)*pol_z(2,0,kg)
                coef_xyz(43)=coef_xyz(43)+coef_xy(1,43)*pol_z(1,0,kg)
                coef_xyz(43)=coef_xyz(43)+coef_xy(2,43)*pol_z(2,0,kg)
                coef_xyz(44)=coef_xyz(44)+coef_xy(1,44)*pol_z(1,0,kg)
                coef_xyz(44)=coef_xyz(44)+coef_xy(2,44)*pol_z(2,0,kg)
                coef_xyz(45)=coef_xyz(45)+coef_xy(1,45)*pol_z(1,0,kg)
                coef_xyz(45)=coef_xyz(45)+coef_xy(2,45)*pol_z(2,0,kg)
                coef_xyz(46)=coef_xyz(46)+coef_xy(1,46)*pol_z(1,0,kg)
                coef_xyz(46)=coef_xyz(46)+coef_xy(2,46)*pol_z(2,0,kg)
                coef_xyz(47)=coef_xyz(47)+coef_xy(1,47)*pol_z(1,0,kg)
                coef_xyz(47)=coef_xyz(47)+coef_xy(2,47)*pol_z(2,0,kg)
                coef_xyz(48)=coef_xyz(48)+coef_xy(1,48)*pol_z(1,0,kg)
                coef_xyz(48)=coef_xyz(48)+coef_xy(2,48)*pol_z(2,0,kg)
                coef_xyz(49)=coef_xyz(49)+coef_xy(1,49)*pol_z(1,0,kg)
                coef_xyz(49)=coef_xyz(49)+coef_xy(2,49)*pol_z(2,0,kg)
                coef_xyz(50)=coef_xyz(50)+coef_xy(1,50)*pol_z(1,0,kg)
                coef_xyz(50)=coef_xyz(50)+coef_xy(2,50)*pol_z(2,0,kg)
                coef_xyz(51)=coef_xyz(51)+coef_xy(1,51)*pol_z(1,0,kg)
                coef_xyz(51)=coef_xyz(51)+coef_xy(2,51)*pol_z(2,0,kg)
                coef_xyz(52)=coef_xyz(52)+coef_xy(1,52)*pol_z(1,0,kg)
                coef_xyz(52)=coef_xyz(52)+coef_xy(2,52)*pol_z(2,0,kg)
                coef_xyz(53)=coef_xyz(53)+coef_xy(1,53)*pol_z(1,0,kg)
                coef_xyz(53)=coef_xyz(53)+coef_xy(2,53)*pol_z(2,0,kg)
                coef_xyz(54)=coef_xyz(54)+coef_xy(1,54)*pol_z(1,0,kg)
                coef_xyz(54)=coef_xyz(54)+coef_xy(2,54)*pol_z(2,0,kg)
                coef_xyz(55)=coef_xyz(55)+coef_xy(1,55)*pol_z(1,0,kg)
                coef_xyz(55)=coef_xyz(55)+coef_xy(2,55)*pol_z(2,0,kg)
                coef_xyz(56)=coef_xyz(56)+coef_xy(1,1)*pol_z(1,1,kg)
                coef_xyz(56)=coef_xyz(56)+coef_xy(2,1)*pol_z(2,1,kg)
                coef_xyz(57)=coef_xyz(57)+coef_xy(1,2)*pol_z(1,1,kg)
                coef_xyz(57)=coef_xyz(57)+coef_xy(2,2)*pol_z(2,1,kg)
                coef_xyz(58)=coef_xyz(58)+coef_xy(1,3)*pol_z(1,1,kg)
                coef_xyz(58)=coef_xyz(58)+coef_xy(2,3)*pol_z(2,1,kg)
                coef_xyz(59)=coef_xyz(59)+coef_xy(1,4)*pol_z(1,1,kg)
                coef_xyz(59)=coef_xyz(59)+coef_xy(2,4)*pol_z(2,1,kg)
                coef_xyz(60)=coef_xyz(60)+coef_xy(1,5)*pol_z(1,1,kg)
                coef_xyz(60)=coef_xyz(60)+coef_xy(2,5)*pol_z(2,1,kg)
                coef_xyz(61)=coef_xyz(61)+coef_xy(1,6)*pol_z(1,1,kg)
                coef_xyz(61)=coef_xyz(61)+coef_xy(2,6)*pol_z(2,1,kg)
                coef_xyz(62)=coef_xyz(62)+coef_xy(1,7)*pol_z(1,1,kg)
                coef_xyz(62)=coef_xyz(62)+coef_xy(2,7)*pol_z(2,1,kg)
                coef_xyz(63)=coef_xyz(63)+coef_xy(1,8)*pol_z(1,1,kg)
                coef_xyz(63)=coef_xyz(63)+coef_xy(2,8)*pol_z(2,1,kg)
                coef_xyz(64)=coef_xyz(64)+coef_xy(1,9)*pol_z(1,1,kg)
                coef_xyz(64)=coef_xyz(64)+coef_xy(2,9)*pol_z(2,1,kg)
                coef_xyz(65)=coef_xyz(65)+coef_xy(1,11)*pol_z(1,1,kg)
                coef_xyz(65)=coef_xyz(65)+coef_xy(2,11)*pol_z(2,1,kg)
                coef_xyz(66)=coef_xyz(66)+coef_xy(1,12)*pol_z(1,1,kg)
                coef_xyz(66)=coef_xyz(66)+coef_xy(2,12)*pol_z(2,1,kg)
                coef_xyz(67)=coef_xyz(67)+coef_xy(1,13)*pol_z(1,1,kg)
                coef_xyz(67)=coef_xyz(67)+coef_xy(2,13)*pol_z(2,1,kg)
                coef_xyz(68)=coef_xyz(68)+coef_xy(1,14)*pol_z(1,1,kg)
                coef_xyz(68)=coef_xyz(68)+coef_xy(2,14)*pol_z(2,1,kg)
                coef_xyz(69)=coef_xyz(69)+coef_xy(1,15)*pol_z(1,1,kg)
                coef_xyz(69)=coef_xyz(69)+coef_xy(2,15)*pol_z(2,1,kg)
                coef_xyz(70)=coef_xyz(70)+coef_xy(1,16)*pol_z(1,1,kg)
                coef_xyz(70)=coef_xyz(70)+coef_xy(2,16)*pol_z(2,1,kg)
                coef_xyz(71)=coef_xyz(71)+coef_xy(1,17)*pol_z(1,1,kg)
                coef_xyz(71)=coef_xyz(71)+coef_xy(2,17)*pol_z(2,1,kg)
                coef_xyz(72)=coef_xyz(72)+coef_xy(1,18)*pol_z(1,1,kg)
                coef_xyz(72)=coef_xyz(72)+coef_xy(2,18)*pol_z(2,1,kg)
                coef_xyz(73)=coef_xyz(73)+coef_xy(1,20)*pol_z(1,1,kg)
                coef_xyz(73)=coef_xyz(73)+coef_xy(2,20)*pol_z(2,1,kg)
                coef_xyz(74)=coef_xyz(74)+coef_xy(1,21)*pol_z(1,1,kg)
                coef_xyz(74)=coef_xyz(74)+coef_xy(2,21)*pol_z(2,1,kg)
                coef_xyz(75)=coef_xyz(75)+coef_xy(1,22)*pol_z(1,1,kg)
                coef_xyz(75)=coef_xyz(75)+coef_xy(2,22)*pol_z(2,1,kg)
                coef_xyz(76)=coef_xyz(76)+coef_xy(1,23)*pol_z(1,1,kg)
                coef_xyz(76)=coef_xyz(76)+coef_xy(2,23)*pol_z(2,1,kg)
                coef_xyz(77)=coef_xyz(77)+coef_xy(1,24)*pol_z(1,1,kg)
                coef_xyz(77)=coef_xyz(77)+coef_xy(2,24)*pol_z(2,1,kg)
                coef_xyz(78)=coef_xyz(78)+coef_xy(1,25)*pol_z(1,1,kg)
                coef_xyz(78)=coef_xyz(78)+coef_xy(2,25)*pol_z(2,1,kg)
                coef_xyz(79)=coef_xyz(79)+coef_xy(1,26)*pol_z(1,1,kg)
                coef_xyz(79)=coef_xyz(79)+coef_xy(2,26)*pol_z(2,1,kg)
                coef_xyz(80)=coef_xyz(80)+coef_xy(1,28)*pol_z(1,1,kg)
                coef_xyz(80)=coef_xyz(80)+coef_xy(2,28)*pol_z(2,1,kg)
                coef_xyz(81)=coef_xyz(81)+coef_xy(1,29)*pol_z(1,1,kg)
                coef_xyz(81)=coef_xyz(81)+coef_xy(2,29)*pol_z(2,1,kg)
                coef_xyz(82)=coef_xyz(82)+coef_xy(1,30)*pol_z(1,1,kg)
                coef_xyz(82)=coef_xyz(82)+coef_xy(2,30)*pol_z(2,1,kg)
                coef_xyz(83)=coef_xyz(83)+coef_xy(1,31)*pol_z(1,1,kg)
                coef_xyz(83)=coef_xyz(83)+coef_xy(2,31)*pol_z(2,1,kg)
                coef_xyz(84)=coef_xyz(84)+coef_xy(1,32)*pol_z(1,1,kg)
                coef_xyz(84)=coef_xyz(84)+coef_xy(2,32)*pol_z(2,1,kg)
                coef_xyz(85)=coef_xyz(85)+coef_xy(1,33)*pol_z(1,1,kg)
                coef_xyz(85)=coef_xyz(85)+coef_xy(2,33)*pol_z(2,1,kg)
                coef_xyz(86)=coef_xyz(86)+coef_xy(1,35)*pol_z(1,1,kg)
                coef_xyz(86)=coef_xyz(86)+coef_xy(2,35)*pol_z(2,1,kg)
                coef_xyz(87)=coef_xyz(87)+coef_xy(1,36)*pol_z(1,1,kg)
                coef_xyz(87)=coef_xyz(87)+coef_xy(2,36)*pol_z(2,1,kg)
                coef_xyz(88)=coef_xyz(88)+coef_xy(1,37)*pol_z(1,1,kg)
                coef_xyz(88)=coef_xyz(88)+coef_xy(2,37)*pol_z(2,1,kg)
                coef_xyz(89)=coef_xyz(89)+coef_xy(1,38)*pol_z(1,1,kg)
                coef_xyz(89)=coef_xyz(89)+coef_xy(2,38)*pol_z(2,1,kg)
                coef_xyz(90)=coef_xyz(90)+coef_xy(1,39)*pol_z(1,1,kg)
                coef_xyz(90)=coef_xyz(90)+coef_xy(2,39)*pol_z(2,1,kg)
                coef_xyz(91)=coef_xyz(91)+coef_xy(1,41)*pol_z(1,1,kg)
                coef_xyz(91)=coef_xyz(91)+coef_xy(2,41)*pol_z(2,1,kg)
                coef_xyz(92)=coef_xyz(92)+coef_xy(1,42)*pol_z(1,1,kg)
                coef_xyz(92)=coef_xyz(92)+coef_xy(2,42)*pol_z(2,1,kg)
                coef_xyz(93)=coef_xyz(93)+coef_xy(1,43)*pol_z(1,1,kg)
                coef_xyz(93)=coef_xyz(93)+coef_xy(2,43)*pol_z(2,1,kg)
                coef_xyz(94)=coef_xyz(94)+coef_xy(1,44)*pol_z(1,1,kg)
                coef_xyz(94)=coef_xyz(94)+coef_xy(2,44)*pol_z(2,1,kg)
                coef_xyz(95)=coef_xyz(95)+coef_xy(1,46)*pol_z(1,1,kg)
                coef_xyz(95)=coef_xyz(95)+coef_xy(2,46)*pol_z(2,1,kg)
                coef_xyz(96)=coef_xyz(96)+coef_xy(1,47)*pol_z(1,1,kg)
                coef_xyz(96)=coef_xyz(96)+coef_xy(2,47)*pol_z(2,1,kg)
                coef_xyz(97)=coef_xyz(97)+coef_xy(1,48)*pol_z(1,1,kg)
                coef_xyz(97)=coef_xyz(97)+coef_xy(2,48)*pol_z(2,1,kg)
                coef_xyz(98)=coef_xyz(98)+coef_xy(1,50)*pol_z(1,1,kg)
                coef_xyz(98)=coef_xyz(98)+coef_xy(2,50)*pol_z(2,1,kg)
                coef_xyz(99)=coef_xyz(99)+coef_xy(1,51)*pol_z(1,1,kg)
                coef_xyz(99)=coef_xyz(99)+coef_xy(2,51)*pol_z(2,1,kg)
                coef_xyz(100)=coef_xyz(100)+coef_xy(1,53)*pol_z(1,1,kg)
                coef_xyz(100)=coef_xyz(100)+coef_xy(2,53)*pol_z(2,1,kg)
                coef_xyz(101)=coef_xyz(101)+coef_xy(1,1)*pol_z(1,2,kg)
                coef_xyz(101)=coef_xyz(101)+coef_xy(2,1)*pol_z(2,2,kg)
                coef_xyz(102)=coef_xyz(102)+coef_xy(1,2)*pol_z(1,2,kg)
                coef_xyz(102)=coef_xyz(102)+coef_xy(2,2)*pol_z(2,2,kg)
                coef_xyz(103)=coef_xyz(103)+coef_xy(1,3)*pol_z(1,2,kg)
                coef_xyz(103)=coef_xyz(103)+coef_xy(2,3)*pol_z(2,2,kg)
                coef_xyz(104)=coef_xyz(104)+coef_xy(1,4)*pol_z(1,2,kg)
                coef_xyz(104)=coef_xyz(104)+coef_xy(2,4)*pol_z(2,2,kg)
                coef_xyz(105)=coef_xyz(105)+coef_xy(1,5)*pol_z(1,2,kg)
                coef_xyz(105)=coef_xyz(105)+coef_xy(2,5)*pol_z(2,2,kg)
                coef_xyz(106)=coef_xyz(106)+coef_xy(1,6)*pol_z(1,2,kg)
                coef_xyz(106)=coef_xyz(106)+coef_xy(2,6)*pol_z(2,2,kg)
                coef_xyz(107)=coef_xyz(107)+coef_xy(1,7)*pol_z(1,2,kg)
                coef_xyz(107)=coef_xyz(107)+coef_xy(2,7)*pol_z(2,2,kg)
                coef_xyz(108)=coef_xyz(108)+coef_xy(1,8)*pol_z(1,2,kg)
                coef_xyz(108)=coef_xyz(108)+coef_xy(2,8)*pol_z(2,2,kg)
                coef_xyz(109)=coef_xyz(109)+coef_xy(1,11)*pol_z(1,2,kg)
                coef_xyz(109)=coef_xyz(109)+coef_xy(2,11)*pol_z(2,2,kg)
                coef_xyz(110)=coef_xyz(110)+coef_xy(1,12)*pol_z(1,2,kg)
                coef_xyz(110)=coef_xyz(110)+coef_xy(2,12)*pol_z(2,2,kg)
                coef_xyz(111)=coef_xyz(111)+coef_xy(1,13)*pol_z(1,2,kg)
                coef_xyz(111)=coef_xyz(111)+coef_xy(2,13)*pol_z(2,2,kg)
                coef_xyz(112)=coef_xyz(112)+coef_xy(1,14)*pol_z(1,2,kg)
                coef_xyz(112)=coef_xyz(112)+coef_xy(2,14)*pol_z(2,2,kg)
                coef_xyz(113)=coef_xyz(113)+coef_xy(1,15)*pol_z(1,2,kg)
                coef_xyz(113)=coef_xyz(113)+coef_xy(2,15)*pol_z(2,2,kg)
                coef_xyz(114)=coef_xyz(114)+coef_xy(1,16)*pol_z(1,2,kg)
                coef_xyz(114)=coef_xyz(114)+coef_xy(2,16)*pol_z(2,2,kg)
                coef_xyz(115)=coef_xyz(115)+coef_xy(1,17)*pol_z(1,2,kg)
                coef_xyz(115)=coef_xyz(115)+coef_xy(2,17)*pol_z(2,2,kg)
                coef_xyz(116)=coef_xyz(116)+coef_xy(1,20)*pol_z(1,2,kg)
                coef_xyz(116)=coef_xyz(116)+coef_xy(2,20)*pol_z(2,2,kg)
                coef_xyz(117)=coef_xyz(117)+coef_xy(1,21)*pol_z(1,2,kg)
                coef_xyz(117)=coef_xyz(117)+coef_xy(2,21)*pol_z(2,2,kg)
                coef_xyz(118)=coef_xyz(118)+coef_xy(1,22)*pol_z(1,2,kg)
                coef_xyz(118)=coef_xyz(118)+coef_xy(2,22)*pol_z(2,2,kg)
                coef_xyz(119)=coef_xyz(119)+coef_xy(1,23)*pol_z(1,2,kg)
                coef_xyz(119)=coef_xyz(119)+coef_xy(2,23)*pol_z(2,2,kg)
                coef_xyz(120)=coef_xyz(120)+coef_xy(1,24)*pol_z(1,2,kg)
                coef_xyz(120)=coef_xyz(120)+coef_xy(2,24)*pol_z(2,2,kg)
                coef_xyz(121)=coef_xyz(121)+coef_xy(1,25)*pol_z(1,2,kg)
                coef_xyz(121)=coef_xyz(121)+coef_xy(2,25)*pol_z(2,2,kg)
                coef_xyz(122)=coef_xyz(122)+coef_xy(1,28)*pol_z(1,2,kg)
                coef_xyz(122)=coef_xyz(122)+coef_xy(2,28)*pol_z(2,2,kg)
                coef_xyz(123)=coef_xyz(123)+coef_xy(1,29)*pol_z(1,2,kg)
                coef_xyz(123)=coef_xyz(123)+coef_xy(2,29)*pol_z(2,2,kg)
                coef_xyz(124)=coef_xyz(124)+coef_xy(1,30)*pol_z(1,2,kg)
                coef_xyz(124)=coef_xyz(124)+coef_xy(2,30)*pol_z(2,2,kg)
                coef_xyz(125)=coef_xyz(125)+coef_xy(1,31)*pol_z(1,2,kg)
                coef_xyz(125)=coef_xyz(125)+coef_xy(2,31)*pol_z(2,2,kg)
                coef_xyz(126)=coef_xyz(126)+coef_xy(1,32)*pol_z(1,2,kg)
                coef_xyz(126)=coef_xyz(126)+coef_xy(2,32)*pol_z(2,2,kg)
                coef_xyz(127)=coef_xyz(127)+coef_xy(1,35)*pol_z(1,2,kg)
                coef_xyz(127)=coef_xyz(127)+coef_xy(2,35)*pol_z(2,2,kg)
                coef_xyz(128)=coef_xyz(128)+coef_xy(1,36)*pol_z(1,2,kg)
                coef_xyz(128)=coef_xyz(128)+coef_xy(2,36)*pol_z(2,2,kg)
                coef_xyz(129)=coef_xyz(129)+coef_xy(1,37)*pol_z(1,2,kg)
                coef_xyz(129)=coef_xyz(129)+coef_xy(2,37)*pol_z(2,2,kg)
                coef_xyz(130)=coef_xyz(130)+coef_xy(1,38)*pol_z(1,2,kg)
                coef_xyz(130)=coef_xyz(130)+coef_xy(2,38)*pol_z(2,2,kg)
                coef_xyz(131)=coef_xyz(131)+coef_xy(1,41)*pol_z(1,2,kg)
                coef_xyz(131)=coef_xyz(131)+coef_xy(2,41)*pol_z(2,2,kg)
                coef_xyz(132)=coef_xyz(132)+coef_xy(1,42)*pol_z(1,2,kg)
                coef_xyz(132)=coef_xyz(132)+coef_xy(2,42)*pol_z(2,2,kg)
                coef_xyz(133)=coef_xyz(133)+coef_xy(1,43)*pol_z(1,2,kg)
                coef_xyz(133)=coef_xyz(133)+coef_xy(2,43)*pol_z(2,2,kg)
                coef_xyz(134)=coef_xyz(134)+coef_xy(1,46)*pol_z(1,2,kg)
                coef_xyz(134)=coef_xyz(134)+coef_xy(2,46)*pol_z(2,2,kg)
                coef_xyz(135)=coef_xyz(135)+coef_xy(1,47)*pol_z(1,2,kg)
                coef_xyz(135)=coef_xyz(135)+coef_xy(2,47)*pol_z(2,2,kg)
                coef_xyz(136)=coef_xyz(136)+coef_xy(1,50)*pol_z(1,2,kg)
                coef_xyz(136)=coef_xyz(136)+coef_xy(2,50)*pol_z(2,2,kg)
                coef_xyz(137)=coef_xyz(137)+coef_xy(1,1)*pol_z(1,3,kg)
                coef_xyz(137)=coef_xyz(137)+coef_xy(2,1)*pol_z(2,3,kg)
                coef_xyz(138)=coef_xyz(138)+coef_xy(1,2)*pol_z(1,3,kg)
                coef_xyz(138)=coef_xyz(138)+coef_xy(2,2)*pol_z(2,3,kg)
                coef_xyz(139)=coef_xyz(139)+coef_xy(1,3)*pol_z(1,3,kg)
                coef_xyz(139)=coef_xyz(139)+coef_xy(2,3)*pol_z(2,3,kg)
                coef_xyz(140)=coef_xyz(140)+coef_xy(1,4)*pol_z(1,3,kg)
                coef_xyz(140)=coef_xyz(140)+coef_xy(2,4)*pol_z(2,3,kg)
                coef_xyz(141)=coef_xyz(141)+coef_xy(1,5)*pol_z(1,3,kg)
                coef_xyz(141)=coef_xyz(141)+coef_xy(2,5)*pol_z(2,3,kg)
                coef_xyz(142)=coef_xyz(142)+coef_xy(1,6)*pol_z(1,3,kg)
                coef_xyz(142)=coef_xyz(142)+coef_xy(2,6)*pol_z(2,3,kg)
                coef_xyz(143)=coef_xyz(143)+coef_xy(1,7)*pol_z(1,3,kg)
                coef_xyz(143)=coef_xyz(143)+coef_xy(2,7)*pol_z(2,3,kg)
                coef_xyz(144)=coef_xyz(144)+coef_xy(1,11)*pol_z(1,3,kg)
                coef_xyz(144)=coef_xyz(144)+coef_xy(2,11)*pol_z(2,3,kg)
                coef_xyz(145)=coef_xyz(145)+coef_xy(1,12)*pol_z(1,3,kg)
                coef_xyz(145)=coef_xyz(145)+coef_xy(2,12)*pol_z(2,3,kg)
                coef_xyz(146)=coef_xyz(146)+coef_xy(1,13)*pol_z(1,3,kg)
                coef_xyz(146)=coef_xyz(146)+coef_xy(2,13)*pol_z(2,3,kg)
                coef_xyz(147)=coef_xyz(147)+coef_xy(1,14)*pol_z(1,3,kg)
                coef_xyz(147)=coef_xyz(147)+coef_xy(2,14)*pol_z(2,3,kg)
                coef_xyz(148)=coef_xyz(148)+coef_xy(1,15)*pol_z(1,3,kg)
                coef_xyz(148)=coef_xyz(148)+coef_xy(2,15)*pol_z(2,3,kg)
                coef_xyz(149)=coef_xyz(149)+coef_xy(1,16)*pol_z(1,3,kg)
                coef_xyz(149)=coef_xyz(149)+coef_xy(2,16)*pol_z(2,3,kg)
                coef_xyz(150)=coef_xyz(150)+coef_xy(1,20)*pol_z(1,3,kg)
                coef_xyz(150)=coef_xyz(150)+coef_xy(2,20)*pol_z(2,3,kg)
                coef_xyz(151)=coef_xyz(151)+coef_xy(1,21)*pol_z(1,3,kg)
                coef_xyz(151)=coef_xyz(151)+coef_xy(2,21)*pol_z(2,3,kg)
                coef_xyz(152)=coef_xyz(152)+coef_xy(1,22)*pol_z(1,3,kg)
                coef_xyz(152)=coef_xyz(152)+coef_xy(2,22)*pol_z(2,3,kg)
                coef_xyz(153)=coef_xyz(153)+coef_xy(1,23)*pol_z(1,3,kg)
                coef_xyz(153)=coef_xyz(153)+coef_xy(2,23)*pol_z(2,3,kg)
                coef_xyz(154)=coef_xyz(154)+coef_xy(1,24)*pol_z(1,3,kg)
                coef_xyz(154)=coef_xyz(154)+coef_xy(2,24)*pol_z(2,3,kg)
                coef_xyz(155)=coef_xyz(155)+coef_xy(1,28)*pol_z(1,3,kg)
                coef_xyz(155)=coef_xyz(155)+coef_xy(2,28)*pol_z(2,3,kg)
                coef_xyz(156)=coef_xyz(156)+coef_xy(1,29)*pol_z(1,3,kg)
                coef_xyz(156)=coef_xyz(156)+coef_xy(2,29)*pol_z(2,3,kg)
                coef_xyz(157)=coef_xyz(157)+coef_xy(1,30)*pol_z(1,3,kg)
                coef_xyz(157)=coef_xyz(157)+coef_xy(2,30)*pol_z(2,3,kg)
                coef_xyz(158)=coef_xyz(158)+coef_xy(1,31)*pol_z(1,3,kg)
                coef_xyz(158)=coef_xyz(158)+coef_xy(2,31)*pol_z(2,3,kg)
                coef_xyz(159)=coef_xyz(159)+coef_xy(1,35)*pol_z(1,3,kg)
                coef_xyz(159)=coef_xyz(159)+coef_xy(2,35)*pol_z(2,3,kg)
                coef_xyz(160)=coef_xyz(160)+coef_xy(1,36)*pol_z(1,3,kg)
                coef_xyz(160)=coef_xyz(160)+coef_xy(2,36)*pol_z(2,3,kg)
                coef_xyz(161)=coef_xyz(161)+coef_xy(1,37)*pol_z(1,3,kg)
                coef_xyz(161)=coef_xyz(161)+coef_xy(2,37)*pol_z(2,3,kg)
                coef_xyz(162)=coef_xyz(162)+coef_xy(1,41)*pol_z(1,3,kg)
                coef_xyz(162)=coef_xyz(162)+coef_xy(2,41)*pol_z(2,3,kg)
                coef_xyz(163)=coef_xyz(163)+coef_xy(1,42)*pol_z(1,3,kg)
                coef_xyz(163)=coef_xyz(163)+coef_xy(2,42)*pol_z(2,3,kg)
                coef_xyz(164)=coef_xyz(164)+coef_xy(1,46)*pol_z(1,3,kg)
                coef_xyz(164)=coef_xyz(164)+coef_xy(2,46)*pol_z(2,3,kg)
                coef_xyz(165)=coef_xyz(165)+coef_xy(1,1)*pol_z(1,4,kg)
                coef_xyz(165)=coef_xyz(165)+coef_xy(2,1)*pol_z(2,4,kg)
                coef_xyz(166)=coef_xyz(166)+coef_xy(1,2)*pol_z(1,4,kg)
                coef_xyz(166)=coef_xyz(166)+coef_xy(2,2)*pol_z(2,4,kg)
                coef_xyz(167)=coef_xyz(167)+coef_xy(1,3)*pol_z(1,4,kg)
                coef_xyz(167)=coef_xyz(167)+coef_xy(2,3)*pol_z(2,4,kg)
                coef_xyz(168)=coef_xyz(168)+coef_xy(1,4)*pol_z(1,4,kg)
                coef_xyz(168)=coef_xyz(168)+coef_xy(2,4)*pol_z(2,4,kg)
                coef_xyz(169)=coef_xyz(169)+coef_xy(1,5)*pol_z(1,4,kg)
                coef_xyz(169)=coef_xyz(169)+coef_xy(2,5)*pol_z(2,4,kg)
                coef_xyz(170)=coef_xyz(170)+coef_xy(1,6)*pol_z(1,4,kg)
                coef_xyz(170)=coef_xyz(170)+coef_xy(2,6)*pol_z(2,4,kg)
                coef_xyz(171)=coef_xyz(171)+coef_xy(1,11)*pol_z(1,4,kg)
                coef_xyz(171)=coef_xyz(171)+coef_xy(2,11)*pol_z(2,4,kg)
                coef_xyz(172)=coef_xyz(172)+coef_xy(1,12)*pol_z(1,4,kg)
                coef_xyz(172)=coef_xyz(172)+coef_xy(2,12)*pol_z(2,4,kg)
                coef_xyz(173)=coef_xyz(173)+coef_xy(1,13)*pol_z(1,4,kg)
                coef_xyz(173)=coef_xyz(173)+coef_xy(2,13)*pol_z(2,4,kg)
                coef_xyz(174)=coef_xyz(174)+coef_xy(1,14)*pol_z(1,4,kg)
                coef_xyz(174)=coef_xyz(174)+coef_xy(2,14)*pol_z(2,4,kg)
                coef_xyz(175)=coef_xyz(175)+coef_xy(1,15)*pol_z(1,4,kg)
                coef_xyz(175)=coef_xyz(175)+coef_xy(2,15)*pol_z(2,4,kg)
                coef_xyz(176)=coef_xyz(176)+coef_xy(1,20)*pol_z(1,4,kg)
                coef_xyz(176)=coef_xyz(176)+coef_xy(2,20)*pol_z(2,4,kg)
                coef_xyz(177)=coef_xyz(177)+coef_xy(1,21)*pol_z(1,4,kg)
                coef_xyz(177)=coef_xyz(177)+coef_xy(2,21)*pol_z(2,4,kg)
                coef_xyz(178)=coef_xyz(178)+coef_xy(1,22)*pol_z(1,4,kg)
                coef_xyz(178)=coef_xyz(178)+coef_xy(2,22)*pol_z(2,4,kg)
                coef_xyz(179)=coef_xyz(179)+coef_xy(1,23)*pol_z(1,4,kg)
                coef_xyz(179)=coef_xyz(179)+coef_xy(2,23)*pol_z(2,4,kg)
                coef_xyz(180)=coef_xyz(180)+coef_xy(1,28)*pol_z(1,4,kg)
                coef_xyz(180)=coef_xyz(180)+coef_xy(2,28)*pol_z(2,4,kg)
                coef_xyz(181)=coef_xyz(181)+coef_xy(1,29)*pol_z(1,4,kg)
                coef_xyz(181)=coef_xyz(181)+coef_xy(2,29)*pol_z(2,4,kg)
                coef_xyz(182)=coef_xyz(182)+coef_xy(1,30)*pol_z(1,4,kg)
                coef_xyz(182)=coef_xyz(182)+coef_xy(2,30)*pol_z(2,4,kg)
                coef_xyz(183)=coef_xyz(183)+coef_xy(1,35)*pol_z(1,4,kg)
                coef_xyz(183)=coef_xyz(183)+coef_xy(2,35)*pol_z(2,4,kg)
                coef_xyz(184)=coef_xyz(184)+coef_xy(1,36)*pol_z(1,4,kg)
                coef_xyz(184)=coef_xyz(184)+coef_xy(2,36)*pol_z(2,4,kg)
                coef_xyz(185)=coef_xyz(185)+coef_xy(1,41)*pol_z(1,4,kg)
                coef_xyz(185)=coef_xyz(185)+coef_xy(2,41)*pol_z(2,4,kg)
                coef_xyz(186)=coef_xyz(186)+coef_xy(1,1)*pol_z(1,5,kg)
                coef_xyz(186)=coef_xyz(186)+coef_xy(2,1)*pol_z(2,5,kg)
                coef_xyz(187)=coef_xyz(187)+coef_xy(1,2)*pol_z(1,5,kg)
                coef_xyz(187)=coef_xyz(187)+coef_xy(2,2)*pol_z(2,5,kg)
                coef_xyz(188)=coef_xyz(188)+coef_xy(1,3)*pol_z(1,5,kg)
                coef_xyz(188)=coef_xyz(188)+coef_xy(2,3)*pol_z(2,5,kg)
                coef_xyz(189)=coef_xyz(189)+coef_xy(1,4)*pol_z(1,5,kg)
                coef_xyz(189)=coef_xyz(189)+coef_xy(2,4)*pol_z(2,5,kg)
                coef_xyz(190)=coef_xyz(190)+coef_xy(1,5)*pol_z(1,5,kg)
                coef_xyz(190)=coef_xyz(190)+coef_xy(2,5)*pol_z(2,5,kg)
                coef_xyz(191)=coef_xyz(191)+coef_xy(1,11)*pol_z(1,5,kg)
                coef_xyz(191)=coef_xyz(191)+coef_xy(2,11)*pol_z(2,5,kg)
                coef_xyz(192)=coef_xyz(192)+coef_xy(1,12)*pol_z(1,5,kg)
                coef_xyz(192)=coef_xyz(192)+coef_xy(2,12)*pol_z(2,5,kg)
                coef_xyz(193)=coef_xyz(193)+coef_xy(1,13)*pol_z(1,5,kg)
                coef_xyz(193)=coef_xyz(193)+coef_xy(2,13)*pol_z(2,5,kg)
                coef_xyz(194)=coef_xyz(194)+coef_xy(1,14)*pol_z(1,5,kg)
                coef_xyz(194)=coef_xyz(194)+coef_xy(2,14)*pol_z(2,5,kg)
                coef_xyz(195)=coef_xyz(195)+coef_xy(1,20)*pol_z(1,5,kg)
                coef_xyz(195)=coef_xyz(195)+coef_xy(2,20)*pol_z(2,5,kg)
                coef_xyz(196)=coef_xyz(196)+coef_xy(1,21)*pol_z(1,5,kg)
                coef_xyz(196)=coef_xyz(196)+coef_xy(2,21)*pol_z(2,5,kg)
                coef_xyz(197)=coef_xyz(197)+coef_xy(1,22)*pol_z(1,5,kg)
                coef_xyz(197)=coef_xyz(197)+coef_xy(2,22)*pol_z(2,5,kg)
                coef_xyz(198)=coef_xyz(198)+coef_xy(1,28)*pol_z(1,5,kg)
                coef_xyz(198)=coef_xyz(198)+coef_xy(2,28)*pol_z(2,5,kg)
                coef_xyz(199)=coef_xyz(199)+coef_xy(1,29)*pol_z(1,5,kg)
                coef_xyz(199)=coef_xyz(199)+coef_xy(2,29)*pol_z(2,5,kg)
                coef_xyz(200)=coef_xyz(200)+coef_xy(1,35)*pol_z(1,5,kg)
                coef_xyz(200)=coef_xyz(200)+coef_xy(2,35)*pol_z(2,5,kg)
                coef_xyz(201)=coef_xyz(201)+coef_xy(1,1)*pol_z(1,6,kg)
                coef_xyz(201)=coef_xyz(201)+coef_xy(2,1)*pol_z(2,6,kg)
                coef_xyz(202)=coef_xyz(202)+coef_xy(1,2)*pol_z(1,6,kg)
                coef_xyz(202)=coef_xyz(202)+coef_xy(2,2)*pol_z(2,6,kg)
                coef_xyz(203)=coef_xyz(203)+coef_xy(1,3)*pol_z(1,6,kg)
                coef_xyz(203)=coef_xyz(203)+coef_xy(2,3)*pol_z(2,6,kg)
                coef_xyz(204)=coef_xyz(204)+coef_xy(1,4)*pol_z(1,6,kg)
                coef_xyz(204)=coef_xyz(204)+coef_xy(2,4)*pol_z(2,6,kg)
                coef_xyz(205)=coef_xyz(205)+coef_xy(1,11)*pol_z(1,6,kg)
                coef_xyz(205)=coef_xyz(205)+coef_xy(2,11)*pol_z(2,6,kg)
                coef_xyz(206)=coef_xyz(206)+coef_xy(1,12)*pol_z(1,6,kg)
                coef_xyz(206)=coef_xyz(206)+coef_xy(2,12)*pol_z(2,6,kg)
                coef_xyz(207)=coef_xyz(207)+coef_xy(1,13)*pol_z(1,6,kg)
                coef_xyz(207)=coef_xyz(207)+coef_xy(2,13)*pol_z(2,6,kg)
                coef_xyz(208)=coef_xyz(208)+coef_xy(1,20)*pol_z(1,6,kg)
                coef_xyz(208)=coef_xyz(208)+coef_xy(2,20)*pol_z(2,6,kg)
                coef_xyz(209)=coef_xyz(209)+coef_xy(1,21)*pol_z(1,6,kg)
                coef_xyz(209)=coef_xyz(209)+coef_xy(2,21)*pol_z(2,6,kg)
                coef_xyz(210)=coef_xyz(210)+coef_xy(1,28)*pol_z(1,6,kg)
                coef_xyz(210)=coef_xyz(210)+coef_xy(2,28)*pol_z(2,6,kg)
                coef_xyz(211)=coef_xyz(211)+coef_xy(1,1)*pol_z(1,7,kg)
                coef_xyz(211)=coef_xyz(211)+coef_xy(2,1)*pol_z(2,7,kg)
                coef_xyz(212)=coef_xyz(212)+coef_xy(1,2)*pol_z(1,7,kg)
                coef_xyz(212)=coef_xyz(212)+coef_xy(2,2)*pol_z(2,7,kg)
                coef_xyz(213)=coef_xyz(213)+coef_xy(1,3)*pol_z(1,7,kg)
                coef_xyz(213)=coef_xyz(213)+coef_xy(2,3)*pol_z(2,7,kg)
                coef_xyz(214)=coef_xyz(214)+coef_xy(1,11)*pol_z(1,7,kg)
                coef_xyz(214)=coef_xyz(214)+coef_xy(2,11)*pol_z(2,7,kg)
                coef_xyz(215)=coef_xyz(215)+coef_xy(1,12)*pol_z(1,7,kg)
                coef_xyz(215)=coef_xyz(215)+coef_xy(2,12)*pol_z(2,7,kg)
                coef_xyz(216)=coef_xyz(216)+coef_xy(1,20)*pol_z(1,7,kg)
                coef_xyz(216)=coef_xyz(216)+coef_xy(2,20)*pol_z(2,7,kg)
                coef_xyz(217)=coef_xyz(217)+coef_xy(1,1)*pol_z(1,8,kg)
                coef_xyz(217)=coef_xyz(217)+coef_xy(2,1)*pol_z(2,8,kg)
                coef_xyz(218)=coef_xyz(218)+coef_xy(1,2)*pol_z(1,8,kg)
                coef_xyz(218)=coef_xyz(218)+coef_xy(2,2)*pol_z(2,8,kg)
                coef_xyz(219)=coef_xyz(219)+coef_xy(1,11)*pol_z(1,8,kg)
                coef_xyz(219)=coef_xyz(219)+coef_xy(2,11)*pol_z(2,8,kg)
                coef_xyz(220)=coef_xyz(220)+coef_xy(1,1)*pol_z(1,9,kg)
                coef_xyz(220)=coef_xyz(220)+coef_xy(2,1)*pol_z(2,9,kg)
    END DO

  END SUBROUTINE integrate_core_9





! **************************************************************************************************
!> \brief ...
!> \param grid ...
!> \param xdat ...
!> \param ydat ...
!> \param zdat ...
!> \param bo1 ...
!> \param bo2 ...
!> \param zlb ...
!> \param zub ...
!> \param ylb ...
!> \param yub ...
!> \param xlb ...
!> \param xub ...
! **************************************************************************************************
SUBROUTINE collocate_gf_npbc(grid, xdat, ydat, zdat, bo1, bo2, zlb, zub, ylb, yub, xlb, xub)
    USE kinds,                        ONLY: dp
   INTEGER, INTENT(IN)                      :: bo1(2, 3)
   REAL(dp), INTENT(INOUT) :: &
      grid(bo1(1, 1):bo1(2, 1), bo1(1, 2):bo1(2, 2), bo1(1, 3):bo1(2, 3))
   INTEGER, INTENT(IN)                      :: bo2(2, 3)
   REAL(dp), INTENT(IN)                     :: zdat(bo2(1, 3):bo2(2, 3)), &
                                               ydat(bo2(1, 2):bo2(2, 2)), &
                                               xdat(bo2(1, 1):bo2(2, 1))
   INTEGER, INTENT(IN)                      :: zlb, zub, ylb, yub, xlb, xub

   INTEGER                                  :: ix, iy, iz
   REAL(dp)                                 :: tmp1

   DO iz = zlb, zub
      DO iy = ylb, yub
         tmp1 = zdat(iz)*ydat(iy)
         DO ix = xlb, xub
            grid(ix, iy, iz) = grid(ix, iy, iz)+xdat(ix)*tmp1
         END DO ! Loop on x
      END DO ! Loop on y
   END DO ! Loop on z

END SUBROUTINE

! **************************************************************************************************
!> \brief ...
!> \param grid ...
!> \param xdat ...
!> \param ydat ...
!> \param zdat ...
!> \param bo ...
!> \param zlb ...
!> \param zub ...
!> \param ylb ...
!> \param yub ...
!> \param xlb ...
!> \param xub ...
!> \param force ...
! **************************************************************************************************
SUBROUTINE integrate_gf_npbc(grid, xdat, ydat, zdat, bo, zlb, zub, ylb, yub, xlb, xub, force)
    USE kinds,                        ONLY: dp
   INTEGER, INTENT(IN)                      :: bo(2, 3)
   REAL(dp), INTENT(IN)                     :: zdat(2, bo(1, 3):bo(2, 3)), &
                                               ydat(2, bo(1, 2):bo(2, 2)), &
                                               xdat(2, bo(1, 1):bo(2, 1))
   REAL(dp), INTENT(INOUT) :: &
      grid(bo(1, 1):bo(2, 1), bo(1, 2):bo(2, 2), bo(1, 3):bo(2, 3))
   INTEGER, INTENT(IN)                      :: zlb, zub, ylb, yub, xlb, xub
   REAL(dp), INTENT(INOUT)                  :: force(3)

   INTEGER                                  :: ix, iy, iy2, iz
   REAL(dp)                                 :: fx1, fx2, fyz1, fyz2, g1, g2, &
                                               x1, x2

   DO iz = zlb, zub
      iy2 = HUGE(0)
      ! unroll by 2
      DO iy = ylb, yub-1, 2
         iy2 = iy+1
         fx1 = 0.0_dp
         fyz1 = 0.0_dp
         fx2 = 0.0_dp
         fyz2 = 0.0_dp
         DO ix = xlb, xub
            g1 = grid(ix, iy, iz)
            g2 = grid(ix, iy2, iz)
            x1 = xdat(1, ix)
            x2 = xdat(2, ix)
            fyz1 = fyz1+g1*x1
            fx1 = fx1+g1*x2
            fyz2 = fyz2+g2*x1
            fx2 = fx2+g2*x2
         END DO ! Loop on x
         force(1) = force(1)+fx1*zdat(1, iz)*ydat(1, iy)
         force(2) = force(2)+fyz1*zdat(1, iz)*ydat(2, iy)
         force(3) = force(3)+fyz1*zdat(2, iz)*ydat(1, iy)
         force(1) = force(1)+fx2*zdat(1, iz)*ydat(1, iy2)
         force(2) = force(2)+fyz2*zdat(1, iz)*ydat(2, iy2)
         force(3) = force(3)+fyz2*zdat(2, iz)*ydat(1, iy2)
      END DO ! Loop on y

      ! cleanup loop: check if the last loop element has done
      IF (iy2 .NE. yub) THEN
         iy2 = yub
         fx2 = 0.0_dp
         fyz2 = 0.0_dp
         DO ix = xlb, xub
            g2 = grid(ix, iy2, iz)
            x1 = xdat(1, ix)
            x2 = xdat(2, ix)
            fyz2 = fyz2+g2*x1
            fx2 = fx2+g2*x2
         END DO ! Loop on x
         force(1) = force(1)+fx2*zdat(1, iz)*ydat(1, iy2)
         force(2) = force(2)+fyz2*zdat(1, iz)*ydat(2, iy2)
         force(3) = force(3)+fyz2*zdat(2, iz)*ydat(1, iy2)
      ENDIF

   END DO ! Loop on z

END SUBROUTINE
!--------------------------------------------------------------------------------------------------!
!   CP2K: A general program to perform molecular dynamics simulations                              !
!   Copyright (C) 2000 - 2016  CP2K developers group                                               !
!--------------------------------------------------------------------------------------------------!

! **************************************************************************************************
!> \brief provides specific implementations for common cases of xyz_to_vab routine.
!>
!> \note
!>      Specific implementations for la_max_local,lb_max_local <=4 are contained
!>      in the this file, tuned for AMD Interlagos processor using the
!>      gfortran compiler
!>      If grid operations (subroutines integrate_v_rspace, calculate_rho_elec and similar)
!>      consume significant amounts of time, users are recommended to build an auto-tuned
!>      version of these kernels for their own system.
!>      Instructions on building an optimised 'libgrid' library can be found in cp2k/tools/autotune_grid
!>
!> \author Ruyman Reyes
! **************************************************************************************************
! We use the caller routine in libgrid.a (which may have a different range of la/lb_max_local implemented


! Use the default caller routine
! **************************************************************************************************
!> \brief ...
!> \param prefactor ...
!> \param coef_xyz ...
!> \param lp ...
!> \param la_max_local ...
!> \param lb_max_local ...
!> \param rp ...
!> \param ra ...
!> \param rab ...
!> \param vab ...
!> \param coset ...
!> \param la_min_local ...
!> \param lb_min_local ...
!> \param maxl ...
!> \param lvab ...
!> \param hvab ...
! **************************************************************************************************
SUBROUTINE call_to_xyz_to_vab(prefactor, coef_xyz, lp, &
                              la_max_local, lb_max_local, &
                              rp, ra, rab, vab, coset, &
                              la_min_local, lb_min_local, &
                              maxl, lvab, hvab)

              USE kinds,                        ONLY: dp

   REAL(KIND=dp), INTENT(IN) :: prefactor
   INTEGER, INTENT(IN) :: lp
   REAL(kind=dp), DIMENSION(((lp+1)*(lp+2)*(lp+3))/6), &
      INTENT(INOUT) :: coef_xyz
   INTEGER, INTENT(IN) :: la_max_local, lb_max_local, &
                          la_min_local, lb_min_local
   INTEGER, INTENT(IN) :: maxl, lvab, hvab
   REAL(kind=dp), DIMENSION(3), INTENT(IN) :: rp, ra, rab
   REAL(kind=dp), DIMENSION(lvab, hvab), INTENT(INOUT)  :: vab
   INTEGER, DIMENSION(-1:maxl, -1:maxl, -1:maxl), &
      INTENT(IN)     :: coset

   SELECT CASE (la_max_local)
   CASE (0)
      SELECT CASE (lb_max_local)
      CASE (0)
         CALL xyz_to_vab_0_0(prefactor, coef_xyz, lp, &
                             vab, &
                             lvab, hvab)

      CASE (1)
         CALL xyz_to_vab_0_1(prefactor, coef_xyz, lp, &
                             rp, ra, rab, vab, coset, &
                             lb_min_local, maxl, lvab, hvab)

      CASE (2)
         CALL xyz_to_vab_0_2(prefactor, coef_xyz, lp, &
                             rp, ra, rab, vab, coset, &
                             lb_min_local, maxl, lvab, hvab)

      CASE (3)
         CALL xyz_to_vab_0_3(prefactor, coef_xyz, lp, &
                             rp, ra, rab, vab, coset, &
                             lb_min_local, maxl, lvab, hvab)

      CASE (4)
         CALL xyz_to_vab_0_4(prefactor, coef_xyz, lp, &
                             rp, ra, rab, vab, coset, &
                             lb_min_local, maxl, lvab, hvab)

      CASE DEFAULT
         CALL xyz_to_vab(prefactor, coef_xyz, lp, &
                         la_max_local, lb_max_local, &
                         rp, ra, rab, vab, coset, la_min_local, &
                         lb_min_local, maxl, lvab, hvab)
      END SELECT
   CASE (1)
      SELECT CASE (lb_max_local)
      CASE (0)
         CALL xyz_to_vab_1_0(prefactor, coef_xyz, lp, &
                             rp, ra, vab, coset, la_min_local, &
                             maxl, lvab, hvab)

      CASE (1)
         CALL xyz_to_vab_1_1(prefactor, coef_xyz, lp, &
                             rp, ra, rab, vab, coset, la_min_local, &
                             lb_min_local, maxl, lvab, hvab)

      CASE (2)
         CALL xyz_to_vab_1_2(prefactor, coef_xyz, lp, &
                             rp, ra, rab, vab, coset, la_min_local, &
                             lb_min_local, maxl, lvab, hvab)

      CASE (3)
         CALL xyz_to_vab_1_3(prefactor, coef_xyz, lp, &
                             rp, ra, rab, vab, coset, la_min_local, &
                             lb_min_local, maxl, lvab, hvab)

      CASE (4)
         CALL xyz_to_vab_1_4(prefactor, coef_xyz, lp, &
                             rp, ra, rab, vab, coset, la_min_local, &
                             lb_min_local, maxl, lvab, hvab)

      CASE DEFAULT
         CALL xyz_to_vab(prefactor, coef_xyz, lp, &
                         la_max_local, lb_max_local, &
                         rp, ra, rab, vab, coset, la_min_local, &
                         lb_min_local, maxl, lvab, hvab)
      END SELECT
   CASE (2)
      SELECT CASE (lb_max_local)
      CASE (0)
         CALL xyz_to_vab_2_0(prefactor, coef_xyz, lp, &
                             rp, ra, vab, coset, la_min_local, &
                             maxl, lvab, hvab)

      CASE (1)
         CALL xyz_to_vab_2_1(prefactor, coef_xyz, lp, &
                             rp, ra, rab, vab, coset, la_min_local, &
                             lb_min_local, maxl, lvab, hvab)

      CASE (2)
         CALL xyz_to_vab_2_2(prefactor, coef_xyz, lp, &
                             rp, ra, rab, vab, coset, la_min_local, &
                             lb_min_local, maxl, lvab, hvab)

      CASE (3)
         CALL xyz_to_vab_2_3(prefactor, coef_xyz, lp, &
                             rp, ra, rab, vab, coset, la_min_local, &
                             lb_min_local, maxl, lvab, hvab)

      CASE (4)
         CALL xyz_to_vab_2_4(prefactor, coef_xyz, lp, &
                             rp, ra, rab, vab, coset, la_min_local, &
                             lb_min_local, maxl, lvab, hvab)

      CASE DEFAULT
         CALL xyz_to_vab(prefactor, coef_xyz, lp, &
                         la_max_local, lb_max_local, &
                         rp, ra, rab, vab, coset, la_min_local, &
                         lb_min_local, maxl, lvab, hvab)
      END SELECT
   CASE (3)
      SELECT CASE (lb_max_local)
      CASE (0)
         CALL xyz_to_vab_3_0(prefactor, coef_xyz, lp, &
                             rp, ra, vab, coset, la_min_local, &
                             maxl, lvab, hvab)

      CASE (1)
         CALL xyz_to_vab_3_1(prefactor, coef_xyz, lp, &
                             rp, ra, rab, vab, coset, la_min_local, &
                             lb_min_local, maxl, lvab, hvab)

      CASE (2)
         CALL xyz_to_vab_3_2(prefactor, coef_xyz, lp, &
                             rp, ra, rab, vab, coset, la_min_local, &
                             lb_min_local, maxl, lvab, hvab)

      CASE (3)
         CALL xyz_to_vab_3_3(prefactor, coef_xyz, lp, &
                             rp, ra, rab, vab, coset, la_min_local, &
                             lb_min_local, maxl, lvab, hvab)

      CASE (4)
         CALL xyz_to_vab_3_4(prefactor, coef_xyz, lp, &
                             rp, ra, rab, vab, coset, la_min_local, &
                             lb_min_local, maxl, lvab, hvab)

      CASE DEFAULT
         CALL xyz_to_vab(prefactor, coef_xyz, lp, &
                         la_max_local, lb_max_local, &
                         rp, ra, rab, vab, coset, la_min_local, &
                         lb_min_local, maxl, lvab, hvab)
      END SELECT
   CASE (4)
      SELECT CASE (lb_max_local)
      CASE (0)
         CALL xyz_to_vab_4_0(prefactor, coef_xyz, lp, &
                             rp, ra, vab, coset, la_min_local, &
                             maxl, lvab, hvab)

      CASE (1)
         CALL xyz_to_vab_4_1(prefactor, coef_xyz, lp, &
                             rp, ra, rab, vab, coset, la_min_local, &
                             lb_min_local, maxl, lvab, hvab)

      CASE (2)
         CALL xyz_to_vab_4_2(prefactor, coef_xyz, lp, &
                             rp, ra, rab, vab, coset, la_min_local, &
                             lb_min_local, maxl, lvab, hvab)

      CASE (3)
         CALL xyz_to_vab_4_3(prefactor, coef_xyz, lp, &
                             rp, ra, rab, vab, coset, la_min_local, &
                             lb_min_local, maxl, lvab, hvab)

      CASE (4)
         CALL xyz_to_vab_4_4(prefactor, coef_xyz, lp, &
                             rp, ra, rab, vab, coset, la_min_local, &
                             lb_min_local, maxl, lvab, hvab)

      CASE DEFAULT
         CALL xyz_to_vab(prefactor, coef_xyz, lp, &
                         la_max_local, lb_max_local, &
                         rp, ra, rab, vab, coset, la_min_local, &
                         lb_min_local, maxl, lvab, hvab)
      END SELECT
   CASE DEFAULT
      CALL xyz_to_vab(prefactor, coef_xyz, lp, &
                      la_max_local, lb_max_local, &
                      rp, ra, rab, vab, coset, la_min_local, lb_min_local, maxl, lvab, hvab)

   END SELECT

END SUBROUTINE call_to_xyz_to_vab

!!! Cloned routines
! Fully unrolled version
! **************************************************************************************************
!> \brief ...
!> \param prefactor ...
!> \param coef_xyz ...
!> \param lp ...
!> \param vab ...
!> \param lvab ...
!> \param hvab ...
! **************************************************************************************************
SUBROUTINE xyz_to_vab_0_0(prefactor, coef_xyz, &
                          lp, vab, &
                          lvab, hvab)

       USE kinds,                        ONLY: dp

   ! PARAMETERS
   REAL(KIND=dp), INTENT(IN) :: prefactor
   INTEGER, INTENT(IN) :: lp, &
                          lvab, hvab

   REAL(kind=dp), DIMENSION(((lp+1)*(lp+2)*(lp+3))/6), &
      INTENT(INOUT) :: coef_xyz
   REAL(kind=dp), DIMENSION(lvab, hvab), INTENT(INOUT) :: vab

   ! LOCAL
   INTEGER :: ico, jco

   coef_xyz = coef_xyz*prefactor

   jco = 1 ! coset(0,0,0)
   ico = 1 ! coset(0,0,0)
   vab(ico, jco) = vab(ico, jco)+coef_xyz(1)

END SUBROUTINE xyz_to_vab_0_0

! Fully unrolled version
! **************************************************************************************************
!> \brief ...
!> \param prefactor ...
!> \param coef_xyz ...
!> \param lp ...
!> \param rp ...
!> \param ra ...
!> \param rab ...
!> \param vab ...
!> \param coset ...
!> \param lb_min_local ...
!> \param maxl ...
!> \param lvab ...
!> \param hvab ...
! **************************************************************************************************
SUBROUTINE xyz_to_vab_0_1(prefactor, coef_xyz, &
                          lp, rp, ra, rab, vab, coset, &
                          lb_min_local, maxl, lvab, hvab)

       USE kinds,                        ONLY: dp

   ! PARAMETERS
   REAL(KIND=dp), INTENT(IN) :: prefactor
   INTEGER, INTENT(IN) :: lp, &
                          lb_min_local, maxl, &
                          lvab, hvab

   INTEGER, DIMENSION(-1:maxl, -1:maxl, -1:maxl), &
      INTENT(IN)     :: coset

   REAL(kind=dp), DIMENSION(((lp+1)*(lp+2)*(lp+3))/6), &
      INTENT(INOUT) :: coef_xyz
   REAL(kind=dp), DIMENSION(lvab, hvab), INTENT(INOUT) :: vab
   REAL(kind=dp), DIMENSION(3), INTENT(IN) :: rp, ra, rab

   ! LOCAL
   INTEGER :: ico, jco, lxa, lxb, lxp, lxyz, lyp
   REAL(KIND=dp) :: a, b, binomial_k_lxa, binomial_l_lxb, const_rp

   REAL(kind=dp) :: alpha_1(0:(0+1), 0:0, 0:1)
   REAL(kind=dp) :: alpha_2(0:(0+1), 0:0, 0:1)
   REAL(kind=dp) :: alpha_3(0:(0+1), 0:0, 0:1)
   REAL(kind=dp) :: coef_ttz(0:0, 0:1)
   REAL(kind=dp) :: coef_tyz(0:0, 0:1, 0:0, 0:1)
   coef_xyz = coef_xyz*prefactor

   alpha_1(:, :, :) = 0.0_dp
   alpha_2(:, :, :) = 0.0_dp
   alpha_3(:, :, :) = 0.0_dp

   const_rp = rp(1)-(ra(1)+rab(1))
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(0, 0, 0) = alpha_1(0, 0, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(1, 0, 1) = alpha_1(1, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_1(0, 0, 1) = alpha_1(0, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   const_rp = rp(2)-(ra(2)+rab(2))
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(0, 0, 0) = alpha_2(0, 0, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(1, 0, 1) = alpha_2(1, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_2(0, 0, 1) = alpha_2(0, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   const_rp = rp(3)-(ra(3)+rab(3))
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(0, 0, 0) = alpha_3(0, 0, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(1, 0, 1) = alpha_3(1, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_3(0, 0, 1) = alpha_3(0, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0

   lxyz = 0
   coef_tyz = 0.0_dp
   DO lyp = 0, 0+1-0
      coef_ttz = 0.0_dp
      DO lxp = 0, 0+1-0-lyp
         lxyz = lxyz+1
         coef_ttz(0, 0) = coef_ttz(0, 0)+coef_xyz(lxyz)*alpha_1(lxp, 0, 0)
         coef_ttz(0, 1) = coef_ttz(0, 1)+coef_xyz(lxyz)*alpha_1(lxp, 0, 1)
      ENDDO

      coef_tyz(0, 0, 0, 0) = coef_tyz(0, 0, 0, 0)+coef_ttz(0, 0)*alpha_2(lyp, 0, 0)
      coef_tyz(0, 1, 0, 0) = coef_tyz(0, 1, 0, 0)+coef_ttz(0, 1)*alpha_2(lyp, 0, 0)

      coef_tyz(0, 0, 0, 1) = coef_tyz(0, 0, 0, 1)+coef_ttz(0, 0)*alpha_2(lyp, 0, 1)

   ENDDO

   DO lxb = lb_min_local, 1
      jco = coset(lxb, 0, 0)

      lxa = 0
      !ico=coset(0,0,0)
      ico = 1

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(0, 0, 0)

   ENDDO
   lxb = 0
   !jco=coset(0,1,0)
   jco = 3

   lxa = 0
   !ico=coset(0,0,0)
   ico = 1

   vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 1)*alpha_3(0, 0, 0)

   lxb = 0
   !jco=coset(0,0,1)
   jco = 4

   lxa = 0
   !ico=coset(0,0,0)
   ico = 1

   vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(0, 0, 1)

   coef_tyz = 0.0_dp
   DO lyp = 0, 0+1-1
      coef_ttz = 0.0_dp
      DO lxp = 0, 0+1-1-lyp
         lxyz = lxyz+1
         coef_ttz(0, 0) = coef_ttz(0, 0)+coef_xyz(lxyz)*alpha_1(lxp, 0, 0)
         coef_ttz(0, 1) = coef_ttz(0, 1)+coef_xyz(lxyz)*alpha_1(lxp, 0, 1)
      ENDDO

      coef_tyz(0, 0, 0, 0) = coef_tyz(0, 0, 0, 0)+coef_ttz(0, 0)*alpha_2(lyp, 0, 0)
      coef_tyz(0, 1, 0, 0) = coef_tyz(0, 1, 0, 0)+coef_ttz(0, 1)*alpha_2(lyp, 0, 0)

      coef_tyz(0, 0, 0, 1) = coef_tyz(0, 0, 0, 1)+coef_ttz(0, 0)*alpha_2(lyp, 0, 1)

   ENDDO

   DO lxb = lb_min_local, 1
      jco = coset(lxb, 0, 0)

      lxa = 0
      !ico=coset(0,0,0)
      ico = 1

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(1, 0, 0)

   ENDDO
   lxb = 0
   !jco=coset(0,1,0)
   jco = 3

   lxa = 0
   !ico=coset(0,0,0)
   ico = 1

   vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 1)*alpha_3(1, 0, 0)

   lxb = 0
   !jco=coset(0,0,1)
   jco = 4

   lxa = 0
   !ico=coset(0,0,0)
   ico = 1

   vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(1, 0, 1)

END SUBROUTINE xyz_to_vab_0_1

! Fully unrolled version
! **************************************************************************************************
!> \brief ...
!> \param prefactor ...
!> \param coef_xyz ...
!> \param lp ...
!> \param rp ...
!> \param ra ...
!> \param rab ...
!> \param vab ...
!> \param coset ...
!> \param lb_min_local ...
!> \param maxl ...
!> \param lvab ...
!> \param hvab ...
! **************************************************************************************************
SUBROUTINE xyz_to_vab_0_2(prefactor, coef_xyz, &
                          lp, rp, ra, rab, vab, coset, &
                          lb_min_local, maxl, lvab, hvab)

       USE kinds,                        ONLY: dp

   ! PARAMETERS
   REAL(KIND=dp), INTENT(IN) :: prefactor
   INTEGER, INTENT(IN) :: lp, &
                          lb_min_local, maxl, &
                          lvab, hvab

   INTEGER, DIMENSION(-1:maxl, -1:maxl, -1:maxl), &
      INTENT(IN)     :: coset

   REAL(kind=dp), DIMENSION(((lp+1)*(lp+2)*(lp+3))/6), &
      INTENT(INOUT) :: coef_xyz
   REAL(kind=dp), DIMENSION(lvab, hvab), INTENT(INOUT) :: vab
   REAL(kind=dp), DIMENSION(3), INTENT(IN) :: rp, ra, rab

   ! LOCAL
   INTEGER :: ico, jco, lxa, lxb, lxp, lxyz, lyp, lzp
   REAL(KIND=dp) :: a, b, binomial_k_lxa, binomial_l_lxb, const_rp

   REAL(kind=dp) :: alpha_1(0:(0+2), 0:0, 0:2)
   REAL(kind=dp) :: alpha_2(0:(0+2), 0:0, 0:2)
   REAL(kind=dp) :: alpha_3(0:(0+2), 0:0, 0:2)
   REAL(kind=dp) :: coef_ttz(0:0, 0:2)
   REAL(kind=dp) :: coef_tyz(0:0, 0:2, 0:0, 0:2)
   coef_xyz = coef_xyz*prefactor

   alpha_1(:, :, :) = 0.0_dp
   alpha_2(:, :, :) = 0.0_dp
   alpha_3(:, :, :) = 0.0_dp

   const_rp = rp(1)-(ra(1)+rab(1))
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(0, 0, 0) = alpha_1(0, 0, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(1, 0, 1) = alpha_1(1, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_1(0, 0, 1) = alpha_1(0, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(2, 0, 2) = alpha_1(2, 0, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(2, dp)
   b = b*const_rp
   alpha_1(1, 0, 2) = alpha_1(1, 0, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(2, dp)
   b = b*const_rp
   alpha_1(0, 0, 2) = alpha_1(0, 0, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   const_rp = rp(2)-(ra(2)+rab(2))
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(0, 0, 0) = alpha_2(0, 0, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(1, 0, 1) = alpha_2(1, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_2(0, 0, 1) = alpha_2(0, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(2, 0, 2) = alpha_2(2, 0, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(2, dp)
   b = b*const_rp
   alpha_2(1, 0, 2) = alpha_2(1, 0, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(2, dp)
   b = b*const_rp
   alpha_2(0, 0, 2) = alpha_2(0, 0, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   const_rp = rp(3)-(ra(3)+rab(3))
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(0, 0, 0) = alpha_3(0, 0, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(1, 0, 1) = alpha_3(1, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_3(0, 0, 1) = alpha_3(0, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(2, 0, 2) = alpha_3(2, 0, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(2, dp)
   b = b*const_rp
   alpha_3(1, 0, 2) = alpha_3(1, 0, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(2, dp)
   b = b*const_rp
   alpha_3(0, 0, 2) = alpha_3(0, 0, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0

   lxyz = 0
   DO lzp = 0, 0+2
      coef_tyz = 0.0_dp
      DO lyp = 0, 0+2-lzp
         coef_ttz = 0.0_dp
         DO lxp = 0, 0+2-lzp-lyp
            lxyz = lxyz+1
            coef_ttz(0, 0) = coef_ttz(0, 0)+coef_xyz(lxyz)*alpha_1(lxp, 0, 0)
            coef_ttz(0, 1) = coef_ttz(0, 1)+coef_xyz(lxyz)*alpha_1(lxp, 0, 1)
            coef_ttz(0, 2) = coef_ttz(0, 2)+coef_xyz(lxyz)*alpha_1(lxp, 0, 2)
         ENDDO

         coef_tyz(0, 0, 0, 0) = coef_tyz(0, 0, 0, 0)+coef_ttz(0, 0)*alpha_2(lyp, 0, 0)
         coef_tyz(0, 1, 0, 0) = coef_tyz(0, 1, 0, 0)+coef_ttz(0, 1)*alpha_2(lyp, 0, 0)
         coef_tyz(0, 2, 0, 0) = coef_tyz(0, 2, 0, 0)+coef_ttz(0, 2)*alpha_2(lyp, 0, 0)
         coef_tyz(0, 0, 0, 1) = coef_tyz(0, 0, 0, 1)+coef_ttz(0, 0)*alpha_2(lyp, 0, 1)
         coef_tyz(0, 1, 0, 1) = coef_tyz(0, 1, 0, 1)+coef_ttz(0, 1)*alpha_2(lyp, 0, 1)
         coef_tyz(0, 0, 0, 2) = coef_tyz(0, 0, 0, 2)+coef_ttz(0, 0)*alpha_2(lyp, 0, 2)
      ENDDO

      DO lxb = lb_min_local, 2
         jco = coset(lxb, 0, 0)

         lxa = 0
         !ico=coset(0,0,0)
         ico = 1

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 0, 0)

      ENDDO
      DO lxb = MAX(lb_min_local-0-1, 0), 1
         jco = coset(lxb, 1, 0)

         lxa = 0
         !ico=coset(0,0,0)
         ico = 1

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 1)*alpha_3(lzp, 0, 0)

      ENDDO
      lxb = 0
      !jco=coset(0,2,0)
      jco = 8

      lxa = 0
      !ico=coset(0,0,0)
      ico = 1

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 2)*alpha_3(lzp, 0, 0)

      DO lxb = MAX(lb_min_local-1-0, 0), 1
         jco = coset(lxb, 0, 1)

         lxa = 0
         !ico=coset(0,0,0)
         ico = 1

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 0, 1)

      ENDDO
      lxb = 0
      !jco=coset(0,1,1)
      jco = 9

      lxa = 0
      !ico=coset(0,0,0)
      ico = 1

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 1)*alpha_3(lzp, 0, 1)

      lxb = 0
      !jco=coset(0,0,2)
      jco = 10

      lxa = 0
      !ico=coset(0,0,0)
      ico = 1

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 0, 2)

   ENDDO

END SUBROUTINE xyz_to_vab_0_2

! Fully unrolled version
! **************************************************************************************************
!> \brief ...
!> \param prefactor ...
!> \param coef_xyz ...
!> \param lp ...
!> \param rp ...
!> \param ra ...
!> \param rab ...
!> \param vab ...
!> \param coset ...
!> \param lb_min_local ...
!> \param maxl ...
!> \param lvab ...
!> \param hvab ...
! **************************************************************************************************
SUBROUTINE xyz_to_vab_0_3(prefactor, coef_xyz, &
                          lp, rp, ra, rab, vab, coset, &
                          lb_min_local, maxl, lvab, hvab)

       USE kinds,                        ONLY: dp

   ! PARAMETERS
   REAL(KIND=dp), INTENT(IN) :: prefactor
   INTEGER, INTENT(IN) :: lp, &
                          lb_min_local, maxl, &
                          lvab, hvab

   INTEGER, DIMENSION(-1:maxl, -1:maxl, -1:maxl), &
      INTENT(IN)     :: coset

   REAL(kind=dp), DIMENSION(((lp+1)*(lp+2)*(lp+3))/6), &
      INTENT(INOUT) :: coef_xyz
   REAL(kind=dp), DIMENSION(lvab, hvab), INTENT(INOUT) :: vab
   REAL(kind=dp), DIMENSION(3), INTENT(IN) :: rp, ra, rab

   ! LOCAL
   INTEGER :: ico, jco, lxa, lxb, lxp, lxyz, lyp, lzp
   REAL(KIND=dp) :: a, b, binomial_k_lxa, binomial_l_lxb, const_rp

   REAL(kind=dp) :: alpha_1(0:(0+3), 0:0, 0:3)
   REAL(kind=dp) :: alpha_2(0:(0+3), 0:0, 0:3)
   REAL(kind=dp) :: alpha_3(0:(0+3), 0:0, 0:3)
   REAL(kind=dp) :: coef_ttz(0:0, 0:3)
   REAL(kind=dp) :: coef_tyz(0:0, 0:3, 0:0, 0:3)
   coef_xyz = coef_xyz*prefactor

   alpha_1(:, :, :) = 0.0_dp
   alpha_2(:, :, :) = 0.0_dp
   alpha_3(:, :, :) = 0.0_dp

   const_rp = rp(1)-(ra(1)+rab(1))
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(0, 0, 0) = alpha_1(0, 0, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(1, 0, 1) = alpha_1(1, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_1(0, 0, 1) = alpha_1(0, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(2, 0, 2) = alpha_1(2, 0, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(2, dp)
   b = b*const_rp
   alpha_1(1, 0, 2) = alpha_1(1, 0, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(2, dp)
   b = b*const_rp
   alpha_1(0, 0, 2) = alpha_1(0, 0, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(3, 0, 3) = alpha_1(3, 0, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(3, dp)
   b = b*const_rp
   alpha_1(2, 0, 3) = alpha_1(2, 0, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_1(1, 0, 3) = alpha_1(1, 0, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(3, dp)
   b = b*const_rp
   alpha_1(0, 0, 3) = alpha_1(0, 0, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   const_rp = rp(2)-(ra(2)+rab(2))
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(0, 0, 0) = alpha_2(0, 0, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(1, 0, 1) = alpha_2(1, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_2(0, 0, 1) = alpha_2(0, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(2, 0, 2) = alpha_2(2, 0, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(2, dp)
   b = b*const_rp
   alpha_2(1, 0, 2) = alpha_2(1, 0, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(2, dp)
   b = b*const_rp
   alpha_2(0, 0, 2) = alpha_2(0, 0, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(3, 0, 3) = alpha_2(3, 0, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(3, dp)
   b = b*const_rp
   alpha_2(2, 0, 3) = alpha_2(2, 0, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_2(1, 0, 3) = alpha_2(1, 0, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(3, dp)
   b = b*const_rp
   alpha_2(0, 0, 3) = alpha_2(0, 0, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   const_rp = rp(3)-(ra(3)+rab(3))
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(0, 0, 0) = alpha_3(0, 0, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(1, 0, 1) = alpha_3(1, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_3(0, 0, 1) = alpha_3(0, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(2, 0, 2) = alpha_3(2, 0, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(2, dp)
   b = b*const_rp
   alpha_3(1, 0, 2) = alpha_3(1, 0, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(2, dp)
   b = b*const_rp
   alpha_3(0, 0, 2) = alpha_3(0, 0, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(3, 0, 3) = alpha_3(3, 0, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(3, dp)
   b = b*const_rp
   alpha_3(2, 0, 3) = alpha_3(2, 0, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_3(1, 0, 3) = alpha_3(1, 0, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(3, dp)
   b = b*const_rp
   alpha_3(0, 0, 3) = alpha_3(0, 0, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0

   lxyz = 0
   DO lzp = 0, 0+3
      coef_tyz = 0.0_dp
      DO lyp = 0, 0+3-lzp
         coef_ttz = 0.0_dp
         DO lxp = 0, 0+3-lzp-lyp
            lxyz = lxyz+1
            coef_ttz(0, 0) = coef_ttz(0, 0)+coef_xyz(lxyz)*alpha_1(lxp, 0, 0)
            coef_ttz(0, 1) = coef_ttz(0, 1)+coef_xyz(lxyz)*alpha_1(lxp, 0, 1)
            coef_ttz(0, 2) = coef_ttz(0, 2)+coef_xyz(lxyz)*alpha_1(lxp, 0, 2)
            coef_ttz(0, 3) = coef_ttz(0, 3)+coef_xyz(lxyz)*alpha_1(lxp, 0, 3)
         ENDDO

         coef_tyz(0, 0, 0, 0) = coef_tyz(0, 0, 0, 0)+coef_ttz(0, 0)*alpha_2(lyp, 0, 0)
         coef_tyz(0, 1, 0, 0) = coef_tyz(0, 1, 0, 0)+coef_ttz(0, 1)*alpha_2(lyp, 0, 0)
         coef_tyz(0, 2, 0, 0) = coef_tyz(0, 2, 0, 0)+coef_ttz(0, 2)*alpha_2(lyp, 0, 0)
         coef_tyz(0, 3, 0, 0) = coef_tyz(0, 3, 0, 0)+coef_ttz(0, 3)*alpha_2(lyp, 0, 0)
         coef_tyz(0, 0, 0, 1) = coef_tyz(0, 0, 0, 1)+coef_ttz(0, 0)*alpha_2(lyp, 0, 1)
         coef_tyz(0, 1, 0, 1) = coef_tyz(0, 1, 0, 1)+coef_ttz(0, 1)*alpha_2(lyp, 0, 1)
         coef_tyz(0, 2, 0, 1) = coef_tyz(0, 2, 0, 1)+coef_ttz(0, 2)*alpha_2(lyp, 0, 1)
         coef_tyz(0, 0, 0, 2) = coef_tyz(0, 0, 0, 2)+coef_ttz(0, 0)*alpha_2(lyp, 0, 2)
         coef_tyz(0, 1, 0, 2) = coef_tyz(0, 1, 0, 2)+coef_ttz(0, 1)*alpha_2(lyp, 0, 2)
         coef_tyz(0, 0, 0, 3) = coef_tyz(0, 0, 0, 3)+coef_ttz(0, 0)*alpha_2(lyp, 0, 3)
      ENDDO

      DO lxb = lb_min_local, 3
         jco = coset(lxb, 0, 0)

         lxa = 0
         !ico=coset(0,0,0)
         ico = 1

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 0, 0)

      ENDDO
      DO lxb = MAX(lb_min_local-0-1, 0), 2
         jco = coset(lxb, 1, 0)

         lxa = 0
         !ico=coset(0,0,0)
         ico = 1

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 1)*alpha_3(lzp, 0, 0)

      ENDDO
      DO lxb = MAX(lb_min_local-0-2, 0), 1
         jco = coset(lxb, 2, 0)

         lxa = 0
         !ico=coset(0,0,0)
         ico = 1

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 2)*alpha_3(lzp, 0, 0)

      ENDDO
      lxb = 0
      !jco=coset(0,3,0)
      jco = 17

      lxa = 0
      !ico=coset(0,0,0)
      ico = 1

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 3)*alpha_3(lzp, 0, 0)

      DO lxb = MAX(lb_min_local-1-0, 0), 2
         jco = coset(lxb, 0, 1)

         lxa = 0
         !ico=coset(0,0,0)
         ico = 1

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 0, 1)

      ENDDO
      DO lxb = MAX(lb_min_local-1-1, 0), 1
         jco = coset(lxb, 1, 1)

         lxa = 0
         !ico=coset(0,0,0)
         ico = 1

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 1)*alpha_3(lzp, 0, 1)

      ENDDO
      lxb = 0
      !jco=coset(0,2,1)
      jco = 18

      lxa = 0
      !ico=coset(0,0,0)
      ico = 1

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 2)*alpha_3(lzp, 0, 1)

      DO lxb = MAX(lb_min_local-2-0, 0), 1
         jco = coset(lxb, 0, 2)

         lxa = 0
         !ico=coset(0,0,0)
         ico = 1

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 0, 2)

      ENDDO
      lxb = 0
      !jco=coset(0,1,2)
      jco = 19

      lxa = 0
      !ico=coset(0,0,0)
      ico = 1

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 1)*alpha_3(lzp, 0, 2)

      lxb = 0
      !jco=coset(0,0,3)
      jco = 20

      lxa = 0
      !ico=coset(0,0,0)
      ico = 1

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 0, 3)

   ENDDO

END SUBROUTINE xyz_to_vab_0_3

! Fully unrolled version
! **************************************************************************************************
!> \brief ...
!> \param prefactor ...
!> \param coef_xyz ...
!> \param lp ...
!> \param rp ...
!> \param ra ...
!> \param rab ...
!> \param vab ...
!> \param coset ...
!> \param lb_min_local ...
!> \param maxl ...
!> \param lvab ...
!> \param hvab ...
! **************************************************************************************************
SUBROUTINE xyz_to_vab_0_4(prefactor, coef_xyz, &
                          lp, rp, ra, rab, vab, coset, &
                          lb_min_local, maxl, lvab, hvab)

       USE kinds,                        ONLY: dp

   ! PARAMETERS
   REAL(KIND=dp), INTENT(IN) :: prefactor
   INTEGER, INTENT(IN) :: lp, &
                          lb_min_local, maxl, &
                          lvab, hvab

   INTEGER, DIMENSION(-1:maxl, -1:maxl, -1:maxl), &
      INTENT(IN)     :: coset

   REAL(kind=dp), DIMENSION(((lp+1)*(lp+2)*(lp+3))/6), &
      INTENT(INOUT) :: coef_xyz
   REAL(kind=dp), DIMENSION(lvab, hvab), INTENT(INOUT) :: vab
   REAL(kind=dp), DIMENSION(3), INTENT(IN) :: rp, ra, rab

   ! LOCAL
   INTEGER :: ico, jco, lxa, lxb, lxp, lxyz, lyp, lzp
   REAL(KIND=dp) :: a, b, binomial_k_lxa, binomial_l_lxb, const_rp

   REAL(kind=dp) :: alpha_1(0:(0+4), 0:0, 0:4)
   REAL(kind=dp) :: alpha_2(0:(0+4), 0:0, 0:4)
   REAL(kind=dp) :: alpha_3(0:(0+4), 0:0, 0:4)
   REAL(kind=dp) :: coef_ttz(0:0, 0:4)
   REAL(kind=dp) :: coef_tyz(0:0, 0:4, 0:0, 0:4)
   coef_xyz = coef_xyz*prefactor

   alpha_1(:, :, :) = 0.0_dp
   alpha_2(:, :, :) = 0.0_dp
   alpha_3(:, :, :) = 0.0_dp

   const_rp = rp(1)-(ra(1)+rab(1))
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(0, 0, 0) = alpha_1(0, 0, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(1, 0, 1) = alpha_1(1, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_1(0, 0, 1) = alpha_1(0, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(2, 0, 2) = alpha_1(2, 0, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(2, dp)
   b = b*const_rp
   alpha_1(1, 0, 2) = alpha_1(1, 0, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(2, dp)
   b = b*const_rp
   alpha_1(0, 0, 2) = alpha_1(0, 0, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(3, 0, 3) = alpha_1(3, 0, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(3, dp)
   b = b*const_rp
   alpha_1(2, 0, 3) = alpha_1(2, 0, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_1(1, 0, 3) = alpha_1(1, 0, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(3, dp)
   b = b*const_rp
   alpha_1(0, 0, 3) = alpha_1(0, 0, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(4, 0, 4) = alpha_1(4, 0, 4)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(4, dp)
   b = b*const_rp
   alpha_1(3, 0, 4) = alpha_1(3, 0, 4)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(3, dp)/REAL(2, dp)
   b = b*const_rp
   alpha_1(2, 0, 4) = alpha_1(2, 0, 4)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(2, dp)/REAL(3, dp)
   b = b*const_rp
   alpha_1(1, 0, 4) = alpha_1(1, 0, 4)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(4, dp)
   b = b*const_rp
   alpha_1(0, 0, 4) = alpha_1(0, 0, 4)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   const_rp = rp(2)-(ra(2)+rab(2))
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(0, 0, 0) = alpha_2(0, 0, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(1, 0, 1) = alpha_2(1, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_2(0, 0, 1) = alpha_2(0, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(2, 0, 2) = alpha_2(2, 0, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(2, dp)
   b = b*const_rp
   alpha_2(1, 0, 2) = alpha_2(1, 0, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(2, dp)
   b = b*const_rp
   alpha_2(0, 0, 2) = alpha_2(0, 0, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(3, 0, 3) = alpha_2(3, 0, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(3, dp)
   b = b*const_rp
   alpha_2(2, 0, 3) = alpha_2(2, 0, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_2(1, 0, 3) = alpha_2(1, 0, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(3, dp)
   b = b*const_rp
   alpha_2(0, 0, 3) = alpha_2(0, 0, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(4, 0, 4) = alpha_2(4, 0, 4)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(4, dp)
   b = b*const_rp
   alpha_2(3, 0, 4) = alpha_2(3, 0, 4)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(3, dp)/REAL(2, dp)
   b = b*const_rp
   alpha_2(2, 0, 4) = alpha_2(2, 0, 4)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(2, dp)/REAL(3, dp)
   b = b*const_rp
   alpha_2(1, 0, 4) = alpha_2(1, 0, 4)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(4, dp)
   b = b*const_rp
   alpha_2(0, 0, 4) = alpha_2(0, 0, 4)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   const_rp = rp(3)-(ra(3)+rab(3))
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(0, 0, 0) = alpha_3(0, 0, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(1, 0, 1) = alpha_3(1, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_3(0, 0, 1) = alpha_3(0, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(2, 0, 2) = alpha_3(2, 0, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(2, dp)
   b = b*const_rp
   alpha_3(1, 0, 2) = alpha_3(1, 0, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(2, dp)
   b = b*const_rp
   alpha_3(0, 0, 2) = alpha_3(0, 0, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(3, 0, 3) = alpha_3(3, 0, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(3, dp)
   b = b*const_rp
   alpha_3(2, 0, 3) = alpha_3(2, 0, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_3(1, 0, 3) = alpha_3(1, 0, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(3, dp)
   b = b*const_rp
   alpha_3(0, 0, 3) = alpha_3(0, 0, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(4, 0, 4) = alpha_3(4, 0, 4)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(4, dp)
   b = b*const_rp
   alpha_3(3, 0, 4) = alpha_3(3, 0, 4)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(3, dp)/REAL(2, dp)
   b = b*const_rp
   alpha_3(2, 0, 4) = alpha_3(2, 0, 4)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(2, dp)/REAL(3, dp)
   b = b*const_rp
   alpha_3(1, 0, 4) = alpha_3(1, 0, 4)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(4, dp)
   b = b*const_rp
   alpha_3(0, 0, 4) = alpha_3(0, 0, 4)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0

   lxyz = 0
   DO lzp = 0, 0+4
      coef_tyz = 0.0_dp
      DO lyp = 0, 0+4-lzp
         coef_ttz = 0.0_dp
         DO lxp = 0, 0+4-lzp-lyp
            lxyz = lxyz+1
            coef_ttz(0, 0) = coef_ttz(0, 0)+coef_xyz(lxyz)*alpha_1(lxp, 0, 0)
            coef_ttz(0, 1) = coef_ttz(0, 1)+coef_xyz(lxyz)*alpha_1(lxp, 0, 1)
            coef_ttz(0, 2) = coef_ttz(0, 2)+coef_xyz(lxyz)*alpha_1(lxp, 0, 2)
            coef_ttz(0, 3) = coef_ttz(0, 3)+coef_xyz(lxyz)*alpha_1(lxp, 0, 3)
            coef_ttz(0, 4) = coef_ttz(0, 4)+coef_xyz(lxyz)*alpha_1(lxp, 0, 4)
         ENDDO

         coef_tyz(0, 0, 0, 0) = coef_tyz(0, 0, 0, 0)+coef_ttz(0, 0)*alpha_2(lyp, 0, 0)
         coef_tyz(0, 1, 0, 0) = coef_tyz(0, 1, 0, 0)+coef_ttz(0, 1)*alpha_2(lyp, 0, 0)
         coef_tyz(0, 2, 0, 0) = coef_tyz(0, 2, 0, 0)+coef_ttz(0, 2)*alpha_2(lyp, 0, 0)
         coef_tyz(0, 3, 0, 0) = coef_tyz(0, 3, 0, 0)+coef_ttz(0, 3)*alpha_2(lyp, 0, 0)
         coef_tyz(0, 4, 0, 0) = coef_tyz(0, 4, 0, 0)+coef_ttz(0, 4)*alpha_2(lyp, 0, 0)
         coef_tyz(0, 0, 0, 1) = coef_tyz(0, 0, 0, 1)+coef_ttz(0, 0)*alpha_2(lyp, 0, 1)
         coef_tyz(0, 1, 0, 1) = coef_tyz(0, 1, 0, 1)+coef_ttz(0, 1)*alpha_2(lyp, 0, 1)
         coef_tyz(0, 2, 0, 1) = coef_tyz(0, 2, 0, 1)+coef_ttz(0, 2)*alpha_2(lyp, 0, 1)
         coef_tyz(0, 3, 0, 1) = coef_tyz(0, 3, 0, 1)+coef_ttz(0, 3)*alpha_2(lyp, 0, 1)
         coef_tyz(0, 0, 0, 2) = coef_tyz(0, 0, 0, 2)+coef_ttz(0, 0)*alpha_2(lyp, 0, 2)
         coef_tyz(0, 1, 0, 2) = coef_tyz(0, 1, 0, 2)+coef_ttz(0, 1)*alpha_2(lyp, 0, 2)
         coef_tyz(0, 2, 0, 2) = coef_tyz(0, 2, 0, 2)+coef_ttz(0, 2)*alpha_2(lyp, 0, 2)
         coef_tyz(0, 0, 0, 3) = coef_tyz(0, 0, 0, 3)+coef_ttz(0, 0)*alpha_2(lyp, 0, 3)
         coef_tyz(0, 1, 0, 3) = coef_tyz(0, 1, 0, 3)+coef_ttz(0, 1)*alpha_2(lyp, 0, 3)
         coef_tyz(0, 0, 0, 4) = coef_tyz(0, 0, 0, 4)+coef_ttz(0, 0)*alpha_2(lyp, 0, 4)
      ENDDO

      DO lxb = lb_min_local, 4
         jco = coset(lxb, 0, 0)

         lxa = 0
         !ico=coset(0,0,0)
         ico = 1

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 0, 0)

      ENDDO
      DO lxb = MAX(lb_min_local-0-1, 0), 3
         jco = coset(lxb, 1, 0)

         lxa = 0
         !ico=coset(0,0,0)
         ico = 1

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 1)*alpha_3(lzp, 0, 0)

      ENDDO
      DO lxb = MAX(lb_min_local-0-2, 0), 2
         jco = coset(lxb, 2, 0)

         lxa = 0
         !ico=coset(0,0,0)
         ico = 1

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 2)*alpha_3(lzp, 0, 0)

      ENDDO
      DO lxb = MAX(lb_min_local-0-3, 0), 1
         jco = coset(lxb, 3, 0)

         lxa = 0
         !ico=coset(0,0,0)
         ico = 1

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 3)*alpha_3(lzp, 0, 0)

      ENDDO
      lxb = 0
      !jco=coset(0,4,0)
      jco = 31

      lxa = 0
      !ico=coset(0,0,0)
      ico = 1

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 4)*alpha_3(lzp, 0, 0)

      DO lxb = MAX(lb_min_local-1-0, 0), 3
         jco = coset(lxb, 0, 1)

         lxa = 0
         !ico=coset(0,0,0)
         ico = 1

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 0, 1)

      ENDDO
      DO lxb = MAX(lb_min_local-1-1, 0), 2
         jco = coset(lxb, 1, 1)

         lxa = 0
         !ico=coset(0,0,0)
         ico = 1

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 1)*alpha_3(lzp, 0, 1)

      ENDDO
      DO lxb = MAX(lb_min_local-1-2, 0), 1
         jco = coset(lxb, 2, 1)

         lxa = 0
         !ico=coset(0,0,0)
         ico = 1

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 2)*alpha_3(lzp, 0, 1)

      ENDDO
      lxb = 0
      !jco=coset(0,3,1)
      jco = 32

      lxa = 0
      !ico=coset(0,0,0)
      ico = 1

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 3)*alpha_3(lzp, 0, 1)

      DO lxb = MAX(lb_min_local-2-0, 0), 2
         jco = coset(lxb, 0, 2)

         lxa = 0
         !ico=coset(0,0,0)
         ico = 1

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 0, 2)

      ENDDO
      DO lxb = MAX(lb_min_local-2-1, 0), 1
         jco = coset(lxb, 1, 2)

         lxa = 0
         !ico=coset(0,0,0)
         ico = 1

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 1)*alpha_3(lzp, 0, 2)

      ENDDO
      lxb = 0
      !jco=coset(0,2,2)
      jco = 33

      lxa = 0
      !ico=coset(0,0,0)
      ico = 1

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 2)*alpha_3(lzp, 0, 2)

      DO lxb = MAX(lb_min_local-3-0, 0), 1
         jco = coset(lxb, 0, 3)

         lxa = 0
         !ico=coset(0,0,0)
         ico = 1

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 0, 3)

      ENDDO
      lxb = 0
      !jco=coset(0,1,3)
      jco = 34

      lxa = 0
      !ico=coset(0,0,0)
      ico = 1

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 1)*alpha_3(lzp, 0, 3)

      lxb = 0
      !jco=coset(0,0,4)
      jco = 35

      lxa = 0
      !ico=coset(0,0,0)
      ico = 1

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 0, 4)

   ENDDO

END SUBROUTINE xyz_to_vab_0_4

! Fully unrolled version
! **************************************************************************************************
!> \brief ...
!> \param prefactor ...
!> \param coef_xyz ...
!> \param lp ...
!> \param rp ...
!> \param ra ...
!> \param vab ...
!> \param coset ...
!> \param la_min_local ...
!> \param maxl ...
!> \param lvab ...
!> \param hvab ...
! **************************************************************************************************
SUBROUTINE xyz_to_vab_1_0(prefactor, coef_xyz, &
                          lp, rp, ra, vab, coset, &
                          la_min_local, maxl, lvab, hvab)

       USE kinds,                        ONLY: dp

   ! PARAMETERS
   REAL(KIND=dp), INTENT(IN) :: prefactor
   INTEGER, INTENT(IN) :: lp, &
                          la_min_local, maxl, &
                          lvab, hvab

   INTEGER, DIMENSION(-1:maxl, -1:maxl, -1:maxl), &
      INTENT(IN)     :: coset

   REAL(kind=dp), DIMENSION(((lp+1)*(lp+2)*(lp+3))/6), &
      INTENT(INOUT) :: coef_xyz
   REAL(kind=dp), DIMENSION(lvab, hvab), INTENT(INOUT) :: vab
   REAL(kind=dp), DIMENSION(3), INTENT(IN) :: rp, ra

   ! LOCAL
   INTEGER :: ico, jco, lxa, lxb, lxp, lxyz, lyp
   REAL(KIND=dp) :: a, b, binomial_k_lxa, binomial_l_lxb, const_ra

   REAL(kind=dp) :: alpha_1(0:(1+0), 0:1, 0:0)
   REAL(kind=dp) :: alpha_2(0:(1+0), 0:1, 0:0)
   REAL(kind=dp) :: alpha_3(0:(1+0), 0:1, 0:0)
   REAL(kind=dp) :: coef_ttz(0:1, 0:0)
   REAL(kind=dp) :: coef_tyz(0:1, 0:0, 0:1, 0:0)
   coef_xyz = coef_xyz*prefactor

   alpha_1(:, :, :) = 0.0_dp
   alpha_2(:, :, :) = 0.0_dp
   alpha_3(:, :, :) = 0.0_dp

   const_ra = (-ra(1)+rp(1))
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(0, 0, 0) = alpha_1(0, 0, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(1, 1, 0) = alpha_1(1, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(0, 1, 0) = alpha_1(0, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   const_ra = (-ra(2)+rp(2))
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(0, 0, 0) = alpha_2(0, 0, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(1, 1, 0) = alpha_2(1, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(0, 1, 0) = alpha_2(0, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   const_ra = (-ra(3)+rp(3))
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(0, 0, 0) = alpha_3(0, 0, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(1, 1, 0) = alpha_3(1, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(0, 1, 0) = alpha_3(0, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0

   lxyz = 0
   coef_tyz = 0.0_dp
   DO lyp = 0, 1+0-0
      coef_ttz = 0.0_dp
      DO lxp = 0, 1+0-0-lyp
         lxyz = lxyz+1
         coef_ttz(0, 0) = coef_ttz(0, 0)+coef_xyz(lxyz)*alpha_1(lxp, 0, 0)
         coef_ttz(1, 0) = coef_ttz(1, 0)+coef_xyz(lxyz)*alpha_1(lxp, 1, 0)
      ENDDO

      coef_tyz(0, 0, 0, 0) = coef_tyz(0, 0, 0, 0)+coef_ttz(0, 0)*alpha_2(lyp, 0, 0)
      coef_tyz(1, 0, 0, 0) = coef_tyz(1, 0, 0, 0)+coef_ttz(1, 0)*alpha_2(lyp, 0, 0)
      coef_tyz(0, 0, 1, 0) = coef_tyz(0, 0, 1, 0)+coef_ttz(0, 0)*alpha_2(lyp, 1, 0)

   ENDDO

   lxb = 0
   !jco=coset(0,0,0)
   jco = 1

   DO lxa = la_min_local, 1
      ico = coset(lxa, 0, 0)

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(0, 0, 0)

   ENDDO

   lxb = 0
   !jco=coset(0,0,0)
   jco = 1

   lxa = 0
   !ico=coset(0,1,0)
   ico = 3

   vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 1, 0)*alpha_3(0, 0, 0)

   lxb = 0
   !jco=coset(0,0,0)
   jco = 1

   lxa = 0
   !ico=coset(0,0,1)
   ico = 4

   vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(0, 1, 0)

   coef_tyz = 0.0_dp
   DO lyp = 0, 1+0-1
      coef_ttz = 0.0_dp
      DO lxp = 0, 1+0-1-lyp
         lxyz = lxyz+1
         coef_ttz(0, 0) = coef_ttz(0, 0)+coef_xyz(lxyz)*alpha_1(lxp, 0, 0)
         coef_ttz(1, 0) = coef_ttz(1, 0)+coef_xyz(lxyz)*alpha_1(lxp, 1, 0)
      ENDDO

      coef_tyz(0, 0, 0, 0) = coef_tyz(0, 0, 0, 0)+coef_ttz(0, 0)*alpha_2(lyp, 0, 0)
      coef_tyz(1, 0, 0, 0) = coef_tyz(1, 0, 0, 0)+coef_ttz(1, 0)*alpha_2(lyp, 0, 0)
      coef_tyz(0, 0, 1, 0) = coef_tyz(0, 0, 1, 0)+coef_ttz(0, 0)*alpha_2(lyp, 1, 0)

   ENDDO

   lxb = 0
   !jco=coset(0,0,0)
   jco = 1

   DO lxa = la_min_local, 1
      ico = coset(lxa, 0, 0)

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(1, 0, 0)

   ENDDO

   lxb = 0
   !jco=coset(0,0,0)
   jco = 1

   lxa = 0
   !ico=coset(0,1,0)
   ico = 3

   vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 1, 0)*alpha_3(1, 0, 0)

   lxb = 0
   !jco=coset(0,0,0)
   jco = 1

   lxa = 0
   !ico=coset(0,0,1)
   ico = 4

   vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(1, 1, 0)

END SUBROUTINE xyz_to_vab_1_0

! Fully unrolled version
! **************************************************************************************************
!> \brief ...
!> \param prefactor ...
!> \param coef_xyz ...
!> \param lp ...
!> \param rp ...
!> \param ra ...
!> \param rab ...
!> \param vab ...
!> \param coset ...
!> \param la_min_local ...
!> \param lb_min_local ...
!> \param maxl ...
!> \param lvab ...
!> \param hvab ...
! **************************************************************************************************
SUBROUTINE xyz_to_vab_1_1(prefactor, coef_xyz, &
                          lp, rp, ra, rab, vab, coset, &
                          la_min_local, lb_min_local, maxl, lvab, hvab)

       USE kinds,                        ONLY: dp

   ! PARAMETERS
   REAL(KIND=dp), INTENT(IN) :: prefactor
   INTEGER, INTENT(IN) :: lp, &
                          lb_min_local, la_min_local, maxl, &
                          lvab, hvab

   INTEGER, DIMENSION(-1:maxl, -1:maxl, -1:maxl), &
      INTENT(IN)     :: coset

   REAL(kind=dp), DIMENSION(((lp+1)*(lp+2)*(lp+3))/6), &
      INTENT(INOUT) :: coef_xyz
   REAL(kind=dp), DIMENSION(lvab, hvab), INTENT(INOUT) :: vab
   REAL(kind=dp), DIMENSION(3), INTENT(IN) :: rp, ra, rab

   ! LOCAL
   INTEGER :: ico, jco, lxa, lxb, lxp, lxyz, lyp, lzp
   REAL(KIND=dp) :: a, b, binomial_k_lxa, binomial_l_lxb, const_rp, const_ra

   REAL(kind=dp) :: alpha_1(0:(1+1), 0:1, 0:1)
   REAL(kind=dp) :: alpha_2(0:(1+1), 0:1, 0:1)
   REAL(kind=dp) :: alpha_3(0:(1+1), 0:1, 0:1)
   REAL(kind=dp) :: coef_ttz(0:1, 0:1)
   REAL(kind=dp) :: coef_tyz(0:1, 0:1, 0:1, 0:1)
   coef_xyz = coef_xyz*prefactor

   alpha_1(:, :, :) = 0.0_dp
   alpha_2(:, :, :) = 0.0_dp
   alpha_3(:, :, :) = 0.0_dp

   const_rp = rp(1)-(ra(1)+rab(1))
   const_ra = (-ra(1)+rp(1))
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(0, 0, 0) = alpha_1(0, 0, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(1, 0, 1) = alpha_1(1, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_1(0, 0, 1) = alpha_1(0, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(1, 1, 0) = alpha_1(1, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(0, 1, 0) = alpha_1(0, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(2, 1, 1) = alpha_1(2, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_1(1, 1, 1) = alpha_1(1, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(1, 1, 1) = alpha_1(1, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_1(0, 1, 1) = alpha_1(0, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   const_rp = rp(2)-(ra(2)+rab(2))
   const_ra = (-ra(2)+rp(2))
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(0, 0, 0) = alpha_2(0, 0, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(1, 0, 1) = alpha_2(1, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_2(0, 0, 1) = alpha_2(0, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(1, 1, 0) = alpha_2(1, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(0, 1, 0) = alpha_2(0, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(2, 1, 1) = alpha_2(2, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_2(1, 1, 1) = alpha_2(1, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(1, 1, 1) = alpha_2(1, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_2(0, 1, 1) = alpha_2(0, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   const_rp = rp(3)-(ra(3)+rab(3))
   const_ra = (-ra(3)+rp(3))
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(0, 0, 0) = alpha_3(0, 0, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(1, 0, 1) = alpha_3(1, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_3(0, 0, 1) = alpha_3(0, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(1, 1, 0) = alpha_3(1, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(0, 1, 0) = alpha_3(0, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(2, 1, 1) = alpha_3(2, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_3(1, 1, 1) = alpha_3(1, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(1, 1, 1) = alpha_3(1, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_3(0, 1, 1) = alpha_3(0, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0

   lxyz = 0
   DO lzp = 0, 1+1
      coef_tyz = 0.0_dp
      DO lyp = 0, 1+1-lzp
         coef_ttz = 0.0_dp
         DO lxp = 0, 1+1-lzp-lyp
            lxyz = lxyz+1
            coef_ttz(0, 0) = coef_ttz(0, 0)+coef_xyz(lxyz)*alpha_1(lxp, 0, 0)
            coef_ttz(1, 0) = coef_ttz(1, 0)+coef_xyz(lxyz)*alpha_1(lxp, 1, 0)
            coef_ttz(0, 1) = coef_ttz(0, 1)+coef_xyz(lxyz)*alpha_1(lxp, 0, 1)
            coef_ttz(1, 1) = coef_ttz(1, 1)+coef_xyz(lxyz)*alpha_1(lxp, 1, 1)
         ENDDO

         coef_tyz(0, 0, 0, 0) = coef_tyz(0, 0, 0, 0)+coef_ttz(0, 0)*alpha_2(lyp, 0, 0)
         coef_tyz(1, 0, 0, 0) = coef_tyz(1, 0, 0, 0)+coef_ttz(1, 0)*alpha_2(lyp, 0, 0)
         coef_tyz(0, 1, 0, 0) = coef_tyz(0, 1, 0, 0)+coef_ttz(0, 1)*alpha_2(lyp, 0, 0)
         coef_tyz(1, 1, 0, 0) = coef_tyz(1, 1, 0, 0)+coef_ttz(1, 1)*alpha_2(lyp, 0, 0)
         coef_tyz(0, 0, 1, 0) = coef_tyz(0, 0, 1, 0)+coef_ttz(0, 0)*alpha_2(lyp, 1, 0)
         coef_tyz(0, 1, 1, 0) = coef_tyz(0, 1, 1, 0)+coef_ttz(0, 1)*alpha_2(lyp, 1, 0)
         coef_tyz(0, 0, 0, 1) = coef_tyz(0, 0, 0, 1)+coef_ttz(0, 0)*alpha_2(lyp, 0, 1)
         coef_tyz(1, 0, 0, 1) = coef_tyz(1, 0, 0, 1)+coef_ttz(1, 0)*alpha_2(lyp, 0, 1)
         coef_tyz(0, 0, 1, 1) = coef_tyz(0, 0, 1, 1)+coef_ttz(0, 0)*alpha_2(lyp, 1, 1)
      ENDDO

      DO lxb = lb_min_local, 1
         jco = coset(lxb, 0, 0)

         DO lxa = la_min_local, 1

            ico = coset(lxa, 0, 0)

            vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 0, 0)

         ENDDO

      ENDDO
      DO lxb = lb_min_local, 1
         jco = coset(lxb, 0, 0)

         lxa = 0
         !ico=coset(0,1,0)
         ico = 3

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 1, 0)*alpha_3(lzp, 0, 0)

      ENDDO
      lxb = 0
      !jco=coset(0,1,0)
      jco = 3

      DO lxa = la_min_local, 1

         ico = coset(lxa, 0, 0)

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 1)*alpha_3(lzp, 0, 0)

      ENDDO

      lxb = 0
      !jco=coset(0,1,0)
      jco = 3

      lxa = 0
      !ico=coset(0,1,0)
      ico = 3

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 1, 1)*alpha_3(lzp, 0, 0)

      DO lxb = lb_min_local, 1
         jco = coset(lxb, 0, 0)

         lxa = 0
         !ico=coset(0,0,1)
         ico = 4

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 1, 0)

      ENDDO
      lxb = 0
      !jco=coset(0,1,0)
      jco = 3

      lxa = 0
      !ico=coset(0,0,1)
      ico = 4

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 1)*alpha_3(lzp, 1, 0)

      lxb = 0
      !jco=coset(0,0,1)
      jco = 4

      DO lxa = la_min_local, 1

         ico = coset(lxa, 0, 0)

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 0, 1)

      ENDDO

      lxb = 0
      !jco=coset(0,0,1)
      jco = 4

      lxa = 0
      !ico=coset(0,1,0)
      ico = 3

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 1, 0)*alpha_3(lzp, 0, 1)

      lxb = 0
      !jco=coset(0,0,1)
      jco = 4

      lxa = 0
      !ico=coset(0,0,1)
      ico = 4

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 1, 1)

   ENDDO

END SUBROUTINE xyz_to_vab_1_1

! Fully unrolled version
! **************************************************************************************************
!> \brief ...
!> \param prefactor ...
!> \param coef_xyz ...
!> \param lp ...
!> \param rp ...
!> \param ra ...
!> \param rab ...
!> \param vab ...
!> \param coset ...
!> \param la_min_local ...
!> \param lb_min_local ...
!> \param maxl ...
!> \param lvab ...
!> \param hvab ...
! **************************************************************************************************
SUBROUTINE xyz_to_vab_1_2(prefactor, coef_xyz, &
                          lp, rp, ra, rab, vab, coset, &
                          la_min_local, lb_min_local, maxl, lvab, hvab)

       USE kinds,                        ONLY: dp

   ! PARAMETERS
   REAL(KIND=dp), INTENT(IN) :: prefactor
   INTEGER, INTENT(IN) :: lp, &
                          lb_min_local, la_min_local, maxl, &
                          lvab, hvab

   INTEGER, DIMENSION(-1:maxl, -1:maxl, -1:maxl), &
      INTENT(IN)     :: coset

   REAL(kind=dp), DIMENSION(((lp+1)*(lp+2)*(lp+3))/6), &
      INTENT(INOUT) :: coef_xyz
   REAL(kind=dp), DIMENSION(lvab, hvab), INTENT(INOUT) :: vab
   REAL(kind=dp), DIMENSION(3), INTENT(IN) :: rp, ra, rab

   ! LOCAL
   INTEGER :: ico, jco, lxa, lxb, lxp, lxyz, lyp, lzp
   REAL(KIND=dp) :: a, b, binomial_k_lxa, binomial_l_lxb, const_rp, const_ra

   REAL(kind=dp) :: alpha_1(0:(1+2), 0:1, 0:2)
   REAL(kind=dp) :: alpha_2(0:(1+2), 0:1, 0:2)
   REAL(kind=dp) :: alpha_3(0:(1+2), 0:1, 0:2)
   REAL(kind=dp) :: coef_ttz(0:1, 0:2)
   REAL(kind=dp) :: coef_tyz(0:1, 0:2, 0:1, 0:2)
   coef_xyz = coef_xyz*prefactor

   alpha_1(:, :, :) = 0.0_dp
   alpha_2(:, :, :) = 0.0_dp
   alpha_3(:, :, :) = 0.0_dp

   const_rp = rp(1)-(ra(1)+rab(1))
   const_ra = (-ra(1)+rp(1))
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(0, 0, 0) = alpha_1(0, 0, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(1, 0, 1) = alpha_1(1, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_1(0, 0, 1) = alpha_1(0, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(2, 0, 2) = alpha_1(2, 0, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(2, dp)
   b = b*const_rp
   alpha_1(1, 0, 2) = alpha_1(1, 0, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(2, dp)
   b = b*const_rp
   alpha_1(0, 0, 2) = alpha_1(0, 0, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(1, 1, 0) = alpha_1(1, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(0, 1, 0) = alpha_1(0, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(2, 1, 1) = alpha_1(2, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_1(1, 1, 1) = alpha_1(1, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(1, 1, 1) = alpha_1(1, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_1(0, 1, 1) = alpha_1(0, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(3, 1, 2) = alpha_1(3, 1, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(2, dp)
   b = b*const_rp
   alpha_1(2, 1, 2) = alpha_1(2, 1, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(2, dp)
   b = b*const_rp
   alpha_1(1, 1, 2) = alpha_1(1, 1, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(2, 1, 2) = alpha_1(2, 1, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(2, dp)
   b = b*const_rp
   alpha_1(1, 1, 2) = alpha_1(1, 1, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(2, dp)
   b = b*const_rp
   alpha_1(0, 1, 2) = alpha_1(0, 1, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   const_rp = rp(2)-(ra(2)+rab(2))
   const_ra = (-ra(2)+rp(2))
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(0, 0, 0) = alpha_2(0, 0, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(1, 0, 1) = alpha_2(1, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_2(0, 0, 1) = alpha_2(0, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(2, 0, 2) = alpha_2(2, 0, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(2, dp)
   b = b*const_rp
   alpha_2(1, 0, 2) = alpha_2(1, 0, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(2, dp)
   b = b*const_rp
   alpha_2(0, 0, 2) = alpha_2(0, 0, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(1, 1, 0) = alpha_2(1, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(0, 1, 0) = alpha_2(0, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(2, 1, 1) = alpha_2(2, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_2(1, 1, 1) = alpha_2(1, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(1, 1, 1) = alpha_2(1, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_2(0, 1, 1) = alpha_2(0, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(3, 1, 2) = alpha_2(3, 1, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(2, dp)
   b = b*const_rp
   alpha_2(2, 1, 2) = alpha_2(2, 1, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(2, dp)
   b = b*const_rp
   alpha_2(1, 1, 2) = alpha_2(1, 1, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(2, 1, 2) = alpha_2(2, 1, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(2, dp)
   b = b*const_rp
   alpha_2(1, 1, 2) = alpha_2(1, 1, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(2, dp)
   b = b*const_rp
   alpha_2(0, 1, 2) = alpha_2(0, 1, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   const_rp = rp(3)-(ra(3)+rab(3))
   const_ra = (-ra(3)+rp(3))
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(0, 0, 0) = alpha_3(0, 0, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(1, 0, 1) = alpha_3(1, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_3(0, 0, 1) = alpha_3(0, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(2, 0, 2) = alpha_3(2, 0, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(2, dp)
   b = b*const_rp
   alpha_3(1, 0, 2) = alpha_3(1, 0, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(2, dp)
   b = b*const_rp
   alpha_3(0, 0, 2) = alpha_3(0, 0, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(1, 1, 0) = alpha_3(1, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(0, 1, 0) = alpha_3(0, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(2, 1, 1) = alpha_3(2, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_3(1, 1, 1) = alpha_3(1, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(1, 1, 1) = alpha_3(1, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_3(0, 1, 1) = alpha_3(0, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(3, 1, 2) = alpha_3(3, 1, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(2, dp)
   b = b*const_rp
   alpha_3(2, 1, 2) = alpha_3(2, 1, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(2, dp)
   b = b*const_rp
   alpha_3(1, 1, 2) = alpha_3(1, 1, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(2, 1, 2) = alpha_3(2, 1, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(2, dp)
   b = b*const_rp
   alpha_3(1, 1, 2) = alpha_3(1, 1, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(2, dp)
   b = b*const_rp
   alpha_3(0, 1, 2) = alpha_3(0, 1, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0

   lxyz = 0
   DO lzp = 0, 1+2
      coef_tyz = 0.0_dp
      DO lyp = 0, 1+2-lzp
         coef_ttz = 0.0_dp
         DO lxp = 0, 1+2-lzp-lyp
            lxyz = lxyz+1
            coef_ttz(0, 0) = coef_ttz(0, 0)+coef_xyz(lxyz)*alpha_1(lxp, 0, 0)
            coef_ttz(1, 0) = coef_ttz(1, 0)+coef_xyz(lxyz)*alpha_1(lxp, 1, 0)
            coef_ttz(0, 1) = coef_ttz(0, 1)+coef_xyz(lxyz)*alpha_1(lxp, 0, 1)
            coef_ttz(1, 1) = coef_ttz(1, 1)+coef_xyz(lxyz)*alpha_1(lxp, 1, 1)
            coef_ttz(0, 2) = coef_ttz(0, 2)+coef_xyz(lxyz)*alpha_1(lxp, 0, 2)
            coef_ttz(1, 2) = coef_ttz(1, 2)+coef_xyz(lxyz)*alpha_1(lxp, 1, 2)
         ENDDO

         coef_tyz(0, 0, 0, 0) = coef_tyz(0, 0, 0, 0)+coef_ttz(0, 0)*alpha_2(lyp, 0, 0)
         coef_tyz(1, 0, 0, 0) = coef_tyz(1, 0, 0, 0)+coef_ttz(1, 0)*alpha_2(lyp, 0, 0)
         coef_tyz(0, 1, 0, 0) = coef_tyz(0, 1, 0, 0)+coef_ttz(0, 1)*alpha_2(lyp, 0, 0)
         coef_tyz(1, 1, 0, 0) = coef_tyz(1, 1, 0, 0)+coef_ttz(1, 1)*alpha_2(lyp, 0, 0)
         coef_tyz(0, 2, 0, 0) = coef_tyz(0, 2, 0, 0)+coef_ttz(0, 2)*alpha_2(lyp, 0, 0)
         coef_tyz(1, 2, 0, 0) = coef_tyz(1, 2, 0, 0)+coef_ttz(1, 2)*alpha_2(lyp, 0, 0)
         coef_tyz(0, 0, 1, 0) = coef_tyz(0, 0, 1, 0)+coef_ttz(0, 0)*alpha_2(lyp, 1, 0)
         coef_tyz(0, 1, 1, 0) = coef_tyz(0, 1, 1, 0)+coef_ttz(0, 1)*alpha_2(lyp, 1, 0)
         coef_tyz(0, 2, 1, 0) = coef_tyz(0, 2, 1, 0)+coef_ttz(0, 2)*alpha_2(lyp, 1, 0)
         coef_tyz(0, 0, 0, 1) = coef_tyz(0, 0, 0, 1)+coef_ttz(0, 0)*alpha_2(lyp, 0, 1)
         coef_tyz(1, 0, 0, 1) = coef_tyz(1, 0, 0, 1)+coef_ttz(1, 0)*alpha_2(lyp, 0, 1)
         coef_tyz(0, 1, 0, 1) = coef_tyz(0, 1, 0, 1)+coef_ttz(0, 1)*alpha_2(lyp, 0, 1)
         coef_tyz(1, 1, 0, 1) = coef_tyz(1, 1, 0, 1)+coef_ttz(1, 1)*alpha_2(lyp, 0, 1)
         coef_tyz(0, 0, 1, 1) = coef_tyz(0, 0, 1, 1)+coef_ttz(0, 0)*alpha_2(lyp, 1, 1)
         coef_tyz(0, 1, 1, 1) = coef_tyz(0, 1, 1, 1)+coef_ttz(0, 1)*alpha_2(lyp, 1, 1)
         coef_tyz(0, 0, 0, 2) = coef_tyz(0, 0, 0, 2)+coef_ttz(0, 0)*alpha_2(lyp, 0, 2)
         coef_tyz(1, 0, 0, 2) = coef_tyz(1, 0, 0, 2)+coef_ttz(1, 0)*alpha_2(lyp, 0, 2)
         coef_tyz(0, 0, 1, 2) = coef_tyz(0, 0, 1, 2)+coef_ttz(0, 0)*alpha_2(lyp, 1, 2)
      ENDDO

      DO lxb = lb_min_local, 2
         jco = coset(lxb, 0, 0)

         DO lxa = la_min_local, 1

            ico = coset(lxa, 0, 0)

            vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 0, 0)

         ENDDO

      ENDDO
      DO lxb = lb_min_local, 2
         jco = coset(lxb, 0, 0)

         lxa = 0
         !ico=coset(0,1,0)
         ico = 3

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 1, 0)*alpha_3(lzp, 0, 0)

      ENDDO
      DO lxb = MAX(lb_min_local-0-1, 0), 1
         jco = coset(lxb, 1, 0)

         DO lxa = la_min_local, 1

            ico = coset(lxa, 0, 0)

            vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 1)*alpha_3(lzp, 0, 0)

         ENDDO

      ENDDO
      DO lxb = MAX(lb_min_local-0-1, 0), 1
         jco = coset(lxb, 1, 0)

         lxa = 0
         !ico=coset(0,1,0)
         ico = 3

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 1, 1)*alpha_3(lzp, 0, 0)

      ENDDO
      lxb = 0
      !jco=coset(0,2,0)
      jco = 8

      DO lxa = la_min_local, 1

         ico = coset(lxa, 0, 0)

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 2)*alpha_3(lzp, 0, 0)

      ENDDO

      lxb = 0
      !jco=coset(0,2,0)
      jco = 8

      lxa = 0
      !ico=coset(0,1,0)
      ico = 3

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 1, 2)*alpha_3(lzp, 0, 0)

      DO lxb = lb_min_local, 2
         jco = coset(lxb, 0, 0)

         lxa = 0
         !ico=coset(0,0,1)
         ico = 4

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 1, 0)

      ENDDO
      DO lxb = MAX(lb_min_local-0-1, 0), 1
         jco = coset(lxb, 1, 0)

         lxa = 0
         !ico=coset(0,0,1)
         ico = 4

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 1)*alpha_3(lzp, 1, 0)

      ENDDO
      lxb = 0
      !jco=coset(0,2,0)
      jco = 8

      lxa = 0
      !ico=coset(0,0,1)
      ico = 4

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 2)*alpha_3(lzp, 1, 0)

      DO lxb = MAX(lb_min_local-1-0, 0), 1
         jco = coset(lxb, 0, 1)

         DO lxa = la_min_local, 1

            ico = coset(lxa, 0, 0)

            vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 0, 1)

         ENDDO

      ENDDO
      DO lxb = MAX(lb_min_local-1-0, 0), 1
         jco = coset(lxb, 0, 1)

         lxa = 0
         !ico=coset(0,1,0)
         ico = 3

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 1, 0)*alpha_3(lzp, 0, 1)

      ENDDO
      lxb = 0
      !jco=coset(0,1,1)
      jco = 9

      DO lxa = la_min_local, 1

         ico = coset(lxa, 0, 0)

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 1)*alpha_3(lzp, 0, 1)

      ENDDO

      lxb = 0
      !jco=coset(0,1,1)
      jco = 9

      lxa = 0
      !ico=coset(0,1,0)
      ico = 3

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 1, 1)*alpha_3(lzp, 0, 1)

      DO lxb = MAX(lb_min_local-1-0, 0), 1
         jco = coset(lxb, 0, 1)

         lxa = 0
         !ico=coset(0,0,1)
         ico = 4

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 1, 1)

      ENDDO
      lxb = 0
      !jco=coset(0,1,1)
      jco = 9

      lxa = 0
      !ico=coset(0,0,1)
      ico = 4

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 1)*alpha_3(lzp, 1, 1)

      lxb = 0
      !jco=coset(0,0,2)
      jco = 10

      DO lxa = la_min_local, 1

         ico = coset(lxa, 0, 0)

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 0, 2)

      ENDDO

      lxb = 0
      !jco=coset(0,0,2)
      jco = 10

      lxa = 0
      !ico=coset(0,1,0)
      ico = 3

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 1, 0)*alpha_3(lzp, 0, 2)

      lxb = 0
      !jco=coset(0,0,2)
      jco = 10

      lxa = 0
      !ico=coset(0,0,1)
      ico = 4

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 1, 2)

   ENDDO

END SUBROUTINE xyz_to_vab_1_2

! Fully unrolled version
! **************************************************************************************************
!> \brief ...
!> \param prefactor ...
!> \param coef_xyz ...
!> \param lp ...
!> \param rp ...
!> \param ra ...
!> \param rab ...
!> \param vab ...
!> \param coset ...
!> \param la_min_local ...
!> \param lb_min_local ...
!> \param maxl ...
!> \param lvab ...
!> \param hvab ...
! **************************************************************************************************
SUBROUTINE xyz_to_vab_1_3(prefactor, coef_xyz, &
                          lp, rp, ra, rab, vab, coset, &
                          la_min_local, lb_min_local, maxl, lvab, hvab)

       USE kinds,                        ONLY: dp

   ! PARAMETERS
   REAL(KIND=dp), INTENT(IN) :: prefactor
   INTEGER, INTENT(IN) :: lp, &
                          lb_min_local, la_min_local, maxl, &
                          lvab, hvab

   INTEGER, DIMENSION(-1:maxl, -1:maxl, -1:maxl), &
      INTENT(IN)     :: coset

   REAL(kind=dp), DIMENSION(((lp+1)*(lp+2)*(lp+3))/6), &
      INTENT(INOUT) :: coef_xyz
   REAL(kind=dp), DIMENSION(lvab, hvab), INTENT(INOUT) :: vab
   REAL(kind=dp), DIMENSION(3), INTENT(IN) :: rp, ra, rab

   ! LOCAL
   INTEGER :: ico, jco, lxa, lxb, lxp, lxyz, lyp, lzp
   REAL(KIND=dp) :: a, b, binomial_k_lxa, binomial_l_lxb, const_rp, const_ra

   REAL(kind=dp) :: alpha_1(0:(1+3), 0:1, 0:3)
   REAL(kind=dp) :: alpha_2(0:(1+3), 0:1, 0:3)
   REAL(kind=dp) :: alpha_3(0:(1+3), 0:1, 0:3)
   REAL(kind=dp) :: coef_ttz(0:1, 0:3)
   REAL(kind=dp) :: coef_tyz(0:1, 0:3, 0:1, 0:3)
   coef_xyz = coef_xyz*prefactor

   alpha_1(:, :, :) = 0.0_dp
   alpha_2(:, :, :) = 0.0_dp
   alpha_3(:, :, :) = 0.0_dp

   const_rp = rp(1)-(ra(1)+rab(1))
   const_ra = (-ra(1)+rp(1))
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(0, 0, 0) = alpha_1(0, 0, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(1, 0, 1) = alpha_1(1, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_1(0, 0, 1) = alpha_1(0, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(2, 0, 2) = alpha_1(2, 0, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(2, dp)
   b = b*const_rp
   alpha_1(1, 0, 2) = alpha_1(1, 0, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(2, dp)
   b = b*const_rp
   alpha_1(0, 0, 2) = alpha_1(0, 0, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(3, 0, 3) = alpha_1(3, 0, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(3, dp)
   b = b*const_rp
   alpha_1(2, 0, 3) = alpha_1(2, 0, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_1(1, 0, 3) = alpha_1(1, 0, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(3, dp)
   b = b*const_rp
   alpha_1(0, 0, 3) = alpha_1(0, 0, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(1, 1, 0) = alpha_1(1, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(0, 1, 0) = alpha_1(0, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(2, 1, 1) = alpha_1(2, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_1(1, 1, 1) = alpha_1(1, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(1, 1, 1) = alpha_1(1, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_1(0, 1, 1) = alpha_1(0, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(3, 1, 2) = alpha_1(3, 1, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(2, dp)
   b = b*const_rp
   alpha_1(2, 1, 2) = alpha_1(2, 1, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(2, dp)
   b = b*const_rp
   alpha_1(1, 1, 2) = alpha_1(1, 1, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(2, 1, 2) = alpha_1(2, 1, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(2, dp)
   b = b*const_rp
   alpha_1(1, 1, 2) = alpha_1(1, 1, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(2, dp)
   b = b*const_rp
   alpha_1(0, 1, 2) = alpha_1(0, 1, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(4, 1, 3) = alpha_1(4, 1, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(3, dp)
   b = b*const_rp
   alpha_1(3, 1, 3) = alpha_1(3, 1, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_1(2, 1, 3) = alpha_1(2, 1, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(3, dp)
   b = b*const_rp
   alpha_1(1, 1, 3) = alpha_1(1, 1, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(3, 1, 3) = alpha_1(3, 1, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(3, dp)
   b = b*const_rp
   alpha_1(2, 1, 3) = alpha_1(2, 1, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_1(1, 1, 3) = alpha_1(1, 1, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(3, dp)
   b = b*const_rp
   alpha_1(0, 1, 3) = alpha_1(0, 1, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   const_rp = rp(2)-(ra(2)+rab(2))
   const_ra = (-ra(2)+rp(2))
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(0, 0, 0) = alpha_2(0, 0, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(1, 0, 1) = alpha_2(1, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_2(0, 0, 1) = alpha_2(0, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(2, 0, 2) = alpha_2(2, 0, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(2, dp)
   b = b*const_rp
   alpha_2(1, 0, 2) = alpha_2(1, 0, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(2, dp)
   b = b*const_rp
   alpha_2(0, 0, 2) = alpha_2(0, 0, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(3, 0, 3) = alpha_2(3, 0, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(3, dp)
   b = b*const_rp
   alpha_2(2, 0, 3) = alpha_2(2, 0, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_2(1, 0, 3) = alpha_2(1, 0, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(3, dp)
   b = b*const_rp
   alpha_2(0, 0, 3) = alpha_2(0, 0, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(1, 1, 0) = alpha_2(1, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(0, 1, 0) = alpha_2(0, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(2, 1, 1) = alpha_2(2, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_2(1, 1, 1) = alpha_2(1, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(1, 1, 1) = alpha_2(1, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_2(0, 1, 1) = alpha_2(0, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(3, 1, 2) = alpha_2(3, 1, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(2, dp)
   b = b*const_rp
   alpha_2(2, 1, 2) = alpha_2(2, 1, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(2, dp)
   b = b*const_rp
   alpha_2(1, 1, 2) = alpha_2(1, 1, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(2, 1, 2) = alpha_2(2, 1, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(2, dp)
   b = b*const_rp
   alpha_2(1, 1, 2) = alpha_2(1, 1, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(2, dp)
   b = b*const_rp
   alpha_2(0, 1, 2) = alpha_2(0, 1, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(4, 1, 3) = alpha_2(4, 1, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(3, dp)
   b = b*const_rp
   alpha_2(3, 1, 3) = alpha_2(3, 1, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_2(2, 1, 3) = alpha_2(2, 1, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(3, dp)
   b = b*const_rp
   alpha_2(1, 1, 3) = alpha_2(1, 1, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(3, 1, 3) = alpha_2(3, 1, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(3, dp)
   b = b*const_rp
   alpha_2(2, 1, 3) = alpha_2(2, 1, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_2(1, 1, 3) = alpha_2(1, 1, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(3, dp)
   b = b*const_rp
   alpha_2(0, 1, 3) = alpha_2(0, 1, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   const_rp = rp(3)-(ra(3)+rab(3))
   const_ra = (-ra(3)+rp(3))
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(0, 0, 0) = alpha_3(0, 0, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(1, 0, 1) = alpha_3(1, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_3(0, 0, 1) = alpha_3(0, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(2, 0, 2) = alpha_3(2, 0, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(2, dp)
   b = b*const_rp
   alpha_3(1, 0, 2) = alpha_3(1, 0, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(2, dp)
   b = b*const_rp
   alpha_3(0, 0, 2) = alpha_3(0, 0, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(3, 0, 3) = alpha_3(3, 0, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(3, dp)
   b = b*const_rp
   alpha_3(2, 0, 3) = alpha_3(2, 0, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_3(1, 0, 3) = alpha_3(1, 0, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(3, dp)
   b = b*const_rp
   alpha_3(0, 0, 3) = alpha_3(0, 0, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(1, 1, 0) = alpha_3(1, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(0, 1, 0) = alpha_3(0, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(2, 1, 1) = alpha_3(2, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_3(1, 1, 1) = alpha_3(1, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(1, 1, 1) = alpha_3(1, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_3(0, 1, 1) = alpha_3(0, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(3, 1, 2) = alpha_3(3, 1, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(2, dp)
   b = b*const_rp
   alpha_3(2, 1, 2) = alpha_3(2, 1, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(2, dp)
   b = b*const_rp
   alpha_3(1, 1, 2) = alpha_3(1, 1, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(2, 1, 2) = alpha_3(2, 1, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(2, dp)
   b = b*const_rp
   alpha_3(1, 1, 2) = alpha_3(1, 1, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(2, dp)
   b = b*const_rp
   alpha_3(0, 1, 2) = alpha_3(0, 1, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(4, 1, 3) = alpha_3(4, 1, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(3, dp)
   b = b*const_rp
   alpha_3(3, 1, 3) = alpha_3(3, 1, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_3(2, 1, 3) = alpha_3(2, 1, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(3, dp)
   b = b*const_rp
   alpha_3(1, 1, 3) = alpha_3(1, 1, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(3, 1, 3) = alpha_3(3, 1, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(3, dp)
   b = b*const_rp
   alpha_3(2, 1, 3) = alpha_3(2, 1, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_3(1, 1, 3) = alpha_3(1, 1, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(3, dp)
   b = b*const_rp
   alpha_3(0, 1, 3) = alpha_3(0, 1, 3)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0

   lxyz = 0
   DO lzp = 0, 1+3
      coef_tyz = 0.0_dp
      DO lyp = 0, 1+3-lzp
         coef_ttz = 0.0_dp
         DO lxp = 0, 1+3-lzp-lyp
            lxyz = lxyz+1
            coef_ttz(0, 0) = coef_ttz(0, 0)+coef_xyz(lxyz)*alpha_1(lxp, 0, 0)
            coef_ttz(1, 0) = coef_ttz(1, 0)+coef_xyz(lxyz)*alpha_1(lxp, 1, 0)
            coef_ttz(0, 1) = coef_ttz(0, 1)+coef_xyz(lxyz)*alpha_1(lxp, 0, 1)
            coef_ttz(1, 1) = coef_ttz(1, 1)+coef_xyz(lxyz)*alpha_1(lxp, 1, 1)
            coef_ttz(0, 2) = coef_ttz(0, 2)+coef_xyz(lxyz)*alpha_1(lxp, 0, 2)
            coef_ttz(1, 2) = coef_ttz(1, 2)+coef_xyz(lxyz)*alpha_1(lxp, 1, 2)
            coef_ttz(0, 3) = coef_ttz(0, 3)+coef_xyz(lxyz)*alpha_1(lxp, 0, 3)
            coef_ttz(1, 3) = coef_ttz(1, 3)+coef_xyz(lxyz)*alpha_1(lxp, 1, 3)
         ENDDO

         coef_tyz(0, 0, 0, 0) = coef_tyz(0, 0, 0, 0)+coef_ttz(0, 0)*alpha_2(lyp, 0, 0)
         coef_tyz(1, 0, 0, 0) = coef_tyz(1, 0, 0, 0)+coef_ttz(1, 0)*alpha_2(lyp, 0, 0)
         coef_tyz(0, 1, 0, 0) = coef_tyz(0, 1, 0, 0)+coef_ttz(0, 1)*alpha_2(lyp, 0, 0)
         coef_tyz(1, 1, 0, 0) = coef_tyz(1, 1, 0, 0)+coef_ttz(1, 1)*alpha_2(lyp, 0, 0)
         coef_tyz(0, 2, 0, 0) = coef_tyz(0, 2, 0, 0)+coef_ttz(0, 2)*alpha_2(lyp, 0, 0)
         coef_tyz(1, 2, 0, 0) = coef_tyz(1, 2, 0, 0)+coef_ttz(1, 2)*alpha_2(lyp, 0, 0)
         coef_tyz(0, 3, 0, 0) = coef_tyz(0, 3, 0, 0)+coef_ttz(0, 3)*alpha_2(lyp, 0, 0)
         coef_tyz(1, 3, 0, 0) = coef_tyz(1, 3, 0, 0)+coef_ttz(1, 3)*alpha_2(lyp, 0, 0)
         coef_tyz(0, 0, 1, 0) = coef_tyz(0, 0, 1, 0)+coef_ttz(0, 0)*alpha_2(lyp, 1, 0)
         coef_tyz(0, 1, 1, 0) = coef_tyz(0, 1, 1, 0)+coef_ttz(0, 1)*alpha_2(lyp, 1, 0)
         coef_tyz(0, 2, 1, 0) = coef_tyz(0, 2, 1, 0)+coef_ttz(0, 2)*alpha_2(lyp, 1, 0)
         coef_tyz(0, 3, 1, 0) = coef_tyz(0, 3, 1, 0)+coef_ttz(0, 3)*alpha_2(lyp, 1, 0)
         coef_tyz(0, 0, 0, 1) = coef_tyz(0, 0, 0, 1)+coef_ttz(0, 0)*alpha_2(lyp, 0, 1)
         coef_tyz(1, 0, 0, 1) = coef_tyz(1, 0, 0, 1)+coef_ttz(1, 0)*alpha_2(lyp, 0, 1)
         coef_tyz(0, 1, 0, 1) = coef_tyz(0, 1, 0, 1)+coef_ttz(0, 1)*alpha_2(lyp, 0, 1)
         coef_tyz(1, 1, 0, 1) = coef_tyz(1, 1, 0, 1)+coef_ttz(1, 1)*alpha_2(lyp, 0, 1)
         coef_tyz(0, 2, 0, 1) = coef_tyz(0, 2, 0, 1)+coef_ttz(0, 2)*alpha_2(lyp, 0, 1)
         coef_tyz(1, 2, 0, 1) = coef_tyz(1, 2, 0, 1)+coef_ttz(1, 2)*alpha_2(lyp, 0, 1)
         coef_tyz(0, 0, 1, 1) = coef_tyz(0, 0, 1, 1)+coef_ttz(0, 0)*alpha_2(lyp, 1, 1)
         coef_tyz(0, 1, 1, 1) = coef_tyz(0, 1, 1, 1)+coef_ttz(0, 1)*alpha_2(lyp, 1, 1)
         coef_tyz(0, 2, 1, 1) = coef_tyz(0, 2, 1, 1)+coef_ttz(0, 2)*alpha_2(lyp, 1, 1)
         coef_tyz(0, 0, 0, 2) = coef_tyz(0, 0, 0, 2)+coef_ttz(0, 0)*alpha_2(lyp, 0, 2)
         coef_tyz(1, 0, 0, 2) = coef_tyz(1, 0, 0, 2)+coef_ttz(1, 0)*alpha_2(lyp, 0, 2)
         coef_tyz(0, 1, 0, 2) = coef_tyz(0, 1, 0, 2)+coef_ttz(0, 1)*alpha_2(lyp, 0, 2)
         coef_tyz(1, 1, 0, 2) = coef_tyz(1, 1, 0, 2)+coef_ttz(1, 1)*alpha_2(lyp, 0, 2)
         coef_tyz(0, 0, 1, 2) = coef_tyz(0, 0, 1, 2)+coef_ttz(0, 0)*alpha_2(lyp, 1, 2)
         coef_tyz(0, 1, 1, 2) = coef_tyz(0, 1, 1, 2)+coef_ttz(0, 1)*alpha_2(lyp, 1, 2)
         coef_tyz(0, 0, 0, 3) = coef_tyz(0, 0, 0, 3)+coef_ttz(0, 0)*alpha_2(lyp, 0, 3)
         coef_tyz(1, 0, 0, 3) = coef_tyz(1, 0, 0, 3)+coef_ttz(1, 0)*alpha_2(lyp, 0, 3)
         coef_tyz(0, 0, 1, 3) = coef_tyz(0, 0, 1, 3)+coef_ttz(0, 0)*alpha_2(lyp, 1, 3)
      ENDDO

      DO lxb = lb_min_local, 3
         jco = coset(lxb, 0, 0)

         DO lxa = la_min_local, 1

            ico = coset(lxa, 0, 0)

            vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 0, 0)

         ENDDO

      ENDDO
      DO lxb = lb_min_local, 3
         jco = coset(lxb, 0, 0)

         lxa = 0
         !ico=coset(0,1,0)
         ico = 3

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 1, 0)*alpha_3(lzp, 0, 0)

      ENDDO
      DO lxb = MAX(lb_min_local-0-1, 0), 2
         jco = coset(lxb, 1, 0)

         DO lxa = la_min_local, 1

            ico = coset(lxa, 0, 0)

            vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 1)*alpha_3(lzp, 0, 0)

         ENDDO

      ENDDO
      DO lxb = MAX(lb_min_local-0-1, 0), 2
         jco = coset(lxb, 1, 0)

         lxa = 0
         !ico=coset(0,1,0)
         ico = 3

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 1, 1)*alpha_3(lzp, 0, 0)

      ENDDO
      DO lxb = MAX(lb_min_local-0-2, 0), 1
         jco = coset(lxb, 2, 0)

         DO lxa = la_min_local, 1

            ico = coset(lxa, 0, 0)

            vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 2)*alpha_3(lzp, 0, 0)

         ENDDO

      ENDDO
      DO lxb = MAX(lb_min_local-0-2, 0), 1
         jco = coset(lxb, 2, 0)

         lxa = 0
         !ico=coset(0,1,0)
         ico = 3

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 1, 2)*alpha_3(lzp, 0, 0)

      ENDDO
      lxb = 0
      !jco=coset(0,3,0)
      jco = 17

      DO lxa = la_min_local, 1

         ico = coset(lxa, 0, 0)

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 3)*alpha_3(lzp, 0, 0)

      ENDDO

      lxb = 0
      !jco=coset(0,3,0)
      jco = 17

      lxa = 0
      !ico=coset(0,1,0)
      ico = 3

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 1, 3)*alpha_3(lzp, 0, 0)

      DO lxb = lb_min_local, 3
         jco = coset(lxb, 0, 0)

         lxa = 0
         !ico=coset(0,0,1)
         ico = 4

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 1, 0)

      ENDDO
      DO lxb = MAX(lb_min_local-0-1, 0), 2
         jco = coset(lxb, 1, 0)

         lxa = 0
         !ico=coset(0,0,1)
         ico = 4

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 1)*alpha_3(lzp, 1, 0)

      ENDDO
      DO lxb = MAX(lb_min_local-0-2, 0), 1
         jco = coset(lxb, 2, 0)

         lxa = 0
         !ico=coset(0,0,1)
         ico = 4

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 2)*alpha_3(lzp, 1, 0)

      ENDDO
      lxb = 0
      !jco=coset(0,3,0)
      jco = 17

      lxa = 0
      !ico=coset(0,0,1)
      ico = 4

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 3)*alpha_3(lzp, 1, 0)

      DO lxb = MAX(lb_min_local-1-0, 0), 2
         jco = coset(lxb, 0, 1)

         DO lxa = la_min_local, 1

            ico = coset(lxa, 0, 0)

            vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 0, 1)

         ENDDO

      ENDDO
      DO lxb = MAX(lb_min_local-1-0, 0), 2
         jco = coset(lxb, 0, 1)

         lxa = 0
         !ico=coset(0,1,0)
         ico = 3

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 1, 0)*alpha_3(lzp, 0, 1)

      ENDDO
      DO lxb = MAX(lb_min_local-1-1, 0), 1
         jco = coset(lxb, 1, 1)

         DO lxa = la_min_local, 1

            ico = coset(lxa, 0, 0)

            vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 1)*alpha_3(lzp, 0, 1)

         ENDDO

      ENDDO
      DO lxb = MAX(lb_min_local-1-1, 0), 1
         jco = coset(lxb, 1, 1)

         lxa = 0
         !ico=coset(0,1,0)
         ico = 3

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 1, 1)*alpha_3(lzp, 0, 1)

      ENDDO
      lxb = 0
      !jco=coset(0,2,1)
      jco = 18

      DO lxa = la_min_local, 1

         ico = coset(lxa, 0, 0)

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 2)*alpha_3(lzp, 0, 1)

      ENDDO

      lxb = 0
      !jco=coset(0,2,1)
      jco = 18

      lxa = 0
      !ico=coset(0,1,0)
      ico = 3

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 1, 2)*alpha_3(lzp, 0, 1)

      DO lxb = MAX(lb_min_local-1-0, 0), 2
         jco = coset(lxb, 0, 1)

         lxa = 0
         !ico=coset(0,0,1)
         ico = 4

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 1, 1)

      ENDDO
      DO lxb = MAX(lb_min_local-1-1, 0), 1
         jco = coset(lxb, 1, 1)

         lxa = 0
         !ico=coset(0,0,1)
         ico = 4

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 1)*alpha_3(lzp, 1, 1)

      ENDDO
      lxb = 0
      !jco=coset(0,2,1)
      jco = 18

      lxa = 0
      !ico=coset(0,0,1)
      ico = 4

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 2)*alpha_3(lzp, 1, 1)

      DO lxb = MAX(lb_min_local-2-0, 0), 1
         jco = coset(lxb, 0, 2)

         DO lxa = la_min_local, 1

            ico = coset(lxa, 0, 0)

            vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 0, 2)

         ENDDO

      ENDDO
      DO lxb = MAX(lb_min_local-2-0, 0), 1
         jco = coset(lxb, 0, 2)

         lxa = 0
         !ico=coset(0,1,0)
         ico = 3

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 1, 0)*alpha_3(lzp, 0, 2)

      ENDDO
      lxb = 0
      !jco=coset(0,1,2)
      jco = 19

      DO lxa = la_min_local, 1

         ico = coset(lxa, 0, 0)

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 1)*alpha_3(lzp, 0, 2)

      ENDDO

      lxb = 0
      !jco=coset(0,1,2)
      jco = 19

      lxa = 0
      !ico=coset(0,1,0)
      ico = 3

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 1, 1)*alpha_3(lzp, 0, 2)

      DO lxb = MAX(lb_min_local-2-0, 0), 1
         jco = coset(lxb, 0, 2)

         lxa = 0
         !ico=coset(0,0,1)
         ico = 4

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 1, 2)

      ENDDO
      lxb = 0
      !jco=coset(0,1,2)
      jco = 19

      lxa = 0
      !ico=coset(0,0,1)
      ico = 4

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 1)*alpha_3(lzp, 1, 2)

      lxb = 0
      !jco=coset(0,0,3)
      jco = 20

      DO lxa = la_min_local, 1

         ico = coset(lxa, 0, 0)

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 0, 3)

      ENDDO

      lxb = 0
      !jco=coset(0,0,3)
      jco = 20

      lxa = 0
      !ico=coset(0,1,0)
      ico = 3

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 1, 0)*alpha_3(lzp, 0, 3)

      lxb = 0
      !jco=coset(0,0,3)
      jco = 20

      lxa = 0
      !ico=coset(0,0,1)
      ico = 4

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 1, 3)

   ENDDO

END SUBROUTINE xyz_to_vab_1_3

! Partial unrolled version
! **************************************************************************************************
!> \brief ...
!> \param prefactor ...
!> \param coef_xyz ...
!> \param lp ...
!> \param rp ...
!> \param ra ...
!> \param rab ...
!> \param vab ...
!> \param coset ...
!> \param la_min_local ...
!> \param lb_min_local ...
!> \param maxl ...
!> \param lvab ...
!> \param hvab ...
! **************************************************************************************************
SUBROUTINE xyz_to_vab_1_4(prefactor, coef_xyz, &
                          lp, rp, ra, rab, vab, coset, &
                          la_min_local, lb_min_local, maxl, lvab, hvab)

      USE kinds,                        ONLY: dp
   ! PARAMETERS
   INTEGER, INTENT(IN) :: lp, &
                          maxl, lb_min_local, la_min_local, &
                          lvab, hvab

   INTEGER, DIMENSION(-1:maxl, -1:maxl, -1:maxl), &
      INTENT(IN)     :: coset

   REAL(KIND=dp), INTENT(IN) :: prefactor

   REAL(kind=dp), DIMENSION(((lp+1)*(lp+2)*(lp+3))/6), &
      INTENT(INOUT) :: coef_xyz
   REAL(kind=dp), DIMENSION(lvab, hvab), INTENT(INOUT) :: vab
   REAL(kind=dp), DIMENSION(3), INTENT(IN) :: rp, ra, rab

   ! LOCAL
   INTEGER :: ico, jco, lxa, lxb, lxp, lxyz, lyp, lzp, l, k, lya, lyb, lza, lzb
   REAL(KIND=dp) :: a, b, binomial_k_lxa, binomial_l_lxb

   REAL(kind=dp) :: alpha(0:(1+4), 0:1, 0:4, 3)
   REAL(kind=dp) :: coef_ttz(0:1, 0:4)
   REAL(kind=dp) :: coef_tyz(0:1, 0:4, 0:1, 0:4)

   coef_xyz = coef_xyz*prefactor

   alpha(:, :, :, :) = 0.0_dp

   DO lxa = 0, 1
   DO lxb = 0, 4
      binomial_k_lxa = 1.0_dp
      a = 1.0_dp
      DO k = 0, lxa
         binomial_l_lxb = 1.0_dp
         b = 1.0_dp
         DO l = 0, lxb
            alpha(lxa-l+lxb-k, lxa, lxb, 1) = alpha(lxa-l+lxb-k, lxa, lxb, 1)+ &
                                              binomial_k_lxa*binomial_l_lxb*a*b
            binomial_l_lxb = binomial_l_lxb*REAL(lxb-l, dp)/REAL(l+1, dp)
            b = b*(rp(1)-(ra(1)+rab(1)))
         ENDDO
         binomial_k_lxa = binomial_k_lxa*REAL(lxa-k, dp)/REAL(k+1, dp)
         a = a*(-ra(1)+rp(1))
      ENDDO
   ENDDO
   ENDDO
   DO lxa = 0, 1
   DO lxb = 0, 4
      binomial_k_lxa = 1.0_dp
      a = 1.0_dp
      DO k = 0, lxa
         binomial_l_lxb = 1.0_dp
         b = 1.0_dp
         DO l = 0, lxb
            alpha(lxa-l+lxb-k, lxa, lxb, 2) = alpha(lxa-l+lxb-k, lxa, lxb, 2)+ &
                                              binomial_k_lxa*binomial_l_lxb*a*b
            binomial_l_lxb = binomial_l_lxb*REAL(lxb-l, dp)/REAL(l+1, dp)
            b = b*(rp(2)-(ra(2)+rab(2)))
         ENDDO
         binomial_k_lxa = binomial_k_lxa*REAL(lxa-k, dp)/REAL(k+1, dp)
         a = a*(-ra(2)+rp(2))
      ENDDO
   ENDDO
   ENDDO
   DO lxa = 0, 1
   DO lxb = 0, 4
      binomial_k_lxa = 1.0_dp
      a = 1.0_dp
      DO k = 0, lxa
         binomial_l_lxb = 1.0_dp
         b = 1.0_dp
         DO l = 0, lxb
            alpha(lxa-l+lxb-k, lxa, lxb, 3) = alpha(lxa-l+lxb-k, lxa, lxb, 3)+ &
                                              binomial_k_lxa*binomial_l_lxb*a*b
            binomial_l_lxb = binomial_l_lxb*REAL(lxb-l, dp)/REAL(l+1, dp)
            b = b*(rp(3)-(ra(3)+rab(3)))
         ENDDO
         binomial_k_lxa = binomial_k_lxa*REAL(lxa-k, dp)/REAL(k+1, dp)
         a = a*(-ra(3)+rp(3))
      ENDDO
   ENDDO
   ENDDO

   lxyz = 0
   DO lzp = 0, 5
      coef_tyz = 0.0_dp
      DO lyp = 0, 5-lzp
         coef_ttz = 0.0_dp
         DO lxp = 0, 5-lzp-lyp
            lxyz = lxyz+1
            DO lxb = 0, 4
            DO lxa = 0, 1
               coef_ttz(lxa, lxb) = coef_ttz(lxa, lxb)+coef_xyz(lxyz)*alpha(lxp, lxa, lxb, 1)
            ENDDO
            ENDDO

         ENDDO ! lxp

         DO lyb = 0, 4
         DO lya = 0, 1
            DO lxb = 0, 4-lyb
            DO lxa = 0, 1-lya
               coef_tyz(lxa, lxb, lya, lyb) = coef_tyz(lxa, lxb, lya, lyb)+coef_ttz(lxa, lxb)*alpha(lyp, lya, lyb, 2)
            ENDDO
            ENDDO
         ENDDO
         ENDDO

      ENDDO !lyp

      DO lzb = 0, 4
      DO lza = 0, 1
         DO lyb = 0, 4-lzb
         DO lya = 0, 1-lza
            DO lxb = MAX(lb_min_local-lzb-lyb, 0), 4-lzb-lyb
               jco = coset(lxb, lyb, lzb)
               DO lxa = MAX(la_min_local-lza-lya, 0), 1-lza-lya
                  ico = coset(lxa, lya, lza)
                  vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, lya, lyb)*alpha(lzp, lza, lzb, 3)
               ENDDO
            ENDDO
         ENDDO
         ENDDO
      ENDDO
      ENDDO

   ENDDO

END SUBROUTINE xyz_to_vab_1_4

! Fully unrolled version
! **************************************************************************************************
!> \brief ...
!> \param prefactor ...
!> \param coef_xyz ...
!> \param lp ...
!> \param rp ...
!> \param ra ...
!> \param vab ...
!> \param coset ...
!> \param la_min_local ...
!> \param maxl ...
!> \param lvab ...
!> \param hvab ...
! **************************************************************************************************
SUBROUTINE xyz_to_vab_2_0(prefactor, coef_xyz, &
                          lp, rp, ra, vab, coset, &
                          la_min_local, maxl, lvab, hvab)

       USE kinds,                        ONLY: dp

   ! PARAMETERS
   REAL(KIND=dp), INTENT(IN) :: prefactor
   INTEGER, INTENT(IN) :: lp, &
                          la_min_local, maxl, &
                          lvab, hvab

   INTEGER, DIMENSION(-1:maxl, -1:maxl, -1:maxl), &
      INTENT(IN)     :: coset

   REAL(kind=dp), DIMENSION(((lp+1)*(lp+2)*(lp+3))/6), &
      INTENT(INOUT) :: coef_xyz
   REAL(kind=dp), DIMENSION(lvab, hvab), INTENT(INOUT) :: vab
   REAL(kind=dp), DIMENSION(3), INTENT(IN) :: rp, ra

   ! LOCAL
   INTEGER :: ico, jco, lxa, lxb, lxp, lxyz, lyp, lzp
   REAL(KIND=dp) :: a, b, binomial_k_lxa, binomial_l_lxb, const_ra

   REAL(kind=dp) :: alpha_1(0:(2+0), 0:2, 0:0)
   REAL(kind=dp) :: alpha_2(0:(2+0), 0:2, 0:0)
   REAL(kind=dp) :: alpha_3(0:(2+0), 0:2, 0:0)
   REAL(kind=dp) :: coef_ttz(0:2, 0:0)
   REAL(kind=dp) :: coef_tyz(0:2, 0:0, 0:2, 0:0)
   coef_xyz = coef_xyz*prefactor

   alpha_1(:, :, :) = 0.0_dp
   alpha_2(:, :, :) = 0.0_dp
   alpha_3(:, :, :) = 0.0_dp

   const_ra = (-ra(1)+rp(1))
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(0, 0, 0) = alpha_1(0, 0, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(1, 1, 0) = alpha_1(1, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(0, 1, 0) = alpha_1(0, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(2, 2, 0) = alpha_1(2, 2, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(1, 2, 0) = alpha_1(1, 2, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(1, dp)/REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(0, 2, 0) = alpha_1(0, 2, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   const_ra = (-ra(2)+rp(2))
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(0, 0, 0) = alpha_2(0, 0, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(1, 1, 0) = alpha_2(1, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(0, 1, 0) = alpha_2(0, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(2, 2, 0) = alpha_2(2, 2, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(1, 2, 0) = alpha_2(1, 2, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(1, dp)/REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(0, 2, 0) = alpha_2(0, 2, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   const_ra = (-ra(3)+rp(3))
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(0, 0, 0) = alpha_3(0, 0, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(1, 1, 0) = alpha_3(1, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(0, 1, 0) = alpha_3(0, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(2, 2, 0) = alpha_3(2, 2, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(1, 2, 0) = alpha_3(1, 2, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(1, dp)/REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(0, 2, 0) = alpha_3(0, 2, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0

   lxyz = 0
   DO lzp = 0, 2+0
      coef_tyz = 0.0_dp
      DO lyp = 0, 2+0-lzp
         coef_ttz = 0.0_dp
         DO lxp = 0, 2+0-lzp-lyp
            lxyz = lxyz+1
            coef_ttz(0, 0) = coef_ttz(0, 0)+coef_xyz(lxyz)*alpha_1(lxp, 0, 0)
            coef_ttz(1, 0) = coef_ttz(1, 0)+coef_xyz(lxyz)*alpha_1(lxp, 1, 0)
            coef_ttz(2, 0) = coef_ttz(2, 0)+coef_xyz(lxyz)*alpha_1(lxp, 2, 0)
         ENDDO

         coef_tyz(0, 0, 0, 0) = coef_tyz(0, 0, 0, 0)+coef_ttz(0, 0)*alpha_2(lyp, 0, 0)
         coef_tyz(1, 0, 0, 0) = coef_tyz(1, 0, 0, 0)+coef_ttz(1, 0)*alpha_2(lyp, 0, 0)
         coef_tyz(2, 0, 0, 0) = coef_tyz(2, 0, 0, 0)+coef_ttz(2, 0)*alpha_2(lyp, 0, 0)
         coef_tyz(0, 0, 1, 0) = coef_tyz(0, 0, 1, 0)+coef_ttz(0, 0)*alpha_2(lyp, 1, 0)
         coef_tyz(1, 0, 1, 0) = coef_tyz(1, 0, 1, 0)+coef_ttz(1, 0)*alpha_2(lyp, 1, 0)
         coef_tyz(0, 0, 2, 0) = coef_tyz(0, 0, 2, 0)+coef_ttz(0, 0)*alpha_2(lyp, 2, 0)
      ENDDO

      lxb = 0
      !jco=coset(0,0,0)
      jco = 1

      DO lxa = la_min_local, 2

         ico = coset(lxa, 0, 0)

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 0, 0)

      ENDDO

      lxb = 0
      !jco=coset(0,0,0)
      jco = 1

      DO lxa = MAX(la_min_local-0-1, 0), 1

         ico = coset(lxa, 1, 0)

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 1, 0)*alpha_3(lzp, 0, 0)

      ENDDO

      lxb = 0
      !jco=coset(0,0,0)
      jco = 1

      lxa = 0
      !ico=coset(0,2,0)
      ico = 8

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 2, 0)*alpha_3(lzp, 0, 0)

      lxb = 0
      !jco=coset(0,0,0)
      jco = 1

      DO lxa = MAX(la_min_local-1-0, 0), 1

         ico = coset(lxa, 0, 1)

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 1, 0)

      ENDDO

      lxb = 0
      !jco=coset(0,0,0)
      jco = 1

      lxa = 0
      !ico=coset(0,1,1)
      ico = 9

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 1, 0)*alpha_3(lzp, 1, 0)

      lxb = 0
      !jco=coset(0,0,0)
      jco = 1

      lxa = 0
      !ico=coset(0,0,2)
      ico = 10

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 2, 0)

   ENDDO

END SUBROUTINE xyz_to_vab_2_0

! Fully unrolled version
! **************************************************************************************************
!> \brief ...
!> \param prefactor ...
!> \param coef_xyz ...
!> \param lp ...
!> \param rp ...
!> \param ra ...
!> \param rab ...
!> \param vab ...
!> \param coset ...
!> \param la_min_local ...
!> \param lb_min_local ...
!> \param maxl ...
!> \param lvab ...
!> \param hvab ...
! **************************************************************************************************
SUBROUTINE xyz_to_vab_2_1(prefactor, coef_xyz, &
                          lp, rp, ra, rab, vab, coset, &
                          la_min_local, lb_min_local, maxl, lvab, hvab)

       USE kinds,                        ONLY: dp

   ! PARAMETERS
   REAL(KIND=dp), INTENT(IN) :: prefactor
   INTEGER, INTENT(IN) :: lp, &
                          lb_min_local, la_min_local, maxl, &
                          lvab, hvab

   INTEGER, DIMENSION(-1:maxl, -1:maxl, -1:maxl), &
      INTENT(IN)     :: coset

   REAL(kind=dp), DIMENSION(((lp+1)*(lp+2)*(lp+3))/6), &
      INTENT(INOUT) :: coef_xyz
   REAL(kind=dp), DIMENSION(lvab, hvab), INTENT(INOUT) :: vab
   REAL(kind=dp), DIMENSION(3), INTENT(IN) :: rp, ra, rab

   ! LOCAL
   INTEGER :: ico, jco, lxa, lxb, lxp, lxyz, lyp, lzp
   REAL(KIND=dp) :: a, b, binomial_k_lxa, binomial_l_lxb, const_rp, const_ra

   REAL(kind=dp) :: alpha_1(0:(2+1), 0:2, 0:1)
   REAL(kind=dp) :: alpha_2(0:(2+1), 0:2, 0:1)
   REAL(kind=dp) :: alpha_3(0:(2+1), 0:2, 0:1)
   REAL(kind=dp) :: coef_ttz(0:2, 0:1)
   REAL(kind=dp) :: coef_tyz(0:2, 0:1, 0:2, 0:1)
   coef_xyz = coef_xyz*prefactor

   alpha_1(:, :, :) = 0.0_dp
   alpha_2(:, :, :) = 0.0_dp
   alpha_3(:, :, :) = 0.0_dp

   const_rp = rp(1)-(ra(1)+rab(1))
   const_ra = (-ra(1)+rp(1))
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(0, 0, 0) = alpha_1(0, 0, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(1, 0, 1) = alpha_1(1, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_1(0, 0, 1) = alpha_1(0, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(1, 1, 0) = alpha_1(1, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(0, 1, 0) = alpha_1(0, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(2, 1, 1) = alpha_1(2, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_1(1, 1, 1) = alpha_1(1, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(1, 1, 1) = alpha_1(1, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_1(0, 1, 1) = alpha_1(0, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(2, 2, 0) = alpha_1(2, 2, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(1, 2, 0) = alpha_1(1, 2, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(1, dp)/REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(0, 2, 0) = alpha_1(0, 2, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(3, 2, 1) = alpha_1(3, 2, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_1(2, 2, 1) = alpha_1(2, 2, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(2, 2, 1) = alpha_1(2, 2, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_1(1, 2, 1) = alpha_1(1, 2, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(1, dp)/REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(1, 2, 1) = alpha_1(1, 2, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_1(0, 2, 1) = alpha_1(0, 2, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   const_rp = rp(2)-(ra(2)+rab(2))
   const_ra = (-ra(2)+rp(2))
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(0, 0, 0) = alpha_2(0, 0, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(1, 0, 1) = alpha_2(1, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_2(0, 0, 1) = alpha_2(0, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(1, 1, 0) = alpha_2(1, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(0, 1, 0) = alpha_2(0, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(2, 1, 1) = alpha_2(2, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_2(1, 1, 1) = alpha_2(1, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(1, 1, 1) = alpha_2(1, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_2(0, 1, 1) = alpha_2(0, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(2, 2, 0) = alpha_2(2, 2, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(1, 2, 0) = alpha_2(1, 2, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(1, dp)/REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(0, 2, 0) = alpha_2(0, 2, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(3, 2, 1) = alpha_2(3, 2, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_2(2, 2, 1) = alpha_2(2, 2, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(2, 2, 1) = alpha_2(2, 2, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_2(1, 2, 1) = alpha_2(1, 2, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(1, dp)/REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(1, 2, 1) = alpha_2(1, 2, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_2(0, 2, 1) = alpha_2(0, 2, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   const_rp = rp(3)-(ra(3)+rab(3))
   const_ra = (-ra(3)+rp(3))
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(0, 0, 0) = alpha_3(0, 0, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(1, 0, 1) = alpha_3(1, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_3(0, 0, 1) = alpha_3(0, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(1, 1, 0) = alpha_3(1, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(0, 1, 0) = alpha_3(0, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(2, 1, 1) = alpha_3(2, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_3(1, 1, 1) = alpha_3(1, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(1, 1, 1) = alpha_3(1, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_3(0, 1, 1) = alpha_3(0, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(2, 2, 0) = alpha_3(2, 2, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(1, 2, 0) = alpha_3(1, 2, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(1, dp)/REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(0, 2, 0) = alpha_3(0, 2, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(3, 2, 1) = alpha_3(3, 2, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_3(2, 2, 1) = alpha_3(2, 2, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(2, 2, 1) = alpha_3(2, 2, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_3(1, 2, 1) = alpha_3(1, 2, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(1, dp)/REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(1, 2, 1) = alpha_3(1, 2, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_3(0, 2, 1) = alpha_3(0, 2, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0

   lxyz = 0
   DO lzp = 0, 2+1
      coef_tyz = 0.0_dp
      DO lyp = 0, 2+1-lzp
         coef_ttz = 0.0_dp
         DO lxp = 0, 2+1-lzp-lyp
            lxyz = lxyz+1
            coef_ttz(0, 0) = coef_ttz(0, 0)+coef_xyz(lxyz)*alpha_1(lxp, 0, 0)
            coef_ttz(1, 0) = coef_ttz(1, 0)+coef_xyz(lxyz)*alpha_1(lxp, 1, 0)
            coef_ttz(2, 0) = coef_ttz(2, 0)+coef_xyz(lxyz)*alpha_1(lxp, 2, 0)
            coef_ttz(0, 1) = coef_ttz(0, 1)+coef_xyz(lxyz)*alpha_1(lxp, 0, 1)
            coef_ttz(1, 1) = coef_ttz(1, 1)+coef_xyz(lxyz)*alpha_1(lxp, 1, 1)
            coef_ttz(2, 1) = coef_ttz(2, 1)+coef_xyz(lxyz)*alpha_1(lxp, 2, 1)
         ENDDO

         coef_tyz(0, 0, 0, 0) = coef_tyz(0, 0, 0, 0)+coef_ttz(0, 0)*alpha_2(lyp, 0, 0)
         coef_tyz(1, 0, 0, 0) = coef_tyz(1, 0, 0, 0)+coef_ttz(1, 0)*alpha_2(lyp, 0, 0)
         coef_tyz(2, 0, 0, 0) = coef_tyz(2, 0, 0, 0)+coef_ttz(2, 0)*alpha_2(lyp, 0, 0)
         coef_tyz(0, 1, 0, 0) = coef_tyz(0, 1, 0, 0)+coef_ttz(0, 1)*alpha_2(lyp, 0, 0)
         coef_tyz(1, 1, 0, 0) = coef_tyz(1, 1, 0, 0)+coef_ttz(1, 1)*alpha_2(lyp, 0, 0)
         coef_tyz(2, 1, 0, 0) = coef_tyz(2, 1, 0, 0)+coef_ttz(2, 1)*alpha_2(lyp, 0, 0)
         coef_tyz(0, 0, 1, 0) = coef_tyz(0, 0, 1, 0)+coef_ttz(0, 0)*alpha_2(lyp, 1, 0)
         coef_tyz(1, 0, 1, 0) = coef_tyz(1, 0, 1, 0)+coef_ttz(1, 0)*alpha_2(lyp, 1, 0)
         coef_tyz(0, 1, 1, 0) = coef_tyz(0, 1, 1, 0)+coef_ttz(0, 1)*alpha_2(lyp, 1, 0)
         coef_tyz(1, 1, 1, 0) = coef_tyz(1, 1, 1, 0)+coef_ttz(1, 1)*alpha_2(lyp, 1, 0)
         coef_tyz(0, 0, 2, 0) = coef_tyz(0, 0, 2, 0)+coef_ttz(0, 0)*alpha_2(lyp, 2, 0)
         coef_tyz(0, 1, 2, 0) = coef_tyz(0, 1, 2, 0)+coef_ttz(0, 1)*alpha_2(lyp, 2, 0)
         coef_tyz(0, 0, 0, 1) = coef_tyz(0, 0, 0, 1)+coef_ttz(0, 0)*alpha_2(lyp, 0, 1)
         coef_tyz(1, 0, 0, 1) = coef_tyz(1, 0, 0, 1)+coef_ttz(1, 0)*alpha_2(lyp, 0, 1)
         coef_tyz(2, 0, 0, 1) = coef_tyz(2, 0, 0, 1)+coef_ttz(2, 0)*alpha_2(lyp, 0, 1)
         coef_tyz(0, 0, 1, 1) = coef_tyz(0, 0, 1, 1)+coef_ttz(0, 0)*alpha_2(lyp, 1, 1)
         coef_tyz(1, 0, 1, 1) = coef_tyz(1, 0, 1, 1)+coef_ttz(1, 0)*alpha_2(lyp, 1, 1)
         coef_tyz(0, 0, 2, 1) = coef_tyz(0, 0, 2, 1)+coef_ttz(0, 0)*alpha_2(lyp, 2, 1)
      ENDDO

      DO lxb = lb_min_local, 1
         jco = coset(lxb, 0, 0)

         DO lxa = la_min_local, 2

            ico = coset(lxa, 0, 0)

            vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 0, 0)

         ENDDO

      ENDDO
      DO lxb = lb_min_local, 1
         jco = coset(lxb, 0, 0)

         DO lxa = MAX(la_min_local-0-1, 0), 1

            ico = coset(lxa, 1, 0)

            vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 1, 0)*alpha_3(lzp, 0, 0)

         ENDDO

      ENDDO
      DO lxb = lb_min_local, 1
         jco = coset(lxb, 0, 0)

         lxa = 0
         !ico=coset(0,2,0)
         ico = 8

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 2, 0)*alpha_3(lzp, 0, 0)

      ENDDO
      lxb = 0
      !jco=coset(0,1,0)
      jco = 3

      DO lxa = la_min_local, 2

         ico = coset(lxa, 0, 0)

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 1)*alpha_3(lzp, 0, 0)

      ENDDO

      lxb = 0
      !jco=coset(0,1,0)
      jco = 3

      DO lxa = MAX(la_min_local-0-1, 0), 1

         ico = coset(lxa, 1, 0)

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 1, 1)*alpha_3(lzp, 0, 0)

      ENDDO

      lxb = 0
      !jco=coset(0,1,0)
      jco = 3

      lxa = 0
      !ico=coset(0,2,0)
      ico = 8

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 2, 1)*alpha_3(lzp, 0, 0)

      DO lxb = lb_min_local, 1
         jco = coset(lxb, 0, 0)

         DO lxa = MAX(la_min_local-1-0, 0), 1

            ico = coset(lxa, 0, 1)

            vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 1, 0)

         ENDDO

      ENDDO
      DO lxb = lb_min_local, 1
         jco = coset(lxb, 0, 0)

         lxa = 0
         !ico=coset(0,1,1)
         ico = 9

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 1, 0)*alpha_3(lzp, 1, 0)

      ENDDO
      lxb = 0
      !jco=coset(0,1,0)
      jco = 3

      DO lxa = MAX(la_min_local-1-0, 0), 1

         ico = coset(lxa, 0, 1)

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 1)*alpha_3(lzp, 1, 0)

      ENDDO

      lxb = 0
      !jco=coset(0,1,0)
      jco = 3

      lxa = 0
      !ico=coset(0,1,1)
      ico = 9

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 1, 1)*alpha_3(lzp, 1, 0)

      DO lxb = lb_min_local, 1
         jco = coset(lxb, 0, 0)

         lxa = 0
         !ico=coset(0,0,2)
         ico = 10

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 2, 0)

      ENDDO
      lxb = 0
      !jco=coset(0,1,0)
      jco = 3

      lxa = 0
      !ico=coset(0,0,2)
      ico = 10

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 1)*alpha_3(lzp, 2, 0)

      lxb = 0
      !jco=coset(0,0,1)
      jco = 4

      DO lxa = la_min_local, 2

         ico = coset(lxa, 0, 0)

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 0, 1)

      ENDDO

      lxb = 0
      !jco=coset(0,0,1)
      jco = 4

      DO lxa = MAX(la_min_local-0-1, 0), 1

         ico = coset(lxa, 1, 0)

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 1, 0)*alpha_3(lzp, 0, 1)

      ENDDO

      lxb = 0
      !jco=coset(0,0,1)
      jco = 4

      lxa = 0
      !ico=coset(0,2,0)
      ico = 8

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 2, 0)*alpha_3(lzp, 0, 1)

      lxb = 0
      !jco=coset(0,0,1)
      jco = 4

      DO lxa = MAX(la_min_local-1-0, 0), 1

         ico = coset(lxa, 0, 1)

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 1, 1)

      ENDDO

      lxb = 0
      !jco=coset(0,0,1)
      jco = 4

      lxa = 0
      !ico=coset(0,1,1)
      ico = 9

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 1, 0)*alpha_3(lzp, 1, 1)

      lxb = 0
      !jco=coset(0,0,1)
      jco = 4

      lxa = 0
      !ico=coset(0,0,2)
      ico = 10

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 2, 1)

   ENDDO

END SUBROUTINE xyz_to_vab_2_1

! Fully unrolled version
! **************************************************************************************************
!> \brief ...
!> \param prefactor ...
!> \param coef_xyz ...
!> \param lp ...
!> \param rp ...
!> \param ra ...
!> \param rab ...
!> \param vab ...
!> \param coset ...
!> \param la_min_local ...
!> \param lb_min_local ...
!> \param maxl ...
!> \param lvab ...
!> \param hvab ...
! **************************************************************************************************
SUBROUTINE xyz_to_vab_2_2(prefactor, coef_xyz, &
                          lp, rp, ra, rab, vab, coset, &
                          la_min_local, lb_min_local, maxl, lvab, hvab)

       USE kinds,                        ONLY: dp

   ! PARAMETERS
   REAL(KIND=dp), INTENT(IN) :: prefactor
   INTEGER, INTENT(IN) :: lp, &
                          lb_min_local, la_min_local, maxl, &
                          lvab, hvab

   INTEGER, DIMENSION(-1:maxl, -1:maxl, -1:maxl), &
      INTENT(IN)     :: coset

   REAL(kind=dp), DIMENSION(((lp+1)*(lp+2)*(lp+3))/6), &
      INTENT(INOUT) :: coef_xyz
   REAL(kind=dp), DIMENSION(lvab, hvab), INTENT(INOUT) :: vab
   REAL(kind=dp), DIMENSION(3), INTENT(IN) :: rp, ra, rab

   ! LOCAL
   INTEGER :: ico, jco, lxa, lxb, lxp, lxyz, lyp, lzp
   REAL(KIND=dp) :: a, b, binomial_k_lxa, binomial_l_lxb, const_rp, const_ra

   REAL(kind=dp) :: alpha_1(0:(2+2), 0:2, 0:2)
   REAL(kind=dp) :: alpha_2(0:(2+2), 0:2, 0:2)
   REAL(kind=dp) :: alpha_3(0:(2+2), 0:2, 0:2)
   REAL(kind=dp) :: coef_ttz(0:2, 0:2)
   REAL(kind=dp) :: coef_tyz(0:2, 0:2, 0:2, 0:2)
   coef_xyz = coef_xyz*prefactor

   alpha_1(:, :, :) = 0.0_dp
   alpha_2(:, :, :) = 0.0_dp
   alpha_3(:, :, :) = 0.0_dp

   const_rp = rp(1)-(ra(1)+rab(1))
   const_ra = (-ra(1)+rp(1))
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(0, 0, 0) = alpha_1(0, 0, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(1, 0, 1) = alpha_1(1, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_1(0, 0, 1) = alpha_1(0, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(2, 0, 2) = alpha_1(2, 0, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(2, dp)
   b = b*const_rp
   alpha_1(1, 0, 2) = alpha_1(1, 0, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(2, dp)
   b = b*const_rp
   alpha_1(0, 0, 2) = alpha_1(0, 0, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(1, 1, 0) = alpha_1(1, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(0, 1, 0) = alpha_1(0, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(2, 1, 1) = alpha_1(2, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_1(1, 1, 1) = alpha_1(1, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(1, 1, 1) = alpha_1(1, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_1(0, 1, 1) = alpha_1(0, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(3, 1, 2) = alpha_1(3, 1, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(2, dp)
   b = b*const_rp
   alpha_1(2, 1, 2) = alpha_1(2, 1, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(2, dp)
   b = b*const_rp
   alpha_1(1, 1, 2) = alpha_1(1, 1, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(2, 1, 2) = alpha_1(2, 1, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(2, dp)
   b = b*const_rp
   alpha_1(1, 1, 2) = alpha_1(1, 1, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(2, dp)
   b = b*const_rp
   alpha_1(0, 1, 2) = alpha_1(0, 1, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(2, 2, 0) = alpha_1(2, 2, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(1, 2, 0) = alpha_1(1, 2, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(1, dp)/REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(0, 2, 0) = alpha_1(0, 2, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(3, 2, 1) = alpha_1(3, 2, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_1(2, 2, 1) = alpha_1(2, 2, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(2, 2, 1) = alpha_1(2, 2, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_1(1, 2, 1) = alpha_1(1, 2, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(1, dp)/REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(1, 2, 1) = alpha_1(1, 2, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_1(0, 2, 1) = alpha_1(0, 2, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(4, 2, 2) = alpha_1(4, 2, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(2, dp)
   b = b*const_rp
   alpha_1(3, 2, 2) = alpha_1(3, 2, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(2, dp)
   b = b*const_rp
   alpha_1(2, 2, 2) = alpha_1(2, 2, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(3, 2, 2) = alpha_1(3, 2, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(2, dp)
   b = b*const_rp
   alpha_1(2, 2, 2) = alpha_1(2, 2, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(2, dp)
   b = b*const_rp
   alpha_1(1, 2, 2) = alpha_1(1, 2, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(1, dp)/REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(2, 2, 2) = alpha_1(2, 2, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(2, dp)
   b = b*const_rp
   alpha_1(1, 2, 2) = alpha_1(1, 2, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(2, dp)
   b = b*const_rp
   alpha_1(0, 2, 2) = alpha_1(0, 2, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   const_rp = rp(2)-(ra(2)+rab(2))
   const_ra = (-ra(2)+rp(2))
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(0, 0, 0) = alpha_2(0, 0, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(1, 0, 1) = alpha_2(1, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_2(0, 0, 1) = alpha_2(0, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(2, 0, 2) = alpha_2(2, 0, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(2, dp)
   b = b*const_rp
   alpha_2(1, 0, 2) = alpha_2(1, 0, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(2, dp)
   b = b*const_rp
   alpha_2(0, 0, 2) = alpha_2(0, 0, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(1, 1, 0) = alpha_2(1, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(0, 1, 0) = alpha_2(0, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(2, 1, 1) = alpha_2(2, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_2(1, 1, 1) = alpha_2(1, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(1, 1, 1) = alpha_2(1, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_2(0, 1, 1) = alpha_2(0, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(3, 1, 2) = alpha_2(3, 1, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(2, dp)
   b = b*const_rp
   alpha_2(2, 1, 2) = alpha_2(2, 1, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(2, dp)
   b = b*const_rp
   alpha_2(1, 1, 2) = alpha_2(1, 1, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(2, 1, 2) = alpha_2(2, 1, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(2, dp)
   b = b*const_rp
   alpha_2(1, 1, 2) = alpha_2(1, 1, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(2, dp)
   b = b*const_rp
   alpha_2(0, 1, 2) = alpha_2(0, 1, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(2, 2, 0) = alpha_2(2, 2, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(1, 2, 0) = alpha_2(1, 2, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(1, dp)/REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(0, 2, 0) = alpha_2(0, 2, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(3, 2, 1) = alpha_2(3, 2, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_2(2, 2, 1) = alpha_2(2, 2, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(2, 2, 1) = alpha_2(2, 2, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_2(1, 2, 1) = alpha_2(1, 2, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(1, dp)/REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(1, 2, 1) = alpha_2(1, 2, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_2(0, 2, 1) = alpha_2(0, 2, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(4, 2, 2) = alpha_2(4, 2, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(2, dp)
   b = b*const_rp
   alpha_2(3, 2, 2) = alpha_2(3, 2, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(2, dp)
   b = b*const_rp
   alpha_2(2, 2, 2) = alpha_2(2, 2, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(3, 2, 2) = alpha_2(3, 2, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(2, dp)
   b = b*const_rp
   alpha_2(2, 2, 2) = alpha_2(2, 2, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(2, dp)
   b = b*const_rp
   alpha_2(1, 2, 2) = alpha_2(1, 2, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(1, dp)/REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(2, 2, 2) = alpha_2(2, 2, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(2, dp)
   b = b*const_rp
   alpha_2(1, 2, 2) = alpha_2(1, 2, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(2, dp)
   b = b*const_rp
   alpha_2(0, 2, 2) = alpha_2(0, 2, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   const_rp = rp(3)-(ra(3)+rab(3))
   const_ra = (-ra(3)+rp(3))
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(0, 0, 0) = alpha_3(0, 0, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(1, 0, 1) = alpha_3(1, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_3(0, 0, 1) = alpha_3(0, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(2, 0, 2) = alpha_3(2, 0, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(2, dp)
   b = b*const_rp
   alpha_3(1, 0, 2) = alpha_3(1, 0, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(2, dp)
   b = b*const_rp
   alpha_3(0, 0, 2) = alpha_3(0, 0, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(1, 1, 0) = alpha_3(1, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(0, 1, 0) = alpha_3(0, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(2, 1, 1) = alpha_3(2, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_3(1, 1, 1) = alpha_3(1, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(1, 1, 1) = alpha_3(1, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_3(0, 1, 1) = alpha_3(0, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(3, 1, 2) = alpha_3(3, 1, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(2, dp)
   b = b*const_rp
   alpha_3(2, 1, 2) = alpha_3(2, 1, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(2, dp)
   b = b*const_rp
   alpha_3(1, 1, 2) = alpha_3(1, 1, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(2, 1, 2) = alpha_3(2, 1, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(2, dp)
   b = b*const_rp
   alpha_3(1, 1, 2) = alpha_3(1, 1, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(2, dp)
   b = b*const_rp
   alpha_3(0, 1, 2) = alpha_3(0, 1, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(2, 2, 0) = alpha_3(2, 2, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(1, 2, 0) = alpha_3(1, 2, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(1, dp)/REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(0, 2, 0) = alpha_3(0, 2, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(3, 2, 1) = alpha_3(3, 2, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_3(2, 2, 1) = alpha_3(2, 2, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(2, 2, 1) = alpha_3(2, 2, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_3(1, 2, 1) = alpha_3(1, 2, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(1, dp)/REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(1, 2, 1) = alpha_3(1, 2, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_3(0, 2, 1) = alpha_3(0, 2, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(4, 2, 2) = alpha_3(4, 2, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(2, dp)
   b = b*const_rp
   alpha_3(3, 2, 2) = alpha_3(3, 2, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(2, dp)
   b = b*const_rp
   alpha_3(2, 2, 2) = alpha_3(2, 2, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(3, 2, 2) = alpha_3(3, 2, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(2, dp)
   b = b*const_rp
   alpha_3(2, 2, 2) = alpha_3(2, 2, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(2, dp)
   b = b*const_rp
   alpha_3(1, 2, 2) = alpha_3(1, 2, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(1, dp)/REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(2, 2, 2) = alpha_3(2, 2, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(2, dp)
   b = b*const_rp
   alpha_3(1, 2, 2) = alpha_3(1, 2, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   binomial_l_lxb = binomial_l_lxb*REAL(1, dp)/REAL(2, dp)
   b = b*const_rp
   alpha_3(0, 2, 2) = alpha_3(0, 2, 2)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0

   lxyz = 0
   DO lzp = 0, 2+2
      coef_tyz = 0.0_dp
      DO lyp = 0, 2+2-lzp
         coef_ttz = 0.0_dp
         DO lxp = 0, 2+2-lzp-lyp
            lxyz = lxyz+1
            coef_ttz(0, 0) = coef_ttz(0, 0)+coef_xyz(lxyz)*alpha_1(lxp, 0, 0)
            coef_ttz(1, 0) = coef_ttz(1, 0)+coef_xyz(lxyz)*alpha_1(lxp, 1, 0)
            coef_ttz(2, 0) = coef_ttz(2, 0)+coef_xyz(lxyz)*alpha_1(lxp, 2, 0)
            coef_ttz(0, 1) = coef_ttz(0, 1)+coef_xyz(lxyz)*alpha_1(lxp, 0, 1)
            coef_ttz(1, 1) = coef_ttz(1, 1)+coef_xyz(lxyz)*alpha_1(lxp, 1, 1)
            coef_ttz(2, 1) = coef_ttz(2, 1)+coef_xyz(lxyz)*alpha_1(lxp, 2, 1)
            coef_ttz(0, 2) = coef_ttz(0, 2)+coef_xyz(lxyz)*alpha_1(lxp, 0, 2)
            coef_ttz(1, 2) = coef_ttz(1, 2)+coef_xyz(lxyz)*alpha_1(lxp, 1, 2)
            coef_ttz(2, 2) = coef_ttz(2, 2)+coef_xyz(lxyz)*alpha_1(lxp, 2, 2)
         ENDDO

         coef_tyz(0, 0, 0, 0) = coef_tyz(0, 0, 0, 0)+coef_ttz(0, 0)*alpha_2(lyp, 0, 0)
         coef_tyz(1, 0, 0, 0) = coef_tyz(1, 0, 0, 0)+coef_ttz(1, 0)*alpha_2(lyp, 0, 0)
         coef_tyz(2, 0, 0, 0) = coef_tyz(2, 0, 0, 0)+coef_ttz(2, 0)*alpha_2(lyp, 0, 0)
         coef_tyz(0, 1, 0, 0) = coef_tyz(0, 1, 0, 0)+coef_ttz(0, 1)*alpha_2(lyp, 0, 0)
         coef_tyz(1, 1, 0, 0) = coef_tyz(1, 1, 0, 0)+coef_ttz(1, 1)*alpha_2(lyp, 0, 0)
         coef_tyz(2, 1, 0, 0) = coef_tyz(2, 1, 0, 0)+coef_ttz(2, 1)*alpha_2(lyp, 0, 0)
         coef_tyz(0, 2, 0, 0) = coef_tyz(0, 2, 0, 0)+coef_ttz(0, 2)*alpha_2(lyp, 0, 0)
         coef_tyz(1, 2, 0, 0) = coef_tyz(1, 2, 0, 0)+coef_ttz(1, 2)*alpha_2(lyp, 0, 0)
         coef_tyz(2, 2, 0, 0) = coef_tyz(2, 2, 0, 0)+coef_ttz(2, 2)*alpha_2(lyp, 0, 0)
         coef_tyz(0, 0, 1, 0) = coef_tyz(0, 0, 1, 0)+coef_ttz(0, 0)*alpha_2(lyp, 1, 0)
         coef_tyz(1, 0, 1, 0) = coef_tyz(1, 0, 1, 0)+coef_ttz(1, 0)*alpha_2(lyp, 1, 0)
         coef_tyz(0, 1, 1, 0) = coef_tyz(0, 1, 1, 0)+coef_ttz(0, 1)*alpha_2(lyp, 1, 0)
         coef_tyz(1, 1, 1, 0) = coef_tyz(1, 1, 1, 0)+coef_ttz(1, 1)*alpha_2(lyp, 1, 0)
         coef_tyz(0, 2, 1, 0) = coef_tyz(0, 2, 1, 0)+coef_ttz(0, 2)*alpha_2(lyp, 1, 0)
         coef_tyz(1, 2, 1, 0) = coef_tyz(1, 2, 1, 0)+coef_ttz(1, 2)*alpha_2(lyp, 1, 0)
         coef_tyz(0, 0, 2, 0) = coef_tyz(0, 0, 2, 0)+coef_ttz(0, 0)*alpha_2(lyp, 2, 0)
         coef_tyz(0, 1, 2, 0) = coef_tyz(0, 1, 2, 0)+coef_ttz(0, 1)*alpha_2(lyp, 2, 0)
         coef_tyz(0, 2, 2, 0) = coef_tyz(0, 2, 2, 0)+coef_ttz(0, 2)*alpha_2(lyp, 2, 0)
         coef_tyz(0, 0, 0, 1) = coef_tyz(0, 0, 0, 1)+coef_ttz(0, 0)*alpha_2(lyp, 0, 1)
         coef_tyz(1, 0, 0, 1) = coef_tyz(1, 0, 0, 1)+coef_ttz(1, 0)*alpha_2(lyp, 0, 1)
         coef_tyz(2, 0, 0, 1) = coef_tyz(2, 0, 0, 1)+coef_ttz(2, 0)*alpha_2(lyp, 0, 1)
         coef_tyz(0, 1, 0, 1) = coef_tyz(0, 1, 0, 1)+coef_ttz(0, 1)*alpha_2(lyp, 0, 1)
         coef_tyz(1, 1, 0, 1) = coef_tyz(1, 1, 0, 1)+coef_ttz(1, 1)*alpha_2(lyp, 0, 1)
         coef_tyz(2, 1, 0, 1) = coef_tyz(2, 1, 0, 1)+coef_ttz(2, 1)*alpha_2(lyp, 0, 1)
         coef_tyz(0, 0, 1, 1) = coef_tyz(0, 0, 1, 1)+coef_ttz(0, 0)*alpha_2(lyp, 1, 1)
         coef_tyz(1, 0, 1, 1) = coef_tyz(1, 0, 1, 1)+coef_ttz(1, 0)*alpha_2(lyp, 1, 1)
         coef_tyz(0, 1, 1, 1) = coef_tyz(0, 1, 1, 1)+coef_ttz(0, 1)*alpha_2(lyp, 1, 1)
         coef_tyz(1, 1, 1, 1) = coef_tyz(1, 1, 1, 1)+coef_ttz(1, 1)*alpha_2(lyp, 1, 1)
         coef_tyz(0, 0, 2, 1) = coef_tyz(0, 0, 2, 1)+coef_ttz(0, 0)*alpha_2(lyp, 2, 1)
         coef_tyz(0, 1, 2, 1) = coef_tyz(0, 1, 2, 1)+coef_ttz(0, 1)*alpha_2(lyp, 2, 1)
         coef_tyz(0, 0, 0, 2) = coef_tyz(0, 0, 0, 2)+coef_ttz(0, 0)*alpha_2(lyp, 0, 2)
         coef_tyz(1, 0, 0, 2) = coef_tyz(1, 0, 0, 2)+coef_ttz(1, 0)*alpha_2(lyp, 0, 2)
         coef_tyz(2, 0, 0, 2) = coef_tyz(2, 0, 0, 2)+coef_ttz(2, 0)*alpha_2(lyp, 0, 2)
         coef_tyz(0, 0, 1, 2) = coef_tyz(0, 0, 1, 2)+coef_ttz(0, 0)*alpha_2(lyp, 1, 2)
         coef_tyz(1, 0, 1, 2) = coef_tyz(1, 0, 1, 2)+coef_ttz(1, 0)*alpha_2(lyp, 1, 2)
         coef_tyz(0, 0, 2, 2) = coef_tyz(0, 0, 2, 2)+coef_ttz(0, 0)*alpha_2(lyp, 2, 2)
      ENDDO

      DO lxb = lb_min_local, 2
         jco = coset(lxb, 0, 0)

         DO lxa = la_min_local, 2

            ico = coset(lxa, 0, 0)

            vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 0, 0)

         ENDDO

      ENDDO
      DO lxb = lb_min_local, 2
         jco = coset(lxb, 0, 0)

         DO lxa = MAX(la_min_local-0-1, 0), 1

            ico = coset(lxa, 1, 0)

            vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 1, 0)*alpha_3(lzp, 0, 0)

         ENDDO

      ENDDO
      DO lxb = lb_min_local, 2
         jco = coset(lxb, 0, 0)

         lxa = 0
         !ico=coset(0,2,0)
         ico = 8

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 2, 0)*alpha_3(lzp, 0, 0)

      ENDDO
      DO lxb = MAX(lb_min_local-0-1, 0), 1
         jco = coset(lxb, 1, 0)

         DO lxa = la_min_local, 2

            ico = coset(lxa, 0, 0)

            vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 1)*alpha_3(lzp, 0, 0)

         ENDDO

      ENDDO
      DO lxb = MAX(lb_min_local-0-1, 0), 1
         jco = coset(lxb, 1, 0)

         DO lxa = MAX(la_min_local-0-1, 0), 1

            ico = coset(lxa, 1, 0)

            vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 1, 1)*alpha_3(lzp, 0, 0)

         ENDDO

      ENDDO
      DO lxb = MAX(lb_min_local-0-1, 0), 1
         jco = coset(lxb, 1, 0)

         lxa = 0
         !ico=coset(0,2,0)
         ico = 8

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 2, 1)*alpha_3(lzp, 0, 0)

      ENDDO
      lxb = 0
      !jco=coset(0,2,0)
      jco = 8

      DO lxa = la_min_local, 2

         ico = coset(lxa, 0, 0)

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 2)*alpha_3(lzp, 0, 0)

      ENDDO

      lxb = 0
      !jco=coset(0,2,0)
      jco = 8

      DO lxa = MAX(la_min_local-0-1, 0), 1

         ico = coset(lxa, 1, 0)

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 1, 2)*alpha_3(lzp, 0, 0)

      ENDDO

      lxb = 0
      !jco=coset(0,2,0)
      jco = 8

      lxa = 0
      !ico=coset(0,2,0)
      ico = 8

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 2, 2)*alpha_3(lzp, 0, 0)

      DO lxb = lb_min_local, 2
         jco = coset(lxb, 0, 0)

         DO lxa = MAX(la_min_local-1-0, 0), 1

            ico = coset(lxa, 0, 1)

            vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 1, 0)

         ENDDO

      ENDDO
      DO lxb = lb_min_local, 2
         jco = coset(lxb, 0, 0)

         lxa = 0
         !ico=coset(0,1,1)
         ico = 9

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 1, 0)*alpha_3(lzp, 1, 0)

      ENDDO
      DO lxb = MAX(lb_min_local-0-1, 0), 1
         jco = coset(lxb, 1, 0)

         DO lxa = MAX(la_min_local-1-0, 0), 1

            ico = coset(lxa, 0, 1)

            vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 1)*alpha_3(lzp, 1, 0)

         ENDDO

      ENDDO
      DO lxb = MAX(lb_min_local-0-1, 0), 1
         jco = coset(lxb, 1, 0)

         lxa = 0
         !ico=coset(0,1,1)
         ico = 9

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 1, 1)*alpha_3(lzp, 1, 0)

      ENDDO
      lxb = 0
      !jco=coset(0,2,0)
      jco = 8

      DO lxa = MAX(la_min_local-1-0, 0), 1

         ico = coset(lxa, 0, 1)

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 2)*alpha_3(lzp, 1, 0)

      ENDDO

      lxb = 0
      !jco=coset(0,2,0)
      jco = 8

      lxa = 0
      !ico=coset(0,1,1)
      ico = 9

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 1, 2)*alpha_3(lzp, 1, 0)

      DO lxb = lb_min_local, 2
         jco = coset(lxb, 0, 0)

         lxa = 0
         !ico=coset(0,0,2)
         ico = 10

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 2, 0)

      ENDDO
      DO lxb = MAX(lb_min_local-0-1, 0), 1
         jco = coset(lxb, 1, 0)

         lxa = 0
         !ico=coset(0,0,2)
         ico = 10

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 1)*alpha_3(lzp, 2, 0)

      ENDDO
      lxb = 0
      !jco=coset(0,2,0)
      jco = 8

      lxa = 0
      !ico=coset(0,0,2)
      ico = 10

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 2)*alpha_3(lzp, 2, 0)

      DO lxb = MAX(lb_min_local-1-0, 0), 1
         jco = coset(lxb, 0, 1)

         DO lxa = la_min_local, 2

            ico = coset(lxa, 0, 0)

            vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 0, 1)

         ENDDO

      ENDDO
      DO lxb = MAX(lb_min_local-1-0, 0), 1
         jco = coset(lxb, 0, 1)

         DO lxa = MAX(la_min_local-0-1, 0), 1

            ico = coset(lxa, 1, 0)

            vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 1, 0)*alpha_3(lzp, 0, 1)

         ENDDO

      ENDDO
      DO lxb = MAX(lb_min_local-1-0, 0), 1
         jco = coset(lxb, 0, 1)

         lxa = 0
         !ico=coset(0,2,0)
         ico = 8

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 2, 0)*alpha_3(lzp, 0, 1)

      ENDDO
      lxb = 0
      !jco=coset(0,1,1)
      jco = 9

      DO lxa = la_min_local, 2

         ico = coset(lxa, 0, 0)

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 1)*alpha_3(lzp, 0, 1)

      ENDDO

      lxb = 0
      !jco=coset(0,1,1)
      jco = 9

      DO lxa = MAX(la_min_local-0-1, 0), 1

         ico = coset(lxa, 1, 0)

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 1, 1)*alpha_3(lzp, 0, 1)

      ENDDO

      lxb = 0
      !jco=coset(0,1,1)
      jco = 9

      lxa = 0
      !ico=coset(0,2,0)
      ico = 8

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 2, 1)*alpha_3(lzp, 0, 1)

      DO lxb = MAX(lb_min_local-1-0, 0), 1
         jco = coset(lxb, 0, 1)

         DO lxa = MAX(la_min_local-1-0, 0), 1

            ico = coset(lxa, 0, 1)

            vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 1, 1)

         ENDDO

      ENDDO
      DO lxb = MAX(lb_min_local-1-0, 0), 1
         jco = coset(lxb, 0, 1)

         lxa = 0
         !ico=coset(0,1,1)
         ico = 9

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 1, 0)*alpha_3(lzp, 1, 1)

      ENDDO
      lxb = 0
      !jco=coset(0,1,1)
      jco = 9

      DO lxa = MAX(la_min_local-1-0, 0), 1

         ico = coset(lxa, 0, 1)

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 1)*alpha_3(lzp, 1, 1)

      ENDDO

      lxb = 0
      !jco=coset(0,1,1)
      jco = 9

      lxa = 0
      !ico=coset(0,1,1)
      ico = 9

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 1, 1)*alpha_3(lzp, 1, 1)

      DO lxb = MAX(lb_min_local-1-0, 0), 1
         jco = coset(lxb, 0, 1)

         lxa = 0
         !ico=coset(0,0,2)
         ico = 10

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 2, 1)

      ENDDO
      lxb = 0
      !jco=coset(0,1,1)
      jco = 9

      lxa = 0
      !ico=coset(0,0,2)
      ico = 10

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 1)*alpha_3(lzp, 2, 1)

      lxb = 0
      !jco=coset(0,0,2)
      jco = 10

      DO lxa = la_min_local, 2

         ico = coset(lxa, 0, 0)

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 0, 2)

      ENDDO

      lxb = 0
      !jco=coset(0,0,2)
      jco = 10

      DO lxa = MAX(la_min_local-0-1, 0), 1

         ico = coset(lxa, 1, 0)

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 1, 0)*alpha_3(lzp, 0, 2)

      ENDDO

      lxb = 0
      !jco=coset(0,0,2)
      jco = 10

      lxa = 0
      !ico=coset(0,2,0)
      ico = 8

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 2, 0)*alpha_3(lzp, 0, 2)

      lxb = 0
      !jco=coset(0,0,2)
      jco = 10

      DO lxa = MAX(la_min_local-1-0, 0), 1

         ico = coset(lxa, 0, 1)

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 1, 2)

      ENDDO

      lxb = 0
      !jco=coset(0,0,2)
      jco = 10

      lxa = 0
      !ico=coset(0,1,1)
      ico = 9

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 1, 0)*alpha_3(lzp, 1, 2)

      lxb = 0
      !jco=coset(0,0,2)
      jco = 10

      lxa = 0
      !ico=coset(0,0,2)
      ico = 10

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 2, 2)

   ENDDO

END SUBROUTINE xyz_to_vab_2_2

! Partial unrolled version
! **************************************************************************************************
!> \brief ...
!> \param prefactor ...
!> \param coef_xyz ...
!> \param lp ...
!> \param rp ...
!> \param ra ...
!> \param rab ...
!> \param vab ...
!> \param coset ...
!> \param la_min_local ...
!> \param lb_min_local ...
!> \param maxl ...
!> \param lvab ...
!> \param hvab ...
! **************************************************************************************************
SUBROUTINE xyz_to_vab_2_3(prefactor, coef_xyz, &
                          lp, rp, ra, rab, vab, coset, &
                          la_min_local, lb_min_local, maxl, lvab, hvab)

      USE kinds,                        ONLY: dp
   ! PARAMETERS
   INTEGER, INTENT(IN) :: lp, &
                          maxl, lb_min_local, la_min_local, &
                          lvab, hvab

   INTEGER, DIMENSION(-1:maxl, -1:maxl, -1:maxl), &
      INTENT(IN)     :: coset

   REAL(KIND=dp), INTENT(IN) :: prefactor

   REAL(kind=dp), DIMENSION(((lp+1)*(lp+2)*(lp+3))/6), &
      INTENT(INOUT) :: coef_xyz
   REAL(kind=dp), DIMENSION(lvab, hvab), INTENT(INOUT) :: vab
   REAL(kind=dp), DIMENSION(3), INTENT(IN) :: rp, ra, rab

   ! LOCAL
   INTEGER :: ico, jco, lxa, lxb, lxp, lxyz, lyp, lzp, l, k, lya, lyb, lza, lzb
   REAL(KIND=dp) :: a, b, binomial_k_lxa, binomial_l_lxb

   REAL(kind=dp) :: alpha(0:(2+3), 0:2, 0:3, 3)
   REAL(kind=dp) :: coef_ttz(0:2, 0:3)
   REAL(kind=dp) :: coef_tyz(0:2, 0:3, 0:2, 0:3)

   coef_xyz = coef_xyz*prefactor

   alpha(:, :, :, :) = 0.0_dp

   DO lxa = 0, 2
   DO lxb = 0, 3
      binomial_k_lxa = 1.0_dp
      a = 1.0_dp
      DO k = 0, lxa
         binomial_l_lxb = 1.0_dp
         b = 1.0_dp
         DO l = 0, lxb
            alpha(lxa-l+lxb-k, lxa, lxb, 1) = alpha(lxa-l+lxb-k, lxa, lxb, 1)+ &
                                              binomial_k_lxa*binomial_l_lxb*a*b
            binomial_l_lxb = binomial_l_lxb*REAL(lxb-l, dp)/REAL(l+1, dp)
            b = b*(rp(1)-(ra(1)+rab(1)))
         ENDDO
         binomial_k_lxa = binomial_k_lxa*REAL(lxa-k, dp)/REAL(k+1, dp)
         a = a*(-ra(1)+rp(1))
      ENDDO
   ENDDO
   ENDDO
   DO lxa = 0, 2
   DO lxb = 0, 3
      binomial_k_lxa = 1.0_dp
      a = 1.0_dp
      DO k = 0, lxa
         binomial_l_lxb = 1.0_dp
         b = 1.0_dp
         DO l = 0, lxb
            alpha(lxa-l+lxb-k, lxa, lxb, 2) = alpha(lxa-l+lxb-k, lxa, lxb, 2)+ &
                                              binomial_k_lxa*binomial_l_lxb*a*b
            binomial_l_lxb = binomial_l_lxb*REAL(lxb-l, dp)/REAL(l+1, dp)
            b = b*(rp(2)-(ra(2)+rab(2)))
         ENDDO
         binomial_k_lxa = binomial_k_lxa*REAL(lxa-k, dp)/REAL(k+1, dp)
         a = a*(-ra(2)+rp(2))
      ENDDO
   ENDDO
   ENDDO
   DO lxa = 0, 2
   DO lxb = 0, 3
      binomial_k_lxa = 1.0_dp
      a = 1.0_dp
      DO k = 0, lxa
         binomial_l_lxb = 1.0_dp
         b = 1.0_dp
         DO l = 0, lxb
            alpha(lxa-l+lxb-k, lxa, lxb, 3) = alpha(lxa-l+lxb-k, lxa, lxb, 3)+ &
                                              binomial_k_lxa*binomial_l_lxb*a*b
            binomial_l_lxb = binomial_l_lxb*REAL(lxb-l, dp)/REAL(l+1, dp)
            b = b*(rp(3)-(ra(3)+rab(3)))
         ENDDO
         binomial_k_lxa = binomial_k_lxa*REAL(lxa-k, dp)/REAL(k+1, dp)
         a = a*(-ra(3)+rp(3))
      ENDDO
   ENDDO
   ENDDO

   lxyz = 0
   DO lzp = 0, 5
      coef_tyz = 0.0_dp
      DO lyp = 0, 5-lzp
         coef_ttz = 0.0_dp
         DO lxp = 0, 5-lzp-lyp
            lxyz = lxyz+1
            DO lxb = 0, 3
            DO lxa = 0, 2
               coef_ttz(lxa, lxb) = coef_ttz(lxa, lxb)+coef_xyz(lxyz)*alpha(lxp, lxa, lxb, 1)
            ENDDO
            ENDDO

         ENDDO ! lxp

         DO lyb = 0, 3
         DO lya = 0, 2
            DO lxb = 0, 3-lyb
            DO lxa = 0, 2-lya
               coef_tyz(lxa, lxb, lya, lyb) = coef_tyz(lxa, lxb, lya, lyb)+coef_ttz(lxa, lxb)*alpha(lyp, lya, lyb, 2)
            ENDDO
            ENDDO
         ENDDO
         ENDDO

      ENDDO !lyp

      DO lzb = 0, 3
      DO lza = 0, 2
         DO lyb = 0, 3-lzb
         DO lya = 0, 2-lza
            DO lxb = MAX(lb_min_local-lzb-lyb, 0), 3-lzb-lyb
               jco = coset(lxb, lyb, lzb)
               DO lxa = MAX(la_min_local-lza-lya, 0), 2-lza-lya
                  ico = coset(lxa, lya, lza)
                  vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, lya, lyb)*alpha(lzp, lza, lzb, 3)
               ENDDO
            ENDDO
         ENDDO
         ENDDO
      ENDDO
      ENDDO

   ENDDO

END SUBROUTINE xyz_to_vab_2_3

! Partial unrolled version
! **************************************************************************************************
!> \brief ...
!> \param prefactor ...
!> \param coef_xyz ...
!> \param lp ...
!> \param rp ...
!> \param ra ...
!> \param rab ...
!> \param vab ...
!> \param coset ...
!> \param la_min_local ...
!> \param lb_min_local ...
!> \param maxl ...
!> \param lvab ...
!> \param hvab ...
! **************************************************************************************************
SUBROUTINE xyz_to_vab_2_4(prefactor, coef_xyz, &
                          lp, rp, ra, rab, vab, coset, &
                          la_min_local, lb_min_local, maxl, lvab, hvab)

      USE kinds,                        ONLY: dp
   ! PARAMETERS
   INTEGER, INTENT(IN) :: lp, &
                          maxl, lb_min_local, la_min_local, &
                          lvab, hvab

   INTEGER, DIMENSION(-1:maxl, -1:maxl, -1:maxl), &
      INTENT(IN)     :: coset

   REAL(KIND=dp), INTENT(IN) :: prefactor

   REAL(kind=dp), DIMENSION(((lp+1)*(lp+2)*(lp+3))/6), &
      INTENT(INOUT) :: coef_xyz
   REAL(kind=dp), DIMENSION(lvab, hvab), INTENT(INOUT) :: vab
   REAL(kind=dp), DIMENSION(3), INTENT(IN) :: rp, ra, rab

   ! LOCAL
   INTEGER :: ico, jco, lxa, lxb, lxp, lxyz, lyp, lzp, l, k, lya, lyb, lza, lzb
   REAL(KIND=dp) :: a, b, binomial_k_lxa, binomial_l_lxb

   REAL(kind=dp) :: alpha(0:(2+4), 0:2, 0:4, 3)
   REAL(kind=dp) :: coef_ttz(0:2, 0:4)
   REAL(kind=dp) :: coef_tyz(0:2, 0:4, 0:2, 0:4)

   coef_xyz = coef_xyz*prefactor

   alpha(:, :, :, :) = 0.0_dp

   DO lxa = 0, 2
   DO lxb = 0, 4
      binomial_k_lxa = 1.0_dp
      a = 1.0_dp
      DO k = 0, lxa
         binomial_l_lxb = 1.0_dp
         b = 1.0_dp
         DO l = 0, lxb
            alpha(lxa-l+lxb-k, lxa, lxb, 1) = alpha(lxa-l+lxb-k, lxa, lxb, 1)+ &
                                              binomial_k_lxa*binomial_l_lxb*a*b
            binomial_l_lxb = binomial_l_lxb*REAL(lxb-l, dp)/REAL(l+1, dp)
            b = b*(rp(1)-(ra(1)+rab(1)))
         ENDDO
         binomial_k_lxa = binomial_k_lxa*REAL(lxa-k, dp)/REAL(k+1, dp)
         a = a*(-ra(1)+rp(1))
      ENDDO
   ENDDO
   ENDDO
   DO lxa = 0, 2
   DO lxb = 0, 4
      binomial_k_lxa = 1.0_dp
      a = 1.0_dp
      DO k = 0, lxa
         binomial_l_lxb = 1.0_dp
         b = 1.0_dp
         DO l = 0, lxb
            alpha(lxa-l+lxb-k, lxa, lxb, 2) = alpha(lxa-l+lxb-k, lxa, lxb, 2)+ &
                                              binomial_k_lxa*binomial_l_lxb*a*b
            binomial_l_lxb = binomial_l_lxb*REAL(lxb-l, dp)/REAL(l+1, dp)
            b = b*(rp(2)-(ra(2)+rab(2)))
         ENDDO
         binomial_k_lxa = binomial_k_lxa*REAL(lxa-k, dp)/REAL(k+1, dp)
         a = a*(-ra(2)+rp(2))
      ENDDO
   ENDDO
   ENDDO
   DO lxa = 0, 2
   DO lxb = 0, 4
      binomial_k_lxa = 1.0_dp
      a = 1.0_dp
      DO k = 0, lxa
         binomial_l_lxb = 1.0_dp
         b = 1.0_dp
         DO l = 0, lxb
            alpha(lxa-l+lxb-k, lxa, lxb, 3) = alpha(lxa-l+lxb-k, lxa, lxb, 3)+ &
                                              binomial_k_lxa*binomial_l_lxb*a*b
            binomial_l_lxb = binomial_l_lxb*REAL(lxb-l, dp)/REAL(l+1, dp)
            b = b*(rp(3)-(ra(3)+rab(3)))
         ENDDO
         binomial_k_lxa = binomial_k_lxa*REAL(lxa-k, dp)/REAL(k+1, dp)
         a = a*(-ra(3)+rp(3))
      ENDDO
   ENDDO
   ENDDO

   lxyz = 0
   DO lzp = 0, 6
      coef_tyz = 0.0_dp
      DO lyp = 0, 6-lzp
         coef_ttz = 0.0_dp
         DO lxp = 0, 6-lzp-lyp
            lxyz = lxyz+1
            DO lxb = 0, 4
            DO lxa = 0, 2
               coef_ttz(lxa, lxb) = coef_ttz(lxa, lxb)+coef_xyz(lxyz)*alpha(lxp, lxa, lxb, 1)
            ENDDO
            ENDDO

         ENDDO ! lxp

         DO lyb = 0, 4
         DO lya = 0, 2
            DO lxb = 0, 4-lyb
            DO lxa = 0, 2-lya
               coef_tyz(lxa, lxb, lya, lyb) = coef_tyz(lxa, lxb, lya, lyb)+coef_ttz(lxa, lxb)*alpha(lyp, lya, lyb, 2)
            ENDDO
            ENDDO
         ENDDO
         ENDDO

      ENDDO !lyp

      DO lzb = 0, 4
      DO lza = 0, 2
         DO lyb = 0, 4-lzb
         DO lya = 0, 2-lza
            DO lxb = MAX(lb_min_local-lzb-lyb, 0), 4-lzb-lyb
               jco = coset(lxb, lyb, lzb)
               DO lxa = MAX(la_min_local-lza-lya, 0), 2-lza-lya
                  ico = coset(lxa, lya, lza)
                  vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, lya, lyb)*alpha(lzp, lza, lzb, 3)
               ENDDO
            ENDDO
         ENDDO
         ENDDO
      ENDDO
      ENDDO

   ENDDO

END SUBROUTINE xyz_to_vab_2_4

! Fully unrolled version
! **************************************************************************************************
!> \brief ...
!> \param prefactor ...
!> \param coef_xyz ...
!> \param lp ...
!> \param rp ...
!> \param ra ...
!> \param vab ...
!> \param coset ...
!> \param la_min_local ...
!> \param maxl ...
!> \param lvab ...
!> \param hvab ...
! **************************************************************************************************
SUBROUTINE xyz_to_vab_3_0(prefactor, coef_xyz, &
                          lp, rp, ra, vab, coset, &
                          la_min_local, maxl, lvab, hvab)

       USE kinds,                        ONLY: dp

   ! PARAMETERS
   REAL(KIND=dp), INTENT(IN) :: prefactor
   INTEGER, INTENT(IN) :: lp, &
                          la_min_local, maxl, &
                          lvab, hvab

   INTEGER, DIMENSION(-1:maxl, -1:maxl, -1:maxl), &
      INTENT(IN)     :: coset

   REAL(kind=dp), DIMENSION(((lp+1)*(lp+2)*(lp+3))/6), &
      INTENT(INOUT) :: coef_xyz
   REAL(kind=dp), DIMENSION(lvab, hvab), INTENT(INOUT) :: vab
   REAL(kind=dp), DIMENSION(3), INTENT(IN) :: rp, ra

   ! LOCAL
   INTEGER :: ico, jco, lxa, lxb, lxp, lxyz, lyp, lzp
   REAL(KIND=dp) :: a, b, binomial_k_lxa, binomial_l_lxb, const_ra

   REAL(kind=dp) :: alpha_1(0:(3+0), 0:3, 0:0)
   REAL(kind=dp) :: alpha_2(0:(3+0), 0:3, 0:0)
   REAL(kind=dp) :: alpha_3(0:(3+0), 0:3, 0:0)
   REAL(kind=dp) :: coef_ttz(0:3, 0:0)
   REAL(kind=dp) :: coef_tyz(0:3, 0:0, 0:3, 0:0)
   coef_xyz = coef_xyz*prefactor

   alpha_1(:, :, :) = 0.0_dp
   alpha_2(:, :, :) = 0.0_dp
   alpha_3(:, :, :) = 0.0_dp

   const_ra = (-ra(1)+rp(1))
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(0, 0, 0) = alpha_1(0, 0, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(1, 1, 0) = alpha_1(1, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(0, 1, 0) = alpha_1(0, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(2, 2, 0) = alpha_1(2, 2, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(1, 2, 0) = alpha_1(1, 2, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(1, dp)/REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(0, 2, 0) = alpha_1(0, 2, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(3, 3, 0) = alpha_1(3, 3, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(3, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(2, 3, 0) = alpha_1(2, 3, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(1, 3, 0) = alpha_1(1, 3, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(1, dp)/REAL(3, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(0, 3, 0) = alpha_1(0, 3, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   const_ra = (-ra(2)+rp(2))
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(0, 0, 0) = alpha_2(0, 0, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(1, 1, 0) = alpha_2(1, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(0, 1, 0) = alpha_2(0, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(2, 2, 0) = alpha_2(2, 2, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(1, 2, 0) = alpha_2(1, 2, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(1, dp)/REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(0, 2, 0) = alpha_2(0, 2, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(3, 3, 0) = alpha_2(3, 3, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(3, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(2, 3, 0) = alpha_2(2, 3, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(1, 3, 0) = alpha_2(1, 3, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(1, dp)/REAL(3, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(0, 3, 0) = alpha_2(0, 3, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   const_ra = (-ra(3)+rp(3))
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(0, 0, 0) = alpha_3(0, 0, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(1, 1, 0) = alpha_3(1, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(0, 1, 0) = alpha_3(0, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(2, 2, 0) = alpha_3(2, 2, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(1, 2, 0) = alpha_3(1, 2, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(1, dp)/REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(0, 2, 0) = alpha_3(0, 2, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(3, 3, 0) = alpha_3(3, 3, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(3, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(2, 3, 0) = alpha_3(2, 3, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(1, 3, 0) = alpha_3(1, 3, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(1, dp)/REAL(3, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(0, 3, 0) = alpha_3(0, 3, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0

   lxyz = 0
   DO lzp = 0, 3+0
      coef_tyz = 0.0_dp
      DO lyp = 0, 3+0-lzp
         coef_ttz = 0.0_dp
         DO lxp = 0, 3+0-lzp-lyp
            lxyz = lxyz+1
            coef_ttz(0, 0) = coef_ttz(0, 0)+coef_xyz(lxyz)*alpha_1(lxp, 0, 0)
            coef_ttz(1, 0) = coef_ttz(1, 0)+coef_xyz(lxyz)*alpha_1(lxp, 1, 0)
            coef_ttz(2, 0) = coef_ttz(2, 0)+coef_xyz(lxyz)*alpha_1(lxp, 2, 0)
            coef_ttz(3, 0) = coef_ttz(3, 0)+coef_xyz(lxyz)*alpha_1(lxp, 3, 0)
         ENDDO

         coef_tyz(0, 0, 0, 0) = coef_tyz(0, 0, 0, 0)+coef_ttz(0, 0)*alpha_2(lyp, 0, 0)
         coef_tyz(1, 0, 0, 0) = coef_tyz(1, 0, 0, 0)+coef_ttz(1, 0)*alpha_2(lyp, 0, 0)
         coef_tyz(2, 0, 0, 0) = coef_tyz(2, 0, 0, 0)+coef_ttz(2, 0)*alpha_2(lyp, 0, 0)
         coef_tyz(3, 0, 0, 0) = coef_tyz(3, 0, 0, 0)+coef_ttz(3, 0)*alpha_2(lyp, 0, 0)
         coef_tyz(0, 0, 1, 0) = coef_tyz(0, 0, 1, 0)+coef_ttz(0, 0)*alpha_2(lyp, 1, 0)
         coef_tyz(1, 0, 1, 0) = coef_tyz(1, 0, 1, 0)+coef_ttz(1, 0)*alpha_2(lyp, 1, 0)
         coef_tyz(2, 0, 1, 0) = coef_tyz(2, 0, 1, 0)+coef_ttz(2, 0)*alpha_2(lyp, 1, 0)
         coef_tyz(0, 0, 2, 0) = coef_tyz(0, 0, 2, 0)+coef_ttz(0, 0)*alpha_2(lyp, 2, 0)
         coef_tyz(1, 0, 2, 0) = coef_tyz(1, 0, 2, 0)+coef_ttz(1, 0)*alpha_2(lyp, 2, 0)
         coef_tyz(0, 0, 3, 0) = coef_tyz(0, 0, 3, 0)+coef_ttz(0, 0)*alpha_2(lyp, 3, 0)
      ENDDO

      lxb = 0
      !jco=coset(0,0,0)
      jco = 1

      DO lxa = la_min_local, 3

         ico = coset(lxa, 0, 0)

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 0, 0)

      ENDDO

      lxb = 0
      !jco=coset(0,0,0)
      jco = 1

      DO lxa = MAX(la_min_local-0-1, 0), 2

         ico = coset(lxa, 1, 0)

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 1, 0)*alpha_3(lzp, 0, 0)

      ENDDO

      lxb = 0
      !jco=coset(0,0,0)
      jco = 1

      DO lxa = MAX(la_min_local-0-2, 0), 1

         ico = coset(lxa, 2, 0)

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 2, 0)*alpha_3(lzp, 0, 0)

      ENDDO

      lxb = 0
      !jco=coset(0,0,0)
      jco = 1

      lxa = 0
      !ico=coset(0,3,0)
      ico = 17

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 3, 0)*alpha_3(lzp, 0, 0)

      lxb = 0
      !jco=coset(0,0,0)
      jco = 1

      DO lxa = MAX(la_min_local-1-0, 0), 2

         ico = coset(lxa, 0, 1)

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 1, 0)

      ENDDO

      lxb = 0
      !jco=coset(0,0,0)
      jco = 1

      DO lxa = MAX(la_min_local-1-1, 0), 1

         ico = coset(lxa, 1, 1)

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 1, 0)*alpha_3(lzp, 1, 0)

      ENDDO

      lxb = 0
      !jco=coset(0,0,0)
      jco = 1

      lxa = 0
      !ico=coset(0,2,1)
      ico = 18

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 2, 0)*alpha_3(lzp, 1, 0)

      lxb = 0
      !jco=coset(0,0,0)
      jco = 1

      DO lxa = MAX(la_min_local-2-0, 0), 1

         ico = coset(lxa, 0, 2)

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 2, 0)

      ENDDO

      lxb = 0
      !jco=coset(0,0,0)
      jco = 1

      lxa = 0
      !ico=coset(0,1,2)
      ico = 19

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 1, 0)*alpha_3(lzp, 2, 0)

      lxb = 0
      !jco=coset(0,0,0)
      jco = 1

      lxa = 0
      !ico=coset(0,0,3)
      ico = 20

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 3, 0)

   ENDDO

END SUBROUTINE xyz_to_vab_3_0

! Fully unrolled version
! **************************************************************************************************
!> \brief ...
!> \param prefactor ...
!> \param coef_xyz ...
!> \param lp ...
!> \param rp ...
!> \param ra ...
!> \param rab ...
!> \param vab ...
!> \param coset ...
!> \param la_min_local ...
!> \param lb_min_local ...
!> \param maxl ...
!> \param lvab ...
!> \param hvab ...
! **************************************************************************************************
SUBROUTINE xyz_to_vab_3_1(prefactor, coef_xyz, &
                          lp, rp, ra, rab, vab, coset, &
                          la_min_local, lb_min_local, maxl, lvab, hvab)

       USE kinds,                        ONLY: dp

   ! PARAMETERS
   REAL(KIND=dp), INTENT(IN) :: prefactor
   INTEGER, INTENT(IN) :: lp, &
                          lb_min_local, la_min_local, maxl, &
                          lvab, hvab

   INTEGER, DIMENSION(-1:maxl, -1:maxl, -1:maxl), &
      INTENT(IN)     :: coset

   REAL(kind=dp), DIMENSION(((lp+1)*(lp+2)*(lp+3))/6), &
      INTENT(INOUT) :: coef_xyz
   REAL(kind=dp), DIMENSION(lvab, hvab), INTENT(INOUT) :: vab
   REAL(kind=dp), DIMENSION(3), INTENT(IN) :: rp, ra, rab

   ! LOCAL
   INTEGER :: ico, jco, lxa, lxb, lxp, lxyz, lyp, lzp
   REAL(KIND=dp) :: a, b, binomial_k_lxa, binomial_l_lxb, const_rp, const_ra

   REAL(kind=dp) :: alpha_1(0:(3+1), 0:3, 0:1)
   REAL(kind=dp) :: alpha_2(0:(3+1), 0:3, 0:1)
   REAL(kind=dp) :: alpha_3(0:(3+1), 0:3, 0:1)
   REAL(kind=dp) :: coef_ttz(0:3, 0:1)
   REAL(kind=dp) :: coef_tyz(0:3, 0:1, 0:3, 0:1)
   coef_xyz = coef_xyz*prefactor

   alpha_1(:, :, :) = 0.0_dp
   alpha_2(:, :, :) = 0.0_dp
   alpha_3(:, :, :) = 0.0_dp

   const_rp = rp(1)-(ra(1)+rab(1))
   const_ra = (-ra(1)+rp(1))
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(0, 0, 0) = alpha_1(0, 0, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(1, 0, 1) = alpha_1(1, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_1(0, 0, 1) = alpha_1(0, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(1, 1, 0) = alpha_1(1, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(0, 1, 0) = alpha_1(0, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(2, 1, 1) = alpha_1(2, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_1(1, 1, 1) = alpha_1(1, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(1, 1, 1) = alpha_1(1, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_1(0, 1, 1) = alpha_1(0, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(2, 2, 0) = alpha_1(2, 2, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(1, 2, 0) = alpha_1(1, 2, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(1, dp)/REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(0, 2, 0) = alpha_1(0, 2, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(3, 2, 1) = alpha_1(3, 2, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_1(2, 2, 1) = alpha_1(2, 2, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(2, 2, 1) = alpha_1(2, 2, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_1(1, 2, 1) = alpha_1(1, 2, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(1, dp)/REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(1, 2, 1) = alpha_1(1, 2, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_1(0, 2, 1) = alpha_1(0, 2, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(3, 3, 0) = alpha_1(3, 3, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(3, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(2, 3, 0) = alpha_1(2, 3, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(1, 3, 0) = alpha_1(1, 3, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(1, dp)/REAL(3, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(0, 3, 0) = alpha_1(0, 3, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(4, 3, 1) = alpha_1(4, 3, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_1(3, 3, 1) = alpha_1(3, 3, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(3, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(3, 3, 1) = alpha_1(3, 3, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_1(2, 3, 1) = alpha_1(2, 3, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(2, 3, 1) = alpha_1(2, 3, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_1(1, 3, 1) = alpha_1(1, 3, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(1, dp)/REAL(3, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(1, 3, 1) = alpha_1(1, 3, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_1(0, 3, 1) = alpha_1(0, 3, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   const_rp = rp(2)-(ra(2)+rab(2))
   const_ra = (-ra(2)+rp(2))
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(0, 0, 0) = alpha_2(0, 0, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(1, 0, 1) = alpha_2(1, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_2(0, 0, 1) = alpha_2(0, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(1, 1, 0) = alpha_2(1, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(0, 1, 0) = alpha_2(0, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(2, 1, 1) = alpha_2(2, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_2(1, 1, 1) = alpha_2(1, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(1, 1, 1) = alpha_2(1, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_2(0, 1, 1) = alpha_2(0, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(2, 2, 0) = alpha_2(2, 2, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(1, 2, 0) = alpha_2(1, 2, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(1, dp)/REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(0, 2, 0) = alpha_2(0, 2, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(3, 2, 1) = alpha_2(3, 2, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_2(2, 2, 1) = alpha_2(2, 2, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(2, 2, 1) = alpha_2(2, 2, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_2(1, 2, 1) = alpha_2(1, 2, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(1, dp)/REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(1, 2, 1) = alpha_2(1, 2, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_2(0, 2, 1) = alpha_2(0, 2, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(3, 3, 0) = alpha_2(3, 3, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(3, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(2, 3, 0) = alpha_2(2, 3, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(1, 3, 0) = alpha_2(1, 3, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(1, dp)/REAL(3, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(0, 3, 0) = alpha_2(0, 3, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(4, 3, 1) = alpha_2(4, 3, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_2(3, 3, 1) = alpha_2(3, 3, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(3, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(3, 3, 1) = alpha_2(3, 3, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_2(2, 3, 1) = alpha_2(2, 3, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(2, 3, 1) = alpha_2(2, 3, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_2(1, 3, 1) = alpha_2(1, 3, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(1, dp)/REAL(3, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(1, 3, 1) = alpha_2(1, 3, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_2(0, 3, 1) = alpha_2(0, 3, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   const_rp = rp(3)-(ra(3)+rab(3))
   const_ra = (-ra(3)+rp(3))
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(0, 0, 0) = alpha_3(0, 0, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(1, 0, 1) = alpha_3(1, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_3(0, 0, 1) = alpha_3(0, 0, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(1, 1, 0) = alpha_3(1, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(0, 1, 0) = alpha_3(0, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(2, 1, 1) = alpha_3(2, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_3(1, 1, 1) = alpha_3(1, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(1, 1, 1) = alpha_3(1, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_3(0, 1, 1) = alpha_3(0, 1, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(2, 2, 0) = alpha_3(2, 2, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(1, 2, 0) = alpha_3(1, 2, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(1, dp)/REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(0, 2, 0) = alpha_3(0, 2, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(3, 2, 1) = alpha_3(3, 2, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_3(2, 2, 1) = alpha_3(2, 2, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(2, 2, 1) = alpha_3(2, 2, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_3(1, 2, 1) = alpha_3(1, 2, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(1, dp)/REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(1, 2, 1) = alpha_3(1, 2, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_3(0, 2, 1) = alpha_3(0, 2, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(3, 3, 0) = alpha_3(3, 3, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(3, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(2, 3, 0) = alpha_3(2, 3, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(1, 3, 0) = alpha_3(1, 3, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(1, dp)/REAL(3, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(0, 3, 0) = alpha_3(0, 3, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(4, 3, 1) = alpha_3(4, 3, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_3(3, 3, 1) = alpha_3(3, 3, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(3, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(3, 3, 1) = alpha_3(3, 3, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_3(2, 3, 1) = alpha_3(2, 3, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(2, 3, 1) = alpha_3(2, 3, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_3(1, 3, 1) = alpha_3(1, 3, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(1, dp)/REAL(3, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(1, 3, 1) = alpha_3(1, 3, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! binomial_l_lxb=binomial_l_lxb*1
   b = b*const_rp
   alpha_3(0, 3, 1) = alpha_3(0, 3, 1)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0

   lxyz = 0
   DO lzp = 0, 3+1
      coef_tyz = 0.0_dp
      DO lyp = 0, 3+1-lzp
         coef_ttz = 0.0_dp
         DO lxp = 0, 3+1-lzp-lyp
            lxyz = lxyz+1
            coef_ttz(0, 0) = coef_ttz(0, 0)+coef_xyz(lxyz)*alpha_1(lxp, 0, 0)
            coef_ttz(1, 0) = coef_ttz(1, 0)+coef_xyz(lxyz)*alpha_1(lxp, 1, 0)
            coef_ttz(2, 0) = coef_ttz(2, 0)+coef_xyz(lxyz)*alpha_1(lxp, 2, 0)
            coef_ttz(3, 0) = coef_ttz(3, 0)+coef_xyz(lxyz)*alpha_1(lxp, 3, 0)
            coef_ttz(0, 1) = coef_ttz(0, 1)+coef_xyz(lxyz)*alpha_1(lxp, 0, 1)
            coef_ttz(1, 1) = coef_ttz(1, 1)+coef_xyz(lxyz)*alpha_1(lxp, 1, 1)
            coef_ttz(2, 1) = coef_ttz(2, 1)+coef_xyz(lxyz)*alpha_1(lxp, 2, 1)
            coef_ttz(3, 1) = coef_ttz(3, 1)+coef_xyz(lxyz)*alpha_1(lxp, 3, 1)
         ENDDO

         coef_tyz(0, 0, 0, 0) = coef_tyz(0, 0, 0, 0)+coef_ttz(0, 0)*alpha_2(lyp, 0, 0)
         coef_tyz(1, 0, 0, 0) = coef_tyz(1, 0, 0, 0)+coef_ttz(1, 0)*alpha_2(lyp, 0, 0)
         coef_tyz(2, 0, 0, 0) = coef_tyz(2, 0, 0, 0)+coef_ttz(2, 0)*alpha_2(lyp, 0, 0)
         coef_tyz(3, 0, 0, 0) = coef_tyz(3, 0, 0, 0)+coef_ttz(3, 0)*alpha_2(lyp, 0, 0)
         coef_tyz(0, 1, 0, 0) = coef_tyz(0, 1, 0, 0)+coef_ttz(0, 1)*alpha_2(lyp, 0, 0)
         coef_tyz(1, 1, 0, 0) = coef_tyz(1, 1, 0, 0)+coef_ttz(1, 1)*alpha_2(lyp, 0, 0)
         coef_tyz(2, 1, 0, 0) = coef_tyz(2, 1, 0, 0)+coef_ttz(2, 1)*alpha_2(lyp, 0, 0)
         coef_tyz(3, 1, 0, 0) = coef_tyz(3, 1, 0, 0)+coef_ttz(3, 1)*alpha_2(lyp, 0, 0)
         coef_tyz(0, 0, 1, 0) = coef_tyz(0, 0, 1, 0)+coef_ttz(0, 0)*alpha_2(lyp, 1, 0)
         coef_tyz(1, 0, 1, 0) = coef_tyz(1, 0, 1, 0)+coef_ttz(1, 0)*alpha_2(lyp, 1, 0)
         coef_tyz(2, 0, 1, 0) = coef_tyz(2, 0, 1, 0)+coef_ttz(2, 0)*alpha_2(lyp, 1, 0)
         coef_tyz(0, 1, 1, 0) = coef_tyz(0, 1, 1, 0)+coef_ttz(0, 1)*alpha_2(lyp, 1, 0)
         coef_tyz(1, 1, 1, 0) = coef_tyz(1, 1, 1, 0)+coef_ttz(1, 1)*alpha_2(lyp, 1, 0)
         coef_tyz(2, 1, 1, 0) = coef_tyz(2, 1, 1, 0)+coef_ttz(2, 1)*alpha_2(lyp, 1, 0)
         coef_tyz(0, 0, 2, 0) = coef_tyz(0, 0, 2, 0)+coef_ttz(0, 0)*alpha_2(lyp, 2, 0)
         coef_tyz(1, 0, 2, 0) = coef_tyz(1, 0, 2, 0)+coef_ttz(1, 0)*alpha_2(lyp, 2, 0)
         coef_tyz(0, 1, 2, 0) = coef_tyz(0, 1, 2, 0)+coef_ttz(0, 1)*alpha_2(lyp, 2, 0)
         coef_tyz(1, 1, 2, 0) = coef_tyz(1, 1, 2, 0)+coef_ttz(1, 1)*alpha_2(lyp, 2, 0)
         coef_tyz(0, 0, 3, 0) = coef_tyz(0, 0, 3, 0)+coef_ttz(0, 0)*alpha_2(lyp, 3, 0)
         coef_tyz(0, 1, 3, 0) = coef_tyz(0, 1, 3, 0)+coef_ttz(0, 1)*alpha_2(lyp, 3, 0)
         coef_tyz(0, 0, 0, 1) = coef_tyz(0, 0, 0, 1)+coef_ttz(0, 0)*alpha_2(lyp, 0, 1)
         coef_tyz(1, 0, 0, 1) = coef_tyz(1, 0, 0, 1)+coef_ttz(1, 0)*alpha_2(lyp, 0, 1)
         coef_tyz(2, 0, 0, 1) = coef_tyz(2, 0, 0, 1)+coef_ttz(2, 0)*alpha_2(lyp, 0, 1)
         coef_tyz(3, 0, 0, 1) = coef_tyz(3, 0, 0, 1)+coef_ttz(3, 0)*alpha_2(lyp, 0, 1)
         coef_tyz(0, 0, 1, 1) = coef_tyz(0, 0, 1, 1)+coef_ttz(0, 0)*alpha_2(lyp, 1, 1)
         coef_tyz(1, 0, 1, 1) = coef_tyz(1, 0, 1, 1)+coef_ttz(1, 0)*alpha_2(lyp, 1, 1)
         coef_tyz(2, 0, 1, 1) = coef_tyz(2, 0, 1, 1)+coef_ttz(2, 0)*alpha_2(lyp, 1, 1)
         coef_tyz(0, 0, 2, 1) = coef_tyz(0, 0, 2, 1)+coef_ttz(0, 0)*alpha_2(lyp, 2, 1)
         coef_tyz(1, 0, 2, 1) = coef_tyz(1, 0, 2, 1)+coef_ttz(1, 0)*alpha_2(lyp, 2, 1)
         coef_tyz(0, 0, 3, 1) = coef_tyz(0, 0, 3, 1)+coef_ttz(0, 0)*alpha_2(lyp, 3, 1)
      ENDDO

      DO lxb = lb_min_local, 1
         jco = coset(lxb, 0, 0)

         DO lxa = la_min_local, 3

            ico = coset(lxa, 0, 0)

            vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 0, 0)

         ENDDO

      ENDDO
      DO lxb = lb_min_local, 1
         jco = coset(lxb, 0, 0)

         DO lxa = MAX(la_min_local-0-1, 0), 2

            ico = coset(lxa, 1, 0)

            vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 1, 0)*alpha_3(lzp, 0, 0)

         ENDDO

      ENDDO
      DO lxb = lb_min_local, 1
         jco = coset(lxb, 0, 0)

         DO lxa = MAX(la_min_local-0-2, 0), 1

            ico = coset(lxa, 2, 0)

            vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 2, 0)*alpha_3(lzp, 0, 0)

         ENDDO

      ENDDO
      DO lxb = lb_min_local, 1
         jco = coset(lxb, 0, 0)

         lxa = 0
         !ico=coset(0,3,0)
         ico = 17

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 3, 0)*alpha_3(lzp, 0, 0)

      ENDDO
      lxb = 0
      !jco=coset(0,1,0)
      jco = 3

      DO lxa = la_min_local, 3

         ico = coset(lxa, 0, 0)

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 1)*alpha_3(lzp, 0, 0)

      ENDDO

      lxb = 0
      !jco=coset(0,1,0)
      jco = 3

      DO lxa = MAX(la_min_local-0-1, 0), 2

         ico = coset(lxa, 1, 0)

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 1, 1)*alpha_3(lzp, 0, 0)

      ENDDO

      lxb = 0
      !jco=coset(0,1,0)
      jco = 3

      DO lxa = MAX(la_min_local-0-2, 0), 1

         ico = coset(lxa, 2, 0)

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 2, 1)*alpha_3(lzp, 0, 0)

      ENDDO

      lxb = 0
      !jco=coset(0,1,0)
      jco = 3

      lxa = 0
      !ico=coset(0,3,0)
      ico = 17

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 3, 1)*alpha_3(lzp, 0, 0)

      DO lxb = lb_min_local, 1
         jco = coset(lxb, 0, 0)

         DO lxa = MAX(la_min_local-1-0, 0), 2

            ico = coset(lxa, 0, 1)

            vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 1, 0)

         ENDDO

      ENDDO
      DO lxb = lb_min_local, 1
         jco = coset(lxb, 0, 0)

         DO lxa = MAX(la_min_local-1-1, 0), 1

            ico = coset(lxa, 1, 1)

            vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 1, 0)*alpha_3(lzp, 1, 0)

         ENDDO

      ENDDO
      DO lxb = lb_min_local, 1
         jco = coset(lxb, 0, 0)

         lxa = 0
         !ico=coset(0,2,1)
         ico = 18

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 2, 0)*alpha_3(lzp, 1, 0)

      ENDDO
      lxb = 0
      !jco=coset(0,1,0)
      jco = 3

      DO lxa = MAX(la_min_local-1-0, 0), 2

         ico = coset(lxa, 0, 1)

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 1)*alpha_3(lzp, 1, 0)

      ENDDO

      lxb = 0
      !jco=coset(0,1,0)
      jco = 3

      DO lxa = MAX(la_min_local-1-1, 0), 1

         ico = coset(lxa, 1, 1)

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 1, 1)*alpha_3(lzp, 1, 0)

      ENDDO

      lxb = 0
      !jco=coset(0,1,0)
      jco = 3

      lxa = 0
      !ico=coset(0,2,1)
      ico = 18

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 2, 1)*alpha_3(lzp, 1, 0)

      DO lxb = lb_min_local, 1
         jco = coset(lxb, 0, 0)

         DO lxa = MAX(la_min_local-2-0, 0), 1

            ico = coset(lxa, 0, 2)

            vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 2, 0)

         ENDDO

      ENDDO
      DO lxb = lb_min_local, 1
         jco = coset(lxb, 0, 0)

         lxa = 0
         !ico=coset(0,1,2)
         ico = 19

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 1, 0)*alpha_3(lzp, 2, 0)

      ENDDO
      lxb = 0
      !jco=coset(0,1,0)
      jco = 3

      DO lxa = MAX(la_min_local-2-0, 0), 1

         ico = coset(lxa, 0, 2)

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 1)*alpha_3(lzp, 2, 0)

      ENDDO

      lxb = 0
      !jco=coset(0,1,0)
      jco = 3

      lxa = 0
      !ico=coset(0,1,2)
      ico = 19

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 1, 1)*alpha_3(lzp, 2, 0)

      DO lxb = lb_min_local, 1
         jco = coset(lxb, 0, 0)

         lxa = 0
         !ico=coset(0,0,3)
         ico = 20

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 3, 0)

      ENDDO
      lxb = 0
      !jco=coset(0,1,0)
      jco = 3

      lxa = 0
      !ico=coset(0,0,3)
      ico = 20

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 1)*alpha_3(lzp, 3, 0)

      lxb = 0
      !jco=coset(0,0,1)
      jco = 4

      DO lxa = la_min_local, 3

         ico = coset(lxa, 0, 0)

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 0, 1)

      ENDDO

      lxb = 0
      !jco=coset(0,0,1)
      jco = 4

      DO lxa = MAX(la_min_local-0-1, 0), 2

         ico = coset(lxa, 1, 0)

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 1, 0)*alpha_3(lzp, 0, 1)

      ENDDO

      lxb = 0
      !jco=coset(0,0,1)
      jco = 4

      DO lxa = MAX(la_min_local-0-2, 0), 1

         ico = coset(lxa, 2, 0)

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 2, 0)*alpha_3(lzp, 0, 1)

      ENDDO

      lxb = 0
      !jco=coset(0,0,1)
      jco = 4

      lxa = 0
      !ico=coset(0,3,0)
      ico = 17

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 3, 0)*alpha_3(lzp, 0, 1)

      lxb = 0
      !jco=coset(0,0,1)
      jco = 4

      DO lxa = MAX(la_min_local-1-0, 0), 2

         ico = coset(lxa, 0, 1)

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 1, 1)

      ENDDO

      lxb = 0
      !jco=coset(0,0,1)
      jco = 4

      DO lxa = MAX(la_min_local-1-1, 0), 1

         ico = coset(lxa, 1, 1)

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 1, 0)*alpha_3(lzp, 1, 1)

      ENDDO

      lxb = 0
      !jco=coset(0,0,1)
      jco = 4

      lxa = 0
      !ico=coset(0,2,1)
      ico = 18

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 2, 0)*alpha_3(lzp, 1, 1)

      lxb = 0
      !jco=coset(0,0,1)
      jco = 4

      DO lxa = MAX(la_min_local-2-0, 0), 1

         ico = coset(lxa, 0, 2)

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 2, 1)

      ENDDO

      lxb = 0
      !jco=coset(0,0,1)
      jco = 4

      lxa = 0
      !ico=coset(0,1,2)
      ico = 19

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 1, 0)*alpha_3(lzp, 2, 1)

      lxb = 0
      !jco=coset(0,0,1)
      jco = 4

      lxa = 0
      !ico=coset(0,0,3)
      ico = 20

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 3, 1)

   ENDDO

END SUBROUTINE xyz_to_vab_3_1

! Partial unrolled version
! **************************************************************************************************
!> \brief ...
!> \param prefactor ...
!> \param coef_xyz ...
!> \param lp ...
!> \param rp ...
!> \param ra ...
!> \param rab ...
!> \param vab ...
!> \param coset ...
!> \param la_min_local ...
!> \param lb_min_local ...
!> \param maxl ...
!> \param lvab ...
!> \param hvab ...
! **************************************************************************************************
SUBROUTINE xyz_to_vab_3_2(prefactor, coef_xyz, &
                          lp, rp, ra, rab, vab, coset, &
                          la_min_local, lb_min_local, maxl, lvab, hvab)

      USE kinds,                        ONLY: dp
   ! PARAMETERS
   INTEGER, INTENT(IN) :: lp, &
                          maxl, lb_min_local, la_min_local, &
                          lvab, hvab

   INTEGER, DIMENSION(-1:maxl, -1:maxl, -1:maxl), &
      INTENT(IN)     :: coset

   REAL(KIND=dp), INTENT(IN) :: prefactor

   REAL(kind=dp), DIMENSION(((lp+1)*(lp+2)*(lp+3))/6), &
      INTENT(INOUT) :: coef_xyz
   REAL(kind=dp), DIMENSION(lvab, hvab), INTENT(INOUT) :: vab
   REAL(kind=dp), DIMENSION(3), INTENT(IN) :: rp, ra, rab

   ! LOCAL
   INTEGER :: ico, jco, lxa, lxb, lxp, lxyz, lyp, lzp, l, k, lya, lyb, lza, lzb
   REAL(KIND=dp) :: a, b, binomial_k_lxa, binomial_l_lxb

   REAL(kind=dp) :: alpha(0:(3+2), 0:3, 0:2, 3)
   REAL(kind=dp) :: coef_ttz(0:3, 0:2)
   REAL(kind=dp) :: coef_tyz(0:3, 0:2, 0:3, 0:2)

   coef_xyz = coef_xyz*prefactor

   alpha(:, :, :, :) = 0.0_dp

   DO lxa = 0, 3
   DO lxb = 0, 2
      binomial_k_lxa = 1.0_dp
      a = 1.0_dp
      DO k = 0, lxa
         binomial_l_lxb = 1.0_dp
         b = 1.0_dp
         DO l = 0, lxb
            alpha(lxa-l+lxb-k, lxa, lxb, 1) = alpha(lxa-l+lxb-k, lxa, lxb, 1)+ &
                                              binomial_k_lxa*binomial_l_lxb*a*b
            binomial_l_lxb = binomial_l_lxb*REAL(lxb-l, dp)/REAL(l+1, dp)
            b = b*(rp(1)-(ra(1)+rab(1)))
         ENDDO
         binomial_k_lxa = binomial_k_lxa*REAL(lxa-k, dp)/REAL(k+1, dp)
         a = a*(-ra(1)+rp(1))
      ENDDO
   ENDDO
   ENDDO
   DO lxa = 0, 3
   DO lxb = 0, 2
      binomial_k_lxa = 1.0_dp
      a = 1.0_dp
      DO k = 0, lxa
         binomial_l_lxb = 1.0_dp
         b = 1.0_dp
         DO l = 0, lxb
            alpha(lxa-l+lxb-k, lxa, lxb, 2) = alpha(lxa-l+lxb-k, lxa, lxb, 2)+ &
                                              binomial_k_lxa*binomial_l_lxb*a*b
            binomial_l_lxb = binomial_l_lxb*REAL(lxb-l, dp)/REAL(l+1, dp)
            b = b*(rp(2)-(ra(2)+rab(2)))
         ENDDO
         binomial_k_lxa = binomial_k_lxa*REAL(lxa-k, dp)/REAL(k+1, dp)
         a = a*(-ra(2)+rp(2))
      ENDDO
   ENDDO
   ENDDO
   DO lxa = 0, 3
   DO lxb = 0, 2
      binomial_k_lxa = 1.0_dp
      a = 1.0_dp
      DO k = 0, lxa
         binomial_l_lxb = 1.0_dp
         b = 1.0_dp
         DO l = 0, lxb
            alpha(lxa-l+lxb-k, lxa, lxb, 3) = alpha(lxa-l+lxb-k, lxa, lxb, 3)+ &
                                              binomial_k_lxa*binomial_l_lxb*a*b
            binomial_l_lxb = binomial_l_lxb*REAL(lxb-l, dp)/REAL(l+1, dp)
            b = b*(rp(3)-(ra(3)+rab(3)))
         ENDDO
         binomial_k_lxa = binomial_k_lxa*REAL(lxa-k, dp)/REAL(k+1, dp)
         a = a*(-ra(3)+rp(3))
      ENDDO
   ENDDO
   ENDDO

   lxyz = 0
   DO lzp = 0, 5
      coef_tyz = 0.0_dp
      DO lyp = 0, 5-lzp
         coef_ttz = 0.0_dp
         DO lxp = 0, 5-lzp-lyp
            lxyz = lxyz+1
            DO lxb = 0, 2
            DO lxa = 0, 3
               coef_ttz(lxa, lxb) = coef_ttz(lxa, lxb)+coef_xyz(lxyz)*alpha(lxp, lxa, lxb, 1)
            ENDDO
            ENDDO

         ENDDO ! lxp

         DO lyb = 0, 2
         DO lya = 0, 3
            DO lxb = 0, 2-lyb
            DO lxa = 0, 3-lya
               coef_tyz(lxa, lxb, lya, lyb) = coef_tyz(lxa, lxb, lya, lyb)+coef_ttz(lxa, lxb)*alpha(lyp, lya, lyb, 2)
            ENDDO
            ENDDO
         ENDDO
         ENDDO

      ENDDO !lyp

      DO lzb = 0, 2
      DO lza = 0, 3
         DO lyb = 0, 2-lzb
         DO lya = 0, 3-lza
            DO lxb = MAX(lb_min_local-lzb-lyb, 0), 2-lzb-lyb
               jco = coset(lxb, lyb, lzb)
               DO lxa = MAX(la_min_local-lza-lya, 0), 3-lza-lya
                  ico = coset(lxa, lya, lza)
                  vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, lya, lyb)*alpha(lzp, lza, lzb, 3)
               ENDDO
            ENDDO
         ENDDO
         ENDDO
      ENDDO
      ENDDO

   ENDDO

END SUBROUTINE xyz_to_vab_3_2

! Partial unrolled version
! **************************************************************************************************
!> \brief ...
!> \param prefactor ...
!> \param coef_xyz ...
!> \param lp ...
!> \param rp ...
!> \param ra ...
!> \param rab ...
!> \param vab ...
!> \param coset ...
!> \param la_min_local ...
!> \param lb_min_local ...
!> \param maxl ...
!> \param lvab ...
!> \param hvab ...
! **************************************************************************************************
SUBROUTINE xyz_to_vab_3_3(prefactor, coef_xyz, &
                          lp, rp, ra, rab, vab, coset, &
                          la_min_local, lb_min_local, maxl, lvab, hvab)

      USE kinds,                        ONLY: dp
   ! PARAMETERS
   INTEGER, INTENT(IN) :: lp, &
                          maxl, lb_min_local, la_min_local, &
                          lvab, hvab

   INTEGER, DIMENSION(-1:maxl, -1:maxl, -1:maxl), &
      INTENT(IN)     :: coset

   REAL(KIND=dp), INTENT(IN) :: prefactor

   REAL(kind=dp), DIMENSION(((lp+1)*(lp+2)*(lp+3))/6), &
      INTENT(INOUT) :: coef_xyz
   REAL(kind=dp), DIMENSION(lvab, hvab), INTENT(INOUT) :: vab
   REAL(kind=dp), DIMENSION(3), INTENT(IN) :: rp, ra, rab

   ! LOCAL
   INTEGER :: ico, jco, lxa, lxb, lxp, lxyz, lyp, lzp, l, k, lya, lyb, lza, lzb
   REAL(KIND=dp) :: a, b, binomial_k_lxa, binomial_l_lxb

   REAL(kind=dp) :: alpha(0:(3+3), 0:3, 0:3, 3)
   REAL(kind=dp) :: coef_ttz(0:3, 0:3)
   REAL(kind=dp) :: coef_tyz(0:3, 0:3, 0:3, 0:3)

   coef_xyz = coef_xyz*prefactor

   alpha(:, :, :, :) = 0.0_dp

   DO lxa = 0, 3
   DO lxb = 0, 3
      binomial_k_lxa = 1.0_dp
      a = 1.0_dp
      DO k = 0, lxa
         binomial_l_lxb = 1.0_dp
         b = 1.0_dp
         DO l = 0, lxb
            alpha(lxa-l+lxb-k, lxa, lxb, 1) = alpha(lxa-l+lxb-k, lxa, lxb, 1)+ &
                                              binomial_k_lxa*binomial_l_lxb*a*b
            binomial_l_lxb = binomial_l_lxb*REAL(lxb-l, dp)/REAL(l+1, dp)
            b = b*(rp(1)-(ra(1)+rab(1)))
         ENDDO
         binomial_k_lxa = binomial_k_lxa*REAL(lxa-k, dp)/REAL(k+1, dp)
         a = a*(-ra(1)+rp(1))
      ENDDO
   ENDDO
   ENDDO
   DO lxa = 0, 3
   DO lxb = 0, 3
      binomial_k_lxa = 1.0_dp
      a = 1.0_dp
      DO k = 0, lxa
         binomial_l_lxb = 1.0_dp
         b = 1.0_dp
         DO l = 0, lxb
            alpha(lxa-l+lxb-k, lxa, lxb, 2) = alpha(lxa-l+lxb-k, lxa, lxb, 2)+ &
                                              binomial_k_lxa*binomial_l_lxb*a*b
            binomial_l_lxb = binomial_l_lxb*REAL(lxb-l, dp)/REAL(l+1, dp)
            b = b*(rp(2)-(ra(2)+rab(2)))
         ENDDO
         binomial_k_lxa = binomial_k_lxa*REAL(lxa-k, dp)/REAL(k+1, dp)
         a = a*(-ra(2)+rp(2))
      ENDDO
   ENDDO
   ENDDO
   DO lxa = 0, 3
   DO lxb = 0, 3
      binomial_k_lxa = 1.0_dp
      a = 1.0_dp
      DO k = 0, lxa
         binomial_l_lxb = 1.0_dp
         b = 1.0_dp
         DO l = 0, lxb
            alpha(lxa-l+lxb-k, lxa, lxb, 3) = alpha(lxa-l+lxb-k, lxa, lxb, 3)+ &
                                              binomial_k_lxa*binomial_l_lxb*a*b
            binomial_l_lxb = binomial_l_lxb*REAL(lxb-l, dp)/REAL(l+1, dp)
            b = b*(rp(3)-(ra(3)+rab(3)))
         ENDDO
         binomial_k_lxa = binomial_k_lxa*REAL(lxa-k, dp)/REAL(k+1, dp)
         a = a*(-ra(3)+rp(3))
      ENDDO
   ENDDO
   ENDDO

   lxyz = 0
   DO lzp = 0, 6
      coef_tyz = 0.0_dp
      DO lyp = 0, 6-lzp
         coef_ttz = 0.0_dp
         DO lxp = 0, 6-lzp-lyp
            lxyz = lxyz+1
            DO lxb = 0, 3
            DO lxa = 0, 3
               coef_ttz(lxa, lxb) = coef_ttz(lxa, lxb)+coef_xyz(lxyz)*alpha(lxp, lxa, lxb, 1)
            ENDDO
            ENDDO

         ENDDO ! lxp

         DO lyb = 0, 3
         DO lya = 0, 3
            DO lxb = 0, 3-lyb
            DO lxa = 0, 3-lya
               coef_tyz(lxa, lxb, lya, lyb) = coef_tyz(lxa, lxb, lya, lyb)+coef_ttz(lxa, lxb)*alpha(lyp, lya, lyb, 2)
            ENDDO
            ENDDO
         ENDDO
         ENDDO

      ENDDO !lyp

      DO lzb = 0, 3
      DO lza = 0, 3
         DO lyb = 0, 3-lzb
         DO lya = 0, 3-lza
            DO lxb = MAX(lb_min_local-lzb-lyb, 0), 3-lzb-lyb
               jco = coset(lxb, lyb, lzb)
               DO lxa = MAX(la_min_local-lza-lya, 0), 3-lza-lya
                  ico = coset(lxa, lya, lza)
                  vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, lya, lyb)*alpha(lzp, lza, lzb, 3)
               ENDDO
            ENDDO
         ENDDO
         ENDDO
      ENDDO
      ENDDO

   ENDDO

END SUBROUTINE xyz_to_vab_3_3

! Partial unrolled version
! **************************************************************************************************
!> \brief ...
!> \param prefactor ...
!> \param coef_xyz ...
!> \param lp ...
!> \param rp ...
!> \param ra ...
!> \param rab ...
!> \param vab ...
!> \param coset ...
!> \param la_min_local ...
!> \param lb_min_local ...
!> \param maxl ...
!> \param lvab ...
!> \param hvab ...
! **************************************************************************************************
SUBROUTINE xyz_to_vab_3_4(prefactor, coef_xyz, &
                          lp, rp, ra, rab, vab, coset, &
                          la_min_local, lb_min_local, maxl, lvab, hvab)

      USE kinds,                        ONLY: dp
   ! PARAMETERS
   INTEGER, INTENT(IN) :: lp, &
                          maxl, lb_min_local, la_min_local, &
                          lvab, hvab

   INTEGER, DIMENSION(-1:maxl, -1:maxl, -1:maxl), &
      INTENT(IN)     :: coset

   REAL(KIND=dp), INTENT(IN) :: prefactor

   REAL(kind=dp), DIMENSION(((lp+1)*(lp+2)*(lp+3))/6), &
      INTENT(INOUT) :: coef_xyz
   REAL(kind=dp), DIMENSION(lvab, hvab), INTENT(INOUT) :: vab
   REAL(kind=dp), DIMENSION(3), INTENT(IN) :: rp, ra, rab

   ! LOCAL
   INTEGER :: ico, jco, lxa, lxb, lxp, lxyz, lyp, lzp, l, k, lya, lyb, lza, lzb
   REAL(KIND=dp) :: a, b, binomial_k_lxa, binomial_l_lxb

   REAL(kind=dp) :: alpha(0:(3+4), 0:3, 0:4, 3)
   REAL(kind=dp) :: coef_ttz(0:3, 0:4)
   REAL(kind=dp) :: coef_tyz(0:3, 0:4, 0:3, 0:4)

   coef_xyz = coef_xyz*prefactor

   alpha(:, :, :, :) = 0.0_dp

   DO lxa = 0, 3
   DO lxb = 0, 4
      binomial_k_lxa = 1.0_dp
      a = 1.0_dp
      DO k = 0, lxa
         binomial_l_lxb = 1.0_dp
         b = 1.0_dp
         DO l = 0, lxb
            alpha(lxa-l+lxb-k, lxa, lxb, 1) = alpha(lxa-l+lxb-k, lxa, lxb, 1)+ &
                                              binomial_k_lxa*binomial_l_lxb*a*b
            binomial_l_lxb = binomial_l_lxb*REAL(lxb-l, dp)/REAL(l+1, dp)
            b = b*(rp(1)-(ra(1)+rab(1)))
         ENDDO
         binomial_k_lxa = binomial_k_lxa*REAL(lxa-k, dp)/REAL(k+1, dp)
         a = a*(-ra(1)+rp(1))
      ENDDO
   ENDDO
   ENDDO
   DO lxa = 0, 3
   DO lxb = 0, 4
      binomial_k_lxa = 1.0_dp
      a = 1.0_dp
      DO k = 0, lxa
         binomial_l_lxb = 1.0_dp
         b = 1.0_dp
         DO l = 0, lxb
            alpha(lxa-l+lxb-k, lxa, lxb, 2) = alpha(lxa-l+lxb-k, lxa, lxb, 2)+ &
                                              binomial_k_lxa*binomial_l_lxb*a*b
            binomial_l_lxb = binomial_l_lxb*REAL(lxb-l, dp)/REAL(l+1, dp)
            b = b*(rp(2)-(ra(2)+rab(2)))
         ENDDO
         binomial_k_lxa = binomial_k_lxa*REAL(lxa-k, dp)/REAL(k+1, dp)
         a = a*(-ra(2)+rp(2))
      ENDDO
   ENDDO
   ENDDO
   DO lxa = 0, 3
   DO lxb = 0, 4
      binomial_k_lxa = 1.0_dp
      a = 1.0_dp
      DO k = 0, lxa
         binomial_l_lxb = 1.0_dp
         b = 1.0_dp
         DO l = 0, lxb
            alpha(lxa-l+lxb-k, lxa, lxb, 3) = alpha(lxa-l+lxb-k, lxa, lxb, 3)+ &
                                              binomial_k_lxa*binomial_l_lxb*a*b
            binomial_l_lxb = binomial_l_lxb*REAL(lxb-l, dp)/REAL(l+1, dp)
            b = b*(rp(3)-(ra(3)+rab(3)))
         ENDDO
         binomial_k_lxa = binomial_k_lxa*REAL(lxa-k, dp)/REAL(k+1, dp)
         a = a*(-ra(3)+rp(3))
      ENDDO
   ENDDO
   ENDDO

   lxyz = 0
   DO lzp = 0, 7
      coef_tyz = 0.0_dp
      DO lyp = 0, 7-lzp
         coef_ttz = 0.0_dp
         DO lxp = 0, 7-lzp-lyp
            lxyz = lxyz+1
            DO lxb = 0, 4
            DO lxa = 0, 3
               coef_ttz(lxa, lxb) = coef_ttz(lxa, lxb)+coef_xyz(lxyz)*alpha(lxp, lxa, lxb, 1)
            ENDDO
            ENDDO

         ENDDO ! lxp

         DO lyb = 0, 4
         DO lya = 0, 3
            DO lxb = 0, 4-lyb
            DO lxa = 0, 3-lya
               coef_tyz(lxa, lxb, lya, lyb) = coef_tyz(lxa, lxb, lya, lyb)+coef_ttz(lxa, lxb)*alpha(lyp, lya, lyb, 2)
            ENDDO
            ENDDO
         ENDDO
         ENDDO

      ENDDO !lyp

      DO lzb = 0, 4
      DO lza = 0, 3
         DO lyb = 0, 4-lzb
         DO lya = 0, 3-lza
            DO lxb = MAX(lb_min_local-lzb-lyb, 0), 4-lzb-lyb
               jco = coset(lxb, lyb, lzb)
               DO lxa = MAX(la_min_local-lza-lya, 0), 3-lza-lya
                  ico = coset(lxa, lya, lza)
                  vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, lya, lyb)*alpha(lzp, lza, lzb, 3)
               ENDDO
            ENDDO
         ENDDO
         ENDDO
      ENDDO
      ENDDO

   ENDDO

END SUBROUTINE xyz_to_vab_3_4

! Fully unrolled version
! **************************************************************************************************
!> \brief ...
!> \param prefactor ...
!> \param coef_xyz ...
!> \param lp ...
!> \param rp ...
!> \param ra ...
!> \param vab ...
!> \param coset ...
!> \param la_min_local ...
!> \param maxl ...
!> \param lvab ...
!> \param hvab ...
! **************************************************************************************************
SUBROUTINE xyz_to_vab_4_0(prefactor, coef_xyz, &
                          lp, rp, ra, vab, coset, &
                          la_min_local, maxl, lvab, hvab)

       USE kinds,                        ONLY: dp

   ! PARAMETERS
   REAL(KIND=dp), INTENT(IN) :: prefactor
   INTEGER, INTENT(IN) :: lp, &
                          la_min_local, maxl, &
                          lvab, hvab

   INTEGER, DIMENSION(-1:maxl, -1:maxl, -1:maxl), &
      INTENT(IN)     :: coset

   REAL(kind=dp), DIMENSION(((lp+1)*(lp+2)*(lp+3))/6), &
      INTENT(INOUT) :: coef_xyz
   REAL(kind=dp), DIMENSION(lvab, hvab), INTENT(INOUT) :: vab
   REAL(kind=dp), DIMENSION(3), INTENT(IN) :: rp, ra

   ! LOCAL
   INTEGER :: ico, jco, lxa, lxb, lxp, lxyz, lyp, lzp
   REAL(KIND=dp) :: a, b, binomial_k_lxa, binomial_l_lxb, const_ra

   REAL(kind=dp) :: alpha_1(0:(4+0), 0:4, 0:0)
   REAL(kind=dp) :: alpha_2(0:(4+0), 0:4, 0:0)
   REAL(kind=dp) :: alpha_3(0:(4+0), 0:4, 0:0)
   REAL(kind=dp) :: coef_ttz(0:4, 0:0)
   REAL(kind=dp) :: coef_tyz(0:4, 0:0, 0:4, 0:0)
   coef_xyz = coef_xyz*prefactor

   alpha_1(:, :, :) = 0.0_dp
   alpha_2(:, :, :) = 0.0_dp
   alpha_3(:, :, :) = 0.0_dp

   const_ra = (-ra(1)+rp(1))
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(0, 0, 0) = alpha_1(0, 0, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(1, 1, 0) = alpha_1(1, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(0, 1, 0) = alpha_1(0, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(2, 2, 0) = alpha_1(2, 2, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(1, 2, 0) = alpha_1(1, 2, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(1, dp)/REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(0, 2, 0) = alpha_1(0, 2, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(3, 3, 0) = alpha_1(3, 3, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(3, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(2, 3, 0) = alpha_1(2, 3, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(1, 3, 0) = alpha_1(1, 3, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(1, dp)/REAL(3, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(0, 3, 0) = alpha_1(0, 3, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(4, 4, 0) = alpha_1(4, 4, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(4, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(3, 4, 0) = alpha_1(3, 4, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(3, dp)/REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(2, 4, 0) = alpha_1(2, 4, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(2, dp)/REAL(3, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(1, 4, 0) = alpha_1(1, 4, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(1, dp)/REAL(4, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_1(0, 4, 0) = alpha_1(0, 4, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   const_ra = (-ra(2)+rp(2))
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(0, 0, 0) = alpha_2(0, 0, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(1, 1, 0) = alpha_2(1, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(0, 1, 0) = alpha_2(0, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(2, 2, 0) = alpha_2(2, 2, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(1, 2, 0) = alpha_2(1, 2, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(1, dp)/REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(0, 2, 0) = alpha_2(0, 2, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(3, 3, 0) = alpha_2(3, 3, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(3, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(2, 3, 0) = alpha_2(2, 3, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(1, 3, 0) = alpha_2(1, 3, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(1, dp)/REAL(3, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(0, 3, 0) = alpha_2(0, 3, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(4, 4, 0) = alpha_2(4, 4, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(4, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(3, 4, 0) = alpha_2(3, 4, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(3, dp)/REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(2, 4, 0) = alpha_2(2, 4, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(2, dp)/REAL(3, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(1, 4, 0) = alpha_2(1, 4, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(1, dp)/REAL(4, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_2(0, 4, 0) = alpha_2(0, 4, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   const_ra = (-ra(3)+rp(3))
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(0, 0, 0) = alpha_3(0, 0, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(1, 1, 0) = alpha_3(1, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(0, 1, 0) = alpha_3(0, 1, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(2, 2, 0) = alpha_3(2, 2, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(1, 2, 0) = alpha_3(1, 2, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(1, dp)/REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(0, 2, 0) = alpha_3(0, 2, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(3, 3, 0) = alpha_3(3, 3, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(3, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(2, 3, 0) = alpha_3(2, 3, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(1, 3, 0) = alpha_3(1, 3, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(1, dp)/REAL(3, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(0, 3, 0) = alpha_3(0, 3, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0
   binomial_k_lxa = 1.0_dp
   a = 1.0_dp
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(4, 4, 0) = alpha_3(4, 4, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(4, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(3, 4, 0) = alpha_3(3, 4, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(3, dp)/REAL(2, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(2, 4, 0) = alpha_3(2, 4, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(2, dp)/REAL(3, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(1, 4, 0) = alpha_3(1, 4, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   binomial_k_lxa = binomial_k_lxa*REAL(1, dp)/REAL(4, dp)
   a = a*const_ra
   binomial_l_lxb = 1.0_dp
   b = 1.0_dp
   alpha_3(0, 4, 0) = alpha_3(0, 4, 0)+ &
                      binomial_k_lxa*binomial_l_lxb*a*b
   ! Removed here as lxb - l = 0
   ! Removed  here as lxa-k == 0

   lxyz = 0
   DO lzp = 0, 4+0
      coef_tyz = 0.0_dp
      DO lyp = 0, 4+0-lzp
         coef_ttz = 0.0_dp
         DO lxp = 0, 4+0-lzp-lyp
            lxyz = lxyz+1
            coef_ttz(0, 0) = coef_ttz(0, 0)+coef_xyz(lxyz)*alpha_1(lxp, 0, 0)
            coef_ttz(1, 0) = coef_ttz(1, 0)+coef_xyz(lxyz)*alpha_1(lxp, 1, 0)
            coef_ttz(2, 0) = coef_ttz(2, 0)+coef_xyz(lxyz)*alpha_1(lxp, 2, 0)
            coef_ttz(3, 0) = coef_ttz(3, 0)+coef_xyz(lxyz)*alpha_1(lxp, 3, 0)
            coef_ttz(4, 0) = coef_ttz(4, 0)+coef_xyz(lxyz)*alpha_1(lxp, 4, 0)
         ENDDO

         coef_tyz(0, 0, 0, 0) = coef_tyz(0, 0, 0, 0)+coef_ttz(0, 0)*alpha_2(lyp, 0, 0)
         coef_tyz(1, 0, 0, 0) = coef_tyz(1, 0, 0, 0)+coef_ttz(1, 0)*alpha_2(lyp, 0, 0)
         coef_tyz(2, 0, 0, 0) = coef_tyz(2, 0, 0, 0)+coef_ttz(2, 0)*alpha_2(lyp, 0, 0)
         coef_tyz(3, 0, 0, 0) = coef_tyz(3, 0, 0, 0)+coef_ttz(3, 0)*alpha_2(lyp, 0, 0)
         coef_tyz(4, 0, 0, 0) = coef_tyz(4, 0, 0, 0)+coef_ttz(4, 0)*alpha_2(lyp, 0, 0)
         coef_tyz(0, 0, 1, 0) = coef_tyz(0, 0, 1, 0)+coef_ttz(0, 0)*alpha_2(lyp, 1, 0)
         coef_tyz(1, 0, 1, 0) = coef_tyz(1, 0, 1, 0)+coef_ttz(1, 0)*alpha_2(lyp, 1, 0)
         coef_tyz(2, 0, 1, 0) = coef_tyz(2, 0, 1, 0)+coef_ttz(2, 0)*alpha_2(lyp, 1, 0)
         coef_tyz(3, 0, 1, 0) = coef_tyz(3, 0, 1, 0)+coef_ttz(3, 0)*alpha_2(lyp, 1, 0)
         coef_tyz(0, 0, 2, 0) = coef_tyz(0, 0, 2, 0)+coef_ttz(0, 0)*alpha_2(lyp, 2, 0)
         coef_tyz(1, 0, 2, 0) = coef_tyz(1, 0, 2, 0)+coef_ttz(1, 0)*alpha_2(lyp, 2, 0)
         coef_tyz(2, 0, 2, 0) = coef_tyz(2, 0, 2, 0)+coef_ttz(2, 0)*alpha_2(lyp, 2, 0)
         coef_tyz(0, 0, 3, 0) = coef_tyz(0, 0, 3, 0)+coef_ttz(0, 0)*alpha_2(lyp, 3, 0)
         coef_tyz(1, 0, 3, 0) = coef_tyz(1, 0, 3, 0)+coef_ttz(1, 0)*alpha_2(lyp, 3, 0)
         coef_tyz(0, 0, 4, 0) = coef_tyz(0, 0, 4, 0)+coef_ttz(0, 0)*alpha_2(lyp, 4, 0)
      ENDDO

      lxb = 0
      !jco=coset(0,0,0)
      jco = 1

      DO lxa = la_min_local, 4

         ico = coset(lxa, 0, 0)

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 0, 0)

      ENDDO

      lxb = 0
      !jco=coset(0,0,0)
      jco = 1

      DO lxa = MAX(la_min_local-0-1, 0), 3

         ico = coset(lxa, 1, 0)

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 1, 0)*alpha_3(lzp, 0, 0)

      ENDDO

      lxb = 0
      !jco=coset(0,0,0)
      jco = 1

      DO lxa = MAX(la_min_local-0-2, 0), 2

         ico = coset(lxa, 2, 0)

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 2, 0)*alpha_3(lzp, 0, 0)

      ENDDO

      lxb = 0
      !jco=coset(0,0,0)
      jco = 1

      DO lxa = MAX(la_min_local-0-3, 0), 1

         ico = coset(lxa, 3, 0)

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 3, 0)*alpha_3(lzp, 0, 0)

      ENDDO

      lxb = 0
      !jco=coset(0,0,0)
      jco = 1

      lxa = 0
      !ico=coset(0,4,0)
      ico = 31

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 4, 0)*alpha_3(lzp, 0, 0)

      lxb = 0
      !jco=coset(0,0,0)
      jco = 1

      DO lxa = MAX(la_min_local-1-0, 0), 3

         ico = coset(lxa, 0, 1)

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 1, 0)

      ENDDO

      lxb = 0
      !jco=coset(0,0,0)
      jco = 1

      DO lxa = MAX(la_min_local-1-1, 0), 2

         ico = coset(lxa, 1, 1)

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 1, 0)*alpha_3(lzp, 1, 0)

      ENDDO

      lxb = 0
      !jco=coset(0,0,0)
      jco = 1

      DO lxa = MAX(la_min_local-1-2, 0), 1

         ico = coset(lxa, 2, 1)

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 2, 0)*alpha_3(lzp, 1, 0)

      ENDDO

      lxb = 0
      !jco=coset(0,0,0)
      jco = 1

      lxa = 0
      !ico=coset(0,3,1)
      ico = 32

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 3, 0)*alpha_3(lzp, 1, 0)

      lxb = 0
      !jco=coset(0,0,0)
      jco = 1

      DO lxa = MAX(la_min_local-2-0, 0), 2

         ico = coset(lxa, 0, 2)

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 2, 0)

      ENDDO

      lxb = 0
      !jco=coset(0,0,0)
      jco = 1

      DO lxa = MAX(la_min_local-2-1, 0), 1

         ico = coset(lxa, 1, 2)

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 1, 0)*alpha_3(lzp, 2, 0)

      ENDDO

      lxb = 0
      !jco=coset(0,0,0)
      jco = 1

      lxa = 0
      !ico=coset(0,2,2)
      ico = 33

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 2, 0)*alpha_3(lzp, 2, 0)

      lxb = 0
      !jco=coset(0,0,0)
      jco = 1

      DO lxa = MAX(la_min_local-3-0, 0), 1

         ico = coset(lxa, 0, 3)

         vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 3, 0)

      ENDDO

      lxb = 0
      !jco=coset(0,0,0)
      jco = 1

      lxa = 0
      !ico=coset(0,1,3)
      ico = 34

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 1, 0)*alpha_3(lzp, 3, 0)

      lxb = 0
      !jco=coset(0,0,0)
      jco = 1

      lxa = 0
      !ico=coset(0,0,4)
      ico = 35

      vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, 0, 0)*alpha_3(lzp, 4, 0)

   ENDDO

END SUBROUTINE xyz_to_vab_4_0

! Partial unrolled version
! **************************************************************************************************
!> \brief ...
!> \param prefactor ...
!> \param coef_xyz ...
!> \param lp ...
!> \param rp ...
!> \param ra ...
!> \param rab ...
!> \param vab ...
!> \param coset ...
!> \param la_min_local ...
!> \param lb_min_local ...
!> \param maxl ...
!> \param lvab ...
!> \param hvab ...
! **************************************************************************************************
SUBROUTINE xyz_to_vab_4_1(prefactor, coef_xyz, &
                          lp, rp, ra, rab, vab, coset, &
                          la_min_local, lb_min_local, maxl, lvab, hvab)

      USE kinds,                        ONLY: dp
   ! PARAMETERS
   INTEGER, INTENT(IN) :: lp, &
                          maxl, lb_min_local, la_min_local, &
                          lvab, hvab

   INTEGER, DIMENSION(-1:maxl, -1:maxl, -1:maxl), &
      INTENT(IN)     :: coset

   REAL(KIND=dp), INTENT(IN) :: prefactor

   REAL(kind=dp), DIMENSION(((lp+1)*(lp+2)*(lp+3))/6), &
      INTENT(INOUT) :: coef_xyz
   REAL(kind=dp), DIMENSION(lvab, hvab), INTENT(INOUT) :: vab
   REAL(kind=dp), DIMENSION(3), INTENT(IN) :: rp, ra, rab

   ! LOCAL
   INTEGER :: ico, jco, lxa, lxb, lxp, lxyz, lyp, lzp, l, k, lya, lyb, lza, lzb
   REAL(KIND=dp) :: a, b, binomial_k_lxa, binomial_l_lxb

   REAL(kind=dp) :: alpha(0:(4+1), 0:4, 0:1, 3)
   REAL(kind=dp) :: coef_ttz(0:4, 0:1)
   REAL(kind=dp) :: coef_tyz(0:4, 0:1, 0:4, 0:1)

   coef_xyz = coef_xyz*prefactor

   alpha(:, :, :, :) = 0.0_dp

   DO lxa = 0, 4
   DO lxb = 0, 1
      binomial_k_lxa = 1.0_dp
      a = 1.0_dp
      DO k = 0, lxa
         binomial_l_lxb = 1.0_dp
         b = 1.0_dp
         DO l = 0, lxb
            alpha(lxa-l+lxb-k, lxa, lxb, 1) = alpha(lxa-l+lxb-k, lxa, lxb, 1)+ &
                                              binomial_k_lxa*binomial_l_lxb*a*b
            binomial_l_lxb = binomial_l_lxb*REAL(lxb-l, dp)/REAL(l+1, dp)
            b = b*(rp(1)-(ra(1)+rab(1)))
         ENDDO
         binomial_k_lxa = binomial_k_lxa*REAL(lxa-k, dp)/REAL(k+1, dp)
         a = a*(-ra(1)+rp(1))
      ENDDO
   ENDDO
   ENDDO
   DO lxa = 0, 4
   DO lxb = 0, 1
      binomial_k_lxa = 1.0_dp
      a = 1.0_dp
      DO k = 0, lxa
         binomial_l_lxb = 1.0_dp
         b = 1.0_dp
         DO l = 0, lxb
            alpha(lxa-l+lxb-k, lxa, lxb, 2) = alpha(lxa-l+lxb-k, lxa, lxb, 2)+ &
                                              binomial_k_lxa*binomial_l_lxb*a*b
            binomial_l_lxb = binomial_l_lxb*REAL(lxb-l, dp)/REAL(l+1, dp)
            b = b*(rp(2)-(ra(2)+rab(2)))
         ENDDO
         binomial_k_lxa = binomial_k_lxa*REAL(lxa-k, dp)/REAL(k+1, dp)
         a = a*(-ra(2)+rp(2))
      ENDDO
   ENDDO
   ENDDO
   DO lxa = 0, 4
   DO lxb = 0, 1
      binomial_k_lxa = 1.0_dp
      a = 1.0_dp
      DO k = 0, lxa
         binomial_l_lxb = 1.0_dp
         b = 1.0_dp
         DO l = 0, lxb
            alpha(lxa-l+lxb-k, lxa, lxb, 3) = alpha(lxa-l+lxb-k, lxa, lxb, 3)+ &
                                              binomial_k_lxa*binomial_l_lxb*a*b
            binomial_l_lxb = binomial_l_lxb*REAL(lxb-l, dp)/REAL(l+1, dp)
            b = b*(rp(3)-(ra(3)+rab(3)))
         ENDDO
         binomial_k_lxa = binomial_k_lxa*REAL(lxa-k, dp)/REAL(k+1, dp)
         a = a*(-ra(3)+rp(3))
      ENDDO
   ENDDO
   ENDDO

   lxyz = 0
   DO lzp = 0, 5
      coef_tyz = 0.0_dp
      DO lyp = 0, 5-lzp
         coef_ttz = 0.0_dp
         DO lxp = 0, 5-lzp-lyp
            lxyz = lxyz+1
            DO lxb = 0, 1
            DO lxa = 0, 4
               coef_ttz(lxa, lxb) = coef_ttz(lxa, lxb)+coef_xyz(lxyz)*alpha(lxp, lxa, lxb, 1)
            ENDDO
            ENDDO

         ENDDO ! lxp

         DO lyb = 0, 1
         DO lya = 0, 4
            DO lxb = 0, 1-lyb
            DO lxa = 0, 4-lya
               coef_tyz(lxa, lxb, lya, lyb) = coef_tyz(lxa, lxb, lya, lyb)+coef_ttz(lxa, lxb)*alpha(lyp, lya, lyb, 2)
            ENDDO
            ENDDO
         ENDDO
         ENDDO

      ENDDO !lyp

      DO lzb = 0, 1
      DO lza = 0, 4
         DO lyb = 0, 1-lzb
         DO lya = 0, 4-lza
            DO lxb = MAX(lb_min_local-lzb-lyb, 0), 1-lzb-lyb
               jco = coset(lxb, lyb, lzb)
               DO lxa = MAX(la_min_local-lza-lya, 0), 4-lza-lya
                  ico = coset(lxa, lya, lza)
                  vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, lya, lyb)*alpha(lzp, lza, lzb, 3)
               ENDDO
            ENDDO
         ENDDO
         ENDDO
      ENDDO
      ENDDO

   ENDDO

END SUBROUTINE xyz_to_vab_4_1

! Partial unrolled version
! **************************************************************************************************
!> \brief ...
!> \param prefactor ...
!> \param coef_xyz ...
!> \param lp ...
!> \param rp ...
!> \param ra ...
!> \param rab ...
!> \param vab ...
!> \param coset ...
!> \param la_min_local ...
!> \param lb_min_local ...
!> \param maxl ...
!> \param lvab ...
!> \param hvab ...
! **************************************************************************************************
SUBROUTINE xyz_to_vab_4_2(prefactor, coef_xyz, &
                          lp, rp, ra, rab, vab, coset, &
                          la_min_local, lb_min_local, maxl, lvab, hvab)

      USE kinds,                        ONLY: dp
   ! PARAMETERS
   INTEGER, INTENT(IN) :: lp, &
                          maxl, lb_min_local, la_min_local, &
                          lvab, hvab

   INTEGER, DIMENSION(-1:maxl, -1:maxl, -1:maxl), &
      INTENT(IN)     :: coset

   REAL(KIND=dp), INTENT(IN) :: prefactor

   REAL(kind=dp), DIMENSION(((lp+1)*(lp+2)*(lp+3))/6), &
      INTENT(INOUT) :: coef_xyz
   REAL(kind=dp), DIMENSION(lvab, hvab), INTENT(INOUT) :: vab
   REAL(kind=dp), DIMENSION(3), INTENT(IN) :: rp, ra, rab

   ! LOCAL
   INTEGER :: ico, jco, lxa, lxb, lxp, lxyz, lyp, lzp, l, k, lya, lyb, lza, lzb
   REAL(KIND=dp) :: a, b, binomial_k_lxa, binomial_l_lxb

   REAL(kind=dp) :: alpha(0:(4+2), 0:4, 0:2, 3)
   REAL(kind=dp) :: coef_ttz(0:4, 0:2)
   REAL(kind=dp) :: coef_tyz(0:4, 0:2, 0:4, 0:2)

   coef_xyz = coef_xyz*prefactor

   alpha(:, :, :, :) = 0.0_dp

   DO lxa = 0, 4
   DO lxb = 0, 2
      binomial_k_lxa = 1.0_dp
      a = 1.0_dp
      DO k = 0, lxa
         binomial_l_lxb = 1.0_dp
         b = 1.0_dp
         DO l = 0, lxb
            alpha(lxa-l+lxb-k, lxa, lxb, 1) = alpha(lxa-l+lxb-k, lxa, lxb, 1)+ &
                                              binomial_k_lxa*binomial_l_lxb*a*b
            binomial_l_lxb = binomial_l_lxb*REAL(lxb-l, dp)/REAL(l+1, dp)
            b = b*(rp(1)-(ra(1)+rab(1)))
         ENDDO
         binomial_k_lxa = binomial_k_lxa*REAL(lxa-k, dp)/REAL(k+1, dp)
         a = a*(-ra(1)+rp(1))
      ENDDO
   ENDDO
   ENDDO
   DO lxa = 0, 4
   DO lxb = 0, 2
      binomial_k_lxa = 1.0_dp
      a = 1.0_dp
      DO k = 0, lxa
         binomial_l_lxb = 1.0_dp
         b = 1.0_dp
         DO l = 0, lxb
            alpha(lxa-l+lxb-k, lxa, lxb, 2) = alpha(lxa-l+lxb-k, lxa, lxb, 2)+ &
                                              binomial_k_lxa*binomial_l_lxb*a*b
            binomial_l_lxb = binomial_l_lxb*REAL(lxb-l, dp)/REAL(l+1, dp)
            b = b*(rp(2)-(ra(2)+rab(2)))
         ENDDO
         binomial_k_lxa = binomial_k_lxa*REAL(lxa-k, dp)/REAL(k+1, dp)
         a = a*(-ra(2)+rp(2))
      ENDDO
   ENDDO
   ENDDO
   DO lxa = 0, 4
   DO lxb = 0, 2
      binomial_k_lxa = 1.0_dp
      a = 1.0_dp
      DO k = 0, lxa
         binomial_l_lxb = 1.0_dp
         b = 1.0_dp
         DO l = 0, lxb
            alpha(lxa-l+lxb-k, lxa, lxb, 3) = alpha(lxa-l+lxb-k, lxa, lxb, 3)+ &
                                              binomial_k_lxa*binomial_l_lxb*a*b
            binomial_l_lxb = binomial_l_lxb*REAL(lxb-l, dp)/REAL(l+1, dp)
            b = b*(rp(3)-(ra(3)+rab(3)))
         ENDDO
         binomial_k_lxa = binomial_k_lxa*REAL(lxa-k, dp)/REAL(k+1, dp)
         a = a*(-ra(3)+rp(3))
      ENDDO
   ENDDO
   ENDDO

   lxyz = 0
   DO lzp = 0, 6
      coef_tyz = 0.0_dp
      DO lyp = 0, 6-lzp
         coef_ttz = 0.0_dp
         DO lxp = 0, 6-lzp-lyp
            lxyz = lxyz+1
            DO lxb = 0, 2
            DO lxa = 0, 4
               coef_ttz(lxa, lxb) = coef_ttz(lxa, lxb)+coef_xyz(lxyz)*alpha(lxp, lxa, lxb, 1)
            ENDDO
            ENDDO

         ENDDO ! lxp

         DO lyb = 0, 2
         DO lya = 0, 4
            DO lxb = 0, 2-lyb
            DO lxa = 0, 4-lya
               coef_tyz(lxa, lxb, lya, lyb) = coef_tyz(lxa, lxb, lya, lyb)+coef_ttz(lxa, lxb)*alpha(lyp, lya, lyb, 2)
            ENDDO
            ENDDO
         ENDDO
         ENDDO

      ENDDO !lyp

      DO lzb = 0, 2
      DO lza = 0, 4
         DO lyb = 0, 2-lzb
         DO lya = 0, 4-lza
            DO lxb = MAX(lb_min_local-lzb-lyb, 0), 2-lzb-lyb
               jco = coset(lxb, lyb, lzb)
               DO lxa = MAX(la_min_local-lza-lya, 0), 4-lza-lya
                  ico = coset(lxa, lya, lza)
                  vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, lya, lyb)*alpha(lzp, lza, lzb, 3)
               ENDDO
            ENDDO
         ENDDO
         ENDDO
      ENDDO
      ENDDO

   ENDDO

END SUBROUTINE xyz_to_vab_4_2

! Partial unrolled version
! **************************************************************************************************
!> \brief ...
!> \param prefactor ...
!> \param coef_xyz ...
!> \param lp ...
!> \param rp ...
!> \param ra ...
!> \param rab ...
!> \param vab ...
!> \param coset ...
!> \param la_min_local ...
!> \param lb_min_local ...
!> \param maxl ...
!> \param lvab ...
!> \param hvab ...
! **************************************************************************************************
SUBROUTINE xyz_to_vab_4_3(prefactor, coef_xyz, &
                          lp, rp, ra, rab, vab, coset, &
                          la_min_local, lb_min_local, maxl, lvab, hvab)

      USE kinds,                        ONLY: dp
   ! PARAMETERS
   INTEGER, INTENT(IN) :: lp, &
                          maxl, lb_min_local, la_min_local, &
                          lvab, hvab

   INTEGER, DIMENSION(-1:maxl, -1:maxl, -1:maxl), &
      INTENT(IN)     :: coset

   REAL(KIND=dp), INTENT(IN) :: prefactor

   REAL(kind=dp), DIMENSION(((lp+1)*(lp+2)*(lp+3))/6), &
      INTENT(INOUT) :: coef_xyz
   REAL(kind=dp), DIMENSION(lvab, hvab), INTENT(INOUT) :: vab
   REAL(kind=dp), DIMENSION(3), INTENT(IN) :: rp, ra, rab

   ! LOCAL
   INTEGER :: ico, jco, lxa, lxb, lxp, lxyz, lyp, lzp, l, k, lya, lyb, lza, lzb
   REAL(KIND=dp) :: a, b, binomial_k_lxa, binomial_l_lxb

   REAL(kind=dp) :: alpha(0:(4+3), 0:4, 0:3, 3)
   REAL(kind=dp) :: coef_ttz(0:4, 0:3)
   REAL(kind=dp) :: coef_tyz(0:4, 0:3, 0:4, 0:3)

   coef_xyz = coef_xyz*prefactor

   alpha(:, :, :, :) = 0.0_dp

   DO lxa = 0, 4
   DO lxb = 0, 3
      binomial_k_lxa = 1.0_dp
      a = 1.0_dp
      DO k = 0, lxa
         binomial_l_lxb = 1.0_dp
         b = 1.0_dp
         DO l = 0, lxb
            alpha(lxa-l+lxb-k, lxa, lxb, 1) = alpha(lxa-l+lxb-k, lxa, lxb, 1)+ &
                                              binomial_k_lxa*binomial_l_lxb*a*b
            binomial_l_lxb = binomial_l_lxb*REAL(lxb-l, dp)/REAL(l+1, dp)
            b = b*(rp(1)-(ra(1)+rab(1)))
         ENDDO
         binomial_k_lxa = binomial_k_lxa*REAL(lxa-k, dp)/REAL(k+1, dp)
         a = a*(-ra(1)+rp(1))
      ENDDO
   ENDDO
   ENDDO
   DO lxa = 0, 4
   DO lxb = 0, 3
      binomial_k_lxa = 1.0_dp
      a = 1.0_dp
      DO k = 0, lxa
         binomial_l_lxb = 1.0_dp
         b = 1.0_dp
         DO l = 0, lxb
            alpha(lxa-l+lxb-k, lxa, lxb, 2) = alpha(lxa-l+lxb-k, lxa, lxb, 2)+ &
                                              binomial_k_lxa*binomial_l_lxb*a*b
            binomial_l_lxb = binomial_l_lxb*REAL(lxb-l, dp)/REAL(l+1, dp)
            b = b*(rp(2)-(ra(2)+rab(2)))
         ENDDO
         binomial_k_lxa = binomial_k_lxa*REAL(lxa-k, dp)/REAL(k+1, dp)
         a = a*(-ra(2)+rp(2))
      ENDDO
   ENDDO
   ENDDO
   DO lxa = 0, 4
   DO lxb = 0, 3
      binomial_k_lxa = 1.0_dp
      a = 1.0_dp
      DO k = 0, lxa
         binomial_l_lxb = 1.0_dp
         b = 1.0_dp
         DO l = 0, lxb
            alpha(lxa-l+lxb-k, lxa, lxb, 3) = alpha(lxa-l+lxb-k, lxa, lxb, 3)+ &
                                              binomial_k_lxa*binomial_l_lxb*a*b
            binomial_l_lxb = binomial_l_lxb*REAL(lxb-l, dp)/REAL(l+1, dp)
            b = b*(rp(3)-(ra(3)+rab(3)))
         ENDDO
         binomial_k_lxa = binomial_k_lxa*REAL(lxa-k, dp)/REAL(k+1, dp)
         a = a*(-ra(3)+rp(3))
      ENDDO
   ENDDO
   ENDDO

   lxyz = 0
   DO lzp = 0, 7
      coef_tyz = 0.0_dp
      DO lyp = 0, 7-lzp
         coef_ttz = 0.0_dp
         DO lxp = 0, 7-lzp-lyp
            lxyz = lxyz+1
            DO lxb = 0, 3
            DO lxa = 0, 4
               coef_ttz(lxa, lxb) = coef_ttz(lxa, lxb)+coef_xyz(lxyz)*alpha(lxp, lxa, lxb, 1)
            ENDDO
            ENDDO

         ENDDO ! lxp

         DO lyb = 0, 3
         DO lya = 0, 4
            DO lxb = 0, 3-lyb
            DO lxa = 0, 4-lya
               coef_tyz(lxa, lxb, lya, lyb) = coef_tyz(lxa, lxb, lya, lyb)+coef_ttz(lxa, lxb)*alpha(lyp, lya, lyb, 2)
            ENDDO
            ENDDO
         ENDDO
         ENDDO

      ENDDO !lyp

      DO lzb = 0, 3
      DO lza = 0, 4
         DO lyb = 0, 3-lzb
         DO lya = 0, 4-lza
            DO lxb = MAX(lb_min_local-lzb-lyb, 0), 3-lzb-lyb
               jco = coset(lxb, lyb, lzb)
               DO lxa = MAX(la_min_local-lza-lya, 0), 4-lza-lya
                  ico = coset(lxa, lya, lza)
                  vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, lya, lyb)*alpha(lzp, lza, lzb, 3)
               ENDDO
            ENDDO
         ENDDO
         ENDDO
      ENDDO
      ENDDO

   ENDDO

END SUBROUTINE xyz_to_vab_4_3

! Partial unrolled version
! **************************************************************************************************
!> \brief ...
!> \param prefactor ...
!> \param coef_xyz ...
!> \param lp ...
!> \param rp ...
!> \param ra ...
!> \param rab ...
!> \param vab ...
!> \param coset ...
!> \param la_min_local ...
!> \param lb_min_local ...
!> \param maxl ...
!> \param lvab ...
!> \param hvab ...
! **************************************************************************************************
SUBROUTINE xyz_to_vab_4_4(prefactor, coef_xyz, &
                          lp, rp, ra, rab, vab, coset, &
                          la_min_local, lb_min_local, maxl, lvab, hvab)

      USE kinds,                        ONLY: dp
   ! PARAMETERS
   INTEGER, INTENT(IN) :: lp, &
                          maxl, lb_min_local, la_min_local, &
                          lvab, hvab

   INTEGER, DIMENSION(-1:maxl, -1:maxl, -1:maxl), &
      INTENT(IN)     :: coset

   REAL(KIND=dp), INTENT(IN) :: prefactor

   REAL(kind=dp), DIMENSION(((lp+1)*(lp+2)*(lp+3))/6), &
      INTENT(INOUT) :: coef_xyz
   REAL(kind=dp), DIMENSION(lvab, hvab), INTENT(INOUT) :: vab
   REAL(kind=dp), DIMENSION(3), INTENT(IN) :: rp, ra, rab

   ! LOCAL
   INTEGER :: ico, jco, lxa, lxb, lxp, lxyz, lyp, lzp, l, k, lya, lyb, lza, lzb
   REAL(KIND=dp) :: a, b, binomial_k_lxa, binomial_l_lxb

   REAL(kind=dp) :: alpha(0:(4+4), 0:4, 0:4, 3)
   REAL(kind=dp) :: coef_ttz(0:4, 0:4)
   REAL(kind=dp) :: coef_tyz(0:4, 0:4, 0:4, 0:4)

   coef_xyz = coef_xyz*prefactor

   alpha(:, :, :, :) = 0.0_dp

   DO lxa = 0, 4
   DO lxb = 0, 4
      binomial_k_lxa = 1.0_dp
      a = 1.0_dp
      DO k = 0, lxa
         binomial_l_lxb = 1.0_dp
         b = 1.0_dp
         DO l = 0, lxb
            alpha(lxa-l+lxb-k, lxa, lxb, 1) = alpha(lxa-l+lxb-k, lxa, lxb, 1)+ &
                                              binomial_k_lxa*binomial_l_lxb*a*b
            binomial_l_lxb = binomial_l_lxb*REAL(lxb-l, dp)/REAL(l+1, dp)
            b = b*(rp(1)-(ra(1)+rab(1)))
         ENDDO
         binomial_k_lxa = binomial_k_lxa*REAL(lxa-k, dp)/REAL(k+1, dp)
         a = a*(-ra(1)+rp(1))
      ENDDO
   ENDDO
   ENDDO
   DO lxa = 0, 4
   DO lxb = 0, 4
      binomial_k_lxa = 1.0_dp
      a = 1.0_dp
      DO k = 0, lxa
         binomial_l_lxb = 1.0_dp
         b = 1.0_dp
         DO l = 0, lxb
            alpha(lxa-l+lxb-k, lxa, lxb, 2) = alpha(lxa-l+lxb-k, lxa, lxb, 2)+ &
                                              binomial_k_lxa*binomial_l_lxb*a*b
            binomial_l_lxb = binomial_l_lxb*REAL(lxb-l, dp)/REAL(l+1, dp)
            b = b*(rp(2)-(ra(2)+rab(2)))
         ENDDO
         binomial_k_lxa = binomial_k_lxa*REAL(lxa-k, dp)/REAL(k+1, dp)
         a = a*(-ra(2)+rp(2))
      ENDDO
   ENDDO
   ENDDO
   DO lxa = 0, 4
   DO lxb = 0, 4
      binomial_k_lxa = 1.0_dp
      a = 1.0_dp
      DO k = 0, lxa
         binomial_l_lxb = 1.0_dp
         b = 1.0_dp
         DO l = 0, lxb
            alpha(lxa-l+lxb-k, lxa, lxb, 3) = alpha(lxa-l+lxb-k, lxa, lxb, 3)+ &
                                              binomial_k_lxa*binomial_l_lxb*a*b
            binomial_l_lxb = binomial_l_lxb*REAL(lxb-l, dp)/REAL(l+1, dp)
            b = b*(rp(3)-(ra(3)+rab(3)))
         ENDDO
         binomial_k_lxa = binomial_k_lxa*REAL(lxa-k, dp)/REAL(k+1, dp)
         a = a*(-ra(3)+rp(3))
      ENDDO
   ENDDO
   ENDDO

   lxyz = 0
   DO lzp = 0, 8
      coef_tyz = 0.0_dp
      DO lyp = 0, 8-lzp
         coef_ttz = 0.0_dp
         DO lxp = 0, 8-lzp-lyp
            lxyz = lxyz+1
            DO lxb = 0, 4
            DO lxa = 0, 4
               coef_ttz(lxa, lxb) = coef_ttz(lxa, lxb)+coef_xyz(lxyz)*alpha(lxp, lxa, lxb, 1)
            ENDDO
            ENDDO

         ENDDO ! lxp

         DO lyb = 0, 4
         DO lya = 0, 4
            DO lxb = 0, 4-lyb
            DO lxa = 0, 4-lya
               coef_tyz(lxa, lxb, lya, lyb) = coef_tyz(lxa, lxb, lya, lyb)+coef_ttz(lxa, lxb)*alpha(lyp, lya, lyb, 2)
            ENDDO
            ENDDO
         ENDDO
         ENDDO

      ENDDO !lyp

      DO lzb = 0, 4
      DO lza = 0, 4
         DO lyb = 0, 4-lzb
         DO lya = 0, 4-lza
            DO lxb = MAX(lb_min_local-lzb-lyb, 0), 4-lzb-lyb
               jco = coset(lxb, lyb, lzb)
               DO lxa = MAX(la_min_local-lza-lya, 0), 4-lza-lya
                  ico = coset(lxa, lya, lza)
                  vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, lya, lyb)*alpha(lzp, lza, lzb, 3)
               ENDDO
            ENDDO
         ENDDO
         ENDDO
      ENDDO
      ENDDO

   ENDDO

END SUBROUTINE xyz_to_vab_4_4

!!! End of cloned routines

! Original routine
! **************************************************************************************************
!> \brief ...
!> \param prefactor ...
!> \param coef_xyz ...
!> \param lp ...
!> \param la_max_local ...
!> \param lb_max_local ...
!> \param rp ...
!> \param ra ...
!> \param rab ...
!> \param vab ...
!> \param coset ...
!> \param la_min_local ...
!> \param lb_min_local ...
!> \param maxl ...
!> \param lvab ...
!> \param hvab ...
! **************************************************************************************************
SUBROUTINE xyz_to_vab(prefactor, coef_xyz, lp, la_max_local, &
                      lb_max_local, rp, ra, rab, vab, coset, &
                      la_min_local, lb_min_local, maxl, &
                      lvab, hvab)

              USE kinds,                        ONLY: dp

   IMPLICIT NONE

   ! PARAMETERS
   REAL(KIND=dp), INTENT(IN) :: prefactor
   INTEGER, INTENT(IN) :: lp, la_max_local, lb_max_local, &
                          la_min_local, lb_min_local, &
                          maxl, lvab, hvab
   INTEGER, DIMENSION(-1:maxl, -1:maxl, -1:maxl), &
      INTENT(IN)     :: coset

   REAL(kind=dp), DIMENSION(((lp+1)*(lp+2)*(lp+3))/6), &
      INTENT(INOUT) :: coef_xyz
   REAL(kind=dp), DIMENSION(3), INTENT(IN) :: rp, ra, rab
   REAL(kind=dp), DIMENSION(lvab, hvab), INTENT(INOUT) :: vab

   ! LOCAL
   INTEGER :: ico, jco, lxa, lxb, lxp, lxyz, lyp, lzp, iaxis, l, k, &
              lya, lyb, lza, lzb
   REAL(KIND=dp) :: a, b, binomial_k_lxa, binomial_l_lxb

   REAL(kind=dp), DIMENSION(:, :, :, :), ALLOCATABLE :: alpha
   REAL(kind=dp), DIMENSION(:, :), ALLOCATABLE :: coef_ttz
   REAL(kind=dp), DIMENSION(:, :, :, :), ALLOCATABLE :: coef_tyz

   coef_xyz = coef_xyz*prefactor

   !   *** initialise the coefficient matrix, we transform the sum
   !
   !   sum_{lxa,lya,lza,lxb,lyb,lzb} P_{lxa,lya,lza,lxb,lyb,lzb} *
   !                  (x-a_x)**lxa (y-a_y)**lya (z-a_z)**lza (x-b_x)**lxb (y-a_y)**lya (z-a_z)**lza
   !
   !   into
   !
   !   sum_{lxp,lyp,lzp} P_{lxp,lyp,lzp} (x-p_x)**lxp (y-p_y)**lyp (z-p_z)**lzp
   !
   !   where p is center of the product gaussian, and lp = la_max + lb_max
   !   (current implementation is l**7)
   !
   !
   !   compute polynomial expansion coefs -> (x-a)**lxa (x-b)**lxb -> sum_{ls} alpha(ls,lxa,lxb,1)*(x-p)**ls
   !
   !   *** make the alpha matrix ***
   ALLOCATE (alpha(0:lp, 0:la_max_local, 0:lb_max_local, 3))
   alpha(:, :, :, :) = 0.0_dp
   DO iaxis = 1, 3
   DO lxa = 0, la_max_local
   DO lxb = 0, lb_max_local
      binomial_k_lxa = 1.0_dp
      a = 1.0_dp
      DO k = 0, lxa
         binomial_l_lxb = 1.0_dp
         b = 1.0_dp
         DO l = 0, lxb
            alpha(lxa-l+lxb-k, lxa, lxb, iaxis) = alpha(lxa-l+lxb-k, lxa, lxb, iaxis)+ &
                                                  binomial_k_lxa*binomial_l_lxb*a*b
            binomial_l_lxb = binomial_l_lxb*REAL(lxb-l, dp)/REAL(l+1, dp)
            b = b*(rp(iaxis)-(ra(iaxis)+rab(iaxis)))
         ENDDO
         binomial_k_lxa = binomial_k_lxa*REAL(lxa-k, dp)/REAL(k+1, dp)
         a = a*(-ra(iaxis)+rp(iaxis))
      ENDDO
   ENDDO
   ENDDO
   ENDDO

   !
   !   compute v_{lxa,lya,lza,lxb,lyb,lzb} given v_{lxp,lyp,lzp} and alpha(ls,lxa,lxb,1)
   !   use a three step procedure
   !

   ALLOCATE (coef_ttz(0:la_max_local, 0:lb_max_local))
   ALLOCATE (coef_tyz(0:la_max_local, 0:lb_max_local, 0:la_max_local, 0:lb_max_local))
   lxyz = 0
   DO lzp = 0, lp
      coef_tyz = 0.0_dp
      DO lyp = 0, lp-lzp
         coef_ttz = 0.0_dp
         DO lxp = 0, lp-lzp-lyp
            lxyz = lxyz+1
            DO lxb = 0, lb_max_local
            DO lxa = 0, la_max_local
               coef_ttz(lxa, lxb) = coef_ttz(lxa, lxb)+coef_xyz(lxyz)*alpha(lxp, lxa, lxb, 1)
            ENDDO
            ENDDO

         ENDDO

         DO lyb = 0, lb_max_local
         DO lya = 0, la_max_local
            DO lxb = 0, lb_max_local-lyb
            DO lxa = 0, la_max_local-lya
               coef_tyz(lxa, lxb, lya, lyb) = coef_tyz(lxa, lxb, lya, lyb)+coef_ttz(lxa, lxb)*alpha(lyp, lya, lyb, 2)
            ENDDO
            ENDDO
         ENDDO
         ENDDO

      ENDDO

      DO lzb = 0, lb_max_local
      DO lza = 0, la_max_local
         DO lyb = 0, lb_max_local-lzb
         DO lya = 0, la_max_local-lza
            DO lxb = MAX(lb_min_local-lzb-lyb, 0), lb_max_local-lzb-lyb
               jco = coset(lxb, lyb, lzb)
               DO lxa = MAX(la_min_local-lza-lya, 0), la_max_local-lza-lya
                  ico = coset(lxa, lya, lza)
                  vab(ico, jco) = vab(ico, jco)+coef_tyz(lxa, lxb, lya, lyb)*alpha(lzp, lza, lzb, 3)
               ENDDO
            ENDDO
         ENDDO
         ENDDO
      ENDDO
      ENDDO

   ENDDO

   DEALLOCATE (coef_tyz)
   DEALLOCATE (coef_ttz)
   DEALLOCATE (alpha)

END SUBROUTINE xyz_to_vab

